@@ -9,6 +9,7 @@
 #include <osg/Texture2D>
 #include <osg/Texture3D>
 #include <osg/BlendFunc>
+#include <osg/Timer>
 
 #include <osgDB/Registry>
 #include <osgDB/ReadFile>
@@ -19,7 +20,7 @@
 #include <osgUtil/Optimizer>
 #include <osgUtil/SmoothingVisitor>
 
-#include <osgProducer/Viewer>
+#include <osgViewer/GraphicsWindow>
 
 #include <iostream>
 
@@ -27,24 +28,49 @@
 
 typedef std::vector<std::string> FileNameList;
 
-class GraphicsContext {
+class MyGraphicsContext {
     public:
-        GraphicsContext()
+        MyGraphicsContext()
         {
-            rs = new Producer::RenderSurface;
-            rs->setWindowRectangle(0,0,1,1);
-            rs->useBorder(false);
-            rs->useConfigEventThread(false);
-            rs->realize();
-            std::cout<<"Realized window"<<std::endl;
-        }
+            osg::ref_ptr<osg::GraphicsContext::Traits> traits = new osg::GraphicsContext::Traits;
+            traits->x = 0;
+            traits->y = 0;
+            traits->width = 1;
+            traits->height = 1;
+            traits->windowDecoration = false;
+            traits->doubleBuffer = false;
+            traits->sharedContext = 0;
+            traits->pbuffer = true;
 
-        virtual ~GraphicsContext()
-        {
+            _gc = osg::GraphicsContext::createGraphicsContext(traits.get());
+
+            if (!_gc)
+            {
+                osg::notify(osg::NOTICE)<<"Failed to create pbuffer, failing back to normal graphics window."<<std::endl;
+                
+                traits->pbuffer = false;
+                _gc = osg::GraphicsContext::createGraphicsContext(traits.get());
+            }
+
+            if (_gc.valid()) 
+            {
+                _gc->realize();
+                _gc->makeCurrent();
+                if (dynamic_cast<osgViewer::GraphicsWindow*>(_gc.get()))
+                {
+                    std::cout<<"Realized graphics window for OpenGL operations."<<std::endl;
+                }
+                else
+                {
+                    std::cout<<"Realized pbuffer for OpenGL operations."<<std::endl;
+                }
+            }
         }
         
+        bool valid() const { return _gc.valid() && _gc->isRealized(); }
+        
     private:
-        Producer::ref_ptr<Producer::RenderSurface> rs;
+        osg::ref_ptr<osg::GraphicsContext> _gc;
 };
 
 class CompressTexturesVisitor : public osg::NodeVisitor
@@ -89,7 +115,12 @@ public:
     
     void compress()
     {
-        GraphicsContext context;
+        MyGraphicsContext context;
+        if (!context.valid())
+        {
+            osg::notify(osg::NOTICE)<<"Error: Unable to create graphis context - cannot run compression"<<std::endl;
+            return;
+        }
 
         osg::ref_ptr<osg::State> state = new osg::State;
 
@@ -141,7 +172,7 @@ public:
     {
         NO_TRANSPARANCY_FIXING,
         MAKE_OPAQUE_TEXTURE_STATESET_OPAQUE,
-        MAKE_ALL_STATESET_OPAQUE,
+        MAKE_ALL_STATESET_OPAQUE
     };
 
     FixTransparencyVisitor(FixTransparencyMode mode=MAKE_OPAQUE_TEXTURE_STATESET_OPAQUE):
@@ -580,8 +611,17 @@ int main( int argc, char **argv )
         fileNames.pop_back();
     }
 
+    osg::Timer_t startTick = osg::Timer::instance()->tick();
+
     osg::ref_ptr<osg::Node> root = osgDB::readNodeFiles(fileNames);
 
+    if (root.valid())
+    {
+        osg::Timer_t endTick = osg::Timer::instance()->tick();
+        osg::notify(osg::INFO)<<"Time to load files "<<osg::Timer::instance()->delta_m(startTick, endTick)<<" ms"<<std::endl;
+    }
+
+
     if (pruneStateSet)
     {
         PruneStateSetVisitor pssv;
@@ -630,11 +670,20 @@ int main( int argc, char **argv )
                 std::cout<<"Warning: compressing texture only supported when outputing to .ive"<<std::endl;
             }
         }
-
-        if (osgDB::writeNodeFile(*root,fileNameOut))
+        
+        osgDB::ReaderWriter::WriteResult result = osgDB::Registry::instance()->writeNode(*root,fileNameOut,osgDB::Registry::instance()->getOptions());
+        if (result.success())
         {
             osg::notify(osg::NOTICE)<<"Data written to '"<<fileNameOut<<"'."<< std::endl;
         }
+        else if  (result.message().empty())
+        {
+            osg::notify(osg::NOTICE)<<"Warning: file write to '"<<fileNameOut<<"' no supported."<< std::endl;
+        }
+        else
+        {
+            osg::notify(osg::NOTICE)<<result.message()<< std::endl;
+        }
     }
     else
     {
