@@ -1,6 +1,7 @@
 #include <osg/Version>
 #include <osg/ArgumentParser>
 #include <osg/ApplicationUsage>
+#include <OpenThreads/Version>
 
 #include <set>
 #include <vector>
@@ -33,17 +34,33 @@ bool validName(const std::string& first)
     
     if (first.size()>=2 && (first[1]<'a' || first[1]>'z') && (first[1]!='.')  && (first[1]!=',')) return false; 
 
+    if (first=="Xcode") return false;
     if (first=="Added") return false;
     if (first=="Camera") return false;
     if (first=="CameraNode") return false;
     if (first=="CopyOp") return false;
     if (first=="Fixed") return false;
+    if (first=="View") return false;
+    if (first=="GraphicsContext") return false;
+    if (first=="WindowData") return false;
+    if (first=="ViewPoint") return false;
+    if (first=="PickHandler") return false;
+    if (first=="Program") return false;
+    if (first=="Object") return false;
+    if (first=="OpenSceneGraph") return false;
+    if (first=="SpotExponent") return false;
+    if (first=="Framstamp") return false;
+    if (first=="Stats") return false;
+    if (first=="Group") return false;
+    if (first=="Texture") return false;
+    if (first=="Texture2DArray") return false;
     if (first=="Creator") return false;
     if (first=="CullVisitor") return false;
     if (first=="Drawable") return false;
     if (first=="Geode") return false;
     if (first=="GeoSet") return false;
     if (first=="Image") return false;
+    if (first=="Face") return false;
     if (first=="Images/SolarSystem") return false;
     if (first=="IntersectVisitor") return false;
     if (first=="LongIDRecord") return false;
@@ -184,7 +201,10 @@ bool validName(const std::string& first)
     if (first=="Vec4ub") return false;
     if (first=="Proxy") return false;
     if (first=="CullVistor") return false;
+    if (first=="SimpleViewer") return false;
     if (first=="TexMat(Matrix") return false;
+    if (first=="GraphicsWindowX11") return false;
+    if (first=="OperationThread") return false;
     return true;
 }
 
@@ -196,6 +216,17 @@ std::string typoCorrection(const std::string& name)
     if (name=="") return "";
     if (name=="") return "";
 #endif
+    if (name=="Martsz") return "Martz";
+    if (name=="Froenlich") return "Frhlich";
+    if (name=="Froelich") return "Frhlich";    
+    if (name=="Fruciel") return "Frauciel";
+    if (name=="Hebelin") return "Herbelin";
+    if (name=="Jea-Sebastien") return "Jean-Sebastien";
+    if (name=="Robet") return "Robert";
+    if (name=="Morné") return "Morn";
+    if (name=="Adndre") return "Andre";
+    if (name=="Mellis") return "Melis";
+    if (name=="Rajce") return "Trajce";
     if (name=="Trastenjak") return "Trstenjak";
     if (name=="Baverage") return "Beverage";
     if (name=="Bistroviae") return "Bistrovic";
@@ -203,6 +234,7 @@ std::string typoCorrection(const std::string& name)
     if (name=="Cobin") return "Corbin";
     if (name=="Connel") return "Connell";
     if (name=="Daust") return "Daoust";
+    if (name=="Lugi") return "Luigi";
     if (name=="Daved") return "David";
     if (name=="Fred") return "Frederic";
     if (name=="Fredrick") return "Frederic";
@@ -215,6 +247,7 @@ std::string typoCorrection(const std::string& name)
     if (name=="Heirtlein") return "Hertlein";
     if (name=="Heirtlein") return "Hertlein";
     if (name=="Hertlien") return "Hertlein";
+    if (name=="Narache") return "Marache";
     if (name=="Hi") return "He";
     if (name=="Hooper") return "Hopper";
     if (name=="Inverson") return "Iverson";
@@ -243,7 +276,9 @@ std::string typoCorrection(const std::string& name)
     if (name=="Moloshton") return "Moloshtan";
     if (name=="Moule") return "Moiule";
     if (name=="Nicklov") return "Nikolov";
+    if (name=="Nickolov") return "Nikolov";
     if (name=="Olad") return "Olaf";
+    if (name=="Olar") return "Olaf";
     if (name=="Oritz") return "Ortiz";
     if (name=="Osfied") return "Osfield";
     if (name=="Pail") return "Paul";
@@ -267,13 +302,47 @@ std::string typoCorrection(const std::string& name)
     if (name=="Yefrei") return "Yefei";
     if (name=="Yfei") return "Yefei";
     if (name=="Antonoine") return "Antoine";
+    if (name=="Antione") return "Antoine";
     if (name=="Andew") return "Andrew";
     if (name=="Daneil") return "Daniel";
+    if (name=="André") return "Andr";
+    if (name=="García") return "Garcea";
+    if (name=="Sjölie") return "Sjlie";
+    if (name=="José") return "Jos";
+    if (name=="Fröhlich") return "Frhlich";
+    if (name=="Froehlich") return "Frhlich";
+    if (name=="Eileman") return "Eilemann";
+    if (name=="Skinnder") return "Skinner";
+    if (name=="Mihair")  return "Mihai";
+    if (name=="Mahai")  return "Mihai";
+    
+    if (name=="SimpleViewer") return "";
+    
     return name;
 }
 
 void nameCorrection(NamePair& name)
 {
+    if (name.first=="Radu" && name.second=="Mihai")
+    {
+        name.first = "Mihai";
+        name.second = "Radu";
+    }
+    if (name.first=="Art" && name.second=="Trevs")
+    {
+        name.first = "Art";
+        name.second = "Tevs";
+    }
+    if (name.first=="Tim" && name.second=="More")
+    {
+        name.first = "Tim";
+        name.second = "Moore";
+    }
+    if (name.first=="Andre" && name.second=="Garneau")
+    {
+        name.first = "Andr";
+        name.second = "Garneau";
+    }
     if (name.first=="Eric" && name.second=="Hammil")
     {
         name.first = "Chris";
@@ -351,6 +420,11 @@ void nameCorrection(NamePair& name)
         name.first = "Trajce";
         name.second = "Nikolov";
     }
+    if (name.first=="Frauciel" && name.second=="Luc")
+    {
+        name.first = "Luc";
+        name.second = "Frauciel";
+    }
 }
 
 void lastValidCharacter(const std::string& name, unsigned int& pos,char c)
@@ -434,8 +508,6 @@ bool submissionsSequence(const Words& words, unsigned int& i)
         words[i]=="from" || 
         words[i]=="From:" || 
         words[i]=="from:" || 
-        words[i]=="Applied" || 
-        words[i]=="Added" || 
         words[i]=="Merged" || 
         words[i]=="Integrated") return true;
         
@@ -511,11 +583,11 @@ void readContributors(NameMap& names, const std::string& file)
         }
         else
         {
-            if (words[i]=="robert") 
+            if (words[i]=="robert:") 
             {
                 ++names[NameRobertOsfield];
             }
-            else if (words[i]=="don")
+            else if (words[i]=="don:")
             {
                 ++names[NameDonBurns];
             }
@@ -577,6 +649,7 @@ void buildContributors(NameMap& names)
     ++names[NamePair("Marco","Jez")];
     ++names[NamePair("Mike","Weiblen")];
     ++names[NamePair("Geoff","Michel")];
+    ++names[NamePair("Ben","van Basten")];
     
     // contributors that don't appear in the ChangeLog due to their contributions
     // being before CVS started for the OSG, or before the name logging began.
@@ -595,8 +668,75 @@ int main( int argc, char **argv)
     arguments.getApplicationUsage()->setCommandLineUsage(arguments.getApplicationName()+" [options]");
     arguments.getApplicationUsage()->addCommandLineOption("-h or --help","Display this information");
     arguments.getApplicationUsage()->addCommandLineOption("--entries","Print out number of entries into the ChangeLog file for each contributor.");
+    arguments.getApplicationUsage()->addCommandLineOption("--version-number","Print out version number only");
+    arguments.getApplicationUsage()->addCommandLineOption("--soversion-number","Print out shared object version number only");
+    arguments.getApplicationUsage()->addCommandLineOption("--openthreads-version-number","Print out version number for OpenThreads only");
+    arguments.getApplicationUsage()->addCommandLineOption("--openthreads-soversion-number","Print out shared object version number for OpenThreads only");
     arguments.getApplicationUsage()->addCommandLineOption("-r <file> or --read <file>","Read the ChangeLog to generate an estimated contributors list.");
 
+    if (arguments.read("--version-number"))
+    {
+        std::cout<<osgGetVersion()<<std::endl;
+        return 0;
+    }
+    
+    if (arguments.read("--major-number"))
+    {
+        std::cout<<OPENSCENEGRAPH_MAJOR_VERSION<<std::endl;
+        return 0;
+    }
+    
+    if (arguments.read("--minor-number"))
+    {
+        std::cout<<OPENSCENEGRAPH_MINOR_VERSION<<std::endl;
+        return 0;
+    }
+    
+    if (arguments.read("--patch-number"))
+    {
+        std::cout<<OPENSCENEGRAPH_PATCH_VERSION<<std::endl;
+        return 0;
+    }
+    
+    if (arguments.read("--soversion-number"))
+    {
+        std::cout<<osgGetSOVersion()<<std::endl;
+        return 0;
+    }
+    
+
+    if (arguments.read("--openthreads-version-number"))
+    {
+        std::cout<<OpenThreadsGetVersion()<<std::endl;
+        return 0;
+    }
+    
+    
+    if (arguments.read("--openthreads-major-number"))
+    {
+        std::cout<<OPENTHREADS_MAJOR_VERSION<<std::endl;
+        return 0;
+    }
+    
+    if (arguments.read("--openthreads-minor-number"))
+    {
+        std::cout<<OPENTHREADS_MINOR_VERSION<<std::endl;
+        return 0;
+    }
+    
+    if (arguments.read("--openthreads-patch-number"))
+    {
+        std::cout<<OPENTHREADS_PATCH_VERSION<<std::endl;
+        return 0;
+    }
+    
+
+    if (arguments.read("--openthreads-soversion-number"))
+    {
+        std::cout<<OpenThreadsGetSOVersion()<<std::endl;
+        return 0;
+    }
+    
     std::cout<<osgGetLibraryName()<< " "<< osgGetVersion()<<std::endl<<std::endl;
 
     bool printContributors = false;
