@@ -1,17 +1,21 @@
-/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2006 Robert Osfield 
- *
- * This library is open source and may be redistributed and/or modified under  
- * the terms of the OpenSceneGraph Public License (OSGPL) version 0.0 or 
- * (at your option) any later version.  The full license is in LICENSE file
- * included with this distribution, and on the openscenegraph.org website.
- * 
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
- * OpenSceneGraph Public License for more details.
+/* OpenSceneGraph example, osgcatch.
+*
+*  Permission is hereby granted, free of charge, to any person obtaining a copy
+*  of this software and associated documentation files (the "Software"), to deal
+*  in the Software without restriction, including without limitation the rights
+*  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
+*  copies of the Software, and to permit persons to whom the Software is
+*  furnished to do so, subject to the following conditions:
+*
+*  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+*  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+*  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+*  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+*  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+*  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
+*  THE SOFTWARE.
 */
 
-#include <osgProducer/Viewer>
 #include <osgDB/ReadFile>
 #include <osgDB/WriteFile>
 
@@ -27,13 +31,19 @@
 #include <osg/Switch>
 #include <osg/TexMat>
 #include <osg/Texture2D>
+#include <osg/Timer>
 #include <osg/io_utils>
 
+#include <osgGA/GUIEventHandler>
+
 #include <osgParticle/ExplosionEffect>
 #include <osgParticle/ExplosionDebrisEffect>
 #include <osgParticle/SmokeEffect>
 #include <osgParticle/FireEffect>
 
+#include <osgViewer/Viewer>
+
+#include <iostream>
 #include <sstream>
 
 typedef std::vector<std::string> FileList;
@@ -506,8 +516,6 @@ public:
     
     META_Object(osgStereImageApp,GameEventHandler);
 
-    virtual void accept(osgGA::GUIEventHandlerVisitor& v) { v.visit(*this); }
-
     virtual bool handle(const osgGA::GUIEventAdapter& ea,osgGA::GUIActionAdapter&);
     
     virtual void getUsage(osg::ApplicationUsage& usage) const;
@@ -787,6 +795,8 @@ GameEventHandler::GameEventHandler()
     CatchableObject::setUpCatchablesMap(_benignCatachables);
     
     _currentScore = 0;
+    
+    setFOVY(osg::DegreesToRadians(60.0));
 
 }
 
@@ -1148,6 +1158,7 @@ osg::Node* GameEventHandler::createScene()
             text->setColor(osg::Vec4(0.0f,0.2f,0.2f,1.0f));
             text->setAlignment(osgText::Text::CENTER_CENTER);
             text->setAxisAlignment(osgText::Text::XZ_PLANE);
+            text->setDataVariance(osg::Object::DYNAMIC);
 
             geode->addDrawable(text);
             _scoreTextList.push_back(text);
@@ -1288,6 +1299,7 @@ osg::Node* GameEventHandler::createScene()
             text->setPosition(textPosition);
             text->setCharacterSize(_width.length()*0.04f);
             text->setColor(osg::Vec4(0.0f,0.2f,0.2f,1.0f));
+            text->setDataVariance(osg::Object::DYNAMIC);
             text->setAxisAlignment(osgText::Text::XZ_PLANE);
 
             osg::Geode* geode = new osg::Geode;
@@ -1301,6 +1313,7 @@ osg::Node* GameEventHandler::createScene()
             _levelText->setPosition(textPosition);
             _levelText->setCharacterSize(_width.length()*0.04f);
             _levelText->setColor(osg::Vec4(0.0f,0.2f,0.2f,1.0f));
+            _levelText->setDataVariance(osg::Object::DYNAMIC);
             _levelText->setAxisAlignment(osgText::Text::XZ_PLANE);
 
             geode->addDrawable(_levelText.get());
@@ -1349,31 +1362,26 @@ void GameEventHandler::createNewCatchable()
     _gameGroup->addChild(catchableObject->_object.get());
 }
 
-
-class CompileStateCallback : public osgProducer::OsgCameraGroup::RealizeCallback
+class CompileStateCallback : public osg::Operation
 {
     public:
-        CompileStateCallback(GameEventHandler* eh):_gameEventHandler(eh) {}
+        CompileStateCallback(GameEventHandler* eh):
+            osg::Operation("CompileStateCallback", false),
+            _gameEventHandler(eh) {}
         
-        virtual void operator()( osgProducer::OsgCameraGroup&, osgProducer::OsgSceneHandler& sh, const Producer::RenderSurface& )
-        { 
-            // now safe to construct
-            sh.init();
-          
-            {
-                OpenThreads::ScopedLock<OpenThreads::Mutex> lock(_mutex);
+        virtual void operator () (osg::Object* object)
+        {
+            osg::GraphicsContext* context = dynamic_cast<osg::GraphicsContext*>(object);
+            if (!context) return;
 
-                if (_gameEventHandler)
-                {
-                    _gameEventHandler->compileGLObjects(*(sh.getSceneView()->getState()));
-                }
+            if (_gameEventHandler)
+            {
+                _gameEventHandler->compileGLObjects(*(context->getState()));
             }
         }
         
-        
         OpenThreads::Mutex  _mutex;
         GameEventHandler*  _gameEventHandler;
-        
 };
 
 int main( int argc, char **argv )
@@ -1390,26 +1398,19 @@ int main( int argc, char **argv )
     arguments.getApplicationUsage()->addCommandLineOption("-x <float>","Horizontal offset of left and right images.");
     arguments.getApplicationUsage()->addCommandLineOption("-y <float>","Vertical offset of left and right images.");
     arguments.getApplicationUsage()->addCommandLineOption("-h or --help","Display this information");
-    
 
     // construct the viewer.
-    osgProducer::Viewer viewer(arguments);
+    osgViewer::Viewer viewer;
 
-    // set up the value with sensible default event handlers.
-    viewer.setUpViewer(osgProducer::Viewer::ESCAPE_SETS_DONE);
 
     // register the handler to add keyboard and mosue handling.
     GameEventHandler* seh = new GameEventHandler();
-    viewer.getEventHandlerList().push_front(seh);
+    viewer.addEventHandler(seh);
 
     while (arguments.read("--boy")) seh->addPlayer(GameEventHandler::PLAYER_BOY);
     while (arguments.read("--girl")) seh->addPlayer(GameEventHandler::PLAYER_GIRL);
     
     
-
-    // get details on keyboard and mouse bindings used by the viewer.
-    viewer.getUsage(*arguments.getApplicationUsage());
-
     // if user request help write it out to cout.
     if (arguments.read("-h") || arguments.read("--help"))
     {
@@ -1427,18 +1428,6 @@ int main( int argc, char **argv )
         return 1;
     }
     
-    // now the windows have been realized we switch off the cursor to prevent it
-    // distracting the people seeing the stereo images.
-    float fovy = 1.0f;
-    for( unsigned int i = 0; i < viewer.getNumberOfCameras(); i++ )
-    {
-        Producer::Camera* cam = viewer.getCamera(i);
-        Producer::RenderSurface* rs = cam->getRenderSurface();
-        rs->useCursor(false);
-        fovy = osg::DegreesToRadians(cam->getLensVerticalFov());
-    }
-    
-    seh->setFOVY(fovy);
     
     // enable the image cache so we don't need to keep loading the particle files
     osgDB::ReaderWriter::Options* options = new osgDB::ReaderWriter::Options;
@@ -1449,45 +1438,45 @@ int main( int argc, char **argv )
     // creat the scene from the file list.
     osg::ref_ptr<osg::Node> rootNode = seh->createScene();
 
+    rootNode->getOrCreateStateSet()->setMode(GL_LIGHTING, osg::StateAttribute::OFF);
+
     //osgDB::writeNodeFile(*rootNode,"test.osg");
 
     // set the scene to render
     viewer.setSceneData(rootNode.get());
 
-    viewer.setRealizeCallback(new CompileStateCallback(seh));
+    viewer.setRealizeOperation(new CompileStateCallback(seh));
 
-    // create the windows and run the threads, note has to be single threaded
-    // due to use osgParticle
-    viewer.realize(Producer::CameraGroup::SingleThreaded);
-    
-    viewer.requestWarpPointer(0.5f,0.5f);
-        
+    double fovy, aspectRatio, zNear, zFar;
+    viewer.getCamera()->getProjectionMatrixAsPerspective(fovy, aspectRatio, zNear, zFar);
+    seh->setFOVY(osg::DegreesToRadians(fovy));    
+
+    // todo for osgViewer - create default set up.
+    viewer.setUpViewAcrossAllScreens();
+
+    viewer.realize();
+
+    // switch off the cursor
+    osgViewer::Viewer::Windows windows;
+    viewer.getWindows(windows);
+    for(osgViewer::Viewer::Windows::iterator itr = windows.begin();
+        itr != windows.end();
+        ++itr)
+    {
+        (*itr)->useCursor(false);
+    }
+
+    // todo for osgViewer - implement warp pointer that can be done relative to different coordinate frames
+    // viewer.requestWarpPointer(0.5f,0.5f);        
 
     while( !viewer.done() )
     {
-        // wait for all cull and draw threads to complete.
-        viewer.sync();
-
-        // update the scene by traversing it with the the update visitor which will
-        // call all node update callbacks and animations.
-        viewer.update();
-         
-        viewer.setView(seh->getCameraPosition());
+        viewer.getCamera()->setViewMatrix(seh->getCameraPosition());
 
         // fire off the cull and draw traversals of the scene.
         viewer.frame();
         
     }
-
-    // wait for all cull and draw threads to complete.
-    viewer.sync();
-
-    // run a clean up frame to delete all OpenGL objects.
-    viewer.cleanup_frame();
-
-    // wait for all the clean up frame to complete.
-    viewer.sync();
     
     return 0;
 }
-
