@@ -1,3 +1,21 @@
+/* OpenSceneGraph example, osgfxbrowser.
+*
+*  Permission is hereby granted, free of charge, to any person obtaining a copy
+*  of this software and associated documentation files (the "Software"), to deal
+*  in the Software without restriction, including without limitation the rights
+*  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
+*  copies of the Software, and to permit persons to whom the Software is
+*  furnished to do so, subject to the following conditions:
+*
+*  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+*  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+*  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+*  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+*  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+*  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
+*  THE SOFTWARE.
+*/
+
 #include <osg/Group>
 #include <osg/Geometry>
 #include <osg/Geode>
@@ -6,7 +24,7 @@
 #include <osg/BlendFunc>
 #include <osg/LightSource>
 
-#include <osgProducer/Viewer>
+#include <osgViewer/Viewer>
 
 #include <osgDB/ReadFile>
 #include <osgDB/WriteFile>
@@ -25,6 +43,7 @@
 
 #include <vector>
 #include <string>
+#include <iostream>
 
 class RotateCallback: public osg::NodeCallback {
 public:
@@ -33,7 +52,7 @@ public:
     {
         osg::MatrixTransform *xform = dynamic_cast<osg::MatrixTransform *>(node);
         if (xform && enabled_) {
-            double t = nv->getFrameStamp()->getReferenceTime();
+            double t = nv->getFrameStamp()->getSimulationTime();
             xform->setMatrix(osg::Matrix::rotate(t, osg::Vec3(0, 0, 1)));
         }
         traverse(node, nv);
@@ -163,7 +182,6 @@ protected:
         hints->setColor(_hints_color);
         hints->setAlignment(osgText::Text::CENTER_BOTTOM);
         hints->setCharacterSize(13);
-        hints->setFontResolution(13, 13);
         hints->setPosition(osg::Vec3((client_rect.x0+client_rect.x1)/2, client_rect.y0 + 4, zPos));
         hints->setText("<RETURN> show/hide this panel      <LEFT> previous effect      <RIGHT> next effect      <DEL> enable/disable effects      'x' save to file      'r' rotate/stop");
         addDrawable(hints.get());
@@ -195,7 +213,6 @@ protected:
         ename->setColor(_name_color);
         ename->setAlignment(osgText::Text::CENTER_TOP);
         ename->setCharacterSize(32);
-        ename->setFontResolution(32, 32);
         ename->setPosition(osg::Vec3((client_rect.x0 + client_rect.x1) / 2, client_rect.y1 - 22, zPos));
         ename->setText(effect_name);
         addDrawable(ename.get());
@@ -206,7 +223,6 @@ protected:
         edesc->setColor(_desc_color);
         edesc->setAlignment(osgText::Text::LEFT_TOP);
         edesc->setCharacterSize(16);
-        edesc->setFontResolution(16, 16);
         edesc->setPosition(osg::Vec3(client_rect.x0 + 8, client_rect.y1 - 60, zPos));
         edesc->setText(effect_description);
         addDrawable(edesc.get());
@@ -259,13 +275,13 @@ EffectPanel* build_gui(osg::Group* root)
     return effect_panel.release();
 }
 
-void build_world(osg::Group* root, osg::Node* scene, osgProducer::Viewer& viewer)
+void build_world(osg::Group* root, osg::Node* scene, osgViewer::Viewer& viewer)
 {
     osg::ref_ptr<EffectPanel> effect_panel = build_gui(root);
     effect_panel->setScene(scene);
     effect_panel->rebuild();
 
-    viewer.getEventHandlerList().push_front(new EffectPanel::KeyboardHandler(effect_panel.get()));
+    viewer.addEventHandler(new EffectPanel::KeyboardHandler(effect_panel.get()));
 
     root->addChild(effect_panel->getRoot());
 }
@@ -288,13 +304,7 @@ int main(int argc, char *argv[])
 
 
     // construct the viewer.
-    osgProducer::Viewer viewer(arguments);
-
-    // set up the value with sensible default event handlers.
-    viewer.setUpViewer(osgProducer::Viewer::STANDARD_SETTINGS);
-
-    // get details on keyboard and mouse bindings used by the viewer.
-    viewer.getUsage(*arguments.getApplicationUsage());
+    osgViewer::Viewer viewer;
 
     // if user request help write it out to cout.
     if (arguments.read("-h") || arguments.read("--help")) {
@@ -302,6 +312,14 @@ int main(int argc, char *argv[])
         return 1;
     }
 
+    osgViewer::Viewer::ThreadingModel threading = osgViewer::Viewer::SingleThreaded;
+    while (arguments.read("--SingleThreaded")) threading = osgViewer::Viewer::SingleThreaded;
+    while (arguments.read("--CullDrawThreadPerContext")) threading = osgViewer::Viewer::CullDrawThreadPerContext;
+    while (arguments.read("--DrawThreadPerContext")) threading = osgViewer::Viewer::DrawThreadPerContext;
+    while (arguments.read("--CullThreadPerCameraDrawThreadPerContext")) threading = osgViewer::Viewer::CullThreadPerCameraDrawThreadPerContext;
+
+    viewer.setThreadingModel(threading);
+
     // any option left unread are converted into errors to write out later.
     arguments.reportRemainingOptionsAsUnrecognized();
 
@@ -311,27 +329,18 @@ int main(int argc, char *argv[])
         return 1;
     }
 
-    if (arguments.argc() <= 1) {
-        arguments.getApplicationUsage()->write(std::cout, osg::ApplicationUsage::COMMAND_LINE_OPTION);
-        return 1;
-    }
-
-    osg::Timer timer;
-    osg::Timer_t start_tick = timer.tick();
-
     // read the scene from the list of file specified commandline args.
     osg::ref_ptr<osg::Node> loadedModel = osgDB::readNodeFiles(arguments);
 
-    // if no model has been successfully loaded report failure.
-    if (!loadedModel) {
-        std::cout << arguments.getApplicationName() << ": No data loaded" << std::endl;
+    // if not loaded assume no arguments passed in, try use default mode instead.
+    if (!loadedModel) loadedModel = osgDB::readNodeFile("dumptruck.osg");
+  
+    if (!loadedModel)
+    {
+        std::cout << arguments.getApplicationName() <<": No data loaded" << std::endl;
         return 1;
     }
 
-    osg::Timer_t end_tick = timer.tick();
-
-    std::cout << "Time to load = " << timer.delta_s(start_tick,end_tick) << std::endl;
-
     // optimize the scene graph, remove rendundent nodes and state etc.
     osgUtil::Optimizer optimizer;
     optimizer.optimize(loadedModel.get());
@@ -358,31 +367,5 @@ int main(int argc, char *argv[])
     // set the scene to render
     viewer.setSceneData(root.get());
 
-    // create the windows and run the threads.
-    viewer.realize();
-
-    while(!viewer.done())
-    {
-        // wait for all cull and draw threads to complete.
-        viewer.sync();
-
-        // update the scene by traversing it with the the update visitor which will
-        // call all node update callbacks and animations.
-        viewer.update();
-
-        // fire off the cull and draw traversals of the scene.
-        viewer.frame();
-
-    }
-
-    // wait for all cull and draw threads to complete.
-    viewer.sync();
-
-    // run a clean up frame to delete all OpenGL objects.
-    viewer.cleanup_frame();
-
-    // wait for all the clean up frame to complete.
-    viewer.sync();
-
-    return 0;
+    return viewer.run();
 }
