@@ -1,3 +1,21 @@
+/* OpenSceneGraph example, osghangglide.
+*
+*  Permission is hereby granted, free of charge, to any person obtaining a copy
+*  of this software and associated documentation files (the "Software"), to deal
+*  in the Software without restriction, including without limitation the rights
+*  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
+*  copies of the Software, and to permit persons to whom the Software is
+*  furnished to do so, subject to the following conditions:
+*
+*  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+*  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+*  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+*  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+*  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+*  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
+*  THE SOFTWARE.
+*/
+
 #include "GliderManipulator.h"
 #include <osg/Notify>
 
@@ -7,7 +25,7 @@ using namespace osgGA;
 GliderManipulator::GliderManipulator()
 {
     _modelScale = 0.01f;
-    _velocity = 0.0f;
+    _velocity = 0.2f;
     _yawMode = YAW_AUTOMATICALLY_WHEN_BANKED;
 
     _distance = 1.0f;
@@ -74,7 +92,7 @@ void GliderManipulator::init(const GUIEventAdapter& ea,GUIActionAdapter& us)
 
     us.requestContinuousUpdate(false);
 
-    _velocity = 0.0f;
+    _velocity = 0.2f;
 
     if (ea.getEventType()!=GUIEventAdapter::RESIZE)
     {
@@ -87,6 +105,7 @@ bool GliderManipulator::handle(const GUIEventAdapter& ea,GUIActionAdapter& us)
 {
     switch(ea.getEventType())
     {
+#if 0   
         case(GUIEventAdapter::PUSH):
         {
 
@@ -101,7 +120,7 @@ bool GliderManipulator::handle(const GUIEventAdapter& ea,GUIActionAdapter& us)
 
             addMouseEvent(ea);
             us.requestContinuousUpdate(true);
-            if (calcMovement()) us.requestRedraw();
+            // if (calcMovement()) us.requestRedraw();
             return true;
         }
 
@@ -110,7 +129,7 @@ bool GliderManipulator::handle(const GUIEventAdapter& ea,GUIActionAdapter& us)
 
             addMouseEvent(ea);
             us.requestContinuousUpdate(true);
-            if (calcMovement()) us.requestRedraw();
+            // if (calcMovement()) us.requestRedraw();
             return true;
         }
 
@@ -119,11 +138,11 @@ bool GliderManipulator::handle(const GUIEventAdapter& ea,GUIActionAdapter& us)
 
             addMouseEvent(ea);
             us.requestContinuousUpdate(true);
-            if (calcMovement()) us.requestRedraw();
+            // if (calcMovement()) us.requestRedraw();
 
             return true;
         }
-
+#endif
         case(GUIEventAdapter::KEYDOWN):
             if (ea.getKey()==' ')
             {
@@ -258,6 +277,8 @@ bool GliderManipulator::calcMovement()
 
     float dx = _ga_t0->getXnormalized();
     float dy = _ga_t0->getYnormalized();
+    
+    // osg::notify(osg::NOTICE)<<"dx = "<<dx<<" dy = "<<dy<<"dt = "<<dt<<std::endl;
 
     // mew - flag to reverse mouse-control mapping
     if( getenv( "OSGHANGGLIDE_REVERSE_CONTROLS" ) )
