@@ -1,3 +1,21 @@
+/* OpenSceneGraph example, osglauncher.
+*
+*  Permission is hereby granted, free of charge, to any person obtaining a copy
+*  of this software and associated documentation files (the "Software"), to deal
+*  in the Software without restriction, including without limitation the rights
+*  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
+*  copies of the Software, and to permit persons to whom the Software is
+*  furnished to do so, subject to the following conditions:
+*
+*  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+*  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+*  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+*  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+*  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+*  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
+*  THE SOFTWARE.
+*/
+
 //#include <cstdio>
 //#include <cstdlib>
 #include <iostream>
@@ -19,7 +37,6 @@
 #include <osg/Projection>
 #include <osg/io_utils>
 
-#include <osgUtil/Tesselator>
 #include <osgUtil/CullVisitor>
 #include <osgUtil/Optimizer>
 
@@ -27,7 +44,7 @@
 
 #include <osgGA/TrackballManipulator>
 
-#include <osgProducer/Viewer>
+#include <osgViewer/Viewer>
 
 #include <osgDB/ReadFile>
 #include <osgDB/FileUtils>
@@ -38,7 +55,7 @@ int runApp(std::string xapp);
 class PickHandler : public osgGA::GUIEventHandler {
 public: 
 
-    PickHandler(osgProducer::Viewer* viewer,osgText::Text* updateText):
+    PickHandler(osgViewer::Viewer* viewer,osgText::Text* updateText):
         _viewer(viewer),
         _updateText(updateText) {}
         
@@ -55,7 +72,7 @@ public:
     
 protected:
 
-    osgProducer::Viewer* _viewer;
+    osgViewer::Viewer* _viewer;
     osg::ref_ptr<osgText::Text>  _updateText;
 };
 
@@ -93,16 +110,18 @@ bool PickHandler::handle(const osgGA::GUIEventAdapter& ea,osgGA::GUIActionAdapte
 
 std::string PickHandler::pick(float x, float y)
 {
-    osgUtil::IntersectVisitor::HitList hlist;
-    if (_viewer->computeIntersections(x, y, hlist))
+    osgUtil::LineSegmentIntersector::Intersections intersections;
+    if (_viewer->computeIntersections(x, y, intersections))
     {
-        for(osgUtil::IntersectVisitor::HitList::iterator hitr=hlist.begin();
-            hitr!=hlist.end();
+        for(osgUtil::LineSegmentIntersector::Intersections::iterator hitr = intersections.begin();
+            hitr != intersections.end();
             ++hitr)
         {
-            if (hitr->_geode.valid() && !hitr->_geode->getName().empty()) return hitr->_geode->getName();
+            osg::Node* node = hitr->nodePath.empty() ? 0 : hitr->nodePath.back();
+            if (node && !node->getName().empty()) return node->getName();
         }
     }
+
     return "";
 }
 
@@ -218,7 +237,7 @@ int runApp(std::string xapp)
 } // end printList()
 
 
-void readConfFile(char* confFile)                                                                // read confFile            1
+void readConfFile(const char* confFile)                                                                // read confFile            1
 {
     osg::notify(osg::INFO) << "Start reading confFile" << std::endl;
     
@@ -396,15 +415,13 @@ int main( int argc, char **argv )
     }
     
     // construct the viewer.
-    osgProducer::Viewer viewer;
+    osgViewer::Viewer viewer;
 
-    // set up the value with sensible default event handlers.
-    viewer.setUpViewer(osgProducer::Viewer::STANDARD_SETTINGS);
-   
     osg::ref_ptr<osgText::Text> updateText = new osgText::Text;
+    updateText->setDataVariance(osg::Object::DYNAMIC);
 
     // add the handler for doing the picking
-    viewer.getEventHandlerList().push_front(new PickHandler(&viewer,updateText.get()));
+    viewer.addEventHandler(new PickHandler(&viewer,updateText.get()));
 
     osg::Group* root = new osg::Group();
 
@@ -416,40 +433,18 @@ int main( int argc, char **argv )
     // add model to viewer.
     viewer.setSceneData( root );
 
-    // create the windows and run the threads.
-    viewer.realize();
-    
     osg::Matrix lookAt;
     lookAt.makeLookAt(osg::Vec3(0.0f, -4.0f, 0.0f), centerScope, osg::Vec3(0.0f, 0.0f, 1.0f));
 
-    //viewer.setView(lookAt);
+    viewer.getCamera()->setViewMatrix(lookAt);
         
+    viewer.realize();
+
     while( !viewer.done() )
     {
-        // wait for all cull and draw threads to complete.
-        viewer.sync();
-
-        // update the scene by traversing it with the the update visitor which will
-        // call all node update callbacks and animations.
-        viewer.update();
-
-        // to be able to turn scene, place next call before viewer.update()
-        viewer.setView(lookAt);
-
-    
         // fire off the cull and draw traversals of the scene.
-        viewer.frame();
-        
+        viewer.frame();        
     }
-    
-    // wait for all cull and draw threads to complete.
-    viewer.sync();
-
-    // run a clean up frame to delete all OpenGL objects.
-    viewer.cleanup_frame();
-
-    // wait for all the clean up frame to complete.
-    viewer.sync();
 
     return 0;
 } // end main
