@@ -1,3 +1,21 @@
+/* OpenSceneGraph example, osglogo.
+*
+*  Permission is hereby granted, free of charge, to any person obtaining a copy
+*  of this software and associated documentation files (the "Software"), to deal
+*  in the Software without restriction, including without limitation the rights
+*  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
+*  copies of the Software, and to permit persons to whom the Software is
+*  furnished to do so, subject to the following conditions:
+*
+*  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+*  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+*  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+*  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+*  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+*  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
+*  THE SOFTWARE.
+*/
+
 #include <osg/Geode>
 #include <osg/ShapeDrawable>
 #include <osg/Material>
@@ -8,18 +26,18 @@
 #include <osg/BlendFunc>
 #include <osg/ClearNode>
 
-#include <osgUtil/Tesselator>
+#include <osgUtil/Tessellator>
 #include <osgUtil/CullVisitor>
 
 #include <osgText/Text>
 
 #include <osgGA/TrackballManipulator>
 
-#include <osgProducer/Viewer>
+#include <osgViewer/Viewer>
 
 #include <osgDB/ReadFile>
 
-static bool s_ProfessionalServices = false;
+#include <iostream>
 
 class MyBillboardTransform : public osg::PositionAttitudeTransform
 {
@@ -113,14 +131,14 @@ osg::Geometry* createWing(const osg::Vec3& left, const osg::Vec3& nose, const os
 
     geom->addPrimitiveSet(new osg::DrawArrays(GL_POLYGON,0,vertices->getNumElements()));
     
-    osgUtil::Tesselator tesselator;
-    tesselator.retesselatePolygons(*geom);
+    osgUtil::Tessellator tessellator;
+    tessellator.retessellatePolygons(*geom);
 
     return geom;
     
 }
 
-osg:: Node* createTextBelow(const osg::BoundingBox& bb)
+osg:: Node* createTextBelow(const osg::BoundingBox& bb, const std::string& label, const std::string&)
 {
     osg::Geode* geode = new osg::Geode();
 
@@ -134,14 +152,14 @@ osg:: Node* createTextBelow(const osg::BoundingBox& bb)
     text->setAxisAlignment(osgText::Text::XZ_PLANE);
     text->setPosition(bb.center()-osg::Vec3(0.0f,0.0f,(bb.zMax()-bb.zMin())));
     text->setColor(osg::Vec4(0.37f,0.48f,0.67f,1.0f));
-    text->setText("OpenSceneGraph");
+    text->setText(label);
 
     geode->addDrawable( text );
 
     return geode;
 }
 
-osg:: Node* createTextLeft(const osg::BoundingBox& bb)
+osg:: Node* createTextLeft(const osg::BoundingBox& bb, const std::string& label, const std::string& subscript)
 {
     osg::Geode* geode = new osg::Geode();
 
@@ -163,7 +181,7 @@ osg:: Node* createTextLeft(const osg::BoundingBox& bb)
     text->setPosition(bb.center()-osg::Vec3((bb.xMax()-bb.xMin()),-(bb.yMax()-bb.yMin())*0.5f,(bb.zMax()-bb.zMin())*0.1f));
     //text->setColor(osg::Vec4(0.37f,0.48f,0.67f,1.0f)); // Neil's orignal OSG colour
     text->setColor(osg::Vec4(0.20f,0.45f,0.60f,1.0f)); // OGL logo colour
-    text->setText("OpenSceneGraph");
+    text->setText(label);
 
 #if 1
     text->setBackdropType(osgText::Text::OUTLINE);
@@ -195,19 +213,19 @@ osg:: Node* createTextLeft(const osg::BoundingBox& bb)
     geode->addDrawable( text );
 
 
-    if (s_ProfessionalServices)
+    if (!subscript.empty())
     {
         //osgText::Text* subscript = new  osgText::Text(new osgText::TextureFont(font,45));
 
-        osgText::Text* subscript = new osgText::Text;
-        subscript->setFont(font);
-        subscript->setText("Professional Services");
-        subscript->setAlignment(osgText::Text::RIGHT_CENTER);
-        subscript->setAxisAlignment(osgText::Text::XZ_PLANE);
-        subscript->setPosition(bb.center()-osg::Vec3((bb.xMax()-bb.xMin())*4.3f,-(bb.yMax()-bb.yMin())*0.5f,(bb.zMax()-bb.zMin())*0.6f));
-        subscript->setColor(osg::Vec4(0.0f,0.0f,0.0f,1.0f)); // black
+        osgText::Text* subscriptText = new osgText::Text;
+        subscriptText->setFont(font);
+        subscriptText->setText(subscript);
+        subscriptText->setAlignment(osgText::Text::RIGHT_CENTER);
+        subscriptText->setAxisAlignment(osgText::Text::XZ_PLANE);
+        subscriptText->setPosition(bb.center()-osg::Vec3((bb.xMax()-bb.xMin())*4.3f,-(bb.yMax()-bb.yMin())*0.5f,(bb.zMax()-bb.zMin())*0.6f));
+        subscriptText->setColor(osg::Vec4(0.0f,0.0f,0.0f,1.0f)); // black
 
-        geode->addDrawable( subscript );
+        geode->addDrawable( subscriptText );
     }
         
     return geode;
@@ -371,7 +389,7 @@ osg:: Node* createBackdrop(const osg::Vec3& corner,const osg::Vec3& top,const os
     return geode;    
 }
 
-osg::Node* createLogo(const std::string& filename)
+osg::Node* createLogo(const std::string& filename, const std::string& label, const std::string& subscript)
 {
     osg::BoundingBox bb(osg::Vec3(0.0f,0.0f,0.0f),osg::Vec3(100.0f,100.0f,100.0f));
     float chordRatio = 0.5f; 
@@ -410,7 +428,7 @@ osg::Node* createLogo(const std::string& filename)
 
     // add the text to the group.
     //group->addChild(createTextBelow(bb));
-    logo_group->addChild(createTextLeft(bb));
+    logo_group->addChild(createTextLeft(bb, label, subscript));
     
     
     // create the backdrop to render the shadow to.
@@ -442,30 +460,13 @@ osg::Node* createLogo(const std::string& filename)
 
 int main( int argc, char **argv )
 {
-
     // use an ArgumentParser object to manage the program arguments.
     osg::ArgumentParser arguments(&argc,argv);
-
-    // set up the usage document, in case we need to print out how to use this program.
-    arguments.getApplicationUsage()->setDescription(arguments.getApplicationName()+" is the example which demonstrates both text, animation and billboard via custom transform to create the OpenSceneGraph logo..");
-   
-    arguments.getApplicationUsage()->setCommandLineUsage(arguments.getApplicationName()+"[options] [filename] ...");
-    arguments.getApplicationUsage()->addCommandLineOption("-h or --help","Display this information");
-    arguments.getApplicationUsage()->addCommandLineOption("ps","Render the Professional Services logo");
-   
    
     osg::DisplaySettings::instance()->setMinimumNumAlphaBits(8);
    
     // construct the viewer.
-    osgProducer::Viewer viewer(arguments);
-
-    viewer.setWriteImageFileName("logo.rgb");
-    
-    // set up the value with sensible default event handlers.
-    viewer.setUpViewer(osgProducer::Viewer::STANDARD_SETTINGS);
-
-    // get details on keyboard and mouse bindings used by the viewer.
-    viewer.getUsage(*arguments.getApplicationUsage());
+    osgViewer::Viewer viewer;
 
     // if user request help write it out to cout.
     if (arguments.read("-h") || arguments.read("--help"))
@@ -474,51 +475,19 @@ int main( int argc, char **argv )
         return 1;
     }
     
-    while (arguments.read("ps")) s_ProfessionalServices = true;
+    std::string label = "OpenSceneGraph";
+    std::string subscript = "";
 
-        // any option left unread are converted into errors to write out later.
-    arguments.reportRemainingOptionsAsUnrecognized();
-
-    // report any errors if they have occured when parsing the program aguments.
-    if (arguments.errors())
-    {
-        arguments.writeErrorMessages(std::cout);
-        return 1;
-    }
+    while (arguments.read("--label", label)) {} 
+    while (arguments.read("--subscript", subscript)) {} 
     
-    osg::Node* node = 0;
+    osg::ref_ptr<osg::Node> node;
     
-    if (arguments.argc()>1) createLogo(arguments[1]);
-    else node = createLogo("");
+    if (arguments.argc()>1) node = createLogo(arguments[1], label, subscript);
+    else node = createLogo("", label, subscript);
 
     // add model to viewer.
-    viewer.setSceneData( node );
+    viewer.setSceneData( node.get() );
 
-    // create the windows and run the threads.
-    viewer.realize();
-
-    while( !viewer.done() )
-    {
-        // wait for all cull and draw threads to complete.
-        viewer.sync();
-
-        // update the scene by traversing it with the the update visitor which will
-        // call all node update callbacks and animations.
-        viewer.update();
-         
-        // fire off the cull and draw traversals of the scene.
-        viewer.frame();
-        
-    }
-    
-    // wait for all cull and draw threads to complete.
-    viewer.sync();
-
-    // run a clean up frame to delete all OpenGL objects.
-    viewer.cleanup_frame();
-
-    // wait for all the clean up frame to complete.
-    viewer.sync();
-
-    return 0;
+    return viewer.run();
 }
