@@ -1,6 +1,23 @@
-// -*-c++-*-
+/* OpenSceneGraph example, osgmovie.
+*
+*  Permission is hereby granted, free of charge, to any person obtaining a copy
+*  of this software and associated documentation files (the "Software"), to deal
+*  in the Software without restriction, including without limitation the rights
+*  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
+*  copies of the Software, and to permit persons to whom the Software is
+*  furnished to do so, subject to the following conditions:
+*
+*  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+*  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+*  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+*  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+*  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+*  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
+*  THE SOFTWARE.
+*/
 
-#include <osgProducer/Viewer>
+#include <osgViewer/Viewer>
+#include <osgViewer/ViewerEventHandlers>
 
 #include <osgDB/ReadFile>
 
@@ -10,6 +27,7 @@
 #include <osg/Material>
 #include <osg/Texture2D>
 #include <osg/TextureRectangle>
+#include <osg/TextureCubeMap>
 #include <osg/TexMat>
 #include <osg/CullFace>
 #include <osg/ImageStream>
@@ -18,36 +36,20 @@
 #include <osgGA/TrackballManipulator>
 #include <osgGA/EventVisitor>
 
+#include <iostream>
+
 osg::ImageStream* s_imageStream = 0;
-class PostSwapFinishCallback : public Producer::Camera::Callback
-{
-public:
-
-    PostSwapFinishCallback() {}
-
-    virtual void operator()(const Producer::Camera& camera)
-    {
-        // osg::Timer_t start_tick = osg::Timer::instance()->tick();
-        
-        osgProducer::OsgSceneHandler* sh = const_cast<osgProducer::OsgSceneHandler*>(dynamic_cast<const osgProducer::OsgSceneHandler*>(camera.getSceneHandler()));
-    
-        if (s_imageStream && s_imageStream->getPixelBufferObject()) s_imageStream->getPixelBufferObject()->compileBuffer(*(sh->getSceneView()->getState()));
-        // glFinish();
-
-        //osg::notify(osg::NOTICE)<<"callback after PBO "<<osg::Timer::instance()->delta_m(start_tick,osg::Timer::instance()->tick())<<"ms"<<std::endl;
-    }
-};
-
 class MovieEventHandler : public osgGA::GUIEventHandler
 {
 public:
 
-    MovieEventHandler() {}
+    MovieEventHandler():_trackMouse(false) {}
+    
+    void setMouseTracking(bool track) { _trackMouse = track; }
+    bool getMouseTracking() const { return _trackMouse; }
     
     void set(osg::Node* node);
 
-    virtual void accept(osgGA::GUIEventHandlerVisitor& v) { v.visit(*this); }
-
     virtual bool handle(const osgGA::GUIEventAdapter& ea,osgGA::GUIActionAdapter& aa, osg::Object*, osg::NodeVisitor* nv);
     
     virtual void getUsage(osg::ApplicationUsage& usage) const;
@@ -110,6 +112,7 @@ protected:
     };
 
 
+    bool            _trackMouse;
     ImageStreamList _imageStreamList;
     
 };
@@ -135,55 +138,33 @@ bool MovieEventHandler::handle(const osgGA::GUIEventAdapter& ea,osgGA::GUIAction
         case(osgGA::GUIEventAdapter::PUSH):
         case(osgGA::GUIEventAdapter::RELEASE):
         {
-            osgProducer::Viewer* viewer = dynamic_cast<osgProducer::Viewer*>(&aa);
-            osgUtil::IntersectVisitor::HitList hlist;
-            if (viewer->computeIntersections(ea.getX(),ea.getY(), nv->getNodePath().back(), hlist))
+            if (_trackMouse)
             {
-                if (!hlist.empty())
+                osgViewer::View* view = dynamic_cast<osgViewer::View*>(&aa);
+                osgUtil::LineSegmentIntersector::Intersections intersections;
+                if (view && view->computeIntersections(ea.getX(), ea.getY(), nv->getNodePath(), intersections))
                 {
+
                     // use the nearest intersection                 
-                    osgUtil::Hit& hit = hlist.front();
-                    osg::Drawable* drawable = hit.getDrawable();
+                    const osgUtil::LineSegmentIntersector::Intersection& intersection = *(intersections.begin());
+                    osg::Drawable* drawable = intersection.drawable.get();
                     osg::Geometry* geometry = drawable ? drawable->asGeometry() : 0;
                     osg::Vec3Array* vertices = geometry ? dynamic_cast<osg::Vec3Array*>(geometry->getVertexArray()) : 0;
-
                     if (vertices)
                     {
                         // get the vertex indices.
-                        const osgUtil::Hit::VecIndexList& vil = hit.getVecIndexList();
-                        
-                        if (vil.size()==3)
+                        const osgUtil::LineSegmentIntersector::Intersection::IndexList& indices = intersection.indexList;
+                        const osgUtil::LineSegmentIntersector::Intersection::RatioList& ratios = intersection.ratioList;
+
+                        if (indices.size()==3 && ratios.size()==3)
                         {
+                            unsigned int i1 = indices[0];
+                            unsigned int i2 = indices[1];
+                            unsigned int i3 = indices[2];
 
-                            int i1 = vil[0];
-                            int i2 = vil[1];
-                            int i3 = vil[2];
-                            osg::Vec3 v1 = (*vertices)[i1];
-                            osg::Vec3 v2 = (*vertices)[i2];
-                            osg::Vec3 v3 = (*vertices)[i3];
-                            osg::Vec3 v = hit.getLocalIntersectPoint();
-                            osg::Vec3 p1 = hit.getLocalLineSegment()->start();
-                            osg::Vec3 p2 = hit.getLocalLineSegment()->end();
-                            
-                            osg::Vec3 p12 = p1-p2;
-                            osg::Vec3 v13 = v1-v3;
-                            osg::Vec3 v23 = v2-v3;
-                            osg::Vec3 p1v3 = p1-v3;
-                            
-                            osg::Matrix matrix(p12.x(), v13.x(), v23.x(), 0.0,
-                                               p12.y(), v13.y(), v23.y(), 0.0,
-                                               p12.z(), v13.z(), v23.z(), 0.0,
-                                               0.0,    0.0,    0.0,    1.0);
-                                               
-                            osg::Matrix inverse;
-                            inverse.invert(matrix);
-                            
-                            osg::Vec3 ratio = inverse*p1v3;
-
-                            // extract the baricentric coordinates.                            
-                            float r1 = ratio.y();
-                            float r2 = ratio.z();
-                            float r3 = 1.0f-r1-r2;
+                            float r1 = ratios[0];
+                            float r2 = ratios[1];
+                            float r3 = ratios[2];
 
                             osg::Array* texcoords = (geometry->getNumTexCoordArrays()>0) ? geometry->getTexCoordArray(0) : 0;
                             osg::Vec2Array* texcoords_Vec2Array = dynamic_cast<osg::Vec2Array*>(texcoords);
@@ -194,25 +175,22 @@ bool MovieEventHandler::handle(const osgGA::GUIEventAdapter& ea,osgGA::GUIAction
                                 osg::Vec2 tc2 = (*texcoords_Vec2Array)[i2];
                                 osg::Vec2 tc3 = (*texcoords_Vec2Array)[i3];
                                 osg::Vec2 tc = tc1*r1 + tc2*r2 + tc3*r3;
-                                
+
                                 osg::notify(osg::NOTICE)<<"We hit tex coords "<<tc<<std::endl;
-                                
+
                             }
-                            
-                            
                         }
                         else
                         {
-                            osg::notify(osg::NOTICE)<<"Hit but insufficient indices to work with";
+                            osg::notify(osg::NOTICE)<<"Intersection has insufficient indices to work with";
                         }
-    
-                    }
 
-                } 
-            }
-            else
-            {
-                osg::notify(osg::NOTICE)<<"No hit"<<std::endl;
+                    }
+                }
+                else
+                {
+                    osg::notify(osg::NOTICE)<<"No intersection"<<std::endl;
+                }
             }
             break;
         }
@@ -289,31 +267,42 @@ void MovieEventHandler::getUsage(osg::ApplicationUsage& usage) const
 }
 
 
-osg::Geometry* myCreateTexturedQuadGeometry(const osg::Vec3& pos,float width,float height, osg::Image* image, bool useTextureRectangle)
+osg::Geometry* myCreateTexturedQuadGeometry(const osg::Vec3& pos,float width,float height, osg::Image* image, bool useTextureRectangle, bool xyPlane, bool option_flip)
 {
+    bool flip = image->getOrigin()==osg::Image::TOP_LEFT;
+    if (option_flip) flip = !flip;
+    
     if (useTextureRectangle)
     {
         osg::Geometry* pictureQuad = osg::createTexturedQuadGeometry(pos,
                                            osg::Vec3(width,0.0f,0.0f),
-                                           osg::Vec3(0.0f,0.0f,height),
-                                           0.0f,image->t(), image->s(),0.0f);
+                                           xyPlane ? osg::Vec3(0.0f,height,0.0f) : osg::Vec3(0.0f,0.0f,height),
+                                           0.0f, flip ? image->t() : 0.0, image->s(), flip ? 0.0 : image->t());
 
+        osg::TextureRectangle* texture = new osg::TextureRectangle(image);
+        texture->setWrap(osg::Texture::WRAP_S, osg::Texture::CLAMP_TO_EDGE);
+        texture->setWrap(osg::Texture::WRAP_T, osg::Texture::CLAMP_TO_EDGE);
+        
+        
         pictureQuad->getOrCreateStateSet()->setTextureAttributeAndModes(0,
-                    new osg::TextureRectangle(image),
-                    osg::StateAttribute::ON);
-                    
+                                                                        texture,
+                                                                        osg::StateAttribute::ON);
+        
         return pictureQuad;
     }
     else
     {
         osg::Geometry* pictureQuad = osg::createTexturedQuadGeometry(pos,
                                            osg::Vec3(width,0.0f,0.0f),
-                                           osg::Vec3(0.0f,0.0f,height),
-                                           0.0f,1.0f, 1.0f,0.0f);
+                                           xyPlane ? osg::Vec3(0.0f,height,0.0f) : osg::Vec3(0.0f,0.0f,height),
+                                           0.0f, flip ? 1.0f : 0.0f , 1.0f, flip ? 0.0f : 1.0f);
                                     
         osg::Texture2D* texture = new osg::Texture2D(image);
-        texture->setFilter(osg::Texture::MIN_FILTER,osg::Texture::LINEAR);  
-                                       
+        texture->setFilter(osg::Texture::MIN_FILTER,osg::Texture::LINEAR);
+        texture->setWrap(osg::Texture::WRAP_S, osg::Texture::CLAMP_TO_EDGE);
+        texture->setWrap(osg::Texture::WRAP_T, osg::Texture::CLAMP_TO_EDGE);
+        
+        
         pictureQuad->getOrCreateStateSet()->setTextureAttributeAndModes(0,
                     texture,
                     osg::StateAttribute::ON);
@@ -332,22 +321,26 @@ int main(int argc, char** argv)
     arguments.getApplicationUsage()->setDescription(arguments.getApplicationName()+" example demonstrates the use of ImageStream for rendering movies as textures.");
     arguments.getApplicationUsage()->setCommandLineUsage(arguments.getApplicationName()+" [options] filename ...");
     arguments.getApplicationUsage()->addCommandLineOption("-h or --help","Display this information");
+    arguments.getApplicationUsage()->addCommandLineOption("--texture2D","Use Texture2D rather than TextureRectangle.");
+    arguments.getApplicationUsage()->addCommandLineOption("--shader","Use shaders to post process the video.");
+    arguments.getApplicationUsage()->addCommandLineOption("--interactive","Use camera manipulator to allow movement around movie.");
+    arguments.getApplicationUsage()->addCommandLineOption("--flip","Flip the movie so top becomes bottom.");
     
     bool useTextureRectangle = true;
     bool useShader = false;
 
     // construct the viewer.
-    osgProducer::Viewer viewer(arguments);
+    osgViewer::Viewer viewer(arguments);
     
+    if (arguments.argc()<=1)
+    {
+        arguments.getApplicationUsage()->write(std::cout,osg::ApplicationUsage::COMMAND_LINE_OPTION);
+        return 1;
+    }
+
     while (arguments.read("--texture2D")) useTextureRectangle=false;
     while (arguments.read("--shader")) useShader=true;
 
-    // set up the value with sensible default event handlers.
-    viewer.setUpViewer(osgProducer::Viewer::STANDARD_SETTINGS);
-
-    // get details on keyboard and mouse bindings used by the viewer.
-    viewer.getUsage(*arguments.getApplicationUsage());
-
     // if user request help write it out to cout.
     if (arguments.read("-h") || arguments.read("--help"))
     {
@@ -355,16 +348,18 @@ int main(int argc, char** argv)
         return 1;
     }
 
+    bool fullscreen = !arguments.read("--interactive");
+    bool flip = arguments.read("--flip");
+
     osg::ref_ptr<osg::Geode> geode = new osg::Geode;
-    osg::Vec3 pos(0.0f,0.0f,0.0f);
-        
+
     osg::StateSet* stateset = geode->getOrCreateStateSet();
     stateset->setMode(GL_LIGHTING,osg::StateAttribute::OFF);
 
     if (useShader)
     {
         //useTextureRectangle = false;
-        
+
         static const char *shaderSourceTextureRec = {
             "uniform vec4 cutoff_color;\n"
             "uniform samplerRect movie_texture;\n"
@@ -388,7 +383,7 @@ int main(int argc, char** argv)
         };
 
         osg::Program* program = new osg::Program;
-        
+
         program->addShader(new osg::Shader(osg::Shader::FRAGMENT,
                                            useTextureRectangle ? shaderSourceTextureRec : shaderSourceTexture2D));
 
@@ -399,7 +394,12 @@ int main(int argc, char** argv)
 
     }
 
-
+    osg::Vec3 pos(0.0f,0.0f,0.0f);
+    osg::Vec3 topleft = pos;
+    osg::Vec3 bottomright = pos;
+    
+    bool xyPlane = fullscreen;
+    
     for(int i=1;i<arguments.argc();++i)
     {
         if (arguments.isString(i))
@@ -407,12 +407,14 @@ int main(int argc, char** argv)
             osg::Image* image = osgDB::readImageFile(arguments[i]);
             osg::ImageStream* imagestream = dynamic_cast<osg::ImageStream*>(image);
             if (imagestream) imagestream->play();
-            
+
             if (image)
             {
-                geode->addDrawable(myCreateTexturedQuadGeometry(pos,image->s(),image->t(),image, useTextureRectangle));
-                
-                pos.z() += image->t()*1.5f;
+                geode->addDrawable(myCreateTexturedQuadGeometry(pos,image->s(),image->t(),image, useTextureRectangle, xyPlane, flip));
+        
+                bottomright = pos + osg::Vec3(static_cast<float>(image->s()),static_cast<float>(image->t()),0.0f);
+
+                pos.y() += image->t()*1.5f;
             }
             else
             {
@@ -421,79 +423,46 @@ int main(int argc, char** argv)
         }
     }
     
-    if (geode->getNumDrawables()==0)
-    {
-        // nothing loaded.
-        return 1;
-    }
-    
-    
-
-    // pass the model to the MovieEventHandler so it can pick out ImageStream's to manipulate.
-    MovieEventHandler* meh = new MovieEventHandler();
-    geode->setEventCallback(meh);
-    meh->set(geode.get());
-
-    // report any errors if they have occured when parsing the program aguments.
-    if (arguments.errors())
-    {
-        arguments.writeErrorMessages(std::cout);
-        return 1;
-    }
-    
-    if (arguments.argc()<=1)
-    {
-        arguments.getApplicationUsage()->write(std::cout,osg::ApplicationUsage::COMMAND_LINE_OPTION);
-        return 1;
-    }
-
-
-    // any option left unread are converted into errors to write out later.
-    arguments.reportRemainingOptionsAsUnrecognized();
-
-    // report any errors if they have occured when parsing the program aguments.
-    if (arguments.errors())
-    {
-        arguments.writeErrorMessages(std::cout);
-    }
-/*
-    // set up a post swap callback to flush deleted GL objects and compile new GL objects            
-    for(unsigned int cameraNum=0;cameraNum<viewer.getNumberOfCameras();++cameraNum)
-    {
-        Producer::Camera* camera=viewer.getCamera(cameraNum);
-        camera->addPostSwapCallback(new PostSwapFinishCallback());
-    }
-*/
     // set the scene to render
     viewer.setSceneData(geode.get());
 
-    // create the windows and run the threads.
-    viewer.realize();
-
-    while( !viewer.done() )
+    if (viewer.getSceneData()==0)
     {
-        // wait for all cull and draw threads to complete.
-        viewer.sync();
-        
-        // update the scene by traversing it with the the update visitor which will
-        // call all node update callbacks and animations.
-        viewer.update();
-         
-        // fire off the cull and draw traversals of the scene.
-        viewer.frame();
-        
+        arguments.getApplicationUsage()->write(std::cout);
+        return 1;
     }
+
+    // pass the model to the MovieEventHandler so it can pick out ImageStream's to manipulate.
+    MovieEventHandler* meh = new MovieEventHandler();
+    meh->set(viewer.getSceneData());
+    viewer.addEventHandler(meh);
     
-    // wait for all cull and draw threads to complete.
-    viewer.sync();
+    // add in support for stats
+    viewer.addEventHandler(new osgViewer::StatsHandler());
 
-    // run a clean up frame to delete all OpenGL objects.
-    viewer.cleanup_frame();
+    // report any errors if they have occured when parsing the program aguments.
+    if (arguments.errors())
+    {
+        arguments.writeErrorMessages(std::cout);
+        return 1;
+    }
 
-    // wait for all the clean up frame to complete.
-    viewer.sync();
-
-    return 0;
+    if (fullscreen)
+    {
+        viewer.realize();
 
+        viewer.getCamera()->setViewMatrix(osg::Matrix::identity());
+        viewer.getCamera()->setProjectionMatrixAsOrtho2D(topleft.x(),bottomright.x(),topleft.y(),bottomright.y());
 
+        while(!viewer.done())
+        {
+            viewer.frame();
+        }
+        return 0;
+    }
+    else
+    {
+        // create the windows and run the threads.
+        return viewer.run();
+    }
 }
