@@ -1,13 +1,19 @@
-/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2006 Robert Osfield 
- *
- * This application is open source and may be redistributed and/or modified under  
- * the terms of the GNU Public License (GPL) version 1.0 or 
- * (at your option) any later version.
- * 
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
- * OpenSceneGraph Public License for more details.
+/* OpenSceneGraph example, osgphotoalbum.
+*
+*  Permission is hereby granted, free of charge, to any person obtaining a copy
+*  of this software and associated documentation files (the "Software"), to deal
+*  in the Software without restriction, including without limitation the rights
+*  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
+*  copies of the Software, and to permit persons to whom the Software is
+*  furnished to do so, subject to the following conditions:
+*
+*  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+*  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+*  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+*  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+*  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+*  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
+*  THE SOFTWARE.
 */
 
 #include <osg/Notify>
@@ -22,10 +28,12 @@
 
 #include <osgText/Text>
 
-#include <osgProducer/Viewer>
+#include <osgViewer/Viewer>
 
 #include "ImageReaderWriter.h"
 
+#include <iostream>
+
 using namespace osg;
 
 // now register with Registry to instantiate the above reader/writer,
@@ -422,7 +430,7 @@ void Page::traverse(osg::NodeVisitor& nv)
         const osg::FrameStamp* framestamp = nv.getFrameStamp();
         if (framestamp)
         {
-            double t = framestamp->getReferenceTime();
+            double t = framestamp->getSimulationTime();
             
             if (_rotation!=_targetRotation)
             {
@@ -567,8 +575,6 @@ public:
 
     void set(Album* album, float timePerSlide, bool autoSteppingActive);
 
-    virtual void accept(osgGA::GUIEventHandlerVisitor& v) { v.visit(*this); }
-
     virtual bool handle(const osgGA::GUIEventAdapter& ea,osgGA::GUIActionAdapter&);
     
     virtual void getUsage(osg::ApplicationUsage& usage) const;
@@ -677,19 +683,11 @@ int main( int argc, char **argv )
     
 
     // construct the viewer.
-    osgProducer::Viewer viewer(arguments);
-
-    // set up the value with sensible default event handlers.
-    //viewer.setUpViewer(osgProducer::Viewer::ESCAPE_SETS_DONE);
-    viewer.setUpViewer();
+    osgViewer::Viewer viewer;
 
     // register the handler to add keyboard and mosue handling.
     SlideEventHandler* seh = new SlideEventHandler();
-    viewer.getEventHandlerList().push_front(seh);
-
-
-    // get details on keyboard and mouse bindings used by the viewer.
-    viewer.getUsage(*arguments.getApplicationUsage());
+    viewer.addEventHandler(seh);
 
     // read any time delay argument.
     float timeDelayBetweenSlides = 5.0f;
@@ -742,17 +740,12 @@ int main( int argc, char **argv )
 
     // now the windows have been realized we switch off the cursor to prevent it
     // distracting the people seeing the stereo images.
-    float fovx = 1.25f;
-    float fovy = 1.0f;
-    for( unsigned int i = 0; i < viewer.getNumberOfCameras(); i++ )
-    {
-        Producer::Camera* cam = viewer.getCamera(i);
-        //Producer::RenderSurface* rs = cam->getRenderSurface();
-        //rs->useCursor(false);
-        fovx = cam->getLensHorizontalFov();
-        fovy = cam->getLensVerticalFov();
-    }
+    double fovy, aspectRatio, zNear, zFar;
+    viewer.getCamera()->getProjectionMatrixAsPerspective(fovy, aspectRatio, zNear, zFar);
 
+    fovy = osg::DegreesToRadians(fovy);
+    double fovx = atan(tan(fovy*0.5)*aspectRatio)*2.0;
+    
     float radius = 1.0f;
     float width = 2*radius*tan(fovx*0.5f);
     float height = 2*radius*tan(fovy*0.5f);
@@ -770,42 +763,22 @@ int main( int argc, char **argv )
     // set the scene to render
     viewer.setSceneData(album->getScene());
 
-
     // set up the SlideEventHandler.
     seh->set(album.get(),timeDelayBetweenSlides,autoSteppingActive);
     
-
-    // create the windows and run the threads.
     viewer.realize();
     
-    osg::Matrix homePosition;
-    homePosition.makeLookAt(osg::Vec3(0.0f,0.0f,0.0f),osg::Vec3(0.0f,1.0f,0.0f),osg::Vec3(0.0f,0.0f,1.0f));
-        
-    while( !viewer.done() )
+    // switch off the cursor
+    osgViewer::Viewer::Windows windows;
+    viewer.getWindows(windows);
+    for(osgViewer::Viewer::Windows::iterator itr = windows.begin();
+        itr != windows.end();
+        ++itr)
     {
-        // wait for all cull and draw threads to complete.
-        viewer.sync();
-
-        // update the scene by traversing it with the the update visitor which will
-        // call all node update callbacks and animations.
-        viewer.update();
-         
-        //viewer.setView(homePosition);
-
-        // fire off the cull and draw traversals of the scene.
-        viewer.frame();
-        
+        (*itr)->useCursor(false);
     }
-    
-    // wait for all cull and draw threads to complete.
-    viewer.sync();
 
-    // run a clean up frame to delete all OpenGL objects.
-    viewer.cleanup_frame();
 
-    // wait for all the clean up frame to complete.
-    viewer.sync();
-    
-    return 0;
+    return viewer.run();
 }
 
