@@ -1,3 +1,21 @@
+/* OpenSceneGraph example, osgprerender.
+*
+*  Permission is hereby granted, free of charge, to any person obtaining a copy
+*  of this software and associated documentation files (the "Software"), to deal
+*  in the Software without restriction, including without limitation the rights
+*  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
+*  copies of the Software, and to permit persons to whom the Software is
+*  furnished to do so, subject to the following conditions:
+*
+*  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+*  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+*  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+*  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+*  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+*  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
+*  THE SOFTWARE.
+*/
+
 #include <osg/GLExtensions>
 #include <osg/Node>
 #include <osg/Geometry>
@@ -10,6 +28,7 @@
 #include <osg/Depth>
 #include <osg/Billboard>
 #include <osg/Material>
+#include <osg/AnimationPath>
 
 #include <osgGA/TrackballManipulator>
 #include <osgGA/FlightManipulator>
@@ -20,8 +39,9 @@
 #include <osgDB/Registry>
 #include <osgDB/ReadFile>
 
-#include <osgProducer/Viewer>
+#include <osgViewer/Viewer>
 
+#include <iostream>
 
 // call back which cretes a deformation field to oscilate the model.
 class MyGeometryCallback : 
@@ -47,14 +67,14 @@ class MyGeometryCallback :
         virtual void update(osg::NodeVisitor* nv,osg::Drawable* drawable)
         {
             const osg::FrameStamp* fs = nv->getFrameStamp();
-            double referenceTime = fs->getReferenceTime();
+            double simulationTime = fs->getSimulationTime();
             if (_firstCall)
             {
                 _firstCall = false;
-                _startTime = referenceTime;
+                _startTime = simulationTime;
             }
             
-            _time = referenceTime-_startTime;
+            _time = simulationTime-_startTime;
             
             drawable->accept(*this);
             drawable->dirtyBound();
@@ -107,14 +127,14 @@ class MyGeometryCallback :
         
 };
 
-struct MyCameraPostDrawCallback : public osg::CameraNode::DrawCallback
+struct MyCameraPostDrawCallback : public osg::Camera::DrawCallback
 {
     MyCameraPostDrawCallback(osg::Image* image):
         _image(image)
     {
     }
 
-    virtual void operator () (const osg::CameraNode& /*camera*/) const
+    virtual void operator () (const osg::Camera& /*camera*/) const
     {
         if (_image && _image->getPixelFormat()==GL_RGBA && _image->getDataType()==GL_UNSIGNED_BYTE)
         {
@@ -177,7 +197,7 @@ struct MyCameraPostDrawCallback : public osg::CameraNode::DrawCallback
 };
 
 
-osg::Node* createPreRenderSubGraph(osg::Node* subgraph, unsigned tex_width, unsigned tex_height, osg::CameraNode::RenderTargetImplementation renderImplementation, bool useImage, bool useTextureRectangle, bool useHDR)
+osg::Node* createPreRenderSubGraph(osg::Node* subgraph, unsigned tex_width, unsigned tex_height, osg::Camera::RenderTargetImplementation renderImplementation, bool useImage, bool useTextureRectangle, bool useHDR)
 {
     if (!subgraph) return 0;
 
@@ -288,7 +308,7 @@ osg::Node* createPreRenderSubGraph(osg::Node* subgraph, unsigned tex_width, unsi
 
     // then create the camera node to do the render to texture
     {    
-        osg::CameraNode* camera = new osg::CameraNode;
+        osg::Camera* camera = new osg::Camera;
 
         // set up the background color and clear mask.
         camera->setClearColor(osg::Vec4(0.1f,0.1f,0.3f,1.0f));
@@ -321,7 +341,7 @@ osg::Node* createPreRenderSubGraph(osg::Node* subgraph, unsigned tex_width, unsi
         camera->setViewport(0,0,tex_width,tex_height);
 
         // set the camera to render before the main camera.
-        camera->setRenderOrder(osg::CameraNode::PRE_RENDER);
+        camera->setRenderOrder(osg::Camera::PRE_RENDER);
 
         // tell the camera to use OpenGL frame buffer object where supported.
         camera->setRenderTargetImplementation(renderImplementation);
@@ -334,7 +354,7 @@ osg::Node* createPreRenderSubGraph(osg::Node* subgraph, unsigned tex_width, unsi
             image->allocateImage(tex_width, tex_height, 1, GL_RGBA, GL_FLOAT);
 
             // attach the image so its copied on each frame.
-            camera->attach(osg::CameraNode::COLOR_BUFFER, image);
+            camera->attach(osg::Camera::COLOR_BUFFER, image);
             
             camera->setPostDrawCallback(new MyCameraPostDrawCallback(image));
             
@@ -351,7 +371,7 @@ osg::Node* createPreRenderSubGraph(osg::Node* subgraph, unsigned tex_width, unsi
         else
         {
             // attach the texture and use it as the color buffer.
-            camera->attach(osg::CameraNode::COLOR_BUFFER, texture);
+            camera->attach(osg::Camera::COLOR_BUFFER, texture);
         }
 
 
@@ -384,13 +404,7 @@ int main( int argc, char **argv )
     arguments.getApplicationUsage()->addCommandLineOption("--texture-rectangle","Use osg::TextureRectangle for doing the render to texure to.");
    
     // construct the viewer.
-    osgProducer::Viewer viewer(arguments);
-
-    // set up the value with sensible default event handlers.
-    viewer.setUpViewer(osgProducer::Viewer::STANDARD_SETTINGS);
-
-    // get details on keyboard and mouse bindings used by the viewer.
-    viewer.getUsage(*arguments.getApplicationUsage());
+    osgViewer::Viewer viewer(arguments);
 
     // if user request help write it out to cout.
     if (arguments.read("-h") || arguments.read("--help"))
@@ -404,13 +418,13 @@ int main( int argc, char **argv )
     while (arguments.read("--width", tex_width)) {}
     while (arguments.read("--height", tex_height)) {}
 
-    osg::CameraNode::RenderTargetImplementation renderImplementation = osg::CameraNode::FRAME_BUFFER_OBJECT;
+    osg::Camera::RenderTargetImplementation renderImplementation = osg::Camera::FRAME_BUFFER_OBJECT;
     
-    while (arguments.read("--fbo")) { renderImplementation = osg::CameraNode::FRAME_BUFFER_OBJECT; }
-    while (arguments.read("--pbuffer")) { renderImplementation = osg::CameraNode::PIXEL_BUFFER; }
-    while (arguments.read("--pbuffer-rtt")) { renderImplementation = osg::CameraNode::PIXEL_BUFFER_RTT; }
-    while (arguments.read("--fb")) { renderImplementation = osg::CameraNode::FRAME_BUFFER; }
-    while (arguments.read("--window")) { renderImplementation = osg::CameraNode::SEPERATE_WINDOW; }
+    while (arguments.read("--fbo")) { renderImplementation = osg::Camera::FRAME_BUFFER_OBJECT; }
+    while (arguments.read("--pbuffer")) { renderImplementation = osg::Camera::PIXEL_BUFFER; }
+    while (arguments.read("--pbuffer-rtt")) { renderImplementation = osg::Camera::PIXEL_BUFFER_RTT; }
+    while (arguments.read("--fb")) { renderImplementation = osg::Camera::FRAME_BUFFER; }
+    while (arguments.read("--window")) { renderImplementation = osg::Camera::SEPERATE_WINDOW; }
 
     bool useImage = false;
     while (arguments.read("--image")) { useImage = true; }
@@ -421,25 +435,13 @@ int main( int argc, char **argv )
     bool useHDR = false;
     while (arguments.read("--hdr")) { useHDR = true; }
 
-    // any option left unread are converted into errors to write out later.
-    arguments.reportRemainingOptionsAsUnrecognized();
-
-    // report any errors if they have occured when parsing the program aguments.
-    if (arguments.errors())
-    {
-        arguments.writeErrorMessages(std::cout);
-        return 1;
-    }
-    
-    if (arguments.argc()<=1)
-    {
-        arguments.getApplicationUsage()->write(std::cout,osg::ApplicationUsage::COMMAND_LINE_OPTION);
-        return 1;
-    }
-
     
     // load the nodes from the commandline arguments.
     osg::Node* loadedModel = osgDB::readNodeFiles(arguments);
+    
+    // if not loaded assume no arguments passed in, try use default mode instead.
+    if (!loadedModel) loadedModel = osgDB::readNodeFile("cessna.osg");
+    
     if (!loadedModel)
     {
         return 1;
@@ -455,36 +457,8 @@ int main( int argc, char **argv )
     osg::Group* rootNode = new osg::Group();
     rootNode->addChild(createPreRenderSubGraph(loadedModelTransform,tex_width,tex_height, renderImplementation, useImage, useTextureRectangle, useHDR));
 
-
     // add model to the viewer.
     viewer.setSceneData( rootNode );
 
-
-    // create the windows and run the threads.
-    viewer.realize();
-
-    while( !viewer.done() )
-    {
-        // wait for all cull and draw threads to complete.
-        viewer.sync();
-
-        // update the scene by traversing it with the the update visitor which will
-        // call all node update callbacks and animations.
-        viewer.update();
-         
-        // fire off the cull and draw traversals of the scene.
-        viewer.frame();
-        
-    }
-    
-    // wait for all cull and draw threads to complete.
-    viewer.sync();
-
-    // run a clean up frame to delete all OpenGL objects.
-    viewer.cleanup_frame();
-
-    // wait for all the clean up frame to complete.
-    viewer.sync();
-
-    return 0;
+    return viewer.run();
 }
