@@ -1,4 +1,22 @@
-#include <osgProducer/Viewer>
+/* OpenSceneGraph example, osgspheresegment.
+*
+*  Permission is hereby granted, free of charge, to any person obtaining a copy
+*  of this software and associated documentation files (the "Software"), to deal
+*  in the Software without restriction, including without limitation the rights
+*  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
+*  copies of the Software, and to permit persons to whom the Software is
+*  furnished to do so, subject to the following conditions:
+*
+*  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+*  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+*  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+*  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+*  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+*  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
+*  THE SOFTWARE.
+*/
+
+#include <osgViewer/Viewer>
 
 #include <osg/Group>
 #include <osg/Geode>
@@ -9,6 +27,7 @@
 #include <osg/Geometry>
 
 #include <osgUtil/SmoothingVisitor>
+#include <osgUtil/IntersectVisitor>
 
 #include <osgDB/ReadFile>
 
@@ -24,6 +43,8 @@
 
 #include <osg/io_utils>
 
+#include <iostream>
+
 // for the grid data..
 #include "../osghangglide/terrain_coords.h"
 
@@ -326,7 +347,7 @@ osg::Vec3 computeTerrainIntersection(osg::Node* subgraph,float x,float y)
 // MAIN SCENE GRAPH BUILDING FUNCTION
 //////////////////////////////////////////////////////////////////////////////
 
-void build_world(osg::Group *root, unsigned int testCase)
+void build_world(osg::Group *root, unsigned int testCase, bool useOverlay, osgSim::OverlayNode::OverlayTechnique technique)
 {
 
     // create terrain
@@ -564,8 +585,26 @@ void build_world(osg::Group *root, unsigned int testCase)
     }
     
 
-    root->addChild(terrainGeode.get());
+    if (useOverlay)
+    {
+        osgSim::OverlayNode* overlayNode = new osgSim::OverlayNode(technique);
+        overlayNode->getOrCreateStateSet()->setTextureAttribute(1, new osg::TexEnv(osg::TexEnv::DECAL));
 
+        const osg::BoundingSphere& bs = terrainGeode->getBound();
+        osg::Group* overlaySubgraph = createOverlay(bs.center(), bs.radius()*0.5f);
+        overlaySubgraph->addChild(ss.get());
+        overlayNode->setOverlaySubgraph(overlaySubgraph);
+        overlayNode->setOverlayTextureSizeHint(1024);
+        overlayNode->setOverlayBaseHeight(0.0);
+        overlayNode->addChild(terrainGeode.get());
+
+        root->addChild(overlayNode);
+    }
+    else
+    {
+      root->addChild(terrainGeode.get());
+    }
+    
     // create particle effects
     {    
         osg::Vec3 position = computeTerrainIntersection(terrainGeode.get(),100.0f,100.0f);
@@ -619,18 +658,18 @@ int main(int argc, char **argv)
     
 
     // construct the viewer.
-    osgProducer::Viewer viewer(arguments);
-
-    // set up the value with sensible default event handlers.
-    viewer.setUpViewer(osgProducer::Viewer::STANDARD_SETTINGS);
-
-    // get details on keyboard and mouse bindings used by the viewer.
-    viewer.getUsage(*arguments.getApplicationUsage());
+    osgViewer::Viewer viewer(arguments);
 
     // if user request help write it out to cout.
     unsigned int testCase = 0;
-    if (arguments.read("-t", testCase)) {}
+    while (arguments.read("-t", testCase)) {}
 
+    bool useOverlay = false;
+    osgSim::OverlayNode::OverlayTechnique technique = osgSim::OverlayNode::OBJECT_DEPENDENT_WITH_ORTHOGRAPHIC_OVERLAY;
+    while (arguments.read("--object")) { useOverlay = true; technique = osgSim::OverlayNode::OBJECT_DEPENDENT_WITH_ORTHOGRAPHIC_OVERLAY; }
+    while (arguments.read("--ortho") || arguments.read("--orthographic")) { useOverlay = true; technique = osgSim::OverlayNode::VIEW_DEPENDENT_WITH_ORTHOGRAPHIC_OVERLAY; }
+    while (arguments.read("--persp") || arguments.read("--perspective")) { useOverlay = true; technique = osgSim::OverlayNode::VIEW_DEPENDENT_WITH_PERSPECTIVE_OVERLAY; }
+    
 
     // if user request help write it out to cout.
     if (arguments.read("-h") || arguments.read("--help"))
@@ -650,36 +689,10 @@ int main(int argc, char **argv)
     }
     
     osg::Group *root = new osg::Group;
-    build_world(root, testCase);
+    build_world(root, testCase, useOverlay, technique);
    
     // add a viewport to the viewer and attach the scene graph.
     viewer.setSceneData(root);
         
-    // create the windows and run the threads.
-    viewer.realize();
-
-    while( !viewer.done() )
-    {
-        // wait for all cull and draw threads to complete.
-        viewer.sync();
-
-        // update the scene by traversing it with the the update visitor which will
-        // call all node update callbacks and animations.
-        viewer.update();
-         
-        // fire off the cull and draw traversals of the scene.
-        viewer.frame();
-        
-    }
-    
-    // wait for all cull and draw threads to complete.
-    viewer.sync();
-
-    // run a clean up frame to delete all OpenGL objects.
-    viewer.cleanup_frame();
-
-    // wait for all the clean up frame to complete.
-    viewer.sync();
-
-    return 0;
+    return viewer.run();
 }
