@@ -1,17 +1,22 @@
-/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2006 Robert Osfield 
- *
- * This library is open source and may be redistributed and/or modified under  
- * the terms of the OpenSceneGraph Public License (OSGPL) version 0.0 or 
- * (at your option) any later version.  The full license is in LICENSE file
- * included with this distribution, and on the openscenegraph.org website.
- * 
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
- * OpenSceneGraph Public License for more details.
+/* OpenSceneGraph example, osgstereoimage.
+*
+*  Permission is hereby granted, free of charge, to any person obtaining a copy
+*  of this software and associated documentation files (the "Software"), to deal
+*  in the Software without restriction, including without limitation the rights
+*  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
+*  copies of the Software, and to permit persons to whom the Software is
+*  furnished to do so, subject to the following conditions:
+*
+*  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+*  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+*  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+*  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+*  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+*  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
+*  THE SOFTWARE.
 */
 
-#include <osgProducer/Viewer>
+#include <osgViewer/Viewer>
 #include <osgDB/ReadFile>
 #include <osgDB/WriteFile>
 #include <osgUtil/Optimizer>
@@ -23,8 +28,127 @@
 #include <osg/TexMat>
 #include <osg/Texture2D>
 
+#include <iostream>
+
 typedef std::vector<std::string> FileList;
 
+osg::Geode* createSectorForImage(osg::Image* image, osg::TexMat* texmat, float s,float t, float radius, float height, float length)
+{
+
+    int numSegments = 20;
+    float Theta = length/radius;
+    float dTheta = Theta/(float)(numSegments-1);
+    
+    float ThetaZero = height*s/(t*radius);
+    
+    // set up the texture.
+    osg::Texture2D* texture = new osg::Texture2D;
+    texture->setFilter(osg::Texture2D::MIN_FILTER,osg::Texture2D::LINEAR);
+    texture->setFilter(osg::Texture2D::MAG_FILTER,osg::Texture2D::LINEAR);
+    texture->setWrap(osg::Texture2D::WRAP_S,osg::Texture2D::CLAMP_TO_BORDER);
+    texture->setWrap(osg::Texture2D::WRAP_T,osg::Texture2D::CLAMP_TO_BORDER);
+    texture->setResizeNonPowerOfTwoHint(false);
+    texture->setImage(image);
+
+    // set up the drawstate.
+    osg::StateSet* dstate = new osg::StateSet;
+    dstate->setMode(GL_CULL_FACE,osg::StateAttribute::OFF);
+    dstate->setMode(GL_LIGHTING,osg::StateAttribute::OFF);
+    dstate->setTextureAttributeAndModes(0, texture,osg::StateAttribute::ON);
+    dstate->setTextureAttribute(0, texmat);
+
+    // set up the geoset.
+    osg::Geometry* geom = new osg::Geometry;
+    geom->setStateSet(dstate);
+
+    osg::Vec3Array* coords = new osg::Vec3Array();
+    osg::Vec2Array* tcoords = new osg::Vec2Array();
+   
+    int i;
+    float angle = -Theta/2.0f;
+    for(i=0;
+        i<numSegments;
+        ++i, angle+=dTheta)
+    {
+        coords->push_back(osg::Vec3(sinf(angle)*radius,cosf(angle)*radius,height*0.5f)); // top
+        coords->push_back(osg::Vec3(sinf(angle)*radius,cosf(angle)*radius,-height*0.5f)); // bottom.
+        
+        tcoords->push_back(osg::Vec2(angle/ThetaZero+0.5f,1.0f)); // top
+        tcoords->push_back(osg::Vec2(angle/ThetaZero+0.5f,0.0f)); // bottom.
+
+    }
+
+    osg::Vec4Array* colors = new osg::Vec4Array();
+    colors->push_back(osg::Vec4(1.0f,1.0f,1.0f,1.0f));
+
+    osg::DrawArrays* elements = new osg::DrawArrays(osg::PrimitiveSet::QUAD_STRIP,0,coords->size());
+
+    
+
+    geom->setVertexArray(coords);
+    geom->setTexCoordArray(0,tcoords);
+    geom->setColorArray(colors);
+    geom->setColorBinding(osg::Geometry::BIND_OVERALL);
+
+    geom->addPrimitiveSet(elements);
+
+    // set up the geode.
+    osg::Geode* geode = new osg::Geode;
+    geode->addDrawable(geom);
+
+    return geode;
+
+}
+
+osg::Group * loadImages(std::string image1, std::string image2, osg::TexMat* texmatLeft, osg::TexMat* texmatRight, float radius, float height, float length) {
+        osg::ref_ptr<osg::Image> imageLeft = osgDB::readImageFile(image1);
+        osg::ref_ptr<osg::Image> imageRight = osgDB::readImageFile(image2);
+        if (imageLeft.valid() && imageRight.valid())
+        {
+            float average_s = (imageLeft->s()+imageRight->s())*0.5f;
+            float average_t = (imageLeft->t()+imageRight->t())*0.5f;
+            osg::Geode* geodeLeft = createSectorForImage(imageLeft.get(),texmatLeft,average_s,average_t, radius, height, length);
+            geodeLeft->setNodeMask(0x01);
+
+            osg::Geode* geodeRight = createSectorForImage(imageRight.get(),texmatRight,average_s,average_t, radius, height, length);
+            geodeRight->setNodeMask(0x02);
+
+            osg::Group * imageGroup = new osg::Group;
+            
+            imageGroup->addChild(geodeLeft);
+            imageGroup->addChild(geodeRight);
+            return imageGroup;
+        }
+        else
+        {
+            std::cout << "Warning: Unable to load both image files, '"<<image1<<"' & '"<<image2<<"', required for stereo imaging."<<std::endl;
+            return 0;
+        }
+}
+
+// create a switch containing a set of child each containing a 
+// stereo image pair.
+osg::Switch* createScene(FileList fileList, osg::TexMat* texmatLeft, osg::TexMat* texmatRight, float radius, float height, float length)
+{
+    osg::Switch* sw = new osg::Switch;
+
+    // load the images.
+    for(unsigned int i=0;i+1<fileList.size();i+=2)
+    {
+        osg::Group * imageGroup = loadImages(fileList[i],fileList[i+1],texmatLeft,texmatRight, radius,  height, length);
+        if (imageGroup) sw->addChild(imageGroup);
+    }
+
+
+    if (sw->getNumChildren()>0)
+    {
+        // select first child.
+        sw->setSingleChildOn(0);
+    }
+
+    return sw;
+}
+
 class SlideEventHandler : public osgGA::GUIEventHandler
 {
 public:
@@ -36,7 +160,8 @@ public:
 
     void set(osg::Switch* sw, float offsetX, float offsetY, osg::TexMat* texmatLeft, osg::TexMat* texmatRight, float timePerSlide, bool autoSteppingActive);
 
-    virtual void accept(osgGA::GUIEventHandlerVisitor& v) { v.visit(*this); }
+    void set(FileList fileList, osg::Switch* sw, float offsetX, float offsetY, osg::TexMat* texmatLeft, osg::TexMat* texmatRight, float radius, float height, float length, float timePerSlide, bool autoSteppingActive);
+
 
     virtual bool handle(const osgGA::GUIEventAdapter& ea,osgGA::GUIActionAdapter&);
     
@@ -64,6 +189,9 @@ protected:
     osg::ref_ptr<osg::Switch>   _switch;
     osg::ref_ptr<osg::TexMat>   _texmatLeft;
     osg::ref_ptr<osg::TexMat>   _texmatRight;
+    float                        _radius;
+    float                        _height;
+    float                        _length;
     bool                        _firstTraversal;
     unsigned int                _activeSlide;
     double                      _previousTime;
@@ -73,6 +201,7 @@ protected:
     float                       _currentSeperationX;
     float                       _initSeperationY;
     float                       _currentSeperationY;
+    FileList                     _fileList;
         
 };
 
@@ -109,6 +238,35 @@ void SlideEventHandler::set(osg::Switch* sw, float offsetX, float offsetY, osg::
 
 }
 
+void SlideEventHandler::set(FileList fileList, osg::Switch* sw, float offsetX, float offsetY, osg::TexMat* texmatLeft, osg::TexMat* texmatRight, float radius, float height, float length, float timePerSlide, bool autoSteppingActive)
+{
+    _switch = sw;
+    _switch->setUpdateCallback(this);
+    _fileList=FileList(fileList);
+
+    osg::ref_ptr<osg::Group> imageGroup = loadImages(fileList[0],fileList[1],texmatLeft,texmatRight, radius,  height, length);
+    if (imageGroup.get())_switch->addChild(imageGroup.get());
+
+    _texmatLeft = texmatLeft;
+    _texmatRight = texmatRight;
+
+    _radius=radius;
+    _height=height;
+    _length=length;
+
+    _timePerSlide = timePerSlide;
+    _autoSteppingActive = autoSteppingActive;    
+    
+    _initSeperationX = offsetX;
+    _currentSeperationX = _initSeperationX;
+
+    _initSeperationY = offsetY;
+    _currentSeperationY = _initSeperationY;
+
+    initTexMatrices();
+}
+
+
 bool SlideEventHandler::handle(const osgGA::GUIEventAdapter& ea,osgGA::GUIActionAdapter&)
 {
     switch(ea.getEventType())
@@ -121,22 +279,22 @@ bool SlideEventHandler::handle(const osgGA::GUIEventAdapter& ea,osgGA::GUIAction
                 _previousTime = ea.getTime();
                 return true;
             }
-            else if (ea.getKey()=='n')
+            else if ((ea.getKey()=='n') || (ea.getKey()==osgGA::GUIEventAdapter::KEY_Right)) 
             {
                 nextSlide();
                 return true;
             }
-            else if (ea.getKey()=='p')
+            else if ((ea.getKey()=='p') || (ea.getKey()==osgGA::GUIEventAdapter::KEY_Left)) 
             {
                 previousSlide();
                 return true;
             }
-            else if (ea.getKey()=='w')
+            else if ((ea.getKey()=='w') || (ea.getKey()==osgGA::GUIEventAdapter::KEY_KP_Add)) 
             {
                 scaleImage(0.99f);
                 return true;
             }
-            else if (ea.getKey()=='s')
+            else if ((ea.getKey()=='s') || (ea.getKey()==osgGA::GUIEventAdapter::KEY_KP_Subtract)) 
             {
                 scaleImage(1.01f);
                 return true;
@@ -208,7 +366,7 @@ void SlideEventHandler::operator()(osg::Node* node, osg::NodeVisitor* nv)
 {
     if (_autoSteppingActive && nv->getFrameStamp())
     {
-        double time = nv->getFrameStamp()->getReferenceTime();
+        double time = nv->getFrameStamp()->getSimulationTime();
         
         if (_firstTraversal)
         {
@@ -229,22 +387,38 @@ void SlideEventHandler::operator()(osg::Node* node, osg::NodeVisitor* nv)
 
 void SlideEventHandler::nextSlide()
 {
+
     if (_switch->getNumChildren()==0) return;
 
     ++_activeSlide;
-    if (_activeSlide>=_switch->getNumChildren()) _activeSlide = 0;
 
-    _switch->setSingleChildOn(_activeSlide);
+    if (_fileList.size()>0) {
+        if (_activeSlide>= _fileList.size()/2 ) _activeSlide = 0;
+        osg::ref_ptr<osg::Group> images = loadImages(_fileList[2*_activeSlide],_fileList[2*_activeSlide+1],_texmatLeft.get(),_texmatRight.get(),_radius,_height,_length);
+        if (images.valid()) _switch->replaceChild(_switch->getChild(0),images.get());
+
+    } else {
+        if (_activeSlide>=_switch->getNumChildren()) _activeSlide = 0;
+
+        _switch->setSingleChildOn(_activeSlide);
+    }
 }
 
 void SlideEventHandler::previousSlide()
 {
     if (_switch->getNumChildren()==0) return;
 
-    if (_activeSlide==0) _activeSlide = _switch->getNumChildren()-1;
-    else --_activeSlide;
+    if (_fileList.size()>0) {
+        if (_activeSlide==0) _activeSlide = _fileList.size()/2-1;
+        else --_activeSlide;
+        osg::ref_ptr<osg::Group> images = loadImages(_fileList[2*_activeSlide],_fileList[2*_activeSlide+1],_texmatLeft.get(),_texmatRight.get(),_radius,_height,_length);
+        if (images.valid()) _switch->replaceChild(_switch->getChild(0),images.get());
+    } else {    
+        if (_activeSlide==0) _activeSlide = _switch->getNumChildren()-1;
+        else --_activeSlide;
 
-    _switch->setSingleChildOn(_activeSlide);
+        _switch->setSingleChildOn(_activeSlide);
+    }
 }
 
 void SlideEventHandler::scaleImage(float s)
@@ -276,119 +450,9 @@ void SlideEventHandler::initTexMatrices()
 }
 
 
-osg::Geode* createSectorForImage(osg::Image* image, osg::TexMat* texmat, float s,float t, float radius, float height, float length)
-{
-
-    int numSegments = 20;
-    float Theta = length/radius;
-    float dTheta = Theta/(float)(numSegments-1);
-    
-    float ThetaZero = height*s/(t*radius);
-    
-    // set up the texture.
-    osg::Texture2D* texture = new osg::Texture2D;
-    texture->setFilter(osg::Texture2D::MIN_FILTER,osg::Texture2D::LINEAR);
-    texture->setFilter(osg::Texture2D::MAG_FILTER,osg::Texture2D::LINEAR);
-    texture->setImage(image);
-
-    // set up the drawstate.
-    osg::StateSet* dstate = new osg::StateSet;
-    dstate->setMode(GL_CULL_FACE,osg::StateAttribute::OFF);
-    dstate->setMode(GL_LIGHTING,osg::StateAttribute::OFF);
-    dstate->setTextureAttributeAndModes(0, texture,osg::StateAttribute::ON);
-    dstate->setTextureAttribute(0, texmat);
-
-    // set up the geoset.
-    osg::Geometry* geom = new osg::Geometry;
-    geom->setStateSet(dstate);
-
-    osg::Vec3Array* coords = new osg::Vec3Array();
-    osg::Vec2Array* tcoords = new osg::Vec2Array();
-   
-    int i;
-    float angle = -Theta/2.0f;
-    for(i=0;
-        i<numSegments;
-        ++i, angle+=dTheta)
-    {
-        coords->push_back(osg::Vec3(sinf(angle)*radius,cosf(angle)*radius,height*0.5f)); // top
-        coords->push_back(osg::Vec3(sinf(angle)*radius,cosf(angle)*radius,-height*0.5f)); // bottom.
-        
-        tcoords->push_back(osg::Vec2(angle/ThetaZero+0.5f,1.0f)); // top
-        tcoords->push_back(osg::Vec2(angle/ThetaZero+0.5f,0.0f)); // bottom.
-
-    }
-
-    osg::Vec4Array* colors = new osg::Vec4Array();
-    colors->push_back(osg::Vec4(1.0f,1.0f,1.0f,1.0f));
-
-    osg::DrawArrays* elements = new osg::DrawArrays(osg::PrimitiveSet::QUAD_STRIP,0,coords->size());
-
-    
-
-    geom->setVertexArray(coords);
-    geom->setTexCoordArray(0,tcoords);
-    geom->setColorArray(colors);
-    geom->setColorBinding(osg::Geometry::BIND_OVERALL);
-
-    geom->addPrimitiveSet(elements);
-
-    // set up the geode.
-    osg::Geode* geode = new osg::Geode;
-    geode->addDrawable(geom);
-
-    return geode;
-
-}
-
-// create a switch containing a set of child each containing a 
-// stereo image pair.
-osg::Switch* createScene(const FileList& fileList, osg::TexMat* texmatLeft, osg::TexMat* texmatRight, float radius, float height, float length)
-{
-    osg::Switch* sw = new osg::Switch;
-
-    // load the images.
-    for(unsigned int i=0;i+1<fileList.size();i+=2)
-    {
-        osg::ref_ptr<osg::Image> imageLeft = osgDB::readImageFile(fileList[i]);
-        osg::ref_ptr<osg::Image> imageRight = osgDB::readImageFile(fileList[i+1]);
-        if (imageLeft.valid() && imageRight.valid())
-        {
-            float average_s = (imageLeft->s()+imageRight->s())*0.5f;
-            float average_t = (imageLeft->t()+imageRight->t())*0.5f;
-
-            osg::Geode* geodeLeft = createSectorForImage(imageLeft.get(),texmatLeft,average_s,average_t, radius, height, length);
-            geodeLeft->setNodeMask(0x01);
-
-            osg::Geode* geodeRight = createSectorForImage(imageRight.get(),texmatRight,average_s,average_t, radius, height, length);
-            geodeRight->setNodeMask(0x02);
-
-            osg::ref_ptr<osg::Group> imageGroup = new osg::Group;
-            
-            imageGroup->addChild(geodeLeft);
-            imageGroup->addChild(geodeRight);
-            
-            sw->addChild(imageGroup.get());
-        }
-        else
-        {
-            std::cout << "Warning: Unable to load both image files, '"<<fileList[i]<<"' & '"<<fileList[i+1]<<"', required for stereo imaging."<<std::endl;
-        }
-    }
-
-
-    if (sw->getNumChildren()>0)
-    {
-        // select first child.
-        sw->setSingleChildOn(0);
-    }
-
-    return sw;
-}
 
 int main( int argc, char **argv )
 {
-
     // use an ArgumentParser object to manage the program arguments.
     osg::ArgumentParser arguments(&argc,argv);
     
@@ -399,22 +463,21 @@ int main( int argc, char **argv )
     arguments.getApplicationUsage()->addCommandLineOption("-a","Enter auto advance of image pairs on start up.");
     arguments.getApplicationUsage()->addCommandLineOption("-x <float>","Horizontal offset of left and right images.");
     arguments.getApplicationUsage()->addCommandLineOption("-y <float>","Vertical offset of left and right images.");
+    arguments.getApplicationUsage()->addCommandLineOption("--disk","Keep images on disk");
+    arguments.getApplicationUsage()->addCommandLineOption("-files <filename>","Load filenames from a file");
     arguments.getApplicationUsage()->addCommandLineOption("-h or --help","Display this information");
+    arguments.getApplicationUsage()->addCommandLineOption("--SingleThreaded","Select SingleThreaded threading model for viewer.");
+    arguments.getApplicationUsage()->addCommandLineOption("--CullDrawThreadPerContext","Select CullDrawThreadPerContext threading model for viewer.");
+    arguments.getApplicationUsage()->addCommandLineOption("--DrawThreadPerContext","Select DrawThreadPerContext threading model for viewer.");
+    arguments.getApplicationUsage()->addCommandLineOption("--CullThreadPerCameraDrawThreadPerContext","Select CullThreadPerCameraDrawThreadPerContext threading model for viewer.");
     
 
     // construct the viewer.
-    osgProducer::Viewer viewer(arguments);
-
-    // set up the value with sensible default event handlers.
-    viewer.setUpViewer(osgProducer::Viewer::ESCAPE_SETS_DONE);
+    osgViewer::Viewer viewer;
 
     // register the handler to add keyboard and mosue handling.
     SlideEventHandler* seh = new SlideEventHandler();
-    viewer.getEventHandlerList().push_front(seh);
-
-
-    // get details on keyboard and mouse bindings used by the viewer.
-    viewer.getUsage(*arguments.getApplicationUsage());
+    viewer.addEventHandler(seh);
 
     // read any time delay argument.
     float timeDelayBetweenSlides = 5.0f;
@@ -429,6 +492,22 @@ int main( int argc, char **argv )
     float offsetY=0.0f;
     while (arguments.read("-y",offsetY)) {}
 
+    bool onDisk=false;
+    while (arguments.read("--disk")) { onDisk=true;}
+
+    std::string filename="";
+    FileList fileList;
+    // extract the filenames from the a file, one filename per line.
+    while (arguments.read("-files",filename)) {
+        std::ifstream is(filename.c_str());
+        if (is) {
+                std::string line;
+                while (std::getline(is,line,'\n')) fileList.push_back(line);
+                is.close();
+            }
+    
+    }
+
     // if user request help write it out to cout.
     if (arguments.read("-h") || arguments.read("--help"))
     {
@@ -436,6 +515,14 @@ int main( int argc, char **argv )
         return 1;
     }
 
+    osgViewer::Viewer::ThreadingModel threading = osgViewer::Viewer::SingleThreaded;
+    while (arguments.read("--SingleThreaded")) threading = osgViewer::Viewer::SingleThreaded;
+    while (arguments.read("--CullDrawThreadPerContext")) threading = osgViewer::Viewer::CullDrawThreadPerContext;
+    while (arguments.read("--DrawThreadPerContext")) threading = osgViewer::Viewer::DrawThreadPerContext;
+    while (arguments.read("--CullThreadPerCameraDrawThreadPerContext")) threading = osgViewer::Viewer::CullThreadPerCameraDrawThreadPerContext;
+
+    viewer.setThreadingModel(threading);
+
     // any option left unread are converted into errors to write out later.
     arguments.reportRemainingOptionsAsUnrecognized();
 
@@ -446,45 +533,30 @@ int main( int argc, char **argv )
         return 1;
     }
     
-    if (arguments.argc()<=1)
-    {
-        arguments.getApplicationUsage()->write(std::cout,osg::ApplicationUsage::COMMAND_LINE_OPTION);
-        return 1;
-    }
-
     // extract the filenames from the arguments list.
-    FileList fileList;
     for(int pos=1;pos<arguments.argc();++pos)
     {
         if (arguments.isString(pos)) fileList.push_back(arguments[pos]);
     }
 
-    if (fileList.size()<2)
+    if (fileList.empty())
     {
+        fileList.push_back("Images/dog_left_eye.jpg");
+         fileList.push_back("Images/dog_right_eye.jpg");
+    }
+    else if (fileList.size()<2)
+    {
+        arguments.getApplicationUsage()->write(std::cout,osg::ApplicationUsage::COMMAND_LINE_OPTION);
         return 1;
     }
 
-    // set up the use of stereo by default.
-    osg::DisplaySettings* ds = viewer.getDisplaySettings();
-    if (!ds) ds = osg::DisplaySettings::instance();
-    if (ds) ds->setStereo(true);
-
-    // create the windows and run the threads.
-    viewer.realize();
-
     // now the windows have been realized we switch off the cursor to prevent it
     // distracting the people seeing the stereo images.
-    float fovy = 1.0f;
-    for( unsigned int i = 0; i < viewer.getNumberOfCameras(); i++ )
-    {
-        Producer::Camera* cam = viewer.getCamera(i);
-        Producer::RenderSurface* rs = cam->getRenderSurface();
-        rs->useCursor(false);
-        fovy = osg::DegreesToRadians(cam->getLensVerticalFov());
-    }
+    double fovy, aspectRatio, zNear, zFar;
+    viewer.getCamera()->getProjectionMatrixAsPerspective(fovy, aspectRatio, zNear, zFar);
 
     float radius = 1.0f;
-    float height = 2*radius*tan(fovy*0.5f);
+    float height = 2*radius*tan(osg::DegreesToRadians(fovy)*0.5f);
     float length = osg::PI*radius;  // half a cylinder.
 
     // use a texure matrix to control the placement of the image.
@@ -492,61 +564,56 @@ int main( int argc, char **argv )
     osg::TexMat* texmatRight = new osg::TexMat;
 
     // creat the scene from the file list.
-    osg::ref_ptr<osg::Switch> rootNode = createScene(fileList,texmatLeft,texmatRight,radius,height,length);
-
+    osg::ref_ptr<osg::Switch> rootNode;
+    if (!onDisk)  rootNode = createScene(fileList,texmatLeft,texmatRight,radius,height,length);
+    else rootNode=new osg::Switch();
 
     //osgDB::writeNodeFile(*rootNode,"test.osg");
 
+
+    viewer.getCamera()->setCullMask(0xffffffff);
+    viewer.getCamera()->setCullMaskLeft(0x00000001);
+    viewer.getCamera()->setCullMaskRight(0x00000002);
+
+    // set up the use of stereo by default.
+    osg::DisplaySettings::instance()->setStereo(true);
+
     // set the scene to render
     viewer.setSceneData(rootNode.get());
 
 
-    viewer.getCullSettings().setCullMask(0xffffffff);
-    viewer.getCullSettings().setCullMaskLeft(0x00000001);
-    viewer.getCullSettings().setCullMaskRight(0x00000002);
+    // create the windows and run the threads.
+    viewer.realize();
 
-    // set all the sceneview's up so that their left and right add cull masks are set up.
-    for(osgProducer::OsgCameraGroup::SceneHandlerList::iterator itr=viewer.getSceneHandlerList().begin();
-        itr!=viewer.getSceneHandlerList().end();
+
+    // switch off the cursor
+    osgViewer::Viewer::Windows windows;
+    viewer.getWindows(windows);
+    for(osgViewer::Viewer::Windows::iterator itr = windows.begin();
+        itr != windows.end();
         ++itr)
     {
-        osgUtil::SceneView* sceneview = (*itr)->getSceneView();
-        sceneview->setFusionDistance(osgUtil::SceneView::USE_FUSION_DISTANCE_VALUE,radius);
+        (*itr)->useCursor(false);
     }
 
+    viewer.setFusionDistance(osgUtil::SceneView::USE_FUSION_DISTANCE_VALUE,radius);
 
     // set up the SlideEventHandler.
-    seh->set(rootNode.get(),offsetX,offsetY,texmatLeft,texmatRight,timeDelayBetweenSlides,autoSteppingActive);
-    
+    if (onDisk) seh->set(fileList,rootNode.get(),offsetX,offsetY,texmatLeft,texmatRight,radius,height,length,timeDelayBetweenSlides,autoSteppingActive);
+    else seh->set(rootNode.get(),offsetX,offsetY,texmatLeft,texmatRight,timeDelayBetweenSlides,autoSteppingActive);
     
     osg::Matrix homePosition;
     homePosition.makeLookAt(osg::Vec3(0.0f,0.0f,0.0f),osg::Vec3(0.0f,1.0f,0.0f),osg::Vec3(0.0f,0.0f,1.0f));
         
     while( !viewer.done() )
     {
-        // wait for all cull and draw threads to complete.
-        viewer.sync();
-
-        // update the scene by traversing it with the the update visitor which will
-        // call all node update callbacks and animations.
-        viewer.update();
-         
-        viewer.setView(homePosition);
+        viewer.getCamera()->setViewMatrix(homePosition);
 
         // fire off the cull and draw traversals of the scene.
         viewer.frame();
         
     }
     
-    // wait for all cull and draw threads to complete.
-    viewer.sync();
-
-    // run a clean up frame to delete all OpenGL objects.
-    viewer.cleanup_frame();
-
-    // wait for all the clean up frame to complete.
-    viewer.sync();
-    
     return 0;
 }
 
