@@ -1,14 +1,19 @@
-/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2006 Robert Osfield 
- *
- * This library is open source and may be redistributed and/or modified under  
- * the terms of the OpenSceneGraph Public License (OSGPL) version 0.0 or 
- * (at your option) any later version.  The full license is in LICENSE file
- * included with this distribution, and on the openscenegraph.org website.
- * 
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
- * OpenSceneGraph Public License for more details.
+/* OpenSceneGraph example, osgtext.
+*
+*  Permission is hereby granted, free of charge, to any person obtaining a copy
+*  of this software and associated documentation files (the "Software"), to deal
+*  in the Software without restriction, including without limitation the rights
+*  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
+*  copies of the Software, and to permit persons to whom the Software is
+*  furnished to do so, subject to the following conditions:
+*
+*  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+*  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+*  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+*  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+*  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+*  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
+*  THE SOFTWARE.
 */
 
 #include <osgUtil/Optimizer>
@@ -17,12 +22,15 @@
 #include <osgDB/WriteFile>
 #include <osgDB/Registry>
 
-#include <osgProducer/Viewer>
+#include <osgViewer/Viewer>
+#include <osgViewer/ViewerEventHandlers>
 
 #include <osg/Geode>
-#include <osg/CameraNode>
+#include <osg/Camera>
 #include <osg/ShapeDrawable>
 #include <osg/Sequence>
+#include <osg/PolygonMode>
+#include <osg/io_utils>
 
 #include <osgText/Font>
 #include <osgText/Text>
@@ -469,78 +477,191 @@ osg::Group* create3DText(const osg::Vec3& center,float radius)
     return rootNode;    
 }
 
-int main( int argc, char **argv )
+class UpdateTextOperation : public osg::Operation
 {
-    // use an ArgumentParser object to manage the program arguments.
-    osg::ArgumentParser arguments(&argc,argv);
+public:
+
+    UpdateTextOperation(osg::Group* group):        
+        Operation("UpdateTextOperation", true),
+        _maxNumChildren(200),
+        _maxNumTextPerGeode(10),
+        _group(group)
+    {
+    }
+
+    virtual void operator () (osg::Object* callingObject)
+    {
+        // decided which method to call according to whole has called me.
+        osgViewer::Viewer* viewer = dynamic_cast<osgViewer::Viewer*>(callingObject);
+
+        if (viewer) update();
+        else load();
+    }
     
-    // set up the usage document, in case we need to print out how to use this program.
-    arguments.getApplicationUsage()->setDescription(arguments.getApplicationName()+" is the example which demonstrates use of text.");
-    arguments.getApplicationUsage()->setCommandLineUsage(arguments.getApplicationName()+" [options] [filename] ...");
-    arguments.getApplicationUsage()->addCommandLineOption("-h or --help","Display this information");
+    void update()
+    {
+        // osg::notify(osg::NOTICE)<<"*** Doing update"<<std::endl;
+        
+        OpenThreads::ScopedLock<OpenThreads::Mutex> lock(_mutex);
+        
+        if (_mergeSubgraph.valid())
+        {
+            _group->addChild(_mergeSubgraph.get());
+
+            _mergeSubgraph = 0;
+
+            if (_group->getNumChildren()>_maxNumChildren)
+            {
+                osg::Geode* geode = dynamic_cast<osg::Geode*>(_group->getChild(0));
+                if (geode)
+                {
+                    _availableSubgraph.push_back(geode);
+                    geode->removeDrawables(0,geode->getNumDrawables());
+                }
+                _group->removeChild(0,1);
+            }
+            
+            _waitOnMergeBlock.release();
+        }        
+    }
     
+    void load()
+    {
+    
+        // osg::notify(osg::NOTICE)<<"Doing load"<<std::endl;
+
+        osg::ref_ptr<osg::Geode> geode;
+        {
+            OpenThreads::ScopedLock<OpenThreads::Mutex> lock(_mutex);
+            if (!_availableSubgraph.empty())
+            {
+                geode = _availableSubgraph.front();
+                _availableSubgraph.pop_front();
+            }
+        }
+        
+        if (!geode) geode = new osg::Geode;
+
+        for(unsigned int i=0; i<_maxNumTextPerGeode; ++i)
+        {
+            osg::Vec3 position(float(rand()) / float(RAND_MAX), float(rand()) / float(RAND_MAX), float(i)/float(_maxNumTextPerGeode));
+
+            std::string str;
+            unsigned int _numCharacters = 5;
+            for(unsigned int ni=0; ni<_numCharacters;++ni)
+            {
+                str.push_back(char(32.0 + (float(rand())/float(RAND_MAX))*128.0f));
+            }
+                        
+            osgText::Text* text = new osgText::Text;
+            text->setDataVariance(osg::Object::DYNAMIC);
+            text->setPosition(position);
+            text->setFont("times.ttf");
+            text->setText(str);
+            text->setCharacterSize(0.025f);
+            text->setAxisAlignment(osgText::Text::SCREEN);
+            
+            geode->addDrawable(text);
+        }
+
+
+        {        
+            OpenThreads::ScopedLock<OpenThreads::Mutex> lock(_mutex);
+            _mergeSubgraph = geode;
+        }
+        
+        // osg::notify(osg::NOTICE)<<"Waiting on merge"<<std::endl;
+
+        _waitOnMergeBlock.block();
+
+    }
+    
+    virtual void release()
+    {
+        _waitOnMergeBlock.release();
+    }
+
+    typedef std::list< osg::ref_ptr<osg::Geode> > AvailableList;
+
+    unsigned int                _maxNumChildren;
+    unsigned int                _maxNumTextPerGeode;
+    
+    OpenThreads::Mutex          _mutex;
+    osg::ref_ptr<osg::Group>    _group;
+    osg::ref_ptr<osg::Geode>    _mergeSubgraph;
+    AvailableList               _availableSubgraph;
+    OpenThreads::Block          _waitOnMergeBlock;
+    
+    unsigned int                _counter;
+
+};
+
+
+int main(int argc, char** argv)
+{
+    osg::ArgumentParser arguments(&argc, argv);
 
     // construct the viewer.
-    osgProducer::Viewer viewer(arguments);
-
-    // set up the value with sensible default event handlers.
-    viewer.setUpViewer(osgProducer::Viewer::STANDARD_SETTINGS);
-
-    // get details on keyboard and mouse bindings used by the viewer.
-    viewer.getUsage(*arguments.getApplicationUsage());
-
-    // if user request help write it out to cout.
-    if (arguments.read("-h") || arguments.read("--help"))
-    {
-        arguments.getApplicationUsage()->write(std::cout);
-        return 1;
-    }
-
-    // any option left unread are converted into errors to write out later.
-    arguments.reportRemainingOptionsAsUnrecognized();
-
-    // report any errors if they have occured when parsing the program aguments.
-    if (arguments.errors())
-    {
-        arguments.writeErrorMessages(std::cout);
-        return 1;
-    }
+    osgViewer::Viewer viewer(arguments);
     
+    typedef std::list< osg::ref_ptr<osg::OperationThread> > Threads;
 
-    // read the scene from the list of file specified commandline args.
-    osg::ref_ptr<osg::Node> rootNode = osgDB::readNodeFiles(arguments);
+    Threads operationThreads;
+    osg::ref_ptr<UpdateTextOperation> updateOperation;
 
-    // prepare scene.
+    unsigned int numThreads = 0;
+    if (arguments.read("--mt", numThreads) || arguments.read("--mt"))
     {
+        // construct a multi-threaded text updating test.
+        if (numThreads==0) numThreads = 1;
+        
+        // create a group to add everything into.
+        osg::Group* mainGroup = new osg::Group;
+        
+        for(unsigned int i=0; i<numThreads; ++i)
+        {
+            osg::Group* textGroup = new osg::Group;
+            mainGroup->addChild(textGroup);
 
+            // create the background thread
+            osg::OperationThread* operationThread = new osg::OperationThread;
+            
+            operationThreads.push_back(operationThread);
+
+            // create the operation that will run in the background and
+            // sync once per frame with the main viewer loop.
+            updateOperation = new UpdateTextOperation(textGroup);
+
+            // add the operation to the operation thread and start it.
+            operationThread->add(updateOperation.get());
+            operationThread->startThread();
+
+            // add the operation to the viewer to sync once per frame.
+            viewer.addUpdateOperation(updateOperation.get());
+
+
+            // add a unit cube for the text to appear within.
+            osg::Geode* geode = new osg::Geode;
+            geode->getOrCreateStateSet()->setAttribute(new osg::PolygonMode(osg::PolygonMode::FRONT_AND_BACK,osg::PolygonMode::LINE));
+            geode->addDrawable(new osg::ShapeDrawable(new osg::Box(osg::Vec3(0.5f,0.5f,0.5f),1.0)));
+
+            mainGroup->addChild(geode);
+        }
+                
+        viewer.setSceneData(mainGroup);        
+    }
+    else
+    {
+        // prepare scene.
         osg::Vec3 center(0.0f,0.0f,0.0f);
         float radius = 1.0f;
         
-        if (rootNode.valid())
-        {
-            // optimize the scene graph, remove rendundent nodes and state etc.
-            osgUtil::Optimizer optimizer;
-            optimizer.optimize(rootNode.get());
-            
-            const osg::BoundingSphere& bs = rootNode->getBound();
-            center = bs.center();
-            radius = bs.radius();
-        }
-
         // make sure the root node is group so we can add extra nodes to it.
-        osg::Group* group = dynamic_cast<osg::Group*>(rootNode.get());
-        if (!group)
-        {
-            group = new osg::Group;
-            
-            if (rootNode.valid()) group->addChild(rootNode.get());
-            
-            rootNode = group;
-        }
-
+        osg::Group* group = new osg::Group;
+        
         {
             // create the hud.
-            osg::CameraNode* camera = new osg::CameraNode;
+            osg::Camera* camera = new osg::Camera;
             camera->setReferenceFrame(osg::Transform::ABSOLUTE_RF);
             camera->setProjectionMatrixAsOrtho2D(0,1280,0,1024);
             camera->setViewMatrix(osg::Matrix::identity());
@@ -553,55 +674,26 @@ int main( int argc, char **argv )
 
         group->addChild(create3DText(center,radius));
 
+        // set the scene to render
+        viewer.setSceneData(group);
     }
 
-    // set the scene to render
-    viewer.setSceneData(rootNode.get());
-
-
-    // create the windows and run the threads.
-    viewer.realize();
-
 #if 0
-    // this optional compile block is done as a test against graphics
-    // drivers that claim support for generate mip map, but the actual
-    // implementation is flacky.  It is not compiled by default.
-
-    // go through each graphics context and switch off the generate mip map extension.
-    // note, this must be done after the realize so that init of texture state and as 
-    // result extension structures have been iniatilized.
-    for(unsigned int contextID = 0; 
-        contextID<viewer.getDisplaySettings()->getMaxNumberOfGraphicsContexts();
-        ++contextID)
-    {
-        osg::Texture::Extensions* textureExt = osg::Texture::getExtensions(contextID,false);
-        if (textureExt) textureExt->setGenerateMipMapSupported(false);
-    }
+    osgDB::writeNodeFile(*viewer.getSceneData(),"text.osg");
 #endif
+
+    viewer.addEventHandler(new osgViewer::StatsHandler());
+
+    viewer.run();
     
-    while( !viewer.done() )
+    if (!operationThreads.empty())
     {
-        // wait for all cull and draw threads to complete.
-        viewer.sync();
-
-        // update the scene by traversing it with the the update visitor which will
-        // call all node update callbacks and animations.
-        viewer.update();
-         
-        // fire off the cull and draw traversals of the scene.
-        viewer.frame();
-        
+        for(Threads::iterator itr = operationThreads.begin();
+            itr != operationThreads.end();
+            ++itr)
+        {
+            (*itr)->cancel();
+        }
     }
-    
-    // wait for all cull and draw threads to complete.
-    viewer.sync();
-
-    // run a clean up frame to delete all OpenGL objects.
-    viewer.cleanup_frame();
-
-    // wait for all the clean up frame to complete.
-    viewer.sync();
-
-    return 0;
 }
 
