@@ -1,3 +1,21 @@
+/* OpenSceneGraph example, osgtexture1D.
+*
+*  Permission is hereby granted, free of charge, to any person obtaining a copy
+*  of this software and associated documentation files (the "Software"), to deal
+*  in the Software without restriction, including without limitation the rights
+*  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
+*  copies of the Software, and to permit persons to whom the Software is
+*  furnished to do so, subject to the following conditions:
+*
+*  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+*  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+*  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+*  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+*  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+*  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
+*  THE SOFTWARE.
+*/
+
 #include <osg/Notify>
 #include <osg/Texture1D>
 #include <osg/TexGen>
@@ -6,8 +24,9 @@
 #include <osgDB/Registry>
 #include <osgDB/ReadFile>
 
-#include <osgProducer/Viewer>
+#include <osgViewer/Viewer>
 
+#include <iostream>
 
 // Creates a stateset which contains a 1D texture which is populated by contour banded color
 // this is then used in conjunction with TexGen to create contoured models, either in 
@@ -63,7 +82,7 @@ osg::StateSet* create1DTextureStateToDecorate(osg::Node* loadedModel)
     
     osg::TexGen* texgen = new osg::TexGen;
     texgen->setMode(osg::TexGen::OBJECT_LINEAR);
-    texgen->setPlane(osg::TexGen::S,osg::Vec4(0.0f,0.0f,zScale,-zBase));
+    texgen->setPlane(osg::TexGen::S,osg::Plane(0.0f,0.0f,zScale,-zBase));
     
     osg::Material* material = new osg::Material;
     
@@ -126,7 +145,7 @@ class AnimateStateCallback : public osg::NodeCallback
             if (stateset && nv->getFrameStamp())
             {
                 // we have an exisitng stateset, so lets animate it.
-                animateState(stateset,nv->getFrameStamp()->getReferenceTime());
+                animateState(stateset,nv->getFrameStamp()->getSimulationTime());
             }
 
             // note, callback is repsonsible for scenegraph traversal so
@@ -142,98 +161,35 @@ int main( int argc, char **argv )
 
     // use an ArgumentParser object to manage the program arguments.
     osg::ArgumentParser arguments(&argc,argv);
-
-    // set up the usage document, in case we need to print out how to use this program.
-    arguments.getApplicationUsage()->setDescription(arguments.getApplicationName()+" is the example which demonstrates use of 1D textures.");
-    arguments.getApplicationUsage()->setCommandLineUsage(arguments.getApplicationName()+" [options] filename ...");
-    arguments.getApplicationUsage()->addCommandLineOption("-h or --help","Display this information");
    
     // construct the viewer.
-    osgProducer::Viewer viewer(arguments);
-
-    // set up the value with sensible default event handlers.
-    viewer.setUpViewer(osgProducer::Viewer::STANDARD_SETTINGS);
-
-    // get details on keyboard and mouse bindings used by the viewer.
-    viewer.getUsage(*arguments.getApplicationUsage());
-
-    // if user request help write it out to cout.
-    if (arguments.read("-h") || arguments.read("--help"))
-    {
-        arguments.getApplicationUsage()->write(std::cout);
-        return 1;
-    }
-
-    // any option left unread are converted into errors to write out later.
-    arguments.reportRemainingOptionsAsUnrecognized();
-
-    // report any errors if they have occured when parsing the program aguments.
-    if (arguments.errors())
-    {
-        arguments.writeErrorMessages(std::cout);
-        return 1;
-    }
-    
-    if (arguments.argc()<=1)
-    {
-        arguments.getApplicationUsage()->write(std::cout,osg::ApplicationUsage::COMMAND_LINE_OPTION);
-        return 1;
-    }
+    osgViewer::Viewer viewer;
 
     // load the images specified on command line
     osg::Node* loadedModel = osgDB::readNodeFiles(arguments);
-
+  
+    // if not loaded assume no arguments passed in, try use default mode instead.
+    if (!loadedModel) loadedModel = osgDB::readNodeFile("dumptruck.osg");
     
-    if (loadedModel)
-    {
-
-        osg::StateSet* stateset = create1DTextureStateToDecorate(loadedModel);
-        if (!stateset)
-        {
-            std::cout<<"Error: failed to create 1D texture state."<<std::endl;
-            return 1;
-        }
-
-
-        loadedModel->setStateSet(stateset);
-        loadedModel->setUpdateCallback(new AnimateStateCallback());
-
-        // add model to viewer.
-        viewer.setSceneData( loadedModel );
-
-        // create the windows and run the threads.
-        viewer.realize();
-
-        while( !viewer.done() )
-        {
-            // wait for all cull and draw threads to complete.
-            viewer.sync();
-
-            // update the scene by traversing it with the the update visitor which will
-            // call all node update callbacks and animations.
-            viewer.update();
-
-            // fire off the cull and draw traversals of the scene.
-            viewer.frame();
-
-        }
-    
-        // wait for all cull and draw threads to complete.
-        viewer.sync();
-
-        // run a clean up frame to delete all OpenGL objects.
-        viewer.cleanup_frame();
-
-        // wait for all the clean up frame to complete.
-        viewer.sync();
-    }
-    else
+    if (!loadedModel)
     {
         osg::notify(osg::NOTICE)<<arguments.getApplicationUsage()->getCommandLineUsage()<<std::endl;
         return 0;
     }
-    
-    
-    
-    return 0;
+
+    osg::StateSet* stateset = create1DTextureStateToDecorate(loadedModel);
+    if (!stateset)
+    {
+        std::cout<<"Error: failed to create 1D texture state."<<std::endl;
+        return 1;
+    }
+
+
+    loadedModel->setStateSet(stateset);
+    loadedModel->setUpdateCallback(new AnimateStateCallback());
+
+    // add model to viewer.
+    viewer.setSceneData( loadedModel );
+
+    return viewer.run();
 }
