@@ -1,11 +1,32 @@
-#include <osg/UnitTestFramework>
+/* OpenSceneGraph example, osgunittests.
+*
+*  Permission is hereby granted, free of charge, to any person obtaining a copy
+*  of this software and associated documentation files (the "Software"), to deal
+*  in the Software without restriction, including without limitation the rights
+*  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
+*  copies of the Software, and to permit persons to whom the Software is
+*  furnished to do so, subject to the following conditions:
+*
+*  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+*  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+*  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+*  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+*  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+*  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
+*  THE SOFTWARE.
+*/
+
 #include <osg/ArgumentParser>
 #include <osg/ApplicationUsage>
 
 #include <osg/Vec3>
 #include <osg/Matrix>
+#include <osg/Timer>
 #include <osg/io_utils>
 
+#include <OpenThreads/Thread>
+
+#include "UnitTestFramework.h"
 #include "performance.h"
 
 #include <iostream>
@@ -143,36 +164,52 @@ void sizeOfTest()
 
 }
 
-/// Exercises the Matrix.get(Quat&) function, printout is generated on problems only
-void testGetQuatFromMatrix() {
+/// Exercise the Matrix.getRotate function.
+/// Compare the output of:
+///  q1 * q2 
+/// versus
+///  (mat(q1)*mat(q2)*scale).getRotate()
+/// for a range of rotations
+void testGetQuatFromMatrix(const osg::Vec3d& scale) 
+{
+    
+    // Options
+    
     // acceptable error range
-    double eps=1e-3;
+    double eps=1e-6;
 
+    // scale matrix
+    // To not test with scale, use 1,1,1
+    // Not sure if 0's or negative values are acceptable
+    osg::Matrixd scalemat;
+    scalemat.makeScale(scale);
+    
+    // range of rotations
 #if 1
     // wide range
     double rol1start = 0.0;
     double rol1stop = 360.0;
-    double rol1step = 30.0;
+    double rol1step = 20.0;
 
     double pit1start = 0.0;
     double pit1stop = 90.0;
-    double pit1step = 30.0;
+    double pit1step = 20.0;
 
     double yaw1start = 0.0;
     double yaw1stop = 360.0;
-    double yaw1step = 30.0;
+    double yaw1step = 20.0;
 
     double rol2start = 0.0;
     double rol2stop = 360.0;
-    double rol2step = 30.0;
+    double rol2step = 20.0;
 
     double pit2start = 0.0;
     double pit2stop = 90.0;
-    double pit2step = 30.0;
+    double pit2step = 20.0;
 
     double yaw2start = 0.0;
     double yaw2stop = 360.0;
-    double yaw2step = 30.0;
+    double yaw2step = 20.0;
 #else
     // focussed range
     double rol1start = 0.0;
@@ -200,61 +237,79 @@ void testGetQuatFromMatrix() {
     double yaw2step = 0.1;
 #endif
     
+    std::cout << std::endl << "Starting " << __FUNCTION__ << ", it can take a while ..." << std::endl;
+
+    osg::Timer_t tstart, tstop;
+    tstart = osg::Timer::instance()->tick();
+    int count=0;
     for (double rol1 = rol1start; rol1 <= rol1stop; rol1 += rol1step) {
-	for (double pit1 = pit1start; pit1 <= pit1stop; pit1 += pit1step) {
-	    for (double yaw1 = yaw1start; yaw1 <= yaw1stop; yaw1 += yaw1step) {
-		for (double rol2 = rol2start; rol2 <= rol2stop; rol2 += rol2step) {
-		    for (double pit2 = pit2start; pit2 <= pit2stop; pit2 += pit2step) {
-			for (double yaw2 = yaw2start; yaw2 <= yaw2stop; yaw2 += yaw2step) {
-			    // create two quats based on the roll, pitch and yaw values
-			    osg::Quat rot_quat1 =
-				osg::Quat(osg::DegreesToRadians(rol1),osg::Vec3d(1,0,0),
-					  osg::DegreesToRadians(pit1),osg::Vec3d(0,1,0),
-					  osg::DegreesToRadians(yaw1),osg::Vec3d(0,0,1));
-			    
-			    osg::Quat rot_quat2 =
-				osg::Quat(osg::DegreesToRadians(rol2),osg::Vec3d(1,0,0),
-					  osg::DegreesToRadians(pit2),osg::Vec3d(0,1,0),
-					  osg::DegreesToRadians(yaw2),osg::Vec3d(0,0,1));
-			    
-			    // create an output quat using quaternion math
-			    osg::Quat out_quat1;
-			    out_quat1 = rot_quat2 * rot_quat1;
-			    
-			    // create two matrices based on the input quats
-			    osg::Matrixd mat1,mat2;
-			    mat1.makeRotate(rot_quat1);
-			    mat2.makeRotate(rot_quat2);
-			    
-			    // create an output quat by matrix multiplication and get
-			    osg::Matrixd out_mat;
-			    out_mat = mat2 * mat1;
-			    osg::Quat out_quat2;
-			    out_quat2 = out_mat.getRotate();
-			    
-			    // if the output quat length is not one 
-			    // or if the component magnitudes do not match,
-			    // something is amiss
-			    if (fabs(1.0-out_quat2.length()) > eps ||
-				(fabs(out_quat1.x())-fabs(out_quat2.x())) > eps ||
-				(fabs(out_quat1.y())-fabs(out_quat2.y())) > eps ||
-				(fabs(out_quat1.z())-fabs(out_quat2.z())) > eps ||
-				(fabs(out_quat1.w())-fabs(out_quat2.w())) > eps) {
-				std::cout << "problem at: r1=" << rol1
-					  << " p1=" << pit1
-					  << " y1=" << yaw1
-					  << " r2=" << rol2
-					  << " p2=" << pit2
-					  << " y2=" << yaw2 << "\n";
-				std::cout << "quats:        " << out_quat1 << " length: " << out_quat1.length() << "\n";
-				std::cout << "mats and get: " << out_quat2 << " length: " << out_quat2.length() << "\n\n";
-			    }
-			}
-		    }
-		}
-	    }
-	}
+    for (double pit1 = pit1start; pit1 <= pit1stop; pit1 += pit1step) {
+        for (double yaw1 = yaw1start; yaw1 <= yaw1stop; yaw1 += yaw1step) {
+        for (double rol2 = rol2start; rol2 <= rol2stop; rol2 += rol2step) {
+            for (double pit2 = pit2start; pit2 <= pit2stop; pit2 += pit2step) {
+            for (double yaw2 = yaw2start; yaw2 <= yaw2stop; yaw2 += yaw2step) {
+                count++;
+                // create two quats based on the roll, pitch and yaw values
+                osg::Quat rot_quat1 =
+                osg::Quat(osg::DegreesToRadians(rol1),osg::Vec3d(1,0,0),
+                      osg::DegreesToRadians(pit1),osg::Vec3d(0,1,0),
+                      osg::DegreesToRadians(yaw1),osg::Vec3d(0,0,1));
+                
+                osg::Quat rot_quat2 =
+                osg::Quat(osg::DegreesToRadians(rol2),osg::Vec3d(1,0,0),
+                      osg::DegreesToRadians(pit2),osg::Vec3d(0,1,0),
+                      osg::DegreesToRadians(yaw2),osg::Vec3d(0,0,1));
+                
+                // create an output quat using quaternion math
+                osg::Quat out_quat1;
+                out_quat1 = rot_quat2 * rot_quat1;
+                
+                // create two matrices based on the input quats
+                osg::Matrixd mat1,mat2;
+                mat1.makeRotate(rot_quat1);
+                mat2.makeRotate(rot_quat2);
+        
+                // create an output quat by matrix multiplication and getRotate
+                osg::Matrixd out_mat;
+                out_mat = mat2 * mat1;
+                // add matrix scale for even more nastiness
+                out_mat = out_mat * scalemat;
+                osg::Quat out_quat2;
+                out_quat2 = out_mat.getRotate();
+                
+                // If the quaternion W is <0, then we should reflect
+                // to get it into the positive W
+                if(out_quat1.w()<0) out_quat1 = out_quat1 * -1.0;
+                if(out_quat2.w()<0) out_quat2 = out_quat2 * -1.0;
+
+
+                // if the output quat length is not one 
+                // or if the components do not match,
+                // something is amiss
+                if (fabs(1.0-out_quat2.length()) > eps ||
+                (fabs(out_quat1.x()-out_quat2.x())) > eps ||
+                (fabs(out_quat1.y()-out_quat2.y())) > eps ||
+                (fabs(out_quat1.z()-out_quat2.z())) > eps ||
+                (fabs(out_quat1.w()-out_quat2.w())) > eps) {
+                std::cout << __FUNCTION__ << " problem at: \n"
+                      << " r1=" << rol1
+                      << " p1=" << pit1
+                      << " y1=" << yaw1
+                      << " r2=" << rol2
+                      << " p2=" << pit2
+                      << " y2=" << yaw2 << "\n";
+                std::cout << "quats:        " << out_quat1 << " length: " << out_quat1.length() << "\n";
+                std::cout << "mats and get: " << out_quat2 << " length: " << out_quat2.length() << "\n\n";
+                }
+            }
+            }
+        }
+        }
     }
+    }
+    tstop = osg::Timer::instance()->tick();
+    double duration = osg::Timer::instance()->delta_s(tstart,tstop);
+    std::cout << "Time for " << __FUNCTION__ << " with " << count << " iterations: " << duration << std::endl << std::endl;
 }
 
 void testQuatRotate(const osg::Vec3d& from, const osg::Vec3d& to)
@@ -272,7 +327,7 @@ void testQuatRotate(const osg::Vec3d& from, const osg::Vec3d& to)
     std::cout<<"  from * M4x4(q_original) = "<<from * osg::Matrixd::rotate(q_original)<<std::endl;
 }
 
-void testQuat()
+void testQuat(const osg::Vec3d& quat_scale)
 {
     osg::Quat q1;
     q1.makeRotate(osg::DegreesToRadians(30.0),0.0f,0.0f,1.0f);
@@ -313,9 +368,42 @@ void testQuat()
     testQuatRotate(osg::Vec3d(0.0,0.0,1.0),osg::Vec3d(0.0,0.0,-1.0));
     testQuatRotate(osg::Vec3d(0.0,0.0,-1.0),osg::Vec3d(0.0,0.0,1.0));
 
-    testGetQuatFromMatrix();
+    // Test a range of rotations
+    testGetQuatFromMatrix(quat_scale);
+
+    // This is a specific test case for a matrix containing scale and rotation
+    osg::Matrix matrix(0.5, 0.0, 0.0, 0.0,
+                       0.0, 0.5, 0.0, 0.0,
+                       0.0, 0.0, 0.5, 0.0,
+                       1.0, 1.0, 1.0, 1.0);
+                       
+    osg::Quat quat;
+    matrix.get(quat);
+    
+    osg::notify(osg::NOTICE)<<"Matrix = "<<matrix<<"rotation = "<<quat<<", expected quat = (0,0,0,1)"<<std::endl;
 }
 
+class MyThread : public OpenThreads::Thread {
+public:
+  void run(void) { }
+};
+
+void testThreadInitAndExit()
+{
+    std::cout<<"******   Running thread start and delete test   ****** "<<std::endl;
+
+    {
+        MyThread thread;
+        thread.startThread();
+    }
+    
+    // add a sleep to allow the thread start to fall over it its going to.
+    OpenThreads::Thread::microSleep(500000);
+    
+    std::cout<<"pass    thread start and delete test"<<std::endl<<std::endl;
+}
+
+
 int main( int argc, char** argv )
 {
     osg::ArgumentParser arguments(&argc,argv);
@@ -325,6 +413,8 @@ int main( int argc, char** argv )
     arguments.getApplicationUsage()->setCommandLineUsage(arguments.getApplicationName()+" [options]");
     arguments.getApplicationUsage()->addCommandLineOption("-h or --help","Display this information");
     arguments.getApplicationUsage()->addCommandLineOption("qt","Display qualified tests.");
+    arguments.getApplicationUsage()->addCommandLineOption("quat","Display extended quaternion tests.");
+    arguments.getApplicationUsage()->addCommandLineOption("quat_scaled sx sy sz","Display extended quaternion tests of pre scaled matrix.");
     arguments.getApplicationUsage()->addCommandLineOption("sizeof","Display sizeof tests.");
     arguments.getApplicationUsage()->addCommandLineOption("matrix","Display qualified tests.");
     arguments.getApplicationUsage()->addCommandLineOption("performance","Display qualified tests.");
@@ -346,7 +436,13 @@ int main( int argc, char** argv )
     while (arguments.read("sizeof")) printSizeOfTest = true; 
 
     bool printQuatTest = false; 
-    while (arguments.read("quat")) printQuatTest = true; 
+    while (arguments.read("quat")) printQuatTest = true;
+    
+    bool doTestThreadInitAndExit = false;
+    while (arguments.read("thread")) doTestThreadInitAndExit = true;
+
+    osg::Vec3d quat_scale(1.0,1.0,1.0); 
+    while (arguments.read("quat_scaled", quat_scale.x(), quat_scale.y(), quat_scale.z() )) printQuatTest = true; 
 
     bool performanceTest = false; 
     while (arguments.read("p") || arguments.read("performance")) performanceTest = true; 
@@ -371,7 +467,7 @@ int main( int argc, char** argv )
     
     if (printQuatTest)
     {
-        testQuat();
+        testQuat(quat_scale);
     }
 
 
@@ -430,6 +526,11 @@ int main( int argc, char** argv )
          std::cout<<std::endl;
     }
 
+    if (doTestThreadInitAndExit)
+    {
+        testThreadInitAndExit();
+    }
+
     std::cout<<"******   Running tests   ******"<<std::endl;
 
     // Global Data or Context
