@@ -1,3 +1,21 @@
+/* OpenSceneGraph example, osgvolume.
+*
+*  Permission is hereby granted, free of charge, to any person obtaining a copy
+*  of this software and associated documentation files (the "Software"), to deal
+*  in the Software without restriction, including without limitation the rights
+*  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
+*  copies of the Software, and to permit persons to whom the Software is
+*  furnished to do so, subject to the following conditions:
+*
+*  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+*  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+*  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+*  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+*  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+*  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
+*  THE SOFTWARE.
+*/
+
 #include <osg/Node>
 #include <osg/Geometry>
 #include <osg/Notify>
@@ -25,7 +43,9 @@
 
 #include <osgUtil/CullVisitor>
 
-#include <osgProducer/Viewer>
+#include <osgViewer/Viewer>
+
+#include <iostream>
 
 
 typedef std::vector< osg::ref_ptr<osg::Image> > ImageList;
@@ -714,11 +734,6 @@ class FollowMouseCallback : public osgGA::GUIEventHandler, public osg::StateSet:
             }
             return false;
         }
-                
-        virtual void accept(osgGA::GUIEventHandlerVisitor& v)
-        {
-            v.visit(*this);
-        }
         
         bool _updateTransparency;
         bool _updateAlphaCutOff;
@@ -736,6 +751,8 @@ osg::Node* createShaderModel(osg::ref_ptr<osg::Image>& image_3d, osg::ref_ptr<os
     osg::StateSet* stateset = geode->getOrCreateStateSet();
     
     stateset->setEventCallback(new FollowMouseCallback);
+    
+    stateset->setMode(GL_ALPHA_TEST,osg::StateAttribute::ON);
 
     // set up the 3d texture itself,
     // note, well set the filtering up so that mip mapping is disabled,
@@ -936,9 +953,9 @@ osg::Node* createModel(osg::ref_ptr<osg::Image>& image_3d, osg::ref_ptr<osg::Ima
     osg::TexGenNode* texgenNode_0 = new osg::TexGenNode;
     texgenNode_0->setTextureUnit(0);
     texgenNode_0->getTexGen()->setMode(osg::TexGen::EYE_LINEAR);
-    texgenNode_0->getTexGen()->setPlane(osg::TexGen::S, osg::Vec4(xMultiplier,0.0f,0.0f,0.5f));
-    texgenNode_0->getTexGen()->setPlane(osg::TexGen::T, osg::Vec4(0.0f,yMultiplier,0.0f,0.5f));
-    texgenNode_0->getTexGen()->setPlane(osg::TexGen::R, osg::Vec4(0.0f,0.0f,zMultiplier,0.5f));
+    texgenNode_0->getTexGen()->setPlane(osg::TexGen::S, osg::Plane(xMultiplier,0.0f,0.0f,0.5f));
+    texgenNode_0->getTexGen()->setPlane(osg::TexGen::T, osg::Plane(0.0f,yMultiplier,0.0f,0.5f));
+    texgenNode_0->getTexGen()->setPlane(osg::TexGen::R, osg::Plane(0.0f,0.0f,zMultiplier,0.5f));
     
     if (two_pass)
     {
@@ -1440,13 +1457,7 @@ int main( int argc, char **argv )
 //    arguments.getApplicationUsage()->addCommandLineOption("--raw <sizeX> <sizeY> <sizeZ> <numberBytesPerComponent> <numberOfComponents> <endian> <filename>","read a raw image data");
 
     // construct the viewer.
-    osgProducer::Viewer viewer(arguments);
-
-    // set up the value with sensible default event handlers.
-    viewer.setUpViewer(osgProducer::Viewer::STANDARD_SETTINGS);
-
-    // get details on keyboard and mouse bindings used by the viewer.
-    viewer.getUsage(*arguments.getApplicationUsage());
+    osgViewer::Viewer viewer;
 
     // if user request help write it out to cout.
     if (arguments.read("-h") || arguments.read("--help"))
@@ -1641,31 +1652,8 @@ int main( int argc, char **argv )
         // set the scene to render
         viewer.setSceneData(rootNode);
         
-        // create the windows and run the threads.
-        viewer.realize();
-
-        while( !viewer.done() )
-        {
-            // wait for all cull and draw threads to complete.
-            viewer.sync();
-
-            // update the scene by traversing it with the the update visitor which will
-            // call all node update callbacks and animations.
-            viewer.update();
-
-            // fire off the cull and draw traversals of the scene.
-            viewer.frame();
-
-        }
-        
-        // wait for all cull and draw threads to complete.
-        viewer.sync();
-
-        // run a clean up frame to delete all OpenGL objects.
-        viewer.cleanup_frame();
-
-        // wait for all the clean up frame to complete.
-        viewer.sync();
+        // the the viewers main frame loop
+        viewer.run();
     }    
     
     return 0;
