@@ -1,5 +1,22 @@
+/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2006 Robert Osfield 
+ *
+ * This library is open source and may be redistributed and/or modified under  
+ * the terms of the OpenSceneGraph Public License (OSGPL) version 0.0 or 
+ * (at your option) any later version.  The full license is in LICENSE file
+ * included with this distribution, and on the openscenegraph.org website.
+ * 
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
+ * OpenSceneGraph Public License for more details.
+*/
+
+#include <stdlib.h>
+#include <string.h>
+
 #include <osg/ApplicationUsage>
 #include <osg/Math>
+#include <osg/ref_ptr>
 
 using namespace osg;
 
@@ -10,8 +27,8 @@ ApplicationUsage::ApplicationUsage(const std::string& commandLineUsage):
 
 ApplicationUsage* ApplicationUsage::instance()
 {
-    static ApplicationUsage s_applicationUsage;
-    return &s_applicationUsage;
+    static osg::ref_ptr<ApplicationUsage> s_applicationUsage = new ApplicationUsage;
+    return s_applicationUsage.get();
 }
 
 void ApplicationUsage::addUsageExplanation(Type type,const std::string& option,const std::string& explanation)
@@ -241,7 +258,7 @@ void ApplicationUsage::writeEnvironmentSettings(std::ostream& output)
         std::string::size_type len = citr->first.find_first_of("\n\r\t ");
         if (len == std::string::npos) len = citr->first.size();
         line.replace(optionPos,len,citr->first.substr(0,len));
-        char *cp = getenv(citr->first.substr(0, len).c_str());
+        const char *cp = getenv(citr->first.substr(0, len).c_str());
         if (!cp) cp = "[not set]";
         else if (!*cp) cp = "[set]";
         line += std::string(cp) + "\n";
