@@ -85,7 +85,7 @@ void BlendEquation::setExtensions(unsigned int contextID,Extensions* extensions)
 
 BlendEquation::Extensions::Extensions(unsigned int contextID)
 {
-    setupGLExtenions(contextID);
+    setupGLExtensions(contextID);
 }
 
 BlendEquation::Extensions::Extensions(const Extensions& rhs):
@@ -103,7 +103,7 @@ void BlendEquation::Extensions::lowestCommonDenominator(const Extensions& rhs)
     if (!rhs._glBlendEquation)           _glBlendEquation = 0;
 }
 
-void BlendEquation::Extensions::setupGLExtenions(unsigned int contextID)
+void BlendEquation::Extensions::setupGLExtensions(unsigned int contextID)
 {
     _isBlendEquationSupported = isGLExtensionSupported(contextID,"GL_EXT_blend_equation") ||
                              strncmp((const char*)glGetString(GL_VERSION),"1.2",3)>=0;
@@ -111,15 +111,14 @@ void BlendEquation::Extensions::setupGLExtenions(unsigned int contextID)
     _isSGIXMinMaxSupported = isGLExtensionSupported(contextID, "GL_SGIX_blend_alpha_minmax");
     _isLogicOpSupported = isGLExtensionSupported(contextID, "GL_EXT_blend_logic_op");
 
-    _glBlendEquation = getGLExtensionFuncPtr("glBlendEquation", "glBlendEquationEXT");
+    setGLExtensionFuncPtr(_glBlendEquation, "glBlendEquation", "glBlendEquationEXT");
 }
 
 void BlendEquation::Extensions::glBlendEquation(GLenum mode) const
 {
     if (_glBlendEquation)
     {
-        typedef void (APIENTRY * GLBlendEquationProc) (GLenum mode);
-        ((GLBlendEquationProc)_glBlendEquation)(mode);
+        _glBlendEquation(mode);
     }
     else
     {
