@@ -27,7 +27,7 @@ DisplaySettings* DisplaySettings::instance()
     return s_displaySettings.get();
 }
 
-DisplaySettings::DisplaySettings(const DisplaySettings& vs):Referenced()
+DisplaySettings::DisplaySettings(const DisplaySettings& vs):Referenced(true)
 {
     setDisplaySettings(vs);
 }
@@ -70,6 +70,9 @@ void DisplaySettings::setDisplaySettings(const DisplaySettings& vs)
 
     _maxNumOfGraphicsContexts = vs._maxNumOfGraphicsContexts;
     _numMultiSamples = vs._numMultiSamples;
+    
+    _compileContextsHint = vs._compileContextsHint;
+    _serializeDrawDispatch = vs._serializeDrawDispatch;
 }
 
 void DisplaySettings::merge(const DisplaySettings& vs)
@@ -85,6 +88,9 @@ void DisplaySettings::merge(const DisplaySettings& vs)
     if (vs._minimumNumberAlphaBits>_minimumNumberAlphaBits) _minimumNumberAlphaBits = vs._minimumNumberAlphaBits;
     if (vs._minimumNumberStencilBits>_minimumNumberStencilBits) _minimumNumberStencilBits = vs._minimumNumberStencilBits;
     if (vs._numMultiSamples>_numMultiSamples) _numMultiSamples = vs._numMultiSamples;
+
+    if (vs._compileContextsHint) _compileContextsHint = vs._compileContextsHint;
+    if (vs._serializeDrawDispatch) _serializeDrawDispatch = vs._serializeDrawDispatch;
 }
 
 void DisplaySettings::setDefaults()
@@ -123,6 +129,9 @@ void DisplaySettings::setDefaults()
     // switch on anti-aliasing by default, just in case we have an Onyx :-)
     _numMultiSamples = 4;
     #endif
+    
+    _compileContextsHint = false;
+    _serializeDrawDispatch = true;
 }
 
 void DisplaySettings::setMaxNumberOfGraphicsContexts(unsigned int num)
@@ -157,6 +166,8 @@ static ApplicationUsageProxy DisplaySetting_e9(ApplicationUsage::ENVIRONMENTAL_V
 static ApplicationUsageProxy DisplaySetting_e10(ApplicationUsage::ENVIRONMENTAL_VARIABLE,"OSG_SPLIT_STEREO_AUTO_ADJUST_ASPECT_RATIO <mode>","OFF | ON  Default to ON to compenstate for the compression of the aspect ratio when viewing in split screen stereo.  Note, if you are setting fovx and fovy explicityly OFF should be used.");
 static ApplicationUsageProxy DisplaySetting_e11(ApplicationUsage::ENVIRONMENTAL_VARIABLE,"OSG_SPLIT_STEREO_VERTICAL_SEPARATION <float>","number of pixels between viewports");
 static ApplicationUsageProxy DisplaySetting_e12(ApplicationUsage::ENVIRONMENTAL_VARIABLE,"OSG_MAX_NUMBER_OF_GRAPHICS_CONTEXTS <int>","maximum number of graphics contexts to be used with applications.");
+static ApplicationUsageProxy DisplaySetting_e13(ApplicationUsage::ENVIRONMENTAL_VARIABLE,"OSG_COMPIlE_CONTEXTS <mode>","OFF | ON Enable/disable the use a backgrouind compile contexts and threads.");
+static ApplicationUsageProxy DisplaySetting_e14(ApplicationUsage::ENVIRONMENTAL_VARIABLE,"OSG_SERIALIZE_DRAW_DISPATCH <mode>","OFF | ON Enable/disable the use a muetx to serialize the draw dispatch when there are multiple graphics threads.");
 
 void DisplaySettings::readEnvironmentalVariables()
 {
@@ -219,6 +230,10 @@ void DisplaySettings::readEnvironmentalVariables()
         {
             _stereoMode = VERTICAL_INTERLACE;
         }
+        else if (strcmp(ptr,"CHECKERBOARD")==0)
+        {
+            _stereoMode = CHECKERBOARD;
+        }
     }
 
     if( (ptr = getenv("OSG_STEREO")) != 0)
@@ -308,6 +323,34 @@ void DisplaySettings::readEnvironmentalVariables()
     {
         _maxNumOfGraphicsContexts = atoi(ptr);
     }
+
+    if( (ptr = getenv("OSG_COMPIlE_CONTEXTS")) != 0)
+    {
+        if (strcmp(ptr,"OFF")==0)
+        {
+            _compileContextsHint = false;
+        }
+        else
+        if (strcmp(ptr,"ON")==0)
+        {
+            _compileContextsHint = true;
+        }
+    }
+    
+    if( (ptr = getenv("OSG_SERIALIZE_DRAW_DISPATCH")) != 0)
+    {
+        if (strcmp(ptr,"OFF")==0)
+        {
+            _serializeDrawDispatch = false;
+        }
+        else
+        if (strcmp(ptr,"ON")==0)
+        {
+            _serializeDrawDispatch = true;
+        }
+    }
+
+    
 }
 
 void DisplaySettings::readCommandLine(ArgumentParser& arguments)
@@ -318,12 +361,14 @@ void DisplaySettings::readCommandLine(ArgumentParser& arguments)
     {
         arguments.getApplicationUsage()->addCommandLineOption("--display <type>","MONITOR | POWERWALL | REALITY_CENTER | HEAD_MOUNTED_DISPLAY");
         arguments.getApplicationUsage()->addCommandLineOption("--stereo","Use default stereo mode which is ANAGLYPHIC if not overriden by environmental variable");
-        arguments.getApplicationUsage()->addCommandLineOption("--stereo <mode>","ANAGLYPHIC | QUAD_BUFFER | HORIZONTAL_SPLIT | VERTICAL_SPLIT | LEFT_EYE | RIGHT_EYE | ON | OFF ");
+        arguments.getApplicationUsage()->addCommandLineOption("--stereo <mode>","ANAGLYPHIC | QUAD_BUFFER | HORIZONTAL_SPLIT | VERTICAL_SPLIT | LEFT_EYE | RIGHT_EYE | HORIZONTAL_INTERLACE | VERTICAL_INTERLACE | CHECKERBOARD | ON | OFF ");
         arguments.getApplicationUsage()->addCommandLineOption("--rgba","Request a RGBA color buffer visual");
         arguments.getApplicationUsage()->addCommandLineOption("--stencil","Request a stencil buffer visual");
         arguments.getApplicationUsage()->addCommandLineOption("--accum-rgb","Request a rgb accumulator buffer visual");
         arguments.getApplicationUsage()->addCommandLineOption("--accum-rgba","Request a rgb accumulator buffer visual");
         arguments.getApplicationUsage()->addCommandLineOption("--samples <num>","Request a multisample visual");
+        arguments.getApplicationUsage()->addCommandLineOption("--cc","Request use of compile contexts and threads");
+        arguments.getApplicationUsage()->addCommandLineOption("--serialize-draw <mode>","OFF | ON - set the serialization of draw dispatch");
     }
 
     std::string str;
@@ -344,6 +389,7 @@ void DisplaySettings::readCommandLine(ArgumentParser& arguments)
         else if (arguments.match(pos+1,"VERTICAL_SPLIT"))   { arguments.remove(pos,2); _stereo = true;_stereoMode = VERTICAL_SPLIT; }
         else if (arguments.match(pos+1,"HORIZONTAL_INTERLACE")) { arguments.remove(pos,2); _stereo = true;_stereoMode = HORIZONTAL_INTERLACE; }
         else if (arguments.match(pos+1,"VERTICAL_INTERLACE"))   { arguments.remove(pos,2); _stereo = true;_stereoMode = VERTICAL_INTERLACE; }
+        else if (arguments.match(pos+1,"CHECKERBOARD"))     { arguments.remove(pos,2); _stereo = true;_stereoMode = CHECKERBOARD; }
         else if (arguments.match(pos+1,"LEFT_EYE"))         { arguments.remove(pos,2); _stereo = true;_stereoMode = LEFT_EYE; }
         else if (arguments.match(pos+1,"RIGHT_EYE"))        { arguments.remove(pos,2); _stereo = true;_stereoMode = RIGHT_EYE; }
         else if (arguments.match(pos+1,"ON"))               { arguments.remove(pos,2); _stereo = true; }
@@ -376,4 +422,16 @@ void DisplaySettings::readCommandLine(ArgumentParser& arguments)
     {
         _numMultiSamples = atoi(str.c_str());
     }
+
+    while(arguments.read("--cc"))
+    {
+        _compileContextsHint = true;
+    }
+
+    while(arguments.read("--serialize-draw",str))
+    {
+        if (str=="ON") _serializeDrawDispatch = true;
+        else if (str=="OFF") _serializeDrawDispatch = false;
+    }
+
 }
