@@ -245,6 +245,7 @@ void Drawable::flushDeletedVertexBufferObjects(unsigned int contextID,double /*c
 
 
 Drawable::Drawable()
+    :Object(true)
 {
     _boundingBoxComputed = false;
 
@@ -258,6 +259,7 @@ Drawable::Drawable()
 
     _supportsVertexBufferObjects = false;
     _useVertexBufferObjects = false;
+    // _useVertexBufferObjects = true;
 
     _numChildrenRequiringUpdateTraversal = 0;
     _numChildrenRequiringEventTraversal = 0;
@@ -268,6 +270,7 @@ Drawable::Drawable(const Drawable& drawable,const CopyOp& copyop):
     _parents(), // leave empty as parentList is managed by Geode
     _stateset(copyop(drawable._stateset.get())),
     _initialBound(drawable._initialBound),
+    _computeBoundCallback(drawable._computeBoundCallback),
     _boundingBox(drawable._boundingBox),
     _boundingBoxComputed(drawable._boundingBoxComputed),
     _shape(copyop(drawable._shape.get())),
@@ -292,15 +295,50 @@ Drawable::~Drawable()
     dirtyDisplayList();
 }
 
+void Drawable::computeDataVariance()
+{
+    if (getDataVariance() != UNSPECIFIED) return;
+
+    bool dynamic = false;
+    
+    if (getUpdateCallback() ||
+        getEventCallback() ||
+        getCullCallback()) 
+    {
+        dynamic = true;
+    }
+    
+    setDataVariance(dynamic ? DYNAMIC : STATIC);
+}
+
 void Drawable::addParent(osg::Node* node)
 {
-    _parents.push_back(node);    
+    if (getRefMutex())
+    {
+        OpenThreads::ScopedLock<OpenThreads::Mutex> lock(*getRefMutex());
+
+        _parents.push_back(node);
+    }
+    else
+    {
+        _parents.push_back(node);
+    }
 }
 
 void Drawable::removeParent(osg::Node* node)
 {
-    ParentList::iterator pitr = std::find(_parents.begin(),_parents.end(),node);
-    if (pitr!=_parents.end()) _parents.erase(pitr);
+    if (getRefMutex())
+    {
+        OpenThreads::ScopedLock<OpenThreads::Mutex> lock(*getRefMutex());
+
+        ParentList::iterator pitr = std::find(_parents.begin(),_parents.end(),node);
+        if (pitr!=_parents.end()) _parents.erase(pitr);
+    }
+    else
+    {
+        ParentList::iterator pitr = std::find(_parents.begin(),_parents.end(),node);
+        if (pitr!=_parents.end()) _parents.erase(pitr);
+    }
 }
 
 
@@ -450,13 +488,13 @@ void Drawable::dirtyBound()
     }
 }
 
-void Drawable::compileGLObjects(State& state) const
+void Drawable::compileGLObjects(RenderInfo& renderInfo) const
 {
     if (!_useDisplayList) return;
 
     // get the contextID (user defined ID of 0 upwards) for the 
     // current OpenGL context.
-    unsigned int contextID = state.getContextID();
+    unsigned int contextID = renderInfo.getContextID();
 
     // get the globj for the current contextID.
     GLuint& globj = _globjList[contextID];
@@ -471,18 +509,42 @@ void Drawable::compileGLObjects(State& state) const
     glNewList( globj, GL_COMPILE );
 
     if (_drawCallback.valid())
-        _drawCallback->drawImplementation(state,this);
+        _drawCallback->drawImplementation(renderInfo,this);
     else 
-        drawImplementation(state);
+        drawImplementation(renderInfo);
 
     glEndList();
 
 }
 
+void Drawable::setThreadSafeRefUnref(bool threadSafe)
+{
+    Object::setThreadSafeRefUnref(threadSafe);
+
+    if (_stateset.valid()) _stateset->setThreadSafeRefUnref(threadSafe);
+
+    if (_updateCallback.valid()) _updateCallback->setThreadSafeRefUnref(threadSafe);
+    if (_eventCallback.valid()) _eventCallback->setThreadSafeRefUnref(threadSafe);
+    if (_cullCallback.valid()) _cullCallback->setThreadSafeRefUnref(threadSafe);
+    if (_drawCallback.valid()) _drawCallback->setThreadSafeRefUnref(threadSafe);
+    
+    if (_userData.valid()) _userData->setThreadSafeRefUnref(threadSafe);
+}
+
+void Drawable::resizeGLObjectBuffers(unsigned int maxSize)
+{
+    if (_stateset.valid()) _stateset->resizeGLObjectBuffers(maxSize);
+    if (_drawCallback.valid()) _drawCallback->resizeGLObjectBuffers(maxSize);
+
+    _globjList.resize(maxSize);
+}
+
 void Drawable::releaseGLObjects(State* state) const
 {
     if (_stateset.valid()) _stateset->releaseGLObjects(state);
 
+    if (_drawCallback.valid()) _drawCallback->releaseGLObjects(state);
+
     if (!_useDisplayList) return;
     
     if (state)
@@ -497,7 +559,7 @@ void Drawable::releaseGLObjects(State* state) const
         // call the globj if already set otherwise comple and execute.
         if( globj != 0 )
         {
-            glDeleteLists( globj, 1 );
+            Drawable::deleteDisplayList(contextID,globj, getGLObjectSizeHint());
             globj = 0;
         }    
     }
@@ -563,6 +625,10 @@ void Drawable::setUseDisplayList(bool flag)
 
 void Drawable::setUseVertexBufferObjects(bool flag)
 {
+    // _useVertexBufferObjects = true;
+
+    // osg::notify(osg::NOTICE)<<"Drawable::setUseVertexBufferObjects("<<flag<<")"<<std::endl;
+
     // if value unchanged simply return.
     if (_useVertexBufferObjects==flag) return;
 
@@ -587,15 +653,6 @@ void Drawable::dirtyDisplayList()
             _globjList[i] = 0;
         }
     }
-
-    for(i=0;i<_vboList.size();++i)
-    {
-        if (_vboList[i] != 0)
-        {
-            Drawable::deleteVertexBufferObject(i,_vboList[i]);
-            _vboList[i] = 0;
-        }
-    }
 }
 
 
@@ -783,7 +840,7 @@ void Drawable::setExtensions(unsigned int contextID,Extensions* extensions)
 
 Drawable::Extensions::Extensions(unsigned int contextID)
 {
-    setupGLExtenions(contextID);
+    setupGLExtensions(contextID);
 }
 
 Drawable::Extensions::Extensions(const Extensions& rhs):
@@ -794,6 +851,8 @@ Drawable::Extensions::Extensions(const Extensions& rhs):
     _isFogCoordSupported = rhs._isFogCoordSupported;
     _isMultiTexSupported = rhs._isMultiTexSupported;
     _isOcclusionQuerySupported = rhs._isOcclusionQuerySupported;
+    _isTimerQuerySupported = rhs._isTimerQuerySupported;
+    
     _glFogCoordfv = rhs._glFogCoordfv;
     _glSecondaryColor3ubv = rhs._glSecondaryColor3ubv;
     _glSecondaryColor3fv = rhs._glSecondaryColor3fv;
@@ -828,6 +887,7 @@ Drawable::Extensions::Extensions(const Extensions& rhs):
     _gl_get_queryiv_arb = rhs._gl_get_queryiv_arb;
     _gl_get_query_objectiv_arb = rhs._gl_get_query_objectiv_arb;
     _gl_get_query_objectuiv_arb = rhs._gl_get_query_objectuiv_arb;
+    _gl_get_query_objectui64v = rhs._gl_get_query_objectui64v;
 }
 
 
@@ -840,6 +900,8 @@ void Drawable::Extensions::lowestCommonDenominator(const Extensions& rhs)
     if (!rhs._isOcclusionQuerySupported) _isOcclusionQuerySupported = false;
     if (!rhs._isARBOcclusionQuerySupported) _isARBOcclusionQuerySupported = false;
 
+    if (!rhs._isTimerQuerySupported) _isTimerQuerySupported = false;
+
     if (!rhs._glFogCoordfv) _glFogCoordfv = 0;
     if (!rhs._glSecondaryColor3ubv) _glSecondaryColor3ubv = 0;
     if (!rhs._glSecondaryColor3fv) _glSecondaryColor3fv = 0;
@@ -884,9 +946,10 @@ void Drawable::Extensions::lowestCommonDenominator(const Extensions& rhs)
     if (!rhs._gl_get_queryiv_arb) _gl_get_queryiv_arb = 0;
     if (!rhs._gl_get_query_objectiv_arb) _gl_get_query_objectiv_arb = 0;
     if (!rhs._gl_get_query_objectuiv_arb) _gl_get_query_objectuiv_arb = 0;
+    if (!rhs._gl_get_query_objectui64v) _gl_get_query_objectui64v = 0;
 }
 
-void Drawable::Extensions::setupGLExtenions(unsigned int contextID)
+void Drawable::Extensions::setupGLExtensions(unsigned int contextID)
 {
     _isVertexProgramSupported = isGLExtensionSupported(contextID,"GL_ARB_vertex_program");
     _isSecondaryColorSupported = isGLExtensionSupported(contextID,"GL_EXT_secondary_color");
@@ -895,50 +958,54 @@ void Drawable::Extensions::setupGLExtenions(unsigned int contextID)
     _isOcclusionQuerySupported = osg::isGLExtensionSupported(contextID, "GL_NV_occlusion_query" );
     _isARBOcclusionQuerySupported = osg::isGLExtensionSupported(contextID, "GL_ARB_occlusion_query" );
 
-    _glFogCoordfv = ((FogCoordProc)osg::getGLExtensionFuncPtr("glFogCoordfv","glFogCoordfvEXT"));
-    _glSecondaryColor3ubv = ((SecondaryColor3ubvProc)osg::getGLExtensionFuncPtr("glSecondaryColor3ubv","glSecondaryColor3ubvEXT"));
-    _glSecondaryColor3fv = ((SecondaryColor3fvProc)osg::getGLExtensionFuncPtr("glSecondaryColor3fv","glSecondaryColor3fvEXT"));
-    _glMultiTexCoord1f = ((MultiTexCoord1fProc)osg::getGLExtensionFuncPtr("glMultiTexCoord1f","glMultiTexCoord1fARB"));
-    _glMultiTexCoord2fv = ((MultiTexCoordfvProc)osg::getGLExtensionFuncPtr("glMultiTexCoord2fv","glMultiTexCoord2fvARB"));
-    _glMultiTexCoord3fv = ((MultiTexCoordfvProc)osg::getGLExtensionFuncPtr("glMultiTexCoord3fv","glMultiTexCoord3fvARB"));
-    _glMultiTexCoord4fv = ((MultiTexCoordfvProc)osg::getGLExtensionFuncPtr("glMultiTexCoord4fv","glMultiTexCoord4fvARB"));
+    _isTimerQuerySupported = osg::isGLExtensionSupported(contextID, "GL_EXT_timer_query" );;
 
-    _glVertexAttrib1s = ((VertexAttrib1sProc)osg::getGLExtensionFuncPtr("glVertexAttrib1s","glVertexAttrib1sARB"));
-    _glVertexAttrib1f = ((VertexAttrib1fProc)osg::getGLExtensionFuncPtr("glVertexAttrib1f","glVertexAttrib1fARB"));
-    _glVertexAttrib2fv = ((VertexAttribfvProc)osg::getGLExtensionFuncPtr("glVertexAttrib2fv","glVertexAttrib2fvARB"));
-    _glVertexAttrib3fv = ((VertexAttribfvProc)osg::getGLExtensionFuncPtr("glVertexAttrib3fv","glVertexAttrib3fvARB"));
-    _glVertexAttrib4fv = ((VertexAttribfvProc)osg::getGLExtensionFuncPtr("glVertexAttrib4fv","glVertexAttrib4fvARB"));
-    _glVertexAttrib4ubv = ((VertexAttribubvProc)osg::getGLExtensionFuncPtr("glVertexAttrib4ubv","glVertexAttrib4ubvARB"));
-    _glVertexAttrib4Nubv = ((VertexAttribubvProc)osg::getGLExtensionFuncPtr("glVertexAttrib4Nubv","glVertexAttrib4NubvARB"));
 
-    _glGenBuffers = ((GenBuffersProc)osg::getGLExtensionFuncPtr("glGenBuffers","glGenBuffersARB"));
-    _glBindBuffer = ((BindBufferProc)osg::getGLExtensionFuncPtr("glBindBuffer","glBindBufferARB"));
-    _glBufferData = ((BufferDataProc)osg::getGLExtensionFuncPtr("glBufferData","glBufferDataARB"));
-    _glBufferSubData = ((BufferSubDataProc)osg::getGLExtensionFuncPtr("glBufferSubData","glBufferSubDataARB"));
-    _glDeleteBuffers = ((DeleteBuffersProc)osg::getGLExtensionFuncPtr("glDeleteBuffers","glDeleteBuffersARB"));
-    _glIsBuffer = ((IsBufferProc)osg::getGLExtensionFuncPtr("glIsBuffer","glIsBufferARB"));
-    _glGetBufferSubData = ((GetBufferSubDataProc)osg::getGLExtensionFuncPtr("glGetBufferSubData","glGetBufferSubDataARB"));
-    _glMapBuffer = ((MapBufferProc)osg::getGLExtensionFuncPtr("glMapBuffer","glMapBufferARB"));
-    _glUnmapBuffer = ((UnmapBufferProc)osg::getGLExtensionFuncPtr("glUnmapBuffer","glUnmapBufferARB"));
-    _glGetBufferParameteriv = ((GetBufferParameterivProc)osg::getGLExtensionFuncPtr("glGetBufferParameteriv","glGetBufferParameterivARB"));
-    _glGetBufferPointerv = ((GetBufferPointervProc)osg::getGLExtensionFuncPtr("glGetBufferPointerv","glGetBufferPointervARB"));
+    setGLExtensionFuncPtr(_glFogCoordfv, "glFogCoordfv","glFogCoordfvEXT");
+    setGLExtensionFuncPtr(_glSecondaryColor3ubv, "glSecondaryColor3ubv","glSecondaryColor3ubvEXT");
+    setGLExtensionFuncPtr(_glSecondaryColor3fv, "glSecondaryColor3fv","glSecondaryColor3fvEXT");
+    setGLExtensionFuncPtr(_glMultiTexCoord1f, "glMultiTexCoord1f","glMultiTexCoord1fARB");
+    setGLExtensionFuncPtr(_glMultiTexCoord2fv, "glMultiTexCoord2fv","glMultiTexCoord2fvARB");
+    setGLExtensionFuncPtr(_glMultiTexCoord3fv, "glMultiTexCoord3fv","glMultiTexCoord3fvARB");
+    setGLExtensionFuncPtr(_glMultiTexCoord4fv, "glMultiTexCoord4fv","glMultiTexCoord4fvARB");
 
-    _glGenOcclusionQueries = ((GenOcclusionQueriesProc)osg::getGLExtensionFuncPtr("glGenOcclusionQueries","glGenOcclusionQueriesNV"));
-    _glDeleteOcclusionQueries = ((DeleteOcclusionQueriesProc)osg::getGLExtensionFuncPtr("glDeleteOcclusionQueries","glDeleteOcclusionQueriesNV"));
-    _glIsOcclusionQuery = ((IsOcclusionQueryProc)osg::getGLExtensionFuncPtr("glIsOcclusionQuery","_glIsOcclusionQueryNV"));
-    _glBeginOcclusionQuery = ((BeginOcclusionQueryProc)osg::getGLExtensionFuncPtr("glBeginOcclusionQuery","glBeginOcclusionQueryNV"));
-    _glEndOcclusionQuery = ((EndOcclusionQueryProc)osg::getGLExtensionFuncPtr("glEndOcclusionQuery","glEndOcclusionQueryNV"));
-    _glGetOcclusionQueryiv = ((GetOcclusionQueryivProc)osg::getGLExtensionFuncPtr("glGetOcclusionQueryiv","glGetOcclusionQueryivNV"));
-    _glGetOcclusionQueryuiv = ((GetOcclusionQueryuivProc)osg::getGLExtensionFuncPtr("glGetOcclusionQueryuiv","glGetOcclusionQueryuivNV"));
+    setGLExtensionFuncPtr(_glVertexAttrib1s, "glVertexAttrib1s","glVertexAttrib1sARB");
+    setGLExtensionFuncPtr(_glVertexAttrib1f, "glVertexAttrib1f","glVertexAttrib1fARB");
+    setGLExtensionFuncPtr(_glVertexAttrib2fv, "glVertexAttrib2fv","glVertexAttrib2fvARB");
+    setGLExtensionFuncPtr(_glVertexAttrib3fv, "glVertexAttrib3fv","glVertexAttrib3fvARB");
+    setGLExtensionFuncPtr(_glVertexAttrib4fv, "glVertexAttrib4fv","glVertexAttrib4fvARB");
+    setGLExtensionFuncPtr(_glVertexAttrib4ubv, "glVertexAttrib4ubv","glVertexAttrib4ubvARB");
+    setGLExtensionFuncPtr(_glVertexAttrib4Nubv, "glVertexAttrib4Nubv","glVertexAttrib4NubvARB");
 
-    _gl_gen_queries_arb = (GenQueriesProc)osg::getGLExtensionFuncPtr("glGenQueries", "glGenQueriesARB");
-    _gl_delete_queries_arb = (DeleteQueriesProc)osg::getGLExtensionFuncPtr("glDeleteQueries", "glDeleteQueriesARB");
-    _gl_is_query_arb = (IsQueryProc)osg::getGLExtensionFuncPtr("glIsQuery", "glIsQueryARB");
-    _gl_begin_query_arb = (BeginQueryProc)osg::getGLExtensionFuncPtr("glBeginQuery", "glBeginQueryARB");
-    _gl_end_query_arb = (EndQueryProc)osg::getGLExtensionFuncPtr("glEndQuery", "glEndQueryARB");
-    _gl_get_queryiv_arb = (GetQueryivProc)osg::getGLExtensionFuncPtr("glGetQueryiv", "glGetQueryivARB");
-    _gl_get_query_objectiv_arb = (GetQueryObjectivProc)osg::getGLExtensionFuncPtr("glGetQueryObjectiv","glGetQueryObjectivARB");
-    _gl_get_query_objectuiv_arb = (GetQueryObjectuivProc)osg::getGLExtensionFuncPtr("glGetQueryObjectuiv","glGetQueryObjectuivARB");
+    setGLExtensionFuncPtr(_glGenBuffers, "glGenBuffers","glGenBuffersARB");
+    setGLExtensionFuncPtr(_glBindBuffer, "glBindBuffer","glBindBufferARB");
+    setGLExtensionFuncPtr(_glBufferData, "glBufferData","glBufferDataARB");
+    setGLExtensionFuncPtr(_glBufferSubData, "glBufferSubData","glBufferSubDataARB");
+    setGLExtensionFuncPtr(_glDeleteBuffers, "glDeleteBuffers","glDeleteBuffersARB");
+    setGLExtensionFuncPtr(_glIsBuffer, "glIsBuffer","glIsBufferARB");
+    setGLExtensionFuncPtr(_glGetBufferSubData, "glGetBufferSubData","glGetBufferSubDataARB");
+    setGLExtensionFuncPtr(_glMapBuffer, "glMapBuffer","glMapBufferARB");
+    setGLExtensionFuncPtr(_glUnmapBuffer, "glUnmapBuffer","glUnmapBufferARB");
+    setGLExtensionFuncPtr(_glGetBufferParameteriv, "glGetBufferParameteriv","glGetBufferParameterivARB");
+    setGLExtensionFuncPtr(_glGetBufferPointerv, "glGetBufferPointerv","glGetBufferPointervARB");
+
+    setGLExtensionFuncPtr(_glGenOcclusionQueries, "glGenOcclusionQueries","glGenOcclusionQueriesNV");
+    setGLExtensionFuncPtr(_glDeleteOcclusionQueries, "glDeleteOcclusionQueries","glDeleteOcclusionQueriesNV");
+    setGLExtensionFuncPtr(_glIsOcclusionQuery, "glIsOcclusionQuery","_glIsOcclusionQueryNV");
+    setGLExtensionFuncPtr(_glBeginOcclusionQuery, "glBeginOcclusionQuery","glBeginOcclusionQueryNV");
+    setGLExtensionFuncPtr(_glEndOcclusionQuery, "glEndOcclusionQuery","glEndOcclusionQueryNV");
+    setGLExtensionFuncPtr(_glGetOcclusionQueryiv, "glGetOcclusionQueryiv","glGetOcclusionQueryivNV");
+    setGLExtensionFuncPtr(_glGetOcclusionQueryuiv, "glGetOcclusionQueryuiv","glGetOcclusionQueryuivNV");
+
+    setGLExtensionFuncPtr(_gl_gen_queries_arb, "glGenQueries", "glGenQueriesARB");
+    setGLExtensionFuncPtr(_gl_delete_queries_arb, "glDeleteQueries", "glDeleteQueriesARB");
+    setGLExtensionFuncPtr(_gl_is_query_arb, "glIsQuery", "glIsQueryARB");
+    setGLExtensionFuncPtr(_gl_begin_query_arb, "glBeginQuery", "glBeginQueryARB");
+    setGLExtensionFuncPtr(_gl_end_query_arb, "glEndQuery", "glEndQueryARB");
+    setGLExtensionFuncPtr(_gl_get_queryiv_arb, "glGetQueryiv", "glGetQueryivARB");
+    setGLExtensionFuncPtr(_gl_get_query_objectiv_arb, "glGetQueryObjectiv","glGetQueryObjectivARB");
+    setGLExtensionFuncPtr(_gl_get_query_objectuiv_arb, "glGetQueryObjectuiv","glGetQueryObjectuivARB");
+    setGLExtensionFuncPtr(_gl_get_query_objectui64v, "glGetQueryObjectui64v","glGetQueryObjectui64vEXT");
 }
 
 void Drawable::Extensions::glFogCoordfv(const GLfloat* coord) const
@@ -1337,3 +1404,11 @@ void Drawable::Extensions::glGetQueryObjectuiv(GLuint id, GLenum pname, GLuint *
   else
     osg::notify(osg::WARN) << "Error: glGetQueryObjectuiv not supported by OpenGL driver" << std::endl;
 }
+
+void Drawable::Extensions::glGetQueryObjectui64v(GLuint id, GLenum pname, GLuint64EXT *params) const
+{
+  if (_gl_get_query_objectuiv_arb)
+    _gl_get_query_objectui64v(id, pname, params);
+  else
+    osg::notify(osg::WARN) << "Error: glGetQueryObjectui64v not supported by OpenGL driver" << std::endl;
+}
