@@ -19,6 +19,7 @@
 #include <osg/Texture1D>
 #include <osg/Texture2D>
 #include <osg/Texture3D>
+#include <osg/Texture2DArray>
 #include <osg/TextureCubeMap>
 #include <osg/TextureRectangle>
 #include <osg/Notify>
@@ -26,10 +27,10 @@
 
 using namespace osg;
 
+static buffered_object< ref_ptr<FBOExtensions> > s_extensions;
 
 FBOExtensions* FBOExtensions::instance(unsigned contextID, bool createIfNotInitalized)
 {
-    static buffered_object< ref_ptr<FBOExtensions> > s_extensions;
     if (!s_extensions[contextID] && createIfNotInitalized) s_extensions[contextID] = new FBOExtensions(contextID);
     return s_extensions[contextID].get();
 }
@@ -37,7 +38,7 @@ FBOExtensions* FBOExtensions::instance(unsigned contextID, bool createIfNotInita
 /**************************************************************************
  * FBOExtensions
  **************************************************************************/
-#define LOAD_FBO_EXT(name)    name = (T##name *)getGLExtensionFuncPtr(#name);
+#define LOAD_FBO_EXT(name)    setGLExtensionFuncPtr(name, (#name))
 
 FBOExtensions::FBOExtensions(unsigned int contextID)
 :    _supported(false)
@@ -56,6 +57,7 @@ FBOExtensions::FBOExtensions(unsigned int contextID)
     LOAD_FBO_EXT(glFramebufferTexture1DEXT);
     LOAD_FBO_EXT(glFramebufferTexture2DEXT);
     LOAD_FBO_EXT(glFramebufferTexture3DEXT);
+    LOAD_FBO_EXT(glFramebufferTextureLayerEXT);
     LOAD_FBO_EXT(glFramebufferRenderbufferEXT);
     LOAD_FBO_EXT(glGenerateMipmapEXT);
 
@@ -205,7 +207,8 @@ struct FrameBufferAttachment::Pimpl
         TEXTURE2D,
         TEXTURE3D,
         TEXTURECUBE,
-        TEXTURERECT
+        TEXTURERECT,
+        TEXTURE2DARRAY
     };
     
     TargetType targetType;
@@ -269,6 +272,13 @@ FrameBufferAttachment::FrameBufferAttachment(Texture3D* target, int level, int z
     _ximpl->zoffset = zoffset;
 }
 
+FrameBufferAttachment::FrameBufferAttachment(Texture2DArray* target, int layer, int level)
+{
+    _ximpl = new Pimpl(Pimpl::TEXTURE2DARRAY, level);
+    _ximpl->textureTarget = target;
+    _ximpl->zoffset = layer;
+}
+
 FrameBufferAttachment::FrameBufferAttachment(TextureCubeMap* target, int face, int level)
 {
     _ximpl = new Pimpl(Pimpl::TEXTURECUBE, level);
@@ -282,7 +292,7 @@ FrameBufferAttachment::FrameBufferAttachment(TextureRectangle* target)
     _ximpl->textureTarget = target;
 }
 
-FrameBufferAttachment::FrameBufferAttachment(CameraNode::Attachment& attachment)
+FrameBufferAttachment::FrameBufferAttachment(Camera::Attachment& attachment)
 {
     osg::Texture* texture = attachment._texture.get();
     
@@ -312,6 +322,15 @@ FrameBufferAttachment::FrameBufferAttachment(CameraNode::Attachment& attachment)
             _ximpl->zoffset = attachment._face;
             return;
         }
+        
+        osg::Texture2DArray* texture2DArray = dynamic_cast<osg::Texture2DArray*>(texture);
+        if (texture2DArray)
+        {
+            _ximpl = new Pimpl(Pimpl::TEXTURE2DARRAY, attachment._level);
+            _ximpl->textureTarget = texture2DArray;
+            _ximpl->zoffset = attachment._face;
+            return;
+        }
 
         osg::TextureCubeMap* textureCubeMap = dynamic_cast<osg::TextureCubeMap*>(texture);
         if (textureCubeMap)
@@ -344,12 +363,12 @@ FrameBufferAttachment::FrameBufferAttachment(CameraNode::Attachment& attachment)
         }
         else
         {
-            osg::notify(osg::WARN)<<"Error: FrameBufferAttachment::FrameBufferAttachment(CameraNode::Attachment&) passed an empty osg::Image, image must be allocated first."<<std::endl;
+            osg::notify(osg::WARN)<<"Error: FrameBufferAttachment::FrameBufferAttachment(Camera::Attachment&) passed an empty osg::Image, image must be allocated first."<<std::endl;
         }
         return;
     }
 
-    osg::notify(osg::WARN)<<"Error: FrameBufferAttachment::FrameBufferAttachment(CameraNode::Attachment&) passed an unrecognised Texture type."<<std::endl;
+    osg::notify(osg::WARN)<<"Error: FrameBufferAttachment::FrameBufferAttachment(Camera::Attachment&) passed an unrecognised Texture type."<<std::endl;
 }
 
 
@@ -429,6 +448,9 @@ void FrameBufferAttachment::attach(State &state, GLenum attachment_point, const
     case Pimpl::TEXTURE3D:
         ext->glFramebufferTexture3DEXT(GL_FRAMEBUFFER_EXT, attachment_point, GL_TEXTURE_3D, tobj->_id, _ximpl->level, _ximpl->zoffset);
         break;
+    case Pimpl::TEXTURE2DARRAY:
+        ext->glFramebufferTextureLayerEXT(GL_FRAMEBUFFER_EXT, attachment_point, tobj->_id, _ximpl->level, _ximpl->zoffset);
+        break;
     case Pimpl::TEXTURERECT:
         ext->glFramebufferTexture2DEXT(GL_FRAMEBUFFER_EXT, attachment_point, GL_TEXTURE_RECTANGLE, tobj->_id, 0);
         break;
@@ -588,6 +610,7 @@ void FrameBufferObject::apply(State &state) const
 
     }
     
+   
     ext->glBindFramebufferEXT(GL_FRAMEBUFFER_EXT, fboID);
 
     if (dirtyAttachmentList)
