@@ -21,6 +21,7 @@
 #include <osg/StateSet>
 #include <osg/Texture2D>
 #include <osg/Texture3D>
+#include <osg/Texture2DArray>
 
 #include "dxtctool.h"
 
@@ -28,10 +29,12 @@ using namespace osg;
 using namespace std;
 
 Image::Image()
+    :Object(true)
 {
     setDataVariance(STATIC); 
 
     _fileName               = "";
+    _origin                 = BOTTOM_LEFT;
     _s = _t = _r            = 0;
     _internalTextureFormat  = 0;
     _pixelFormat            = (unsigned int)0;
@@ -47,6 +50,7 @@ Image::Image()
 Image::Image(const Image& image,const CopyOp& copyop):
     Object(image,copyop),
     _fileName(image._fileName),
+    _origin(image._origin),
     _s(image._s), _t(image._t), _r(image._r),
     _internalTextureFormat(image._internalTextureFormat),
     _pixelFormat(image._pixelFormat),
@@ -143,6 +147,79 @@ bool Image::isPackedType(GLenum type)
     }    
 }
 
+
+GLenum Image::computePixelFormat(GLenum format)
+{
+    switch(format)
+    {
+        case(GL_ALPHA16F_ARB):
+        case(GL_ALPHA32F_ARB):
+            return GL_ALPHA;
+        case(GL_LUMINANCE16F_ARB):
+        case(GL_LUMINANCE32F_ARB):
+            return GL_LUMINANCE;
+        case(GL_INTENSITY16F_ARB):
+        case(GL_INTENSITY32F_ARB):
+            return GL_INTENSITY;
+        case(GL_LUMINANCE_ALPHA16F_ARB):
+        case(GL_LUMINANCE_ALPHA32F_ARB):
+            return GL_LUMINANCE_ALPHA;
+        case(GL_RGB32F_ARB):
+        case(GL_RGB16F_ARB):
+            return GL_RGB;
+        case(GL_RGBA32F_ARB):
+        case(GL_RGBA16F_ARB):
+            return GL_RGBA;
+
+        case(GL_ALPHA8I_EXT):
+        case(GL_ALPHA16I_EXT):
+        case(GL_ALPHA32I_EXT):
+        case(GL_ALPHA8UI_EXT):
+        case(GL_ALPHA16UI_EXT):
+        case(GL_ALPHA32UI_EXT):
+            return GL_ALPHA_INTEGER_EXT;
+        case(GL_LUMINANCE8I_EXT):
+        case(GL_LUMINANCE16I_EXT):
+        case(GL_LUMINANCE32I_EXT):
+        case(GL_LUMINANCE8UI_EXT):
+        case(GL_LUMINANCE16UI_EXT):
+        case(GL_LUMINANCE32UI_EXT):
+            return GL_LUMINANCE_INTEGER_EXT;
+        case(GL_INTENSITY8I_EXT):
+        case(GL_INTENSITY16I_EXT):
+        case(GL_INTENSITY32I_EXT):
+        case(GL_INTENSITY8UI_EXT):
+        case(GL_INTENSITY16UI_EXT):
+        case(GL_INTENSITY32UI_EXT):
+            notify(WARN)<<"Image::computePixelFormat("<<std::hex<<format<<std::dec<<") intensity pixel format is not correctly specified, so assume GL_LUMINANCE_INTEGER."<<std::endl;            
+            return GL_LUMINANCE_INTEGER_EXT;
+        case(GL_LUMINANCE_ALPHA8I_EXT):
+        case(GL_LUMINANCE_ALPHA16I_EXT):
+        case(GL_LUMINANCE_ALPHA32I_EXT):
+        case(GL_LUMINANCE_ALPHA8UI_EXT):
+        case(GL_LUMINANCE_ALPHA16UI_EXT):
+        case(GL_LUMINANCE_ALPHA32UI_EXT):
+            return GL_LUMINANCE_ALPHA_INTEGER_EXT;;
+        case(GL_RGB32I_EXT):
+        case(GL_RGB16I_EXT):
+        case(GL_RGB8I_EXT):
+        case(GL_RGB32UI_EXT):
+        case(GL_RGB16UI_EXT):
+        case(GL_RGB8UI_EXT):
+            return GL_RGB_INTEGER_EXT;
+        case(GL_RGBA32I_EXT):
+        case(GL_RGBA16I_EXT):
+        case(GL_RGBA8I_EXT):
+        case(GL_RGBA32UI_EXT):
+        case(GL_RGBA16UI_EXT):
+        case(GL_RGBA8UI_EXT):
+            return GL_RGBA_INTEGER_EXT;;
+
+        default:
+            return format;
+    }
+}
+
 unsigned int Image::computeNumComponents(GLenum pixelFormat)
 {
     switch(pixelFormat)
@@ -158,13 +235,69 @@ unsigned int Image::computeNumComponents(GLenum pixelFormat)
         case(GL_GREEN): return 1;
         case(GL_BLUE): return 1;
         case(GL_ALPHA): return 1;
+        case(GL_ALPHA8I_EXT): return 1;
+        case(GL_ALPHA8UI_EXT): return 1;
+        case(GL_ALPHA16I_EXT): return 1;
+        case(GL_ALPHA16UI_EXT): return 1;
+        case(GL_ALPHA32I_EXT): return 1;
+        case(GL_ALPHA32UI_EXT): return 1;
+        case(GL_ALPHA16F_ARB): return 1;
+        case(GL_ALPHA32F_ARB): return 1;
         case(GL_RGB): return 3;
         case(GL_BGR): return 3;
+        case(GL_RGB8I_EXT): return 3;
+        case(GL_RGB8UI_EXT): return 3;
+        case(GL_RGB16I_EXT): return 3;
+        case(GL_RGB16UI_EXT): return 3;
+        case(GL_RGB32I_EXT): return 3;
+        case(GL_RGB32UI_EXT): return 3;
+        case(GL_RGB16F_ARB): return 3;
+        case(GL_RGB32F_ARB): return 3;
+        case(GL_RGBA16F_ARB): return 4;
+        case(GL_RGBA32F_ARB): return 4;
         case(GL_RGBA): return 4;
         case(GL_BGRA): return 4;
         case(GL_LUMINANCE): return 1;
+        case(GL_LUMINANCE4): return 1;
+        case(GL_LUMINANCE8): return 1;
+        case(GL_LUMINANCE12): return 1;
+        case(GL_LUMINANCE16): return 1;
+        case(GL_LUMINANCE8I_EXT): return 1;
+        case(GL_LUMINANCE8UI_EXT): return 1;
+        case(GL_LUMINANCE16I_EXT): return 1;
+        case(GL_LUMINANCE16UI_EXT): return 1;
+        case(GL_LUMINANCE32I_EXT): return 1;
+        case(GL_LUMINANCE32UI_EXT): return 1;
+        case(GL_LUMINANCE16F_ARB): return 1;
+        case(GL_LUMINANCE32F_ARB): return 1;
+        case(GL_LUMINANCE4_ALPHA4): return 2;
+        case(GL_LUMINANCE6_ALPHA2): return 2;
+        case(GL_LUMINANCE8_ALPHA8): return 2;
+        case(GL_LUMINANCE12_ALPHA4): return 2;
+        case(GL_LUMINANCE12_ALPHA12): return 2;
+        case(GL_LUMINANCE16_ALPHA16): return 2;
         case(GL_INTENSITY): return 1;
+        case(GL_INTENSITY4): return 1;
+        case(GL_INTENSITY8): return 1;
+        case(GL_INTENSITY12): return 1;
+        case(GL_INTENSITY16): return 1;
+        case(GL_INTENSITY8UI_EXT): return 1;
+        case(GL_INTENSITY8I_EXT): return 1;
+        case(GL_INTENSITY16I_EXT): return 1;
+        case(GL_INTENSITY16UI_EXT): return 1;
+        case(GL_INTENSITY32I_EXT): return 1;
+        case(GL_INTENSITY32UI_EXT): return 1;
+        case(GL_INTENSITY16F_ARB): return 1;
+        case(GL_INTENSITY32F_ARB): return 1;
         case(GL_LUMINANCE_ALPHA): return 2;
+        case(GL_LUMINANCE_ALPHA8I_EXT): return 2;
+        case(GL_LUMINANCE_ALPHA8UI_EXT): return 2;
+        case(GL_LUMINANCE_ALPHA16I_EXT): return 2;
+        case(GL_LUMINANCE_ALPHA16UI_EXT): return 2;
+        case(GL_LUMINANCE_ALPHA32I_EXT): return 2;
+        case(GL_LUMINANCE_ALPHA32UI_EXT): return 2;
+        case(GL_LUMINANCE_ALPHA16F_ARB): return 2;
+        case(GL_LUMINANCE_ALPHA32F_ARB): return 2;
         case(GL_HILO_NV): return 2;
         case(GL_DSDT_NV): return 2;
         case(GL_DSDT_MAG_NV): return 3;
@@ -213,6 +346,25 @@ unsigned int Image::computePixelSizeInBits(GLenum format,GLenum type)
         default: break;
     }
 
+    switch(format)
+    {
+        case(GL_LUMINANCE4): return 4;
+        case(GL_LUMINANCE8): return 8;
+        case(GL_LUMINANCE12): return 12;
+        case(GL_LUMINANCE16): return 16;
+        case(GL_LUMINANCE4_ALPHA4): return 8;
+        case(GL_LUMINANCE6_ALPHA2): return 8;
+        case(GL_LUMINANCE8_ALPHA8): return 16;
+        case(GL_LUMINANCE12_ALPHA4): return 16;
+        case(GL_LUMINANCE12_ALPHA12): return 24;
+        case(GL_LUMINANCE16_ALPHA16): return 32;
+        case(GL_INTENSITY4): return 4;
+        case(GL_INTENSITY8): return 8;
+        case(GL_INTENSITY12): return 12;
+        case(GL_INTENSITY16): return 16;
+        default: break;
+    }
+
     switch(type)
     {
    
@@ -277,6 +429,13 @@ int Image::computeNearestPowerOfTwo(int s,float bias)
     return s;
 }
 
+int Image::computeNumberOfMipmapLevels(int s,int t, int r)
+{
+    int w = maximum(s, t);
+    w = maximum(w, r);
+    return 1 + static_cast<int>(floor(logf(w)/logf(2.0f)));
+}
+
 unsigned int Image::getTotalSizeInBytesIncludingMipmaps() const
 {
     if (_mipmapData.empty()) 
@@ -405,7 +564,7 @@ void Image::allocateImage(int s,int t,int r,
         _internalTextureFormat = 0;
     }
     
-    ++_modifiedCount;
+    dirty();
 }
 
 void Image::setImage(int s,int t,int r,
@@ -429,7 +588,7 @@ void Image::setImage(int s,int t,int r,
 
     _packing = packing;
         
-    ++_modifiedCount;
+    dirty();
 
 }
 
@@ -448,14 +607,25 @@ void Image::readImageFromCurrentTexture(unsigned int contextID, bool copyMipMaps
 {
     const osg::Texture::Extensions* extensions = osg::Texture::getExtensions(contextID,true);
     const osg::Texture3D::Extensions* extensions3D = osg::Texture3D::getExtensions(contextID,true);
+    const osg::Texture2DArray::Extensions* extensions2DArray = osg::Texture2DArray::getExtensions(contextID,true);
 
     
-    GLboolean binding1D, binding2D, binding3D;
+    GLboolean binding1D, binding2D, binding3D, binding2DArray;
     glGetBooleanv(GL_TEXTURE_BINDING_1D, &binding1D);
     glGetBooleanv(GL_TEXTURE_BINDING_2D, &binding2D);
     glGetBooleanv(GL_TEXTURE_BINDING_3D, &binding3D);
+    
+    
+    if (extensions2DArray->isTexture2DArraySupported())
+    {
+        glGetBooleanv(GL_TEXTURE_BINDING_2D_ARRAY_EXT, &binding2DArray);
+    }
+    else
+    {
+        binding2DArray = GL_FALSE;
+    }
 
-    GLenum textureMode = binding1D ? GL_TEXTURE_1D : binding2D ? GL_TEXTURE_2D : binding3D ? GL_TEXTURE_3D : 0;
+    GLenum textureMode = binding1D ? GL_TEXTURE_1D : binding2D ? GL_TEXTURE_2D : binding3D ? GL_TEXTURE_3D : binding2DArray ? GL_TEXTURE_2D_ARRAY_EXT : 0;
     
     if (textureMode==0) return;
 
@@ -488,14 +658,21 @@ void Image::readImageFromCurrentTexture(unsigned int contextID, bool copyMipMaps
     {
         if (extensions->isCompressedTexImage2DSupported())
         {
-            glGetTexLevelParameteriv(GL_TEXTURE_2D, 0, GL_TEXTURE_COMPRESSED_ARB,&compressed);
+            glGetTexLevelParameteriv(textureMode, 0, GL_TEXTURE_COMPRESSED_ARB,&compressed);
         }
     }
     else if (textureMode==GL_TEXTURE_3D)
     {
         if (extensions3D->isCompressedTexImage3DSupported())
         {
-            glGetTexLevelParameteriv(GL_TEXTURE_3D, 0, GL_TEXTURE_COMPRESSED_ARB,&compressed);
+            glGetTexLevelParameteriv(textureMode, 0, GL_TEXTURE_COMPRESSED_ARB,&compressed);
+        }
+    }
+    else if (textureMode==GL_TEXTURE_2D_ARRAY_EXT)
+    {
+        if (extensions2DArray->isCompressedTexImage3DSupported())
+        {
+            glGetTexLevelParameteriv(textureMode, 0, GL_TEXTURE_COMPRESSED_ARB,&compressed);
         }
     }
     
@@ -553,7 +730,7 @@ void Image::readImageFromCurrentTexture(unsigned int contextID, bool copyMipMaps
             extensions->glGetCompressedTexImage(textureMode, i, getMipmapData(i));
         }
 
-        ++_modifiedCount;
+        dirty();
     
     }
     else
@@ -600,7 +777,7 @@ void Image::readImageFromCurrentTexture(unsigned int contextID, bool copyMipMaps
         _t = height;
         _r = depth;
         
-        _pixelFormat = internalformat;
+        _pixelFormat = computePixelFormat(internalformat);
         _dataType = type;
         _internalTextureFormat = internalformat;
         _mipmapData = mipMapData;
@@ -612,7 +789,7 @@ void Image::readImageFromCurrentTexture(unsigned int contextID, bool copyMipMaps
             glGetTexImage(textureMode,i,_pixelFormat,_dataType,getMipmapData(i));
         }
 
-        ++_modifiedCount;
+        dirty();
     }    
 }
 
@@ -675,7 +852,7 @@ void Image::scaleImage(int s,int t,int r, GLenum newDataType)
         notify(WARN) << "Error Image::scaleImage() did not succeed : errorString = "<<gluErrorString((GLenum)status)<<std::endl;
     }
     
-    ++_modifiedCount;
+    dirty();
 }
 
 void Image::copySubImage(int s_offset,int t_offset,int r_offset,osg::Image* source)
@@ -774,7 +951,7 @@ void Image::flipHorizontal()
         return;
     }
         
-    ++_modifiedCount;
+    dirty();
 }
 
 void flipImageVertical(unsigned char* top, unsigned char* bottom, unsigned int rowSize)
@@ -858,7 +1035,7 @@ void Image::flipVertical()
        }
     }   
 
-    ++_modifiedCount;
+    dirty();
 }
 
 
@@ -892,6 +1069,17 @@ bool _findLowerAlphaValueInRow(unsigned int num, T* data,T value, unsigned int d
     return false;
 }
 
+template <typename T>    
+bool _maskedFindLowerAlphaValueInRow(unsigned int num, T* data,T value, T mask, unsigned int delta)
+{
+    for(unsigned int i=0;i<num;++i)
+    {
+        if ((*data & mask)<value) return true;
+        data += delta;
+    }
+    return false;
+}
+
 bool Image::isImageTranslucent() const
 {
     unsigned int offset = 0;
@@ -953,6 +1141,47 @@ bool Image::isImageTranslucent() const
                     if (_findLowerAlphaValueInRow(s(), (float*)d + offset, 1.0f, delta))
                         return true;
                     break;
+                case(GL_UNSIGNED_SHORT_5_5_5_1):
+                    if (_maskedFindLowerAlphaValueInRow(s(), (unsigned short*)d,
+                                                        (unsigned short)0x0001,
+                                                        (unsigned short)0x0001, 1))
+                        return true;
+                    break;
+                case(GL_UNSIGNED_SHORT_1_5_5_5_REV):
+                    if (_maskedFindLowerAlphaValueInRow(s(), (unsigned short*)d,
+                                                        (unsigned short)0x8000,
+                                                        (unsigned short)0x8000, 1))
+                        return true;
+                    break;
+                case(GL_UNSIGNED_SHORT_4_4_4_4):
+                    if (_maskedFindLowerAlphaValueInRow(s(), (unsigned short*)d,
+                                                        (unsigned short)0x000f,
+                                                        (unsigned short)0x000f, 1))
+                        return true;
+                    break;
+                case(GL_UNSIGNED_SHORT_4_4_4_4_REV):
+                    if (_maskedFindLowerAlphaValueInRow(s(), (unsigned short*)d,
+                                                        (unsigned short)0xf000,
+                                                        (unsigned short)0xf000, 1))
+                        return true;
+                    break;
+                case(GL_UNSIGNED_INT_10_10_10_2):
+                    if (_maskedFindLowerAlphaValueInRow(s(), (unsigned int*)d,
+                                                        0x00000003u,
+                                                        0x00000003u, 1))
+                        return true;
+                    break;                    
+                case(GL_UNSIGNED_INT_2_10_10_10_REV):
+                    if (_maskedFindLowerAlphaValueInRow(s(), (unsigned int*)d,
+                                                        0xc0000000u,
+                                                        0xc0000000u, 1))
+                        return true;
+                    break;
+                case(GL_HALF_FLOAT_NV):
+                    if (_findLowerAlphaValueInRow(s(), (unsigned short*)d + offset,
+                                                  (unsigned short)0x3c00, delta))
+                        return true;
+                    break;
             }
         }
     }
