@@ -1,4 +1,4 @@
-/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2005 Robert Osfield 
+/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2006 Robert Osfield 
  *
  * This library is open source and may be redistributed and/or modified under  
  * the terms of the OpenSceneGraph Public License (OSGPL) version 0.0 or 
@@ -39,8 +39,6 @@ bool osg::initNotifyLevel()
 
     if (s_NotifyInit) return true;
     
-    s_NotifyInit = true;
-
     // g_NotifyLevel
     // =============
 
@@ -73,10 +71,17 @@ bool osg::initNotifyLevel()
  
     }
 
+    s_NotifyInit = true;
+
     return true;
 
 }
 
+bool osg::isNotifyEnabled( osg::NotifySeverity severity )
+{
+    return severity<=g_NotifyLevel;
+}
+
 #if defined(WIN32) && !(defined(__CYGWIN__) || defined(__MINGW32__))
 const char* NullStreamName = "nul";
 #else
