@@ -15,11 +15,19 @@
 #include <osg/GLU>
 #include <osg/GLExtensions>
 
+#ifndef GL_MAX_TEXTURE_COORDS
+#define GL_MAX_TEXTURE_COORDS 0x8871
+#endif
+
+#ifndef GL_MAX_COMBINED_TEXTURE_IMAGE_UNITS
+#define GL_MAX_COMBINED_TEXTURE_IMAGE_UNITS 0x8B4D
+#endif
 
 using namespace std;
 using namespace osg;
 
-State::State()
+State::State():
+    Referenced(true)
 {
     _graphicsContext = 0;
     _contextID = 0;
@@ -29,11 +37,20 @@ State::State()
     _modelView = _identity;
 
     _abortRenderingPtr = false;    
+
+#if 1
     _checkGLErrors = ONCE_PER_FRAME;
+#else    
+    _checkGLErrors = ONCE_PER_ATTRIBUTE;
+#endif
 
     _currentActiveTextureUnit=0;
     _currentClientActiveTextureUnit=0;
 
+    _currentVBO = 0;
+    _currentEBO = 0;
+    _currentPBO = 0;
+
     _isSecondaryColorSupportResolved = false;
     _isSecondaryColorSupported = false;
 
@@ -53,6 +70,8 @@ State::State()
     _glVertexAttribPointer = 0;
     _glEnableVertexAttribArray = 0;
     _glDisableVertexAttribArray = 0;
+
+    _dynamicObjectCount  = 0;
 }
 
 State::~State()
@@ -123,7 +142,18 @@ void State::reset()
     _projection = _identity;
     
     dirtyAllVertexArrays();
+    
+#if 0
+    // reset active texture unit values and call OpenGL
+    // note, this OpenGL op precludes the use of State::reset() without a
+    // valid graphics context, therefore the new implementation below 
+    // is prefered.
     setActiveTextureUnit(0);
+#else
+    // reset active texture unit values without calling OpenGL
+    _currentActiveTextureUnit = 0;
+    _currentClientActiveTextureUnit = 0;
+#endif
     
     _lastAppliedProgramObject = 0;
 
@@ -160,6 +190,7 @@ void State::setInitialViewMatrix(const osg::RefMatrix* matrix)
 
 void State::pushStateSet(const StateSet* dstate)
 {
+
     _stateStateStack.push_back(dstate);
     if (dstate)
     {
@@ -185,10 +216,14 @@ void State::pushStateSet(const StateSet* dstate)
 
         pushUniformList(_uniformMap,dstate->getUniformList());
     }
+
+    // osg::notify(osg::NOTICE)<<"State::pushStateSet()"<<_stateStateStack.size()<<std::endl;
 }
 
 void State::popAllStateSets()
 {
+    // osg::notify(osg::NOTICE)<<"State::popAllStateSets()"<<_stateStateStack.size()<<std::endl;
+
     while (!_stateStateStack.empty()) popStateSet();
     
     applyProjectionMatrix(0);
@@ -199,8 +234,11 @@ void State::popAllStateSets()
 
 void State::popStateSet()
 {
+    // osg::notify(osg::NOTICE)<<"State::popStateSet()"<<_stateStateStack.size()<<std::endl;
+
     if (_stateStateStack.empty()) return;
     
+    
     const StateSet* dstate = _stateStateStack.back();
 
     if (dstate)
@@ -233,6 +271,58 @@ void State::popStateSet()
     _stateStateStack.pop_back();
 }
 
+void State::insertStateSet(unsigned int pos,const StateSet* dstate)
+{
+    StateSetStack tempStack;
+    
+    // first pop the StateSet above the position we need to insert at
+    while (_stateStateStack.size()>pos)
+    {
+        tempStack.push_back(_stateStateStack.back());
+        popStateSet();
+    }
+
+    // push our new stateset
+    pushStateSet(dstate);
+    
+    // push back the original ones
+    for(StateSetStack::reverse_iterator itr = tempStack.rbegin();
+        itr != tempStack.rend();
+        ++itr)
+    {
+        pushStateSet(*itr);
+    }
+
+}
+
+void State::removeStateSet(unsigned int pos)
+{
+    if (pos >= _stateStateStack.size())
+    {
+        osg::notify(osg::NOTICE)<<"Warning: State::removeStateSet("<<pos<<") out of range"<<std::endl;
+        return;
+    }
+    
+    // record the StateSet above the one we intend to remove
+    StateSetStack tempStack;
+    while (_stateStateStack.size()-1>pos)
+    {
+        tempStack.push_back(_stateStateStack.back());
+        popStateSet();
+    }
+
+    // remove the intended StateSet as well    
+    popStateSet();
+
+    // push back the original ones that were above the remove StateSet
+    for(StateSetStack::reverse_iterator itr = tempStack.rbegin();
+        itr != tempStack.rend();
+        ++itr)
+    {
+        pushStateSet(*itr);
+    }
+}
+
 void State::captureCurrentState(StateSet& stateset) const
 {
     // empty the stateset first.
@@ -650,6 +740,14 @@ void State::initializeExtensionProcs()
     _glVertexAttribPointer =  (VertexAttribPointerProc) osg::getGLExtensionFuncPtr("glVertexAttribPointer","glVertexAttribPointerARB");
     _glEnableVertexAttribArray =  (EnableVertexAttribProc) osg::getGLExtensionFuncPtr("glEnableVertexAttribArray","glEnableVertexAttribArrayARB");
     _glDisableVertexAttribArray =  (DisableVertexAttribProc) osg::getGLExtensionFuncPtr("glDisableVertexAttribArray","glDisableVertexAttribArrayARB");
+    _glBindBuffer = (BindBufferProc) osg::getGLExtensionFuncPtr("glBindBuffer","glBindBufferARB");
+
+    _glMaxTextureCoords = 1;
+    glGetIntegerv(GL_MAX_TEXTURE_COORDS,&_glMaxTextureCoords);
+
+    _glMaxTextureUnits = 1;
+    glGetIntegerv(GL_MAX_COMBINED_TEXTURE_IMAGE_UNITS,&_glMaxTextureUnits);
+    _glMaxTextureUnits = maximum(_glMaxTextureCoords,_glMaxTextureUnits);
 
     _extensionProcsInitialized = true;
 }
@@ -658,9 +756,7 @@ bool State::setClientActiveTextureUnit( unsigned int unit )
 {
     if (unit!=_currentClientActiveTextureUnit)
     {
-        if (!_extensionProcsInitialized) initializeExtensionProcs();
- 
-        if (_glClientActiveTexture)
+        if (_glClientActiveTexture && unit < (unsigned int)_glMaxTextureCoords)
         {
             _glClientActiveTexture(GL_TEXTURE0+unit);
             _currentClientActiveTextureUnit = unit;
@@ -680,9 +776,7 @@ bool State::setActiveTextureUnit( unsigned int unit )
 {
     if (unit!=_currentActiveTextureUnit)
     {
-        if (!_extensionProcsInitialized) initializeExtensionProcs();
-
-        if (_glActiveTexture)
+        if (_glActiveTexture && unit < (unsigned int)_glMaxTextureUnits)
         {
             _glActiveTexture(GL_TEXTURE0+unit);
             _currentActiveTextureUnit = unit;
@@ -697,8 +791,6 @@ bool State::setActiveTextureUnit( unsigned int unit )
 
 void State::setFogCoordPointer(GLenum type, GLsizei stride, const GLvoid *ptr)
 {
-    if (!_extensionProcsInitialized) initializeExtensionProcs();
-
     if (_glFogCoordPointer)
     {
 
@@ -720,8 +812,6 @@ void State::setFogCoordPointer(GLenum type, GLsizei stride, const GLvoid *ptr)
 void State::setSecondaryColorPointer( GLint size, GLenum type,
                                       GLsizei stride, const GLvoid *ptr )
 {
-    if (!_extensionProcsInitialized) initializeExtensionProcs();
-
     if (_glSecondaryColorPointer)
     {
         if (!_secondaryColorArray._enabled || _secondaryColorArray._dirty)
@@ -744,8 +834,6 @@ void State::setVertexAttribPointer( unsigned int index,
                                       GLint size, GLenum type, GLboolean normalized, 
                                     GLsizei stride, const GLvoid *ptr )
 {
-    if (!_extensionProcsInitialized) initializeExtensionProcs();
-
     if (_glVertexAttribPointer)
     {
         if ( index >= _vertexAttribArrayList.size()) _vertexAttribArrayList.resize(index+1);
@@ -770,8 +858,6 @@ void State::setVertexAttribPointer( unsigned int index,
 * note, only updates values that change.*/
 void State::disableVertexAttribPointer( unsigned int index )
 {
-    if (!_extensionProcsInitialized) initializeExtensionProcs();
-
     if (_glDisableVertexAttribArray)
     {
         if ( index >= _vertexAttribArrayList.size()) _vertexAttribArrayList.resize(index+1);
@@ -788,8 +874,6 @@ void State::disableVertexAttribPointer( unsigned int index )
 
 void State::disableVertexAttribPointersAboveAndIncluding( unsigned int index )
 {
-    if (!_extensionProcsInitialized) initializeExtensionProcs();
-
     if (_glDisableVertexAttribArray)
     {
         while (index<_vertexAttribArrayList.size())
