@@ -28,6 +28,7 @@
 
 #include <osg/TextureCubeMap>
 #include <osg/TextureRectangle>
+#include <osg/Texture2DArray>
 
 #include <set>
 #include <algorithm>
@@ -48,7 +49,8 @@ class TextureGLModeSet
             _textureModeSet.insert(GL_TEXTURE_3D);
 
             _textureModeSet.insert(GL_TEXTURE_CUBE_MAP);
-        _textureModeSet.insert(GL_TEXTURE_RECTANGLE_NV);
+            _textureModeSet.insert(GL_TEXTURE_RECTANGLE_NV);
+            _textureModeSet.insert(GL_TEXTURE_2D_ARRAY_EXT);
 
             _textureModeSet.insert(GL_TEXTURE_GEN_Q);
             _textureModeSet.insert(GL_TEXTURE_GEN_R);
@@ -73,10 +75,14 @@ static TextureGLModeSet& getTextureGLModeSet()
     return s_textureGLModeSet;
 }
 
-StateSet::StateSet()
+bool osg::isTextureMode(StateAttribute::GLMode mode)
 {
-    setDataVariance(osg::StateAttribute::STATIC);
+    return getTextureGLModeSet().isTextureMode(mode);
+}
 
+StateSet::StateSet():
+    Object(true)
+{
     _renderingHint = DEFAULT_BIN;
     
     _numChildrenRequiringUpdateTraversal = 0;
@@ -164,17 +170,112 @@ StateSet::~StateSet()
     clear();
 }
 
+void StateSet::computeDataVariance()
+{
+    bool dynamic = false;
+
+    if (_updateCallback.valid() ||
+        _eventCallback.valid())
+    {
+        dynamic = true;
+    }
+
+    // run attribute callbacks
+
+    for(AttributeList::iterator itr=_attributeList.begin();
+        itr!=_attributeList.end();
+        ++itr)
+    {
+        if (itr->second.first->getDataVariance()==UNSPECIFIED && 
+            (itr->second.first->getUpdateCallback() || itr->second.first->getEventCallback()))
+        {
+            itr->second.first->setDataVariance(DYNAMIC);
+        }
+
+        if (itr->second.first->getDataVariance()==DYNAMIC) dynamic = true;
+    }
+
+    // run texture attribute callbacks.
+    for(unsigned int i=0;i<_textureAttributeList.size();++i)
+    {
+        AttributeList& attributeList = _textureAttributeList[i];
+        for(AttributeList::iterator itr=attributeList.begin();
+            itr!=attributeList.end();
+            ++itr)
+        {
+            if (itr->second.first->getDataVariance()==UNSPECIFIED && 
+                (itr->second.first->getUpdateCallback() || itr->second.first->getEventCallback()))
+            {
+                itr->second.first->setDataVariance(DYNAMIC);
+            }
+
+            if (itr->second.first->getDataVariance()==DYNAMIC) dynamic = true;
+        }
+    }
+
+
+    // run uniform callbacks.
+    for(UniformList::iterator uitr = _uniformList.begin();
+        uitr != _uniformList.end();
+        ++uitr)
+    {
+        if (uitr->second.first->getDataVariance()==UNSPECIFIED && 
+            (uitr->second.first->getUpdateCallback() || uitr->second.first->getEventCallback()))
+        {
+            uitr->second.first->setDataVariance(DYNAMIC);
+        }
+
+        if (uitr->second.first->getDataVariance()==DYNAMIC) dynamic = true;
+    }
+
+#if 0
+
+    if (dynamic)
+    {
+        osg::notify(osg::NOTICE)<<"StateSet::computeDataVariance setting to DYNAMIC"<<std::endl;
+    }
+    else
+    {
+        osg::notify(osg::NOTICE)<<"StateSet::computeDataVariance to STATIC"<<std::endl;
+    }
+#endif
+
+    if (getDataVariance()==UNSPECIFIED)
+    {
+        setDataVariance(dynamic ? DYNAMIC : STATIC);
+    }
+}
+
+
 void StateSet::addParent(osg::Object* object)
 {
     // osg::notify(osg::INFO)<<"Adding parent"<<std::endl;
+    if (getRefMutex())
+    {
+        OpenThreads::ScopedLock<OpenThreads::Mutex> lock(*getRefMutex());
 
-    _parents.push_back(object);
+        _parents.push_back(object);
+    }
+    else
+    {
+        _parents.push_back(object);
+    }
 }
 
 void StateSet::removeParent(osg::Object* object)
 {
-    ParentList::iterator pitr = std::find(_parents.begin(),_parents.end(),object);
-    if (pitr!=_parents.end()) _parents.erase(pitr);
+    if (getRefMutex())
+    {
+        OpenThreads::ScopedLock<OpenThreads::Mutex> lock(*getRefMutex());
+
+        ParentList::iterator pitr = std::find(_parents.begin(),_parents.end(),object);
+        if (pitr!=_parents.end()) _parents.erase(pitr);
+    }
+    else
+    {
+        ParentList::iterator pitr = std::find(_parents.begin(),_parents.end(),object);
+        if (pitr!=_parents.end()) _parents.erase(pitr);
+    }
 }
 
 int StateSet::compare(const StateSet& rhs,bool compareAttributeContents) const
@@ -348,6 +449,20 @@ int StateSet::compare(const StateSet& rhs,bool compareAttributeContents) const
     }
     else if (rhs_uniform_itr == rhs._uniformList.end()) return 1;  
 
+    // check render bin details
+    
+    if ( _binMode < rhs._binMode ) return -1;
+    else if ( _binMode > rhs._binMode ) return 1;
+    
+    if ( _binMode != INHERIT_RENDERBIN_DETAILS )
+    {
+        if ( _binNum < rhs._binNum ) return -1;
+        else if ( _binNum > rhs._binNum ) return 1;
+
+        if ( _binName < rhs._binName ) return -1;
+        else if ( _binName > rhs._binName ) return 1;
+    }
+
     return 0;
 }
 
@@ -1161,6 +1276,30 @@ bool StateSet::checkValidityOfAssociatedModes(osg::State& state) const
     return modesValid;
 }
 
+void StateSet::setThreadSafeRefUnref(bool threadSafe)
+{
+    Object::setThreadSafeRefUnref(threadSafe);
+
+    for(AttributeList::const_iterator itr = _attributeList.begin();
+        itr!=_attributeList.end();
+        ++itr)
+    {
+        itr->second.first->setThreadSafeRefUnref(threadSafe);
+    }
+
+    for(TextureAttributeList::const_iterator taitr=_textureAttributeList.begin();
+        taitr!=_textureAttributeList.end();
+        ++taitr)
+    {
+        for(AttributeList::const_iterator itr = taitr->begin();
+            itr!=taitr->end();
+            ++itr)
+        {
+            itr->second.first->setThreadSafeRefUnref(threadSafe);
+        }
+    }
+}
+
 void StateSet::compileGLObjects(State& state) const
 {
     for(AttributeList::const_iterator itr = _attributeList.begin();
@@ -1183,6 +1322,28 @@ void StateSet::compileGLObjects(State& state) const
     }
 }
 
+void StateSet::resizeGLObjectBuffers(unsigned int maxSize)
+{
+    for(AttributeList::const_iterator itr = _attributeList.begin();
+        itr!=_attributeList.end();
+        ++itr)
+    {
+        itr->second.first->resizeGLObjectBuffers(maxSize);
+    }
+
+    for(TextureAttributeList::const_iterator taitr=_textureAttributeList.begin();
+        taitr!=_textureAttributeList.end();
+        ++taitr)
+    {
+        for(AttributeList::const_iterator itr = taitr->begin();
+            itr!=taitr->end();
+            ++itr)
+        {
+            itr->second.first->resizeGLObjectBuffers(maxSize);
+        }
+    }
+}
+
 void StateSet::releaseGLObjects(State* state) const
 {
     for(AttributeList::const_iterator itr = _attributeList.begin();
