@@ -285,8 +285,7 @@ void TextureRectangle::applyTexImage_load(GLenum target, Image* image, State& st
     const PixelBufferObject* pbo = image->getPixelBufferObject();
     if (pbo && pbo->isPBOSupported(contextID))
     {
-        pbo->compileBuffer(state);
-        pbo->bindBuffer(contextID);
+        state.bindPixelBufferObject(pbo);
         dataMinusOffset = image->data();
         dataPlusOffset = reinterpret_cast<unsigned char*>(pbo->offset());
     }
@@ -295,17 +294,26 @@ void TextureRectangle::applyTexImage_load(GLenum target, Image* image, State& st
         pbo = 0;
     }
 
-    // UH: ignoring compressed for now.
-    glTexImage2D(target, 0, _internalFormat,
-                 image->s(), image->t(), 0,
-                 (GLenum)image->getPixelFormat(),
-                 (GLenum)image->getDataType(),
-                 image->data() - dataMinusOffset + dataPlusOffset );
+    if(isCompressedInternalFormat(_internalFormat) && extensions->isCompressedTexImage2DSupported())
+    {
+        extensions->glCompressedTexImage2D(target, 0, _internalFormat, 
+          image->s(), image->t(), 0,
+          image->getImageSizeInBytes(), 
+          image->data() - dataMinusOffset + dataPlusOffset);                
+    }
+    else
+    {
+        glTexImage2D(target, 0, _internalFormat,
+          image->s(), image->t(), 0,
+          (GLenum)image->getPixelFormat(),
+          (GLenum)image->getDataType(),
+          image->data() - dataMinusOffset + dataPlusOffset );
+    }
     
 
     if (pbo)
     {
-        pbo->unbindBuffer(contextID);
+        state.unbindPixelBufferObject();
     }
 
     inwidth = image->s();
@@ -333,6 +341,8 @@ void TextureRectangle::applyTexImage_subload(GLenum target, Image* image, State&
     // get the contextID (user defined ID of 0 upwards) for the 
     // current OpenGL context.
     const unsigned int contextID = state.getContextID();
+    const Extensions* extensions = getExtensions(contextID,true);
+
 
     // update the modified count to show that it is upto date.
     getModifiedCount(contextID) = image->getModifiedCount();
@@ -352,8 +362,7 @@ void TextureRectangle::applyTexImage_subload(GLenum target, Image* image, State&
     const PixelBufferObject* pbo = image->getPixelBufferObject();
     if (pbo && pbo->isPBOSupported(contextID))
     {
-        pbo->compileBuffer(state);
-        pbo->bindBuffer(contextID);
+        state.bindPixelBufferObject(pbo);
         dataMinusOffset = image->data();
         dataPlusOffset = reinterpret_cast<unsigned char*>(pbo->offset()); // -dataMinusOffset+dataPlusOffset
 
@@ -367,18 +376,29 @@ void TextureRectangle::applyTexImage_subload(GLenum target, Image* image, State&
         pbo = 0;
     }
     
-    
-    // UH: ignoring compressed for now.
-    glTexSubImage2D(target, 0, 
-                 0,0,
-                 image->s(), image->t(),
-                 (GLenum)image->getPixelFormat(),
-                 (GLenum)image->getDataType(),
-                 image->data() - dataMinusOffset + dataPlusOffset  );
+
+    if(isCompressedInternalFormat(_internalFormat) && extensions->isCompressedTexSubImage2DSupported())
+    {
+        extensions->glCompressedTexSubImage2D(target, 0, 
+          0,0, 
+          image->s(), image->t(),
+          (GLenum)image->getPixelFormat(),
+          (GLenum)image->getDataType(),
+          image->data() - dataMinusOffset + dataPlusOffset);                
+    }
+    else
+    {
+        glTexSubImage2D(target, 0, 
+          0,0,
+          image->s(), image->t(),
+          (GLenum)image->getPixelFormat(),
+          (GLenum)image->getDataType(),
+          image->data() - dataMinusOffset + dataPlusOffset  );
+    }
 
     if (pbo)
     {
-        pbo->unbindBuffer(contextID);
+        state.unbindPixelBufferObject();
     }
 
 #ifdef DO_TIMING
@@ -389,8 +409,8 @@ void TextureRectangle::applyTexImage_subload(GLenum target, Image* image, State&
 
 void TextureRectangle::computeInternalFormat() const
 {
-    if (_image.valid())
-        computeInternalFormatWithImage(*_image); 
+    if (_image.valid()) computeInternalFormatWithImage(*_image); 
+    else computeInternalFormatType();
 }
 
 void TextureRectangle::copyTexImage2D(State& state, int x, int y, int width, int height )
@@ -522,3 +542,9 @@ void TextureRectangle::copyTexSubImage2D(State& state, int xoffset, int yoffset,
         copyTexImage2D(state,x,y,width,height);
     }
 }
+
+void TextureRectangle::allocateMipmap(State&) const
+{
+    osg::notify(osg::NOTICE)<<"Warning: TextureRectangle::allocateMipmap(State&) called eroneously, GL_TEXTURE_RECTANGLE does not support mipmapping."<<std::endl;
+}
+
