@@ -24,6 +24,8 @@ static osg::ApplicationUsageProxy DatabasePager_e0(osg::ApplicationUsage::ENVIRO
 static osg::ApplicationUsageProxy DatabasePager_e1(osg::ApplicationUsage::ENVIRONMENTAL_VARIABLE,"OSG_MINIMUM_COMPILE_TIME_PER_FRAME <float>","minimum compile time alloted to compiling OpenGL objects per frame in database pager.");
 static osg::ApplicationUsageProxy DatabasePager_e2(osg::ApplicationUsage::ENVIRONMENTAL_VARIABLE,"OSG_MAXIMUM_OBJECTS_TO_COMPILE_PER_FRAME <int>","maximum number of OpenGL objects to compile per frame in database pager.");
 static osg::ApplicationUsageProxy DatabasePager_e3(osg::ApplicationUsage::ENVIRONMENTAL_VARIABLE,"OSG_DATABASE_PAGER_DRAWABLE <mode>","Set the drawable policy for setting of loaded drawable to specified type.  mode can be one of DoNotModify, DisplayList, VBO or VertexArrays>.");
+static osg::ApplicationUsageProxy DatabasePager_e4(osg::ApplicationUsage::ENVIRONMENTAL_VARIABLE,"OSG_DATABASE_PAGER_PRIORITY <mode>", "Set the thread priority to DEFAULT, MIN, LOW, NOMINAL, HIGH or MAX.");
+
 DatabasePager::DatabasePager()
 {
     //osg::notify(osg::INFO)<<"Constructing DatabasePager()"<<std::endl;
@@ -34,14 +36,38 @@ DatabasePager::DatabasePager()
     _acceptNewRequests = true;
     _databasePagerThreadPaused = false;
     
-    _useFrameBlock = false;
     _numFramesActive = 0;
     _frameNumber = 0;
-    _frameBlock = new osg::Block;
-    _databasePagerThreadBlock = new osg::Block;
-
-    _threadPriorityDuringFrame = THREAD_PRIORITY_MIN;
-    _threadPriorityOutwithFrame = THREAD_PRIORITY_MIN;
+    _databasePagerThreadBlock = new osg::RefBlock;
+    
+    const char* str = getenv("OSG_DATABASE_PAGER_PRIORITY");
+    if (str)
+    {
+        if (strcmp(str,"DEFAULT")==0)
+        {
+            setSchedulePriority(OpenThreads::Thread::THREAD_PRIORITY_DEFAULT);
+        }
+        else if (strcmp(str,"MIN")==0)
+        {
+            setSchedulePriority(OpenThreads::Thread::THREAD_PRIORITY_MIN);
+        }
+        else if (strcmp(str,"LOW")==0)
+        {
+            setSchedulePriority(OpenThreads::Thread::THREAD_PRIORITY_LOW);
+        }
+        else if (strcmp(str,"NOMINAL")==0)
+        {
+            setSchedulePriority(OpenThreads::Thread::THREAD_PRIORITY_NOMINAL);
+        }
+        else if (strcmp(str,"HIGH")==0)
+        {
+            setSchedulePriority(OpenThreads::Thread::THREAD_PRIORITY_HIGH);
+        } 
+        else if (strcmp(str,"MAX")==0)
+        {
+            setSchedulePriority(OpenThreads::Thread::THREAD_PRIORITY_MAX);
+        } 
+    }
 
 #if __APPLE__
     // OSX really doesn't like compiling display lists, and performs poorly when they are used,
@@ -51,7 +77,7 @@ DatabasePager::DatabasePager()
     _drawablePolicy = DO_NOT_MODIFY_DRAWABLE_SETTINGS;
 #endif    
     
-    const char* str = getenv("OSG_DATABASE_PAGER_GEOMETRY");
+    str = getenv("OSG_DATABASE_PAGER_GEOMETRY");
     if (!str) str = getenv("OSG_DATABASE_PAGER_DRAWABLE");
     if (str)
     {
@@ -108,6 +134,10 @@ DatabasePager::DatabasePager()
         _maximumNumOfObjectsToCompilePerFrame = atoi(ptr);
     }
 
+
+    // initialize the stats variables
+    resetStats();
+
     // make sure a SharedStateManager exists.
     //osgDB::Registry::instance()->getOrCreateSharedStateManager();
     
@@ -115,11 +145,60 @@ DatabasePager::DatabasePager()
         //osgDB::Registry::instance()->setUseObjectCacheHint(true);
 }
 
+DatabasePager::DatabasePager(const DatabasePager& rhs)
+{
+    //osg::notify(osg::INFO)<<"Constructing DatabasePager(const DatabasePager& )"<<std::endl;
+    
+    _startThreadCalled = false;
+
+    _done = false;
+    _acceptNewRequests = true;
+    _databasePagerThreadPaused = false;
+    
+    _numFramesActive = 0;
+    _frameNumber = 0;
+    _databasePagerThreadBlock = new osg::RefBlock;
+
+    _drawablePolicy = rhs._drawablePolicy;
+
+    _changeAutoUnRef = rhs._changeAutoUnRef;
+    _valueAutoUnRef = rhs._valueAutoUnRef;
+    _changeAnisotropy = rhs._changeAnisotropy;
+    _valueAnisotropy = rhs._valueAnisotropy;
+
+
+    _deleteRemovedSubgraphsInDatabaseThread = rhs._deleteRemovedSubgraphsInDatabaseThread;
+    
+    _expiryDelay = rhs._expiryDelay;
+    _doPreCompile = rhs._doPreCompile;
+    _targetFrameRate = rhs._targetFrameRate;
+    _minimumTimeAvailableForGLCompileAndDeletePerFrame = rhs._minimumTimeAvailableForGLCompileAndDeletePerFrame;
+    _maximumNumOfObjectsToCompilePerFrame = rhs._maximumNumOfObjectsToCompilePerFrame;
+
+    // initialize the stats variables
+    resetStats();
+}
+
+
 DatabasePager::~DatabasePager()
 {
     cancel();
 }
 
+osg::ref_ptr<DatabasePager>& DatabasePager::prototype()
+{
+    static osg::ref_ptr<DatabasePager> s_DatabasePager = new DatabasePager;
+    return s_DatabasePager;
+}
+
+DatabasePager* DatabasePager::create()
+{
+    return DatabasePager::prototype().valid() ? 
+           DatabasePager::prototype()->clone() :
+           new DatabasePager; 
+}
+
+
 int DatabasePager::cancel()
 {
     int result = 0;
@@ -129,11 +208,10 @@ int DatabasePager::cancel()
         _done = true;
 
         // cancel the thread..
-        result = Thread::cancel();
+        // result = Thread::cancel();
         //join();
 
         // release the frameBlock and _databasePagerThreadBlock incase its holding up thread cancelation.
-        _frameBlock->release();
         _databasePagerThreadBlock->release();
 
         // then wait for the the thread to stop running.
@@ -181,7 +259,24 @@ void DatabasePager::clear()
     // _activeGraphicsContexts
 }
 
-void DatabasePager::requestNodeFile(const std::string& fileName,osg::Group* group, float priority, const osg::FrameStamp* framestamp)
+void DatabasePager::resetStats()
+{
+    // initialize the stats variables
+    _minimumTimeToMergeTile = DBL_MAX;
+    _maximumTimeToMergeTile = -DBL_MAX;
+    _totalTimeToMergeTiles = 0.0;
+    _numTilesMerges = 0;
+}
+
+void DatabasePager::requestNodeFile(const std::string& fileName,osg::Group* group,
+                                    float priority, const osg::FrameStamp* framestamp)
+{
+    requestNodeFile(fileName,group,priority,framestamp,Registry::instance()->getOptions());
+}
+
+void DatabasePager::requestNodeFile(const std::string& fileName,osg::Group* group,
+                                    float priority, const osg::FrameStamp* framestamp,
+                                    ReaderWriter::Options* loadOptions)
 {
     if (!_acceptNewRequests) return;
    
@@ -264,6 +359,7 @@ void DatabasePager::requestNodeFile(const std::string& fileName,osg::Group* grou
             databaseRequest->_timestampLastRequest = timestamp;
             databaseRequest->_priorityLastRequest = priority;
             databaseRequest->_groupForAddingLoadedSubgraph = group;
+            databaseRequest->_loadOptions = loadOptions;
 
             _fileRequestList.push_back(databaseRequest);
 
@@ -280,37 +376,24 @@ void DatabasePager::requestNodeFile(const std::string& fileName,osg::Group* grou
             _startThreadCalled = true;
             _done = false;
             osg::notify(osg::DEBUG_INFO)<<"DatabasePager::startThread()"<<std::endl;
-            setSchedulePriority(_threadPriorityDuringFrame);
             startThread();
         }
-                
     }
 }
 
 void DatabasePager::signalBeginFrame(const osg::FrameStamp* framestamp)
 {
-
     if (framestamp)
     {
         //osg::notify(osg::INFO) << "signalBeginFrame "<<framestamp->getFrameNumber()<<">>>>>>>>>>>>>>>>"<<std::endl;
         _frameNumber = framestamp->getFrameNumber();
         
     } //else osg::notify(osg::INFO) << "signalBeginFrame >>>>>>>>>>>>>>>>"<<std::endl;
-
-    updateFrameBlock(1);
-
-    if (_numFramesActive>0 && _threadPriorityDuringFrame!=getSchedulePriority())
-        setSchedulePriority(_threadPriorityDuringFrame);
 }
 
 void DatabasePager::signalEndFrame()
 {
     //osg::notify(osg::INFO) << "signalEndFrame <<<<<<<<<<<<<<<<<<<< "<<std::endl;
-    updateFrameBlock(-1);
-
-    if (_numFramesActive<=0 && _threadPriorityOutwithFrame!=getSchedulePriority())
-        setSchedulePriority(_threadPriorityOutwithFrame);
-
 }
 
 class DatabasePager::FindCompileableGLObjectsVisitor : public osg::NodeVisitor
@@ -446,14 +529,9 @@ void DatabasePager::run()
     
     do
     {
-    
+
         _databasePagerThreadBlock->block();
 
-        if (_useFrameBlock)
-        {
-            _frameBlock->block();
-        }      
-   
         //
         // delete any children if required.
         //
@@ -499,7 +577,7 @@ void DatabasePager::run()
                        
                 // load the data, note safe to write to the databaseRequest since once 
                 // it is created this thread is the only one to write to the _loadedModel pointer.
-                // osg::notify(osg::NOTICE)<<"In DatabasePager thread readNodeFile("<<databaseRequest->_fileName<<")"<<std::endl;
+                //osg::notify(osg::NOTICE)<<"In DatabasePager thread readNodeFile("<<databaseRequest->_fileName<<")"<<std::endl;
                 //osg::Timer_t before = osg::Timer::instance()->tick();
                 
                 
@@ -509,12 +587,14 @@ void DatabasePager::run()
                     // do *not* assume that we only have one DatabasePager, or that reaNodeFile is thread safe...
                     static OpenThreads::Mutex s_serialize_readNodeFile_mutex;
                     OpenThreads::ScopedLock<OpenThreads::Mutex> lock(s_serialize_readNodeFile_mutex);
-                    databaseRequest->_loadedModel = osgDB::readNodeFile(databaseRequest->_fileName);
+                    databaseRequest->_loadedModel = osgDB::readNodeFile(databaseRequest->_fileName,
+                        databaseRequest->_loadOptions.get());
                 }
                 else
                 {
-                    // assume that we only have one DatabasePager, or that reaNodeFile is thread safe...
-                    databaseRequest->_loadedModel = osgDB::readNodeFile(databaseRequest->_fileName);
+                    // assume that we only have one DatabasePager, or that readNodeFile is thread safe...
+                    databaseRequest->_loadedModel = osgDB::readNodeFile(databaseRequest->_fileName,
+                        databaseRequest->_loadOptions.get());
                 }
                     
                 //osg::notify(osg::NOTICE)<<"     node read in "<<osg::Timer::instance()->delta_m(before,osg::Timer::instance()->tick())<<" ms"<<std::endl;
@@ -556,32 +636,61 @@ void DatabasePager::run()
                 }            
 
                 // move the databaseRequest from the front of the fileRequest to the end of
-                // dataLoad list.
-                
-                OpenThreads::ScopedLock<OpenThreads::Mutex> lock(_fileRequestListMutex);
-
-                DatabaseRequestList::iterator itr = std::find(_fileRequestList.begin(),_fileRequestList.end(),databaseRequest);
-                if (itr != _fileRequestList.end()) 
+                // dataToCompile or dataToMerge lists.
                 {
-                    if (databaseRequest->_loadedModel.valid())
+                    OpenThreads::ScopedLock<OpenThreads::Mutex> lock(_fileRequestListMutex);
+
+                    DatabaseRequestList::iterator itr = std::find(_fileRequestList.begin(),_fileRequestList.end(),databaseRequest);
+                    if (itr != _fileRequestList.end()) 
                     {
-                        if (loadedObjectsNeedToBeCompiled)
+                        if (databaseRequest->_loadedModel.valid())
                         {
-                            OpenThreads::ScopedLock<OpenThreads::Mutex> lock(_dataToCompileListMutex);
-                            _dataToCompileList.push_back(databaseRequest);
-                        }
-                        else
-                        {
-                            OpenThreads::ScopedLock<OpenThreads::Mutex> lock(_dataToMergeListMutex);
-                            _dataToMergeList.push_back(databaseRequest);
-                        }
-                    }        
-                    
-                    _fileRequestList.erase(itr);
+                            if (loadedObjectsNeedToBeCompiled)
+                            {
+                                OpenThreads::ScopedLock<OpenThreads::Mutex> lock(_dataToCompileListMutex);
+                                _dataToCompileList.push_back(databaseRequest);
+                            }
+                            else
+                            {
+                                OpenThreads::ScopedLock<OpenThreads::Mutex> lock(_dataToMergeListMutex);
+                                _dataToMergeList.push_back(databaseRequest);
+                            }
+                        }        
+
+                        _fileRequestList.erase(itr);
+                    }
+
+                    updateDatabasePagerThreadBlock();
                 }
+                
+                if (loadedObjectsNeedToBeCompiled)
+                {
+                    for(ActiveGraphicsContexts::iterator itr = _activeGraphicsContexts.begin();
+                        itr != _activeGraphicsContexts.end();
+                        ++itr)
+                    {
+                        osg::GraphicsContext* gc = osg::GraphicsContext::getCompileContext(*itr);
+                        if (gc)
+                        {   
+                            osg::GraphicsThread* gt = gc->getGraphicsThread();
+                            if (gt)
+                            {
+                                gt->add(new DatabasePager::CompileOperation(this));
+                            }
+                            else
+                            {
+                                gc->makeCurrent();
+                                
+                                compileAllGLObjects(*(gc->getState()));
+                                
+                                gc->releaseContext();
+                            }
+                        }
+                    }
 
-                updateDatabasePagerThreadBlock();
-
+                    // osg::notify(osg::NOTICE)<<"Done compiling in paging thread"<<std::endl;                   
+                }
+                
             }
             else
             {
@@ -659,8 +768,18 @@ void DatabasePager::addLoadedDataToSceneGraph(double timeStamp)
             plod->setTimeStamp(plod->getNumChildren(),timeStamp);
         } 
         group->addChild(databaseRequest->_loadedModel.get());
-        osg::notify(osg::INFO)<<"merged subgraph"<<databaseRequest->_fileName<<" after "<<databaseRequest->_numOfRequests<<" requests."<<std::endl;
 
+        osg::notify(osg::INFO)<<"merged subgraph"<<databaseRequest->_fileName<<" after "<<databaseRequest->_numOfRequests<<" requests and time="<<(timeStamp-databaseRequest->_timestampFirstRequest)*1000.0<<std::endl;
+    
+        double timeToMerge = timeStamp-databaseRequest->_timestampFirstRequest;
+
+        if (timeToMerge<_minimumTimeToMergeTile) _minimumTimeToMergeTile = timeToMerge;
+        if (timeToMerge>_maximumTimeToMergeTile) _maximumTimeToMergeTile = timeToMerge;
+        
+        _totalTimeToMergeTiles += timeToMerge;
+        ++_numTilesMerges;
+
+        // osg::notify(osg::NOTICE)<<"curr = "<<timeToMerge<<" min "<<getMinimumTimeToMergeTile()*1000.0<<" max = "<<getMaximumTimeToMergeTile()*1000.0<<" average = "<<getAverageTimToMergeTiles()*1000.0<<std::endl;
     }
 
     // osg::notify(osg::NOTICE)<<"Done DatabasePager::addLoadedDataToSceneGraph"<<osg::Timer::instance()->delta_m(before,osg::Timer::instance()->tick())<<" ms  objects"<<localFileLoadedList.size()<<std::endl;
@@ -805,7 +924,7 @@ void DatabasePager::removeExpiredSubgraphs(double currentFrameTime)
         osgDB::Registry::instance()->getSharedStateManager()->prune();
 
     // update the Registry object cache.
-    osgDB::Registry::instance()->updateTimeStampOfObjectsInCacheWithExtenalReferences(currentFrameTime);
+    osgDB::Registry::instance()->updateTimeStampOfObjectsInCacheWithExternalReferences(currentFrameTime);
     osgDB::Registry::instance()->removeExpiredObjectsInCache(expiryTime);
 
 
@@ -863,9 +982,41 @@ bool DatabasePager::getCompileGLObjectsForContextID(unsigned int contextID)
 }
 
 
+DatabasePager::CompileOperation::CompileOperation(osgDB::DatabasePager* databasePager):
+    osg::GraphicsOperation("DatabasePager::CompileOperation",false),
+    _databasePager(databasePager)
+{
+}
+
+void DatabasePager::CompileOperation::operator () (osg::GraphicsContext* context)
+{
+    // osg::notify(osg::NOTICE)<<"Background thread compiling"<<std::endl;
+
+    if (_databasePager.valid()) _databasePager->compileAllGLObjects(*(context->getState()));
+    
+}
+
+bool DatabasePager::requiresExternalCompileGLObjects(unsigned int contextID) const
+{
+    if (_activeGraphicsContexts.count(contextID)==0) return false;
+
+    return osg::GraphicsContext::getCompileContext(contextID)==0;
+}
+
+void DatabasePager::compileAllGLObjects(osg::State& state)
+{
+    double availableTime = DBL_MAX;
+    compileGLObjects(state, availableTime);
+}
+
 void DatabasePager::compileGLObjects(osg::State& state, double& availableTime)
 {
-//    osg::notify(osg::NOTICE)<<"DatabasePager::compileGLObjects "<<_frameNumber<<std::endl;
+    // osg::notify(osg::NOTICE)<<"DatabasePager::compileGLObjects "<<_frameNumber<<std::endl;
+
+    bool compileAll = (availableTime==DBL_MAX);
+
+    osg::RenderInfo renderInfo;
+    renderInfo.setState(&state);
 
     if (availableTime>0.0)
     {
@@ -921,7 +1072,7 @@ void DatabasePager::compileGLObjects(osg::State& state, double& availableTime)
 
         // while there are valid databaseRequest's in the to compile list and there is
         // sufficient time left compile each databaseRequest's stateset and drawables.
-        while (databaseRequest.valid() && elapsedTime<availableTime && numObjectsCompiled<_maximumNumOfObjectsToCompilePerFrame) 
+        while (databaseRequest.valid() && (compileAll || (elapsedTime<availableTime && numObjectsCompiled<_maximumNumOfObjectsToCompilePerFrame)) ) 
         {
             DataToCompileMap& dcm = databaseRequest->_dataToCompileMap;
             DataToCompile& dtc = dcm[state.getContextID()];
@@ -959,19 +1110,19 @@ void DatabasePager::compileGLObjects(osg::State& state, double& availableTime)
                 sslist.erase(sslist.begin(),itr);
             }
 
-            if (!dtc.second.empty() && (elapsedTime+estimatedDrawableDuration)<availableTime && numObjectsCompiled<_maximumNumOfObjectsToCompilePerFrame)
+            if (!dtc.second.empty() && (compileAll || ((elapsedTime+estimatedDrawableDuration)<availableTime && numObjectsCompiled<_maximumNumOfObjectsToCompilePerFrame)))
             {
                 // we have Drawable's to compile
                 //osg::notify(osg::INFO)<<"Compiling drawables"<<std::endl;
                 DrawableList& dwlist = dtc.second;
                 DrawableList::iterator itr=dwlist.begin();
                 for(;
-                    itr!=dwlist.end() && (elapsedTime+estimatedDrawableDuration)<availableTime && numObjectsCompiled<_maximumNumOfObjectsToCompilePerFrame;
+                    itr!=dwlist.end() && (compileAll || ((elapsedTime+estimatedDrawableDuration)<availableTime && numObjectsCompiled<_maximumNumOfObjectsToCompilePerFrame));
                     ++itr)
                 {
                     //osg::notify(osg::INFO)<<"    Compiling drawable "<<(*itr).get()<<std::endl;
                     double startCompileTime = timer.delta_s(start_tick,timer.tick());
-                    (*itr)->compileGLObjects(state);
+                    (*itr)->compileGLObjects(renderInfo);
                     elapsedTime = timer.delta_s(start_tick,timer.tick());
 
                     // estimate the duration of the compile based on current compile duration.
@@ -1001,6 +1152,8 @@ void DatabasePager::compileGLObjects(osg::State& state, double& availableTime)
             if (allCompiled)
             {
                 // we've compile all of the current databaseRequest so we can now pop it off the
+                // osg::notify(osg::NOTICE)<<"All compiled"<<std::endl;
+
                 // to compile list and place it on the merge list.
                 OpenThreads::ScopedLock<OpenThreads::Mutex> lock(_dataToCompileListMutex);
 
@@ -1021,7 +1174,7 @@ void DatabasePager::compileGLObjects(osg::State& state, double& availableTime)
             }
             else 
             {
-                //osg::notify(osg::INFO)<<"Not all compiled"<<std::endl;
+                // osg::notify(osg::NOTICE)<<"Not all compiled"<<std::endl;
                 databaseRequest = 0;
             }
 
