@@ -18,6 +18,7 @@
 #include <osg/Group>
 #include <osg/Geode>
 #include <osg/ApplicationUsage>
+#include <osg/Version>
 
 #include <osgDB/Registry>
 #include <osgDB/FileUtils>
@@ -29,9 +30,27 @@
 #include <algorithm>
 #include <set>
 
+#if defined(__sgi)
+    #include <ctype.h>
+#elif defined(__GNUC__) || !defined(WIN32) || defined(__MWERKS__)
+    #include <cctype>
+    using std::tolower;
+    using std::strlen;    
+#endif
+
+
 using namespace osg;
 using namespace osgDB;
 
+#if !defined(WIN32) || defined(__CYGWIN__)
+static osg::ApplicationUsageProxy Registry_e0(osg::ApplicationUsage::ENVIRONMENTAL_VARIABLE,"OSG_FILE_PATH <path>[:path]..","Paths for locating datafiles");
+static osg::ApplicationUsageProxy Registry_e1(osg::ApplicationUsage::ENVIRONMENTAL_VARIABLE,"OSG_LIBRARY_PATH <path>[:path]..","Paths for locating libraries/ plugins");
+#else
+static osg::ApplicationUsageProxy Registry_e0(osg::ApplicationUsage::ENVIRONMENTAL_VARIABLE,"OSG_FILE_PATH <path>[;path]..","Paths for locating datafiles");
+static osg::ApplicationUsageProxy Registry_e1(osg::ApplicationUsage::ENVIRONMENTAL_VARIABLE,"OSG_LIBRARY_PATH <path>[;path]..","Paths for locating libraries/ plugins");
+#endif
+
+
 class Registry::AvailableReaderWriterIterator
 {
 public:
@@ -121,7 +140,8 @@ Registry* Registry::instance(bool erase)
 {
     static ref_ptr<Registry> s_registry = new Registry;
     if (erase) 
-    {
+    {   
+        s_registry->destruct();
         s_registry = 0;
     }
     return s_registry.get(); // will return NULL on erase
@@ -136,6 +156,9 @@ Registry::Registry()
 
     _createNodeFromImage = false;
     _openingLibrary = false;
+
+    // add default osga archive extension
+    _archiveExtList.push_back("osga");
     
     initFilePathLists();
 
@@ -155,6 +178,11 @@ Registry::Registry()
     #endif
     }
 
+    addFileExtensionAlias("osgs", "osg");
+    addFileExtensionAlias("shadow",  "osgShadow");
+    addFileExtensionAlias("terrain", "osgTerrain");
+    addFileExtensionAlias("view",  "osgViewer");
+
     addFileExtensionAlias("sgi",  "rgb");
     addFileExtensionAlias("rgba", "rgb");
     addFileExtensionAlias("int",  "rgb");
@@ -205,7 +233,11 @@ Registry::Registry()
     //addFileExtensionAlias("geo",  "lwo");
     addFileExtensionAlias("lw",   "lwo");
 
-    addFileExtensionAlias("wrl",   "iv");
+    #if defined(USE_VRML)
+        addFileExtensionAlias("wrl",   "vrml");    
+    #elif defined(USE_INVENTOR)
+        addFileExtensionAlias("wrl",   "iv");
+    #endif
     
     // add alias for the text/freetype plugin.
     addFileExtensionAlias("ttf",   "freetype");  // true type
@@ -230,6 +262,11 @@ Registry::Registry()
 
 
 Registry::~Registry()
+{
+    destruct();
+}
+
+void Registry::destruct()
 {
     // switch off the pager and its associated thread before we clean up 
     // rest of the Registry.
@@ -253,12 +290,6 @@ Registry::~Registry()
     closeAllLibraries();
 }
 
-#if !defined(WIN32) || defined(__CYGWIN__)
-static osg::ApplicationUsageProxy Registry_e0(osg::ApplicationUsage::ENVIRONMENTAL_VARIABLE,"OSG_FILE_PATH <path>[:path]..","Paths for locating datafiles");
-#else
-static osg::ApplicationUsageProxy Registry_e0(osg::ApplicationUsage::ENVIRONMENTAL_VARIABLE,"OSG_FILE_PATH <path>[;path]..","Paths for locating datafiles");
-#endif
-
 #include <iostream>
 
 void Registry::initDataFilePathList()
@@ -293,11 +324,6 @@ void Registry::setDataFilePathList(const std::string& paths)
 
 void Registry::setLibraryFilePathList(const std::string& paths) { _libraryFilePath.clear(); convertStringPathIntoFilePathList(paths,_libraryFilePath); }
 
-#ifndef WIN32
-static osg::ApplicationUsageProxy Registry_e1(osg::ApplicationUsage::ENVIRONMENTAL_VARIABLE,"OSG_LIBRARY_PATH <path>[:path]..","Paths for locating libraries/ plugins");
-#else
-static osg::ApplicationUsageProxy Registry_e1(osg::ApplicationUsage::ENVIRONMENTAL_VARIABLE,"OSG_LIBRARY_PATH <path>[;path]..","Paths for locating libraries/ plugins");
-#endif
 
 
 void Registry::initLibraryFilePathList()
@@ -473,63 +499,119 @@ void Registry::addFileExtensionAlias(const std::string mapExt, const std::string
     _extAliasMap[mapExt] = toExt;
 }
 
-std::string Registry::createLibraryNameForFile(const std::string& fileName)
+bool Registry::readPluginAliasConfigurationFile( const std::string& file )
 {
-    std::string ext = getLowerCaseFileExtension(fileName);
-    return createLibraryNameForExtension(ext);
+    std::string fileName = osgDB::findDataFile( file );
+    if (fileName.empty())
+    {
+        osg::notify( osg::WARN) << "Can't find plugin alias config file \"" << file << "\"." << std::endl;
+        return false;
+    }
+
+    std::ifstream ifs;
+    ifs.open( fileName.c_str() );
+    if (!ifs.good())
+    {
+        osg::notify( osg::WARN) << "Can't open plugin alias config file \"" << fileName << "\"." << std::endl;
+        return false;
+    }
+
+    int lineNum( 0 );
+    while (ifs.good())
+    {
+        std::string raw;
+        ++lineNum;
+        std::getline( ifs, raw );
+        std::string ln = trim( raw );
+        if (ln.empty()) continue;
+        if (ln[0] == '#') continue;
+
+        std::string::size_type spIdx = ln.find_first_of( " \t" );
+        if (spIdx == ln.npos)
+        {
+            // mapExt and toExt must be on the same line, separated by a space.
+            osg::notify( osg::WARN) << file << ", line " << lineNum << ": Syntax error: missing space in \"" << raw << "\"." << std::endl;
+            continue;
+        }
+
+        const std::string mapExt = trim( ln.substr( 0, spIdx ) );
+        const std::string toExt = trim( ln.substr( spIdx+1 ) );
+        addFileExtensionAlias( mapExt, toExt );
+    }
+    return true;
+}
+
+std::string Registry::trim( const std::string& str )
+{
+    if (!str.size()) return str;
+    std::string::size_type first = str.find_first_not_of( " \t" );
+    std::string::size_type last = str.find_last_not_of( "  \t\r\n" );
+    if ((first==str.npos) || (last==str.npos)) return std::string( "" );
+    return str.substr( first, last-first+1 );
 }
 
 
+std::string Registry::createLibraryNameForFile(const std::string& fileName)
+{
+    return createLibraryNameForExtension(getFileExtension(fileName));
+}
+
 std::string Registry::createLibraryNameForExtension(const std::string& ext)
 {
+    std::string lowercase_ext;
+    for(std::string::const_iterator sitr=ext.begin();
+        sitr!=ext.end();
+        ++sitr)
+    {
+        lowercase_ext.push_back(tolower(*sitr));
+    }
 
-    ExtensionAliasMap::iterator itr=_extAliasMap.find(ext);
+    ExtensionAliasMap::iterator itr=_extAliasMap.find(lowercase_ext);
     if (itr!=_extAliasMap.end() && ext != itr->second) return createLibraryNameForExtension(itr->second);
 
 #ifdef OSG_JAVA_BUILD
-    static std::string prepend = "java";
-#else
+    static std::string prepend = std::string("osgPlugins-")+std::string(osgGetVersion())+std::string("/java");
+#elseif defined(__APPLE__)
+    // OSX is rather a mess in FileUtils.cpp w.r.t its appendPlatformSpecificLibraryFilePaths implementation
+    // as it hardwires the plugin name to PlugIns.  This *needs* fixing to use the naming convention as all
+    // other platforms.
     static std::string prepend = "";
+#else
+    static std::string prepend = std::string("osgPlugins-")+std::string(osgGetVersion())+std::string("/");
 #endif
 
-#if defined(WIN32)
-    // !! recheck evolving Cygwin DLL extension naming protocols !! NHV
-    #ifdef __CYGWIN__
-        return "cyg"+prepend+"osgdb_"+ext+".dll";
-    #elif defined(__MINGW32__)
-        return "lib"+prepend+"osgdb_"+ext+".dll";
+#if defined(__CYGWIN__)
+    return prepend+"cygwin_"+"osgdb_"+lowercase_ext+".dll";
+#elif defined(__MINGW32__)
+    return prepend+"mingw_"+"osgdb_"+lowercase_ext+".dll";
+#elif defined(WIN32)
+    #ifdef _DEBUG
+        return prepend+"osgdb_"+lowercase_ext+"d.dll";
     #else
-        #ifdef _DEBUG
-            return prepend+"osgdb_"+ext+"d.dll";
-        #else
-            return prepend+"osgdb_"+ext+".dll";
-        #endif
+        return prepend+"osgdb_"+lowercase_ext+".dll";
     #endif
 #elif macintosh
-    return prepend+"osgdb_"+ext;
+    return prepend+"osgdb_"+lowercase_ext;
 #elif defined(__hpux__)
     // why don't we use PLUGIN_EXT from the makefiles here?
-    return prepend+"osgdb_"+ext+".sl";
+    return prepend+"osgdb_"+lowercase_ext+".sl";
 #else
-    return prepend+"osgdb_"+ext+".so";
+    return prepend+"osgdb_"+lowercase_ext+".so";
 #endif
 
 }
 
 std::string Registry::createLibraryNameForNodeKit(const std::string& name)
 {
-#if defined(WIN32)
-    // !! recheck evolving Cygwin DLL extension naming protocols !! NHV
-    #ifdef __CYGWIN__ // [
+#if defined(__CYGWIN__)
     return "cyg"+name+".dll";
-    #elif defined(__MINGW32__)
-        return "lib"+name+".dll";
+#elif defined(__MINGW32__)
+    return "lib"+name+".dll";
+#elif defined(WIN32)
+    #ifdef _DEBUG
+        return name+"d.dll";
     #else
-        #ifdef _DEBUG
-            return name+"d.dll";
-        #else
-            return name+".dll";
-        #endif
+        return name+".dll";
     #endif
 #elif macintosh
     return name;
@@ -714,7 +796,7 @@ osg::Object* Registry::readObjectOfType(const basic_type_wrapper &btw,Input& fr)
             bool iteratorAdvanced = false;
             if (fr[0].matchWord("UniqueID") && fr[1].isString())
             {
-                fr.regisiterUniqueIDForObject(fr[1].getStr(),obj);
+                fr.registerUniqueIDForObject(fr[1].getStr(),obj);
                 fr += 2;
                 iteratorAdvanced = true;
             }
@@ -836,7 +918,7 @@ osg::Object* Registry::readObject(DotOsgWrapperMap& dowMap,Input& fr)
             bool iteratorAdvanced = false;
             if (fr[0].matchWord("UniqueID") && fr[1].isString())
             {
-                fr.regisiterUniqueIDForObject(fr[1].getStr(),obj);
+                fr.registerUniqueIDForObject(fr[1].getStr(),obj);
                 fr += 2;
                 iteratorAdvanced = true;
             }
@@ -1053,14 +1135,14 @@ bool Registry::writeObject(const osg::Object& obj,Output& fw)
         std::string uniqueID;
         if (fw.getUniqueIDForObject(&obj,uniqueID))
         {
-            fw.indent() << "Use " << uniqueID << std::endl;
+            fw.writeUseID( uniqueID );
             return true;
         }
     }
 
-    std::string classname = obj.className();
-    std::string libraryName = obj.libraryName();
-    std::string compositeName = libraryName + "::" + classname;
+    const std::string classname( obj.className() );
+    const std::string libraryName( obj.libraryName() );
+    const std::string compositeName( libraryName + "::" + classname );
 
     // try composite name first
     DotOsgWrapperMap::iterator itr = _classNameWrapperMap.find(compositeName);
@@ -1068,11 +1150,11 @@ bool Registry::writeObject(const osg::Object& obj,Output& fw)
     if (itr==_classNameWrapperMap.end())
     {
         // first try the standard nodekit library.
-        std::string nodeKitLibraryName = createLibraryNameForNodeKit(obj.libraryName());
+        std::string nodeKitLibraryName = createLibraryNameForNodeKit(libraryName);
         if (loadLibrary(nodeKitLibraryName)) return writeObject(obj,fw);
 
         // otherwise try the osgdb_ plugin library.
-        std::string pluginLibraryName = createLibraryNameForExtension(obj.libraryName());
+        std::string pluginLibraryName = createLibraryNameForExtension(libraryName);
         if (loadLibrary(pluginLibraryName)) return writeObject(obj,fw);
 
         // otherwise try simple class name
@@ -1085,11 +1167,10 @@ bool Registry::writeObject(const osg::Object& obj,Output& fw)
         DotOsgWrapper* wrapper = itr->second.get();
         const DotOsgWrapper::Associates& assoc = wrapper->getAssociates();
 
-        if (strcmp(obj.libraryName(),"osg")==0)
+        if (libraryName=="osg")
         {
             // member of the core osg, so no need to have composite library::class name.
-            fw.indent() << wrapper->getName() << " {"<< std::endl;
-            fw.moveIn();
+            fw.writeBeginObject( wrapper->getName() );
         }
         else
         {
@@ -1097,15 +1178,14 @@ bool Registry::writeObject(const osg::Object& obj,Output& fw)
             std::string::size_type posDoubleColon = wrapper->getName().find("::");
             if (posDoubleColon != std::string::npos)
             {
-                fw.indent() << wrapper->getName() << " {"<< std::endl;
+                fw.writeBeginObject( wrapper->getName() );
             }
             else
             {
-                fw.indent() << obj.libraryName()<<"::"<< wrapper->getName() << " {"<< std::endl;
+                fw.writeBeginObject( libraryName + "::" + wrapper->getName() );
             }
-
-            fw.moveIn();
         }
+        fw.moveIn();
 
 
         // write out the unique ID if required.
@@ -1114,7 +1194,7 @@ bool Registry::writeObject(const osg::Object& obj,Output& fw)
             std::string uniqueID;
             fw.createUniqueIDForObject(&obj,uniqueID);
             fw.registerUniqueIDForObject(&obj,uniqueID);
-            fw.indent() << "UniqueID " << uniqueID << std::endl;
+            fw.writeUniqueID( uniqueID );
         }
 
         // read the local data by iterating through the associate
@@ -1168,7 +1248,7 @@ bool Registry::writeObject(const osg::Object& obj,Output& fw)
         }
 
         fw.moveOut();
-        fw.indent() << "}"<< std::endl;
+        fw.writeEndObject();
 
         return true;
     }
@@ -1231,30 +1311,47 @@ struct Registry::ReadArchiveFunctor : public Registry::ReadFunctor
 
 };
 
+void Registry::addArchiveExtension(const std::string ext)
+{
+    for(ArchiveExtensionList::iterator aitr=_archiveExtList.begin();
+        aitr!=_archiveExtList.end();
+        ++aitr)
+    {
+        if ( (*aitr) == ext)   // extension already in archive extension list
+            return;
+    }
+    _archiveExtList.push_back(ext);
+}
+
 ReaderWriter::ReadResult Registry::read(const ReadFunctor& readFunctor)
 {
-    std::string archiveName(".osga");
-
-    std::string::size_type positionArchive = readFunctor._filename.find(archiveName+'/');
-    if (positionArchive==std::string::npos) positionArchive = readFunctor._filename.find(archiveName+'\\');
-    if (positionArchive!=std::string::npos)
+    for(ArchiveExtensionList::iterator aitr=_archiveExtList.begin();
+        aitr!=_archiveExtList.end();
+        ++aitr)
     {
-        std::string archiveName(readFunctor._filename.substr(0,positionArchive+5));
-        std::string fileName(readFunctor._filename.substr(positionArchive+6,std::string::npos));
-        osg::notify(osg::INFO)<<"Contains archive : "<<readFunctor._filename<<std::endl;
-        osg::notify(osg::INFO)<<"         archive : "<<archiveName<<std::endl;
-        osg::notify(osg::INFO)<<"         filename : "<<fileName<<std::endl;
-        
-        ReaderWriter::ReadResult result = openArchiveImplementation(archiveName,ReaderWriter::READ, 4096, readFunctor._options);
-        
-        if (!result.validArchive()) return result;
+        std::string archiveName = "." + (*aitr);
 
-        osgDB::Archive* archive = result.getArchive();
+        std::string::size_type positionArchive = readFunctor._filename.find(archiveName+'/');
+        if (positionArchive==std::string::npos) positionArchive = readFunctor._filename.find(archiveName+'\\');
+        if (positionArchive!=std::string::npos)
+        {
+            std::string archiveName(readFunctor._filename.substr(0,positionArchive+5));
+            std::string fileName(readFunctor._filename.substr(positionArchive+6,std::string::npos));
+            osg::notify(osg::INFO)<<"Contains archive : "<<readFunctor._filename<<std::endl;
+            osg::notify(osg::INFO)<<"         archive : "<<archiveName<<std::endl;
+            osg::notify(osg::INFO)<<"         filename : "<<fileName<<std::endl;
         
-        osg::ref_ptr<ReaderWriter::Options> options = new ReaderWriter::Options;
-        options->setDatabasePath(archiveName);
+            ReaderWriter::ReadResult result = openArchiveImplementation(archiveName,ReaderWriter::READ, 4096, readFunctor._options);
+        
+            if (!result.validArchive()) return result;
 
-        return archive->readObject(fileName,options.get());
+            osgDB::Archive* archive = result.getArchive();
+        
+            osg::ref_ptr<ReaderWriter::Options> options = new ReaderWriter::Options;
+            options->setDatabasePath(archiveName);
+
+            return archive->readObject(fileName,options.get());
+        }
     }
 
     // if filename contains archive
@@ -1459,25 +1556,19 @@ ReaderWriter::ReadResult Registry::readImplementation(const ReadFunctor& readFun
 
 ReaderWriter::ReadResult Registry::openArchiveImplementation(const std::string& fileName, ReaderWriter::ArchiveStatus status, unsigned int indexBlockSizeHint, const ReaderWriter::Options* options)
 {
+    osgDB::Archive* archive = getFromArchiveCache(fileName);
+    if (archive) return archive;
+
+    ReaderWriter::ReadResult result = readImplementation(ReadArchiveFunctor(fileName, status, indexBlockSizeHint, options),false);
 
     // default to using chaching archive if no options structure provided, but if options are provided use archives
     // only if supplied.
-    if (!options || (options && (options->getObjectCacheHint() & ReaderWriter::Options::CACHE_ARCHIVES)))
+    if (result.validArchive() &&
+        (!options || (options->getObjectCacheHint() & ReaderWriter::Options::CACHE_ARCHIVES)) )
     {
-        osgDB::Archive* archive = getFromArchiveCache(fileName);
-        if (archive) return archive;
-
-        ReaderWriter::ReadResult result = readImplementation(ReadArchiveFunctor(fileName, status, indexBlockSizeHint, options),false);
-        if (result.validArchive())
-        {
-            addToArchiveCache(fileName,result.getArchive());
-        }
-        return result;
-    }
-    else
-    {
-        return readImplementation(ReadArchiveFunctor(fileName, status, indexBlockSizeHint, _options.get()),false);
+        addToArchiveCache(fileName,result.getArchive());
     }
+    return result;
 }
 
 
@@ -1487,7 +1578,7 @@ ReaderWriter::ReadResult Registry::readObjectImplementation(const std::string& f
                               options ? (options->getObjectCacheHint()&ReaderWriter::Options::CACHE_OBJECTS)!=0: false);
 }
 
-ReaderWriter::WriteResult Registry::writeObjectImplementation(const Object& obj,const std::string& fileName)
+ReaderWriter::WriteResult Registry::writeObjectImplementation(const Object& obj,const std::string& fileName,const ReaderWriter::Options* options)
 {
     // record the errors reported by readerwriters.
     typedef std::vector<ReaderWriter::WriteResult> Results;
@@ -1497,7 +1588,7 @@ ReaderWriter::WriteResult Registry::writeObjectImplementation(const Object& obj,
     AvailableReaderWriterIterator itr(_rwList);
     for(;itr.valid();++itr)
     {
-        ReaderWriter::WriteResult rr = itr->writeObject(obj,fileName,_options.get());
+        ReaderWriter::WriteResult rr = itr->writeObject(obj,fileName,options);
         if (rr.success()) return rr;
         else results.push_back(rr);
     }
@@ -1508,7 +1599,7 @@ ReaderWriter::WriteResult Registry::writeObjectImplementation(const Object& obj,
     {
         for(;itr.valid();++itr)
         {
-            ReaderWriter::WriteResult rr = itr->writeObject(obj,fileName,_options.get());
+            ReaderWriter::WriteResult rr = itr->writeObject(obj,fileName,options);
             if (rr.success()) return rr;
             else results.push_back(rr);
         }
@@ -1519,6 +1610,16 @@ ReaderWriter::WriteResult Registry::writeObjectImplementation(const Object& obj,
         return ReaderWriter::WriteResult("Warning: Could not find plugin to write objects to file \""+fileName+"\".");
     }
 
+    if (results.front().message().empty())
+    {
+        switch(results.front().status())
+        {
+            case(ReaderWriter::WriteResult::FILE_NOT_HANDLED): results.front().message() = "Warning: Write to \""+fileName+"\" not supported."; break;
+            case(ReaderWriter::WriteResult::ERROR_IN_WRITING_FILE): results.front().message() = "Warning: Error in writing to \""+fileName+"\"."; break;
+            default: break;
+        }
+    }
+
     return results.front();
 }
 
@@ -1530,7 +1631,7 @@ ReaderWriter::ReadResult Registry::readImageImplementation(const std::string& fi
                               options ? (options->getObjectCacheHint()&ReaderWriter::Options::CACHE_IMAGES)!=0: false);
 }
 
-ReaderWriter::WriteResult Registry::writeImageImplementation(const Image& image,const std::string& fileName)
+ReaderWriter::WriteResult Registry::writeImageImplementation(const Image& image,const std::string& fileName,const ReaderWriter::Options* options)
 {
     // record the errors reported by readerwriters.
     typedef std::vector<ReaderWriter::WriteResult> Results;
@@ -1540,18 +1641,20 @@ ReaderWriter::WriteResult Registry::writeImageImplementation(const Image& image,
     AvailableReaderWriterIterator itr(_rwList);
     for(;itr.valid();++itr)
     {
-        ReaderWriter::WriteResult rr = itr->writeImage(image,fileName,_options.get());
+        ReaderWriter::WriteResult rr = itr->writeImage(image,fileName,options);
         if (rr.success()) return rr;
         else results.push_back(rr);
     }
 
+    results.clear();
+
     // now look for a plug-in to save the file.
     std::string libraryName = createLibraryNameForFile(fileName);
     if (loadLibrary(libraryName))
     {
         for(;itr.valid();++itr)
         {
-            ReaderWriter::WriteResult rr = itr->writeImage(image,fileName,_options.get());
+            ReaderWriter::WriteResult rr = itr->writeImage(image,fileName,options);
             if (rr.success()) return rr;
             else results.push_back(rr);
         }
@@ -1562,6 +1665,16 @@ ReaderWriter::WriteResult Registry::writeImageImplementation(const Image& image,
         return ReaderWriter::WriteResult("Warning: Could not find plugin to write image to file \""+fileName+"\".");
     }
     
+    if (results.front().message().empty())
+    {
+        switch(results.front().status())
+        {
+            case(ReaderWriter::WriteResult::FILE_NOT_HANDLED): results.front().message() = "Warning: Write to \""+fileName+"\" not supported."; break;
+            case(ReaderWriter::WriteResult::ERROR_IN_WRITING_FILE): results.front().message() = "Warning: Error in writing to \""+fileName+"\"."; break;
+            default: break;
+        }
+    }
+
     return results.front();
 }
 
@@ -1572,7 +1685,7 @@ ReaderWriter::ReadResult Registry::readHeightFieldImplementation(const std::stri
                               options ? (options->getObjectCacheHint()&ReaderWriter::Options::CACHE_HEIGHTFIELDS)!=0: false);
 }
 
-ReaderWriter::WriteResult Registry::writeHeightFieldImplementation(const HeightField& HeightField,const std::string& fileName)
+ReaderWriter::WriteResult Registry::writeHeightFieldImplementation(const HeightField& HeightField,const std::string& fileName,const ReaderWriter::Options* options)
 {
     // record the errors reported by readerwriters.
     typedef std::vector<ReaderWriter::WriteResult> Results;
@@ -1582,18 +1695,20 @@ ReaderWriter::WriteResult Registry::writeHeightFieldImplementation(const HeightF
     AvailableReaderWriterIterator itr(_rwList);
     for(;itr.valid();++itr)
     {
-        ReaderWriter::WriteResult rr = itr->writeHeightField(HeightField,fileName,_options.get());
+        ReaderWriter::WriteResult rr = itr->writeHeightField(HeightField,fileName,options);
         if (rr.success()) return rr;
         else results.push_back(rr);
     }
 
+    results.clear();
+
     // now look for a plug-in to save the file.
     std::string libraryName = createLibraryNameForFile(fileName);
     if (loadLibrary(libraryName))
     {
         for(;itr.valid();++itr)
         {
-            ReaderWriter::WriteResult rr = itr->writeHeightField(HeightField,fileName,_options.get());
+            ReaderWriter::WriteResult rr = itr->writeHeightField(HeightField,fileName,options);
             if (rr.success()) return rr;
             else results.push_back(rr);
         }
@@ -1603,6 +1718,16 @@ ReaderWriter::WriteResult Registry::writeHeightFieldImplementation(const HeightF
     {
         return ReaderWriter::WriteResult("Warning: Could not find plugin to write HeightField to file \""+fileName+"\".");
     }
+
+    if (results.front().message().empty())
+    {
+        switch(results.front().status())
+        {
+            case(ReaderWriter::WriteResult::FILE_NOT_HANDLED): results.front().message() = "Warning: Write to \""+fileName+"\" not supported."; break;
+            case(ReaderWriter::WriteResult::ERROR_IN_WRITING_FILE): results.front().message() = "Warning: Error in writing to \""+fileName+"\"."; break;
+            default: break;
+        }
+    }
     
     return results.front();
 }
@@ -1614,7 +1739,7 @@ ReaderWriter::ReadResult Registry::readNodeImplementation(const std::string& fil
                               options ? (options->getObjectCacheHint()&ReaderWriter::Options::CACHE_NODES)!=0: false);
 }
 
-ReaderWriter::WriteResult Registry::writeNodeImplementation(const Node& node,const std::string& fileName)
+ReaderWriter::WriteResult Registry::writeNodeImplementation(const Node& node,const std::string& fileName,const ReaderWriter::Options* options)
 {
     // record the errors reported by readerwriters.
     typedef std::vector<ReaderWriter::WriteResult> Results;
@@ -1624,18 +1749,23 @@ ReaderWriter::WriteResult Registry::writeNodeImplementation(const Node& node,con
     AvailableReaderWriterIterator itr(_rwList);
     for(;itr.valid();++itr)
     {
-        ReaderWriter::WriteResult rr = itr->writeNode(node,fileName,_options.get());
+        ReaderWriter::WriteResult rr = itr->writeNode(node,fileName,options);
         if (rr.success()) return rr;
         else results.push_back(rr);
     }
 
+    results.clear();
+
     // now look for a plug-in to save the file.
     std::string libraryName = createLibraryNameForFile(fileName);
+
+
     if (loadLibrary(libraryName))
     {
         for(;itr.valid();++itr)
         {
-            ReaderWriter::WriteResult rr = itr->writeNode(node,fileName,_options.get());
+            ReaderWriter::WriteResult rr = itr->writeNode(node,fileName,options);
+    
             if (rr.success()) return rr;
             else results.push_back(rr);
         }
@@ -1646,6 +1776,16 @@ ReaderWriter::WriteResult Registry::writeNodeImplementation(const Node& node,con
         return ReaderWriter::WriteResult("Warning: Could not find plugin to write nodes to file \""+fileName+"\".");
     }
 
+    if (results.front().message().empty())
+    {
+        switch(results.front().status())
+        {
+            case(ReaderWriter::WriteResult::FILE_NOT_HANDLED): results.front().message() = "Warning: Write to \""+fileName+"\" not supported."; break;
+            case(ReaderWriter::WriteResult::ERROR_IN_WRITING_FILE): results.front().message() = "Warning: Error in writing to \""+fileName+"\"."; break;
+            default: break;
+        }
+    }
+
     return results.front();
 }
 
@@ -1663,7 +1803,7 @@ osg::Object* Registry::getFromObjectCache(const std::string& fileName)
     else return 0;
 }
 
-void Registry::updateTimeStampOfObjectsInCacheWithExtenalReferences(double currentTime)
+void Registry::updateTimeStampOfObjectsInCacheWithExternalReferences(double currentTime)
 {
     OpenThreads::ScopedLock<OpenThreads::Mutex> lock(_objectCacheMutex);
 
