@@ -15,6 +15,8 @@
     #pragma warning( disable : 4786 )
 #endif
 
+#include <stdlib.h>
+
 #include <osgGA/DriveManipulator>
 #include <osgUtil/IntersectVisitor>
 #include <osg/Notify>
@@ -198,6 +200,7 @@ void DriveManipulator::home(const GUIEventAdapter& ea,GUIActionAdapter& us)
     _pitch = 0.0;
 
     us.requestRedraw();
+    us.requestContinuousUpdate(false);
 
     us.requestWarpPointer((ea.getXmin()+ea.getXmax())/2.0f,(ea.getYmin()+ea.getYmax())/2.0f);
 
@@ -301,6 +304,23 @@ void DriveManipulator::init(const GUIEventAdapter& ea,GUIActionAdapter& us)
 
 bool DriveManipulator::handle(const GUIEventAdapter& ea,GUIActionAdapter& us)
 {
+    switch(ea.getEventType())
+    {
+        case(GUIEventAdapter::FRAME):
+            addMouseEvent(ea);
+            if (calcMovement()) us.requestRedraw();
+            return false;
+
+        case(GUIEventAdapter::RESIZE):
+            init(ea,us);
+            us.requestRedraw();
+            return true;
+        default:
+            break;
+    }
+
+    if (ea.getHandled()) return false;
+
     switch(ea.getEventType())
     {
         case(GUIEventAdapter::PUSH):
@@ -345,8 +365,6 @@ bool DriveManipulator::handle(const GUIEventAdapter& ea,GUIActionAdapter& us)
             {
                 flushMouseEventStack();
                 home(ea,us);
-                us.requestRedraw();
-                us.requestContinuousUpdate(false);
                 return true;
             }
             else if (ea.getKey()=='q')
@@ -399,22 +417,6 @@ bool DriveManipulator::handle(const GUIEventAdapter& ea,GUIActionAdapter& us)
             return false;
         }
 
-        _pitchDownKeyPressed = false;
-
-        case(GUIEventAdapter::FRAME):
-        {
-            addMouseEvent(ea);
-            if (calcMovement()) us.requestRedraw();
-            return false;
-        }
-
-        case(GUIEventAdapter::RESIZE):
-        {
-            init(ea,us);
-            us.requestRedraw();
-            return true;
-        }
-
         default:
             return false;
     }
