@@ -18,7 +18,10 @@ using namespace osgGA;
 
 EventQueue::EventQueue(GUIEventAdapter::MouseYOrientation mouseYOrientation)
 {
-    _startTick = osg::Timer::instance()->tick();
+    _useFixedMouseInputRange = false;
+
+    _startTick = osg::Timer::instance()->getStartTick();
+
     _accumulateEventState = new GUIEventAdapter();
     _accumulateEventState->setMouseYOrientation(mouseYOrientation);
 }
@@ -41,8 +44,6 @@ void EventQueue::appendEvents(Events& events)
 
 void EventQueue::addEvent(GUIEventAdapter* event)
 {
-    event->setTime(getTime());
-
     OpenThreads::ScopedLock<OpenThreads::Mutex> lock(_eventQueueMutex);
     _eventQueue.push_back(event);
 }
@@ -77,72 +78,78 @@ bool EventQueue::copyEvents(Events& events) const
 }
 
 
-void EventQueue::windowResize(float Xmin, float Ymin, float Xmax, float Ymax)
+void EventQueue::windowResize(int x, int y, int width, int height, double time)
 {
-    _accumulateEventState->setWindowSize(Xmin, Ymin, Xmax, Ymax);
+    _accumulateEventState->setWindowRectangle(x, y, width, height, !_useFixedMouseInputRange);
 
     GUIEventAdapter* event = new GUIEventAdapter(*_accumulateEventState);
     event->setEventType(GUIEventAdapter::RESIZE);
-    
+    event->setTime(time);
+
     addEvent(event);
 }
 
-void EventQueue::penPressure(float pressure)
+void EventQueue::penPressure(float pressure, double time)
 {
     GUIEventAdapter* event = new GUIEventAdapter(*_accumulateEventState);
     event->setEventType(GUIEventAdapter::PEN_PRESSURE);
     event->setPenPressure(pressure);
+    event->setTime(time);
     
     addEvent(event);
 }
 
-void EventQueue::penProximity(GUIEventAdapter::TabletPointerType pt, bool isEntering)
+void EventQueue::penProximity(GUIEventAdapter::TabletPointerType pt, bool isEntering, double time)
 {
     GUIEventAdapter* event = new GUIEventAdapter(*_accumulateEventState);
     event->setEventType( (isEntering) ? GUIEventAdapter::PEN_PROXIMITY_ENTER : GUIEventAdapter::PEN_PROXIMITY_LEAVE);
     event->setTabletPointerType(pt);
+    event->setTime(time);
     
     addEvent(event);
 }
 
-void EventQueue::mouseScroll(GUIEventAdapter::ScrollingMotion sm)
+void EventQueue::mouseScroll(GUIEventAdapter::ScrollingMotion sm, double time)
 {
     GUIEventAdapter* event = new GUIEventAdapter(*_accumulateEventState);
     event->setEventType(GUIEventAdapter::SCROLL);
     event->setScrollingMotion(sm);
+    event->setTime(time);
     
     addEvent(event);
 }
 
-void EventQueue::mouseScroll2D(float x, float y)
+void EventQueue::mouseScroll2D(float x, float y, double time)
 {
     GUIEventAdapter* event = new GUIEventAdapter(*_accumulateEventState);
     event->setEventType(GUIEventAdapter::SCROLL);
     event->setScrollingMotionDelta(x,y);
+    event->setTime(time);
     
     addEvent(event);
 }
 
 
-void EventQueue::mouseWarp(float x, float y)
+void EventQueue::mouseWarped(float x, float y)
 {
     _accumulateEventState->setX(x);
     _accumulateEventState->setY(y);
 }
 
 
-void EventQueue::mouseMotion(float x, float y)
+void EventQueue::mouseMotion(float x, float y, double time)
 {
     _accumulateEventState->setX(x);
     _accumulateEventState->setY(y);
 
     GUIEventAdapter* event = new GUIEventAdapter(*_accumulateEventState);
     event->setEventType(event->getButtonMask() ? GUIEventAdapter::DRAG : GUIEventAdapter::MOVE);
+    event->setTime(time);
     
     addEvent(event);
 }
 
-void EventQueue::mouseButtonPress(float x, float y, unsigned int button)
+void EventQueue::mouseButtonPress(float x, float y, unsigned int button, double time)
 {
     _accumulateEventState->setX(x);
     _accumulateEventState->setY(y);
@@ -162,6 +169,7 @@ void EventQueue::mouseButtonPress(float x, float y, unsigned int button)
 
     GUIEventAdapter* event = new GUIEventAdapter(*_accumulateEventState);
     event->setEventType(GUIEventAdapter::PUSH);
+    event->setTime(time);
 
     switch(button)
     {
@@ -179,7 +187,7 @@ void EventQueue::mouseButtonPress(float x, float y, unsigned int button)
     addEvent(event);
 }
 
-void EventQueue::mouseDoubleButtonPress(float x, float y, unsigned int button)
+void EventQueue::mouseDoubleButtonPress(float x, float y, unsigned int button, double time)
 {
     _accumulateEventState->setX(x);
     _accumulateEventState->setY(y);
@@ -199,6 +207,7 @@ void EventQueue::mouseDoubleButtonPress(float x, float y, unsigned int button)
 
     GUIEventAdapter* event = new GUIEventAdapter(*_accumulateEventState);
     event->setEventType(GUIEventAdapter::DOUBLECLICK);
+    event->setTime(time);
 
     switch(button)
     {
@@ -216,7 +225,7 @@ void EventQueue::mouseDoubleButtonPress(float x, float y, unsigned int button)
     addEvent(event);
 }
 
-void EventQueue::mouseButtonRelease(float x, float y, unsigned int button)
+void EventQueue::mouseButtonRelease(float x, float y, unsigned int button, double time)
 {
     _accumulateEventState->setX(x);
     _accumulateEventState->setY(y);
@@ -236,6 +245,8 @@ void EventQueue::mouseButtonRelease(float x, float y, unsigned int button)
 
     GUIEventAdapter* event = new GUIEventAdapter(*_accumulateEventState);
     event->setEventType(GUIEventAdapter::RELEASE);
+    event->setTime(time);
+
     switch(button)
     {
         case(1): 
@@ -252,7 +263,7 @@ void EventQueue::mouseButtonRelease(float x, float y, unsigned int button)
     addEvent(event);
 }
 
-void EventQueue::keyPress(GUIEventAdapter::KeySymbol key)
+void EventQueue::keyPress(int key, double time)
 {
     switch(key)
     {
@@ -263,7 +274,7 @@ void EventQueue::keyPress(GUIEventAdapter::KeySymbol key)
         case(GUIEventAdapter::KEY_Meta_L):       _accumulateEventState->setModKeyMask(GUIEventAdapter::MODKEY_LEFT_META | _accumulateEventState->getModKeyMask()); break;
         case(GUIEventAdapter::KEY_Meta_R):       _accumulateEventState->setModKeyMask(GUIEventAdapter::MODKEY_RIGHT_META | _accumulateEventState->getModKeyMask()); break;
         case(GUIEventAdapter::KEY_Alt_L):        _accumulateEventState->setModKeyMask(GUIEventAdapter::MODKEY_LEFT_ALT | _accumulateEventState->getModKeyMask()); break;
-        case(GUIEventAdapter::KEY_Alt_R):        _accumulateEventState->setModKeyMask(GUIEventAdapter::MODKEY_LEFT_ALT | _accumulateEventState->getModKeyMask()); break;
+        case(GUIEventAdapter::KEY_Alt_R):        _accumulateEventState->setModKeyMask(GUIEventAdapter::MODKEY_RIGHT_ALT | _accumulateEventState->getModKeyMask()); break;
         case(GUIEventAdapter::KEY_Caps_Lock):
         {
             if ((_accumulateEventState->getModKeyMask() & GUIEventAdapter::MODKEY_CAPS_LOCK)!=0) 
@@ -286,11 +297,12 @@ void EventQueue::keyPress(GUIEventAdapter::KeySymbol key)
     GUIEventAdapter* event = new GUIEventAdapter(*_accumulateEventState);
     event->setEventType(GUIEventAdapter::KEYDOWN);
     event->setKey(key);
+    event->setTime(time);
     
     addEvent(event);
 }
 
-void EventQueue::keyRelease(GUIEventAdapter::KeySymbol key)
+void EventQueue::keyRelease(int key, double time)
 {
     switch(key)
     {
@@ -301,22 +313,42 @@ void EventQueue::keyRelease(GUIEventAdapter::KeySymbol key)
         case(GUIEventAdapter::KEY_Meta_L):       _accumulateEventState->setModKeyMask(~GUIEventAdapter::MODKEY_LEFT_META & _accumulateEventState->getModKeyMask()); break;
         case(GUIEventAdapter::KEY_Meta_R):       _accumulateEventState->setModKeyMask(~GUIEventAdapter::MODKEY_RIGHT_META & _accumulateEventState->getModKeyMask()); break;
         case(GUIEventAdapter::KEY_Alt_L):        _accumulateEventState->setModKeyMask(~GUIEventAdapter::MODKEY_LEFT_ALT & _accumulateEventState->getModKeyMask()); break;
-        case(GUIEventAdapter::KEY_Alt_R):        _accumulateEventState->setModKeyMask(~GUIEventAdapter::MODKEY_LEFT_ALT & _accumulateEventState->getModKeyMask()); break;
+        case(GUIEventAdapter::KEY_Alt_R):        _accumulateEventState->setModKeyMask(~GUIEventAdapter::MODKEY_RIGHT_ALT & _accumulateEventState->getModKeyMask()); break;
         default: break;
     }        
 
     GUIEventAdapter* event = new GUIEventAdapter(*_accumulateEventState);
     event->setEventType(GUIEventAdapter::KEYUP);
     event->setKey(key);
+    event->setTime(time);
     
     addEvent(event);
 }
 
-void EventQueue::frame(double t)
+void EventQueue::closeWindow(double time)
+{
+    GUIEventAdapter* event = new GUIEventAdapter(*_accumulateEventState);
+    event->setEventType(GUIEventAdapter::CLOSE_WINDOW);
+    event->setTime(time);
+  
+    addEvent(event);
+}
+
+void EventQueue::quitApplication(double time)
+{
+    GUIEventAdapter* event = new GUIEventAdapter(*_accumulateEventState);
+    event->setEventType(GUIEventAdapter::QUIT_APPLICATION);
+    event->setTime(time);
+  
+    addEvent(event);
+}
+
+
+void EventQueue::frame(double time)
 {
     GUIEventAdapter* event = new GUIEventAdapter(*_accumulateEventState);
     event->setEventType(GUIEventAdapter::FRAME);
-    event->setTime(t);
+    event->setTime(time);
     
     addEvent(event);
 }
@@ -326,3 +358,15 @@ GUIEventAdapter* EventQueue::createEvent()
     if (_accumulateEventState.valid()) return new GUIEventAdapter(*_accumulateEventState.get()); 
     else return new GUIEventAdapter();    
 }
+
+void EventQueue::userEvent(osg::Referenced* userEventData, double time)
+{
+    GUIEventAdapter* event = new GUIEventAdapter(*_accumulateEventState);
+    event->setEventType(GUIEventAdapter::USER);
+    event->setUserData(userEventData);
+    event->setTime(time);
+    
+    addEvent(event);
+}
+
+
