@@ -15,9 +15,21 @@
 
 using namespace osgGA;
 
+
+osg::ref_ptr<GUIEventAdapter>& GUIEventAdapter::getAccumulatedEventState()
+{
+    static osg::ref_ptr<GUIEventAdapter> s_eventState = new GUIEventAdapter;
+    return s_eventState;
+}
+
 GUIEventAdapter::GUIEventAdapter():
+    _handled(false),
     _eventType(NONE),
     _time(0.0),
+    _windowX(0),
+    _windowY(0),
+    _windowWidth(1280),
+    _windowHeight(1024),
     _key(0),
     _button(0),
     _Xmin(0.0),
@@ -36,10 +48,15 @@ GUIEventAdapter::GUIEventAdapter():
     _tabletPointerType(UNKNOWN)
 {}
 
-GUIEventAdapter::GUIEventAdapter(const GUIEventAdapter& rhs):
-    osg::Referenced(),
+GUIEventAdapter::GUIEventAdapter(const GUIEventAdapter& rhs,const osg::CopyOp& copyop):
+    osg::Object(rhs,copyop),
+    _handled(rhs._handled),
     _eventType(rhs._eventType),
     _time(rhs._time),
+    _windowX(rhs._windowX),
+    _windowY(rhs._windowY),
+    _windowWidth(rhs._windowWidth),
+    _windowHeight(rhs._windowHeight),
     _key(rhs._key),
     _button(rhs._button),
     _Xmin(rhs._Xmin),
@@ -62,7 +79,21 @@ GUIEventAdapter::~GUIEventAdapter()
 {
 }
 
-void GUIEventAdapter::setWindowSize(float Xmin, float Ymin, float Xmax, float Ymax)
+void GUIEventAdapter::setWindowRectangle(int x, int y, int width, int height, bool updateMouseRange)
+{
+    _windowX = x;
+    _windowY = y;
+    _windowWidth = width;
+    _windowHeight = height;
+    
+    if (updateMouseRange)
+    {
+        setInputRange(0, 0, width, height);
+    }
+    
+}
+
+void GUIEventAdapter::setInputRange(float Xmin, float Ymin, float Xmax, float Ymax)
 {
     _Xmin = Xmin;
     _Ymin = Ymin;
