@@ -60,7 +60,7 @@ NodeTrackerManipulator::~NodeTrackerManipulator()
 osg::NodePath NodeTrackerManipulator::getNodePath() const
 {
     osg::NodePath nodePath;
-    for(ObserveredNodePath::const_iterator itr = _trackNodePath.begin();
+    for(ObserverNodePath::const_iterator itr = _trackNodePath.begin();
         itr != _trackNodePath.end();
         ++itr)
     {
@@ -71,14 +71,14 @@ osg::NodePath NodeTrackerManipulator::getNodePath() const
 
 bool NodeTrackerManipulator::validateNodePath() const
 {
-    for(ObserveredNodePath::const_iterator itr = _trackNodePath.begin();
+    for(ObserverNodePath::const_iterator itr = _trackNodePath.begin();
         itr != _trackNodePath.begin();
         ++itr)
     {
         if (*itr==0) 
         {
             osg::notify(osg::NOTICE)<<"Warning: tracked node path has been invalidated by changes in the scene graph."<<std::endl;
-            const_cast<ObserveredNodePath&>(_trackNodePath).clear();
+            const_cast<ObserverNodePath&>(_trackNodePath).clear();
             return false;
         }
     }
@@ -135,10 +135,10 @@ void NodeTrackerManipulator::setTrackNode(osg::Node* node)
         osg::notify(osg::NOTICE)<<"NodeTrackerManipulator::setTrackNode(Node*): Unable to set tracked node due to empty parental path."<<std::endl;
     }
     
-    osg::notify(osg::NOTICE)<<"setTrackNode("<<node->getName()<<")"<<std::endl;
+    osg::notify(osg::INFO)<<"setTrackNode("<<node->getName()<<")"<<std::endl;
     for(unsigned int i=0; i<_trackNodePath.size(); ++i)
     {
-        osg::notify(osg::NOTICE)<<"  "<<_trackNodePath[i]->className()<<" '"<<_trackNodePath[i]->getName()<<"'"<<std::endl;
+        osg::notify(osg::INFO)<<"  "<<_trackNodePath[i]->className()<<" '"<<_trackNodePath[i]->getName()<<"'"<<std::endl;
     }
 
 }
@@ -212,6 +212,9 @@ bool NodeTrackerManipulator::handle(const GUIEventAdapter& ea,GUIActionAdapter&
             if (ea.getButtonMask()==0)
             {
 
+                double timeSinceLastRecordEvent = _ga_t0.valid() ? (ea.getTime() - _ga_t0->getTime()) : DBL_MAX;
+                if (timeSinceLastRecordEvent>0.02) flushMouseEventStack();
+
                 if (isMouseMoving())
                 {
                     if (calcMovement())
@@ -419,9 +422,18 @@ bool NodeTrackerManipulator::calcMovement()
     double dy = _ga_t0->getYnormalized()-_ga_t1->getYnormalized();
 
 
+    float distance = sqrtf(dx*dx + dy*dy);
+    // return if movement is too fast, indicating an error in event values or change in screen.
+    if (distance>0.5)
+    {
+        return false;
+    }
+    
     // return if there is no movement.
-    if (dx==0 && dy==0) return false;
-
+    if (distance==0.0f)
+    {
+        return false;
+    }
 
     osg::Vec3d nodeCenter;
     osg::Quat nodeRotation;
