@@ -162,9 +162,12 @@ void UFOManipulator::init(const GUIEventAdapter&, GUIActionAdapter&)
     _stop();
 }
 
-void UFOManipulator::home(const osgGA::GUIEventAdapter& ea, osgGA::GUIActionAdapter&) 
+void UFOManipulator::home(const osgGA::GUIEventAdapter& ea, osgGA::GUIActionAdapter& us) 
 {
     home(ea.getTime());
+    us.requestRedraw();
+    us.requestContinuousUpdate(false);
+
 }
 
 void UFOManipulator::home(double) 
@@ -189,6 +192,16 @@ void UFOManipulator::home(double)
 
 bool UFOManipulator::handle(const osgGA::GUIEventAdapter& ea,osgGA::GUIActionAdapter &aa)
 {
+    switch(ea.getEventType())
+    {
+        case(osgGA::GUIEventAdapter::FRAME):
+            _frame(ea,aa);
+            return false;
+        default:
+            break;
+    }
+
+    if (ea.getHandled()) return false;
 
     switch(ea.getEventType())
     {
