@@ -89,8 +89,12 @@ void osgParticle::ParticleSystem::update(double dt)
     dirtyBound();
 }
 
-void osgParticle::ParticleSystem::drawImplementation(osg::State& state) const
+void osgParticle::ParticleSystem::drawImplementation(osg::RenderInfo& renderInfo) const
 {
+    osg::State& state = *renderInfo.getState();
+
+    OpenThreads::ScopedReadLock lock(_readWriteMutex);
+
     // update the frame count, so other objects can detect when
     // this particle system is culled
     _last_frame = state.getFrameStamp()->getFrameNumber();
