@@ -10,10 +10,12 @@ using namespace flt;
 
 
 Document::Document() :
+    _replaceClampWithClampToEdge(false),
     _preserveFace(false),
     _preserveObject(false),
     _defaultDOFAnimationState(false),
     _useTextureAlphaForTransparancyBinning(true),
+    _useBillboardCenter(false),
     _doUnitsConversion(true),
     _desiredUnits(METERS),
     _done(false),
@@ -24,7 +26,9 @@ Document::Document() :
     _colorPoolParent(false),
     _texturePoolParent(false),
     _materialPoolParent(false),
+    _lightSourcePoolParent(false),
     _lightPointAppearancePoolParent(false),
+    _lightPointAnimationPoolParent(false),
     _shaderPoolParent(false)
 {
 }
@@ -38,12 +42,14 @@ Document::~Document()
 void Document::pushLevel()
 {
     _levelStack.push_back(_currentPrimaryRecord.get());
+    _levelStack.back()->pushLevel(*this);
     _level++;
 }
 
 
 void Document::popLevel()
 {
+    _levelStack.back()->popLevel(*this);
     _levelStack.pop_back();
 
     if (!_levelStack.empty())
