@@ -125,23 +125,39 @@ class Document
         bool getColorPoolParent() const { return _colorPoolParent; }
 
         void setTexturePool(TexturePool* tp, bool parent=false) { _texturePool = tp; _texturePoolParent=parent; }
+        TexturePool* getTexturePool() { return _texturePool.get(); }
         TexturePool* getOrCreateTexturePool();
         bool getTexturePoolParent() const { return _texturePoolParent; }
 
         void setMaterialPool(MaterialPool* mp, bool parent=false) { _materialPool = mp; _materialPoolParent=parent; }
+        MaterialPool* getMaterialPool() { return _materialPool.get(); }
         MaterialPool* getOrCreateMaterialPool();
         bool getMaterialPoolParent() const { return _materialPoolParent; }
 
+        void setLightSourcePool(LightSourcePool* lsp, bool parent=false) { _lightSourcePool = lsp; _lightSourcePoolParent=parent; }
+        LightSourcePool* getLightSourcePool() { return _lightSourcePool.get(); }
+        LightSourcePool* getOrCreateLightSourcePool();
+        bool getLightSourcePoolParent() const { return _lightSourcePoolParent; }
+
         void setLightPointAppearancePool(LightPointAppearancePool* lpap, bool parent=false) { _lightPointAppearancePool = lpap; _lightPointAppearancePoolParent=parent; }
+        LightPointAppearancePool* getLightPointAppearancePool() { return _lightPointAppearancePool.get(); }
         LightPointAppearancePool* getOrCreateLightPointAppearancePool();
         bool getLightPointAppearancePoolParent() const { return _lightPointAppearancePoolParent; }
 
+        void setLightPointAnimationPool(LightPointAnimationPool* lpap, bool parent=false) { _lightPointAnimationPool = lpap; _lightPointAnimationPoolParent=parent; }
+        LightPointAnimationPool* getLightPointAnimationPool() { return _lightPointAnimationPool.get(); }
+        LightPointAnimationPool* getOrCreateLightPointAnimationPool();
+        bool getLightPointAnimationPoolParent() const { return _lightPointAnimationPoolParent; }
+
         void setShaderPool(ShaderPool* cp, bool parent=false) { _shaderPool = cp; _shaderPoolParent=parent; }
+        ShaderPool* getShaderPool() { return _shaderPool.get(); }
         ShaderPool* getOrCreateShaderPool();
         bool getShaderPoolParent() const { return _shaderPoolParent; }
 
 
         // Options
+        void setReplaceClampWithClampToEdge(bool flag) { _replaceClampWithClampToEdge = flag; }
+        bool getReplaceClampWithClampToEdge() const { return _replaceClampWithClampToEdge; }
         void setPreserveFace(bool flag) { _preserveFace = flag; }
         bool getPreserveFace() const { return _preserveFace; }
         void setPreserveObject(bool flag) { _preserveObject = flag; }
@@ -150,6 +166,8 @@ class Document
         bool getDefaultDOFAnimationState() const { return _defaultDOFAnimationState; }
         void setUseTextureAlphaForTransparancyBinning(bool flag) { _useTextureAlphaForTransparancyBinning=flag; }
         bool getUseTextureAlphaForTransparancyBinning() const { return _useTextureAlphaForTransparancyBinning; }
+        void setUseBillboardCenter(bool flag) { _useBillboardCenter=flag; }
+        bool getUseBillboardCenter() const { return _useBillboardCenter; }
         void setDoUnitsConversion(bool flag) { _doUnitsConversion=flag; }
         bool getDoUnitsConversion() const { return _doUnitsConversion; }
         void setDesiredUnits(CoordUnits units ) { _desiredUnits=units; }
@@ -162,10 +180,12 @@ class Document
 
         // Options
         osg::ref_ptr<const osgDB::ReaderWriter::Options> _options;
+        bool                        _replaceClampWithClampToEdge;
         bool                        _preserveFace;
         bool                        _preserveObject;
         bool                        _defaultDOFAnimationState;
         bool                        _useTextureAlphaForTransparancyBinning;
+        bool                        _useBillboardCenter;
         bool                        _doUnitsConversion;
         CoordUnits                  _desiredUnits;
         
@@ -185,12 +205,16 @@ class Document
         osg::ref_ptr<ColorPool> _colorPool;
         osg::ref_ptr<TexturePool> _texturePool;
         osg::ref_ptr<MaterialPool> _materialPool;
+        osg::ref_ptr<LightSourcePool> _lightSourcePool;
         osg::ref_ptr<LightPointAppearancePool> _lightPointAppearancePool;
+        osg::ref_ptr<LightPointAnimationPool> _lightPointAnimationPool;
         osg::ref_ptr<ShaderPool> _shaderPool;
         bool _colorPoolParent;
         bool _texturePoolParent;
         bool _materialPoolParent;
+        bool _lightSourcePoolParent;
         bool _lightPointAppearancePoolParent;
+        bool _lightPointAnimationPoolParent;
         bool _shaderPoolParent;
 
         osg::ref_ptr<PrimaryRecord> _currentPrimaryRecord;
@@ -220,6 +244,14 @@ inline MaterialPool* Document::getOrCreateMaterialPool()
 }
 
 
+inline LightSourcePool* Document::getOrCreateLightSourcePool()
+{
+    if (!_lightSourcePool.valid())
+        _lightSourcePool = new LightSourcePool;
+    return _lightSourcePool.get();
+}
+
+
 inline LightPointAppearancePool* Document::getOrCreateLightPointAppearancePool()
 {
     if (!_lightPointAppearancePool.valid())
@@ -227,6 +259,13 @@ inline LightPointAppearancePool* Document::getOrCreateLightPointAppearancePool()
     return _lightPointAppearancePool.get();
 }
 
+inline LightPointAnimationPool* Document::getOrCreateLightPointAnimationPool()
+{
+    if (!_lightPointAnimationPool.valid())
+        _lightPointAnimationPool = new LightPointAnimationPool;
+    return _lightPointAnimationPool.get();
+}
+
 
 inline ShaderPool* Document::getOrCreateShaderPool()
 {
