@@ -5,6 +5,7 @@
 //
 
 #include <assert.h>
+#include <osg/Light>
 #include <osg/Texture2D>
 #include <osg/TexEnv>
 #include <osg/BlendFunc>
@@ -246,51 +247,41 @@ protected:
 
     virtual ~TexturePalette() {}
 
-    osg::Texture2D::WrapMode convertWrapMode( int32 wrap )
+    osg::Texture2D::WrapMode convertWrapMode(int32 attrWrapMode, const Document& document) const
     {
-        switch( wrap )
+        osg::Texture2D::WrapMode osgWrapMode = osg::Texture2D::REPEAT;
+        switch (attrWrapMode)
         {
         case AttrData::WRAP_CLAMP:
-            return osg::Texture2D::CLAMP;
+            if (document.getReplaceClampWithClampToEdge())
+                osgWrapMode = osg::Texture2D::CLAMP_TO_EDGE;
+            else
+                osgWrapMode = osg::Texture2D::CLAMP;
             break;
         case AttrData::WRAP_MIRRORED_REPEAT:
-            return osg::Texture2D::MIRROR;
+            osgWrapMode = osg::Texture2D::MIRROR;
             break;
-        default:
         case AttrData::WRAP_REPEAT:
-            return osg::Texture2D::REPEAT;
+            osgWrapMode = osg::Texture2D::REPEAT;
             break;
         }
-        return osg::Texture2D::REPEAT;
+
+        return osgWrapMode;
     }
 
-    virtual void readRecord(RecordInputStream& in, Document& document)
+    osg::StateSet* readTexture(const std::string& filename, const Document& document) const
     {
-        if (document.getTexturePoolParent())
-            // Using parent's texture pool -- ignore this record.
-            return;
+        osg::Image* image = osgDB::readImageFile(filename,document.getOptions());
+        if (!image) return NULL;
 
-        int maxLength = (document.version() < VERSION_14) ? 80 : 200;
-        std::string filename = in.readString(maxLength);
-        int32 index = in.readInt32(-1);
-        /*int32 x =*/ in.readInt32();
-        /*int32 y =*/ in.readInt32();
-
-        osg::ref_ptr<osg::Image> image = osgDB::readImageFile(filename,document.getOptions());
-        if (!image.valid())
-        {
-            osg::notify(osg::WARN) << "Can't find texture (" << index << ") " << filename << std::endl;
-            return;
-        }
-
-        // Create stateset
-        osg::ref_ptr<osg::StateSet> stateset = new osg::StateSet;
+        // Create stateset to hold texture and attributes.
+        osg::StateSet* stateset = new osg::StateSet;
 
         osg::Texture2D* texture = new osg::Texture2D;
         texture->setWrap(osg::Texture2D::WRAP_S,osg::Texture2D::REPEAT);
         texture->setWrap(osg::Texture2D::WRAP_T,osg::Texture2D::REPEAT);
         texture->setResizeNonPowerOfTwoHint(true);
-        texture->setImage(image.get());
+        texture->setImage(image);
         stateset->setTextureAttributeAndModes(0, texture, osg::StateAttribute::ON);
 
         // Read attribute file
@@ -299,10 +290,10 @@ protected:
         if (attr.valid())
         {
             // Wrap mode
-            osg::Texture2D::WrapMode wrap_s = convertWrapMode( attr->wrapMode_u );
+            osg::Texture2D::WrapMode wrap_s = convertWrapMode(attr->wrapMode_u,document);
             texture->setWrap(osg::Texture2D::WRAP_S,wrap_s);
 
-            osg::Texture2D::WrapMode wrap_t = convertWrapMode( attr->wrapMode_v );
+            osg::Texture2D::WrapMode wrap_t = convertWrapMode(attr->wrapMode_v,document);
             texture->setWrap(osg::Texture2D::WRAP_T,wrap_t);
 
             // Min filter
@@ -357,6 +348,42 @@ protected:
                 break;
             }
 
+            // Internal mode
+            switch(attr->intFormat)
+            {
+            case AttrData::INTERNAL_FORMAT_TX_I_12A_4:
+                texture->setInternalFormat(GL_LUMINANCE12_ALPHA4);
+                break;
+            case AttrData::INTERNAL_FORMAT_TX_IA_8:
+                texture->setInternalFormat(GL_LUMINANCE_ALPHA);
+                break;
+            case AttrData::INTERNAL_FORMAT_TX_RGB_5:
+                texture->setInternalFormat(GL_RGB5);
+                break;
+            case AttrData::INTERNAL_FORMAT_TX_RGBA_4:
+                texture->setInternalFormat(GL_RGBA4);
+                break;
+            case AttrData::INTERNAL_FORMAT_TX_IA_12:
+                texture->setInternalFormat(GL_LUMINANCE12_ALPHA12);
+                break;
+            case AttrData::INTERNAL_FORMAT_TX_RGBA_8:
+                texture->setInternalFormat(GL_RGBA8);
+                break;
+            case AttrData::INTERNAL_FORMAT_TX_RGBA_12:
+                texture->setInternalFormat(GL_RGBA12);
+                break;
+            case AttrData::INTERNAL_FORMAT_TX_I_16:
+                texture->setInternalFormat(GL_INTENSITY16);
+                break;
+            case AttrData::INTERNAL_FORMAT_TX_RGB_12:
+                texture->setInternalFormat(GL_RGB12);
+                break;
+            case AttrData::INTERNAL_FORMAT_DEFAULT:
+            default:
+                // Do nothing, just use the image data format
+                break;
+            }
+
             osg::TexEnv* texenv = new osg::TexEnv;
             switch (attr->texEnvMode)
             {
@@ -379,8 +406,44 @@ protected:
             stateset->setTextureAttribute(0, texenv);
         }
 
+        return stateset;
+    }
+
+    virtual void readRecord(RecordInputStream& in, Document& document)
+    {
+        if (document.getTexturePoolParent())
+            // Using parent's texture pool -- ignore this record.
+            return;
+
+        int maxLength = (document.version() < VERSION_14) ? 80 : 200;
+        std::string filename = in.readString(maxLength);
+        int32 index = in.readInt32(-1);
+        /*int32 x =*/ in.readInt32();
+        /*int32 y =*/ in.readInt32();
+
+        // Need full path for unique key in local texture cache.
+        std::string pathname = osgDB::findDataFile(filename,document.getOptions());
+        if (pathname.empty())
+        {
+            osg::notify(osg::WARN) << "Can't find texture (" << index << ") " << filename << std::endl;
+            return;
+        }
+
+        // Is texture in local cache?
+        osg::StateSet* stateset = flt::Registry::instance()->getTextureFromLocalCache(pathname);
+
+        // Read file if not in cache.
+        if (!stateset)
+        {
+            stateset = readTexture(pathname,document);
+
+            // Add to texture cache.
+            flt::Registry::instance()->addTextureToLocalCache(pathname,stateset);
+        }
+
+        // Add to texture pool.
         TexturePool* tp = document.getOrCreateTexturePool();
-        (*tp)[index] = stateset.get();
+        (*tp)[index] = stateset;
     }
 };
 
@@ -449,11 +512,72 @@ public:
 
     META_Record(LightSourcePalette)
 
+    enum LightType 
+    {
+        INFINITE_LIGHT = 0,
+        LOCAL_LIGHT = 1,
+        SPOT_LIGHT = 2
+    };
+
 protected:
 
     virtual ~LightSourcePalette() {}
 
-    virtual void readRecord(RecordInputStream& /*in*/, Document& /*document*/) {}
+    virtual void readRecord(RecordInputStream& in, Document& document)
+    {
+        if (document.getLightSourcePoolParent())
+            // Using parent's texture pool -- ignore this record.
+            return;
+
+        int32 index = in.readInt32(-1);
+        in.forward(2*4);
+        std::string name = in.readString(20);
+        in.forward(4);
+        osg::Vec4f ambient = in.readVec4f();
+        osg::Vec4f diffuse = in.readVec4f();
+        osg::Vec4f specular = in.readVec4f();
+        int32 type = in.readInt32();
+        in.forward(4*10);
+        float32 spotExponent = in.readFloat32();
+        float32 spotCutoff = in.readFloat32();
+        /*float32 yaw =*/ in.readFloat32();
+        /*float32 pitch =*/ in.readFloat32();
+        float32 constantAttenuation = in.readFloat32();
+        float32 linearAttenuation = in.readFloat32();
+        float32 quadraticAttenuation = in.readFloat32();
+        /*int32 active =*/ in.readInt32();
+
+        osg::ref_ptr<osg::Light> light = new osg::Light;
+        light->setAmbient(ambient);
+        light->setDiffuse(diffuse);
+        light->setSpecular(specular);
+
+        switch (type)
+        {
+        case INFINITE_LIGHT:
+            light->setPosition(osg::Vec4(0.0f,0.0f,1.0f,0.0f));
+            break;
+        case LOCAL_LIGHT:
+            light->setPosition(osg::Vec4(0.0f,0.0f,0.0f,1.0f));
+            light->setConstantAttenuation(constantAttenuation);
+            light->setLinearAttenuation(linearAttenuation);
+            light->setQuadraticAttenuation(quadraticAttenuation);
+            break;
+        case SPOT_LIGHT:
+            light->setPosition(osg::Vec4(0.0f,0.0f,0.0f,1.0f));
+            light->setDirection(osg::Vec3(0.0f,1.0f,0.0f));
+            light->setConstantAttenuation(constantAttenuation);
+            light->setLinearAttenuation(linearAttenuation);
+            light->setQuadraticAttenuation(quadraticAttenuation);
+            light->setSpotExponent(spotExponent);
+            light->setSpotCutoff(spotCutoff);
+            break;
+        }
+
+        // Add to pool
+        LightSourcePool* pool = document.getOrCreateLightSourcePool();
+        (*pool)[index] = light.get();
+    }
 };
 
 RegisterRecordProxy<LightSourcePalette> g_LightSourcePalette(LIGHT_SOURCE_PALETTE_OP);
@@ -548,28 +672,46 @@ protected:
 
     virtual ~LightPointAnimationPalette() {}
 
-    virtual void readRecord(RecordInputStream& in, Document& /*document*/)
+    virtual void readRecord(RecordInputStream& in, Document& document)
     {
+         if (document.getLightPointAnimationPoolParent())
+            // Using parent's light point animation pool -- ignore this record.
+            return;
+
+        osg::ref_ptr<LPAnimation> animation = new LPAnimation;
+
         in.forward(4);
-        std::string name = in.readString(256);
-        /*int32 index =*/ in.readInt32(-1);
-        /*float32 animationPeriod =*/ in.readFloat32();
-        /*float32 animationPhaseDelay =*/ in.readFloat32();
-        /*float32 animationEnabledPeriod =*/ in.readFloat32();
-        /*osg::Vec3f axisOfRotation =*/ in.readVec3f();
-        /*uint32 flags =*/ in.readUInt32();
-        /*int32 animationType =*/ in.readInt32();
-        /*int32 morseCodeTiming =*/ in.readInt32();
-        /*int32 wordRate =*/ in.readInt32();
-        /*int32 characterRate =*/ in.readInt32();
-        std::string morseCodeString = in.readString(1024);
+        animation->name = in.readString(256);
+        animation->index = in.readInt32(-1);
+        // Rotating or strobe
+        animation->animationPeriod = in.readFloat32();
+        animation->animationPhaseDelay = in.readFloat32();
+        animation->animationEnabledPeriod = in.readFloat32();
+        animation->axisOfRotation = in.readVec3f();
+        animation->flags = in.readUInt32();
+        animation->animationType = in.readInt32();
+
+        // Morse code
+        animation->morseCodeTiming = in.readInt32();
+        animation->wordRate = in.readInt32();
+        animation->characterRate = in.readInt32();
+        animation->morseCodeString = in.readString(1024);
+
+        // Flashing sequence
         int32 numberOfSequences = in.readInt32();
         for (int n=0; n<numberOfSequences; ++n)
         {
-            /*uint32 sequenceState =*/ in.readUInt32();
-            /*float32 sequenceDuration =*/ in.readFloat32();
-            /*osg::Vec4f sequenceColor =*/ in.readColor32();
+            LPAnimation::Pulse pulse;
+            pulse.state = in.readUInt32();
+            pulse.duration = in.readFloat32();
+            pulse.color = in.readColor32();
+
+            animation->sequence.push_back(pulse);
         }
+
+        // Add to pool
+        LightPointAnimationPool* lpaPool = document.getOrCreateLightPointAnimationPool();
+        (*lpaPool)[animation->index] = animation.get();
     }
 };
 
@@ -674,12 +816,13 @@ protected:
             osg::Program* program = new osg::Program;
             program->setName(name);
 
+            // Read vertex programs
             int idx;
             for( idx=0; idx<vertexProgramFileCount; idx++)
             {
                 std::string vertexProgramFilename = in.readString(1024);
 
-                std::string vertexProgramFilePath = osgDB::findDataFile(vertexProgramFilename);
+                std::string vertexProgramFilePath = osgDB::findDataFile(vertexProgramFilename,document.getOptions());
                 if (!vertexProgramFilePath.empty())
                 {
                     osg::Shader* vertexShader = osg::Shader::readShaderFile(osg::Shader::VERTEX, vertexProgramFilePath);
@@ -687,11 +830,13 @@ protected:
                         program->addShader( vertexShader );
                 }
             }
+
+            // Read fragment programs
             for( idx=0; idx<fragmentProgramFileCount; idx++)
             {
                 std::string fragmentProgramFilename = in.readString(1024);
 
-                std::string fragmentProgramFilePath = osgDB::findDataFile(fragmentProgramFilename);
+                std::string fragmentProgramFilePath = osgDB::findDataFile(fragmentProgramFilename,document.getOptions());
                 if (!fragmentProgramFilePath.empty())
                 {
                     osg::Shader* fragmentShader = osg::Shader::readShaderFile(osg::Shader::FRAGMENT, fragmentProgramFilePath);
@@ -700,6 +845,7 @@ protected:
                 }
             }
 
+            // Add to shader pool
             ShaderPool* shaderPool = document.getOrCreateShaderPool();
             (*shaderPool)[index] = program;
         }
