@@ -12,6 +12,7 @@
 #include <osg/Sequence>
 #include <osg/LOD>
 #include <osg/ProxyNode>
+#include <osg/LightSource>
 #include <osgDB/FileUtils>
 #include <osgSim/DOFTransform>
 #include <osgSim/MultiSwitch>
@@ -118,6 +119,21 @@ protected:
         document.setHeaderNode(_header.get());
     }
 
+    virtual void popLevel(Document& document)
+    {
+        if (_header.valid())
+        {
+            // Preset sampler uniforms.
+            ShaderPool* sp = document.getShaderPool();
+            if (sp && !sp->empty())
+            {
+                _header->getOrCreateStateSet()->addUniform( new osg::Uniform("TextureUnit0", 0) );
+                _header->getOrCreateStateSet()->addUniform( new osg::Uniform("TextureUnit1", 1) );
+                _header->getOrCreateStateSet()->addUniform( new osg::Uniform("TextureUnit2", 2) );
+                _header->getOrCreateStateSet()->addUniform( new osg::Uniform("TextureUnit3", 3) );
+            }
+        }
+    }
 };
 
 RegisterRecordProxy<Header> g_Header(HEADER_OP);
@@ -135,14 +151,22 @@ class Group : public PrimaryRecord
     static const unsigned int BACKWARD_ANIM    = 0x80000000u >> 6;
 
     osg::ref_ptr<osg::Group> _group;
+    uint32 _flags;
     bool _forwardAnim;
     bool _backwardAnim;
+    int32 _loopCount;
+    float32 _loopDuration;
+    float32 _lastFrameDuration;
 
 public:
 
     Group():
+        _flags(0),
         _forwardAnim(false),
-        _backwardAnim(false)
+        _backwardAnim(false),
+        _loopCount(0),
+        _loopDuration(0),
+        _lastFrameDuration(0)
     {}
 
     META_Record(Group)
@@ -157,8 +181,6 @@ public:
 
 protected:
 
-    virtual ~Group() {}
-
     void readRecord(RecordInputStream& in, Document& document)
     {
         std::string id = in.readString(8);
@@ -166,51 +188,33 @@ protected:
 
         /*int16 relativePriority =*/ in.readInt16();
         in.forward(2);
-        uint32 flags = in.readUInt32();
+        _flags = in.readUInt32(0);
         /*uint16 specialId0 =*/ in.readUInt16();
         /*uint16 specialId1 =*/ in.readUInt16();
         /*uint16 significance =*/ in.readUInt16();
         /*int8 layer =*/ in.readInt8();
         in.forward(5);
-        /*uint32 loopCount =*/ in.readUInt32();
-        /*float32 loopDuration =*/ in.readFloat32();
-        /*float32 lastFrameDuration =*/ in.readFloat32();
+        // version >= VERSION_15_8
+        _loopCount = in.readInt32(0);
+        _loopDuration = in.readFloat32(0.0f);
+        _lastFrameDuration = in.readFloat32(0.0f);
 
         // Check for forward animation (sequence)
-        _forwardAnim = (flags & FORWARD_ANIM) != 0;
+        _forwardAnim = (_flags & FORWARD_ANIM) != 0;
 
         // For versions prior to 15.8, the swing bit can be set independently
         // of the animation bit.  This implies forward animation (with swing)
-        if ((document.version() < VERSION_15_8) && (flags & SWING_ANIM))
+        if ((document.version() < VERSION_15_8) && (_flags & SWING_ANIM))
             _forwardAnim = true;
         
         // OpenFlight 15.8 adds backwards animations
         _backwardAnim = ( (document.version() >= VERSION_15_8) &&
-            ((flags & BACKWARD_ANIM) != 0) );
+            ((_flags & BACKWARD_ANIM) != 0) );
 
         if (_forwardAnim || _backwardAnim)
-        {
-            osg::ref_ptr<osg::Sequence> sequence = new osg::Sequence;
-
-            // Regardless of forwards or backwards, animation could have swing bit set
-            const osg::Sequence::LoopMode loopMode = ((flags & SWING_ANIM) == 0) ?
-                osg::Sequence::LOOP : osg::Sequence::SWING;
-
-            if (_forwardAnim)
-                sequence->setInterval(loopMode, 0, -1);
-            else 
-                sequence->setInterval(loopMode, -1, 0);
-
-            float speed=0.1f;
-            sequence->setDuration(speed);
-            sequence->setMode(osg::Sequence::START);
-
-            _group = sequence.get();
-        }
+            _group = new osg::Sequence;
         else
-        {
             _group = new osg::Group;
-        }
 
         _group->setName(id);
 
@@ -219,6 +223,49 @@ protected:
             _parent->addChild(*_group);
     }
 
+    virtual void popLevel(Document& document)
+    {
+        // Children are added!
+        osg::Sequence* sequence = dynamic_cast<osg::Sequence*>(_group.get());
+        if (sequence && sequence->getNumChildren() > 0)
+        {
+            // Regardless of forwards or backwards, animation could have swing bit set.
+            osg::Sequence::LoopMode loopMode = ((_flags & SWING_ANIM) == 0) ?
+                osg::Sequence::LOOP : osg::Sequence::SWING;
+
+            if (_forwardAnim)
+                sequence->setInterval(loopMode, 0, -1);
+            else 
+                sequence->setInterval(loopMode, -1, 0);
+
+            // Loop timing available from version 15.8.
+            if (document.version() >= VERSION_15_8)
+            {
+                // Set frame duration.
+                float frameDuration = _loopDuration / float(sequence->getNumChildren());
+                for (unsigned int i=0; i < sequence->getNumChildren(); i++)
+                    sequence->setTime(i, frameDuration);
+
+                // Set number of repetitions.
+                if (_loopCount > 0)
+                    sequence->setDuration(1.0f, _loopCount);
+                else
+                    sequence->setDuration(1.0f);        // Run continuously
+            }
+            else // No timing available.
+            {
+                // Set frame duration
+                float frameDuration = 0.1f;     // 10Hz
+                for (unsigned int i=0; i < sequence->getNumChildren(); i++)
+                    sequence->setTime(i, frameDuration);
+
+                // Run continuously
+                sequence->setDuration(1.0f);
+            }
+
+            sequence->setMode(osg::Sequence::START);
+        }
+    }
 };
 
 RegisterRecordProxy<Group> g_Group(GROUP_OP);
@@ -611,10 +658,7 @@ protected:
         // Set parent pools as user data
         if (document.version() >= VERSION_14_2)
         {
-            in.forward(2);
-
-            if (document.version() >= VERSION_15_1)
-                in.forward(2);
+            in.forward(4);
 
             uint32 mask = in.readUInt32(~0);
 
@@ -623,18 +667,27 @@ protected:
             if (document.version() == 1541)
                 mask = ~0;
 
-            // Only override light points and shader palettes if 
-            //   we are the correct version.
-            bool parentLightPoints( (document.version() >= VERSION_15_8) &&
-                ((mask & LIGHT_POINT_PALETTE_OVERRIDE) == 0) );
-            bool parentShaders( (document.version() >= VERSION_16_0) &&
-                ((mask & SHADER_PALETTE_OVERRIDE) == 0) );
-            _external->setUserData(new ParentPools(
-                ((mask & COLOR_PALETTE_OVERRIDE) ? NULL : document.getColorPool()),
-                ((mask & MATERIAL_PALETTE_OVERRIDE) ? NULL : document.getOrCreateMaterialPool()),
-                ((mask & TEXTURE_PALETTE_OVERRIDE) ? NULL : document.getOrCreateTexturePool()),
-                ((!parentLightPoints) ? NULL : document.getOrCreateLightPointAppearancePool()),
-                ((!parentShaders) ? NULL : document.getOrCreateShaderPool()) ));
+            ParentPools* parentPools = new ParentPools;
+
+            if ((mask & COLOR_PALETTE_OVERRIDE) == 0)
+                parentPools->setColorPool(document.getColorPool());
+
+            if ((mask & MATERIAL_PALETTE_OVERRIDE) == 0)
+                parentPools->setMaterialPool(document.getMaterialPool());
+
+            if ((mask & TEXTURE_PALETTE_OVERRIDE) == 0)
+                parentPools->setTexturePool(document.getTexturePool());
+
+            if ((document.version() >= VERSION_15_1) && ((mask & LIGHT_SOURCE_PALETTE_OVERRIDE) == 0))
+                parentPools->setLightSourcePool(document.getLightSourcePool());
+
+            if ((document.version() >= VERSION_15_8) && ((mask & LIGHT_POINT_PALETTE_OVERRIDE) == 0))
+                parentPools->setLPAppearancePool(document.getLightPointAppearancePool());
+
+            if ((document.version() >= VERSION_16_0) && ((mask & SHADER_PALETTE_OVERRIDE) == 0))
+                parentPools->setShaderPool(document.getShaderPool());
+
+            _external->setUserData(parentPools);
         }
 
         // Add this implementation to parent implementation.
@@ -867,6 +920,91 @@ protected:
 
 RegisterRecordProxy<Object> g_Object(OBJECT_OP);
 
+
+/** LightSource
+*/
+class LightSource : public PrimaryRecord
+{
+    static const unsigned int ENABLED = 0x80000000u >> 0;
+    static const unsigned int GLOBAL  = 0x80000000u >> 1;
+    static const unsigned int EXPORT  = 0x80000000u >> 3;
+
+    osg::ref_ptr<osg::LightSource> _lightSource;
+
+public:
+
+    LightSource() {}
+
+    META_Record(LightSource)
+
+    META_setID(_lightSource)
+    META_setComment(_lightSource)
+    META_setMatrix(_lightSource)
+
+protected:
+
+    virtual ~LightSource() {}
+
+    virtual void readRecord(RecordInputStream& in, Document& document)
+    {
+        std::string id = in.readString(8);
+        in.forward(4);
+        int32 index = in.readInt32();
+        in.forward(4);
+        uint32 flags = in.readUInt32();
+        in.forward(4);
+        osg::Vec3d pos = in.readVec3d();
+        float32 yaw = in.readFloat32();
+        float32 pitch = in.readFloat32();
+
+        _lightSource = new osg::LightSource;  
+        _lightSource->setName(id);
+
+        LightSourcePool* pool = document.getOrCreateLightSourcePool();
+        osg::Light* lightFromPool = pool->get(index);
+        if (lightFromPool)
+        {
+            // Make a clone of light in pool.
+            osg::Light* light = new osg::Light(*lightFromPool);
+
+            // TODO: Find a better way to set light number.
+            light->setLightNum(1);
+
+            // Position
+            float w = lightFromPool->getPosition().w();
+            if (w > 0.0) // positional light?
+                light->setPosition(osg::Vec4(pos,w));
+
+            // Apply yaw and pitch for infinite and spot light.
+            if ((w==0.0) || (light->getSpotCutoff()<180.0))
+            {
+                // TODO: What direction is zero yaw and pitch?
+                osg::Quat rotation = ( osg::Quat(osg::inDegrees(double(yaw)),osg::Vec3(0.0,0.0,1.0)) *
+                                       osg::Quat(osg::inDegrees(double(pitch)),osg::Vec3(1.0,0.0,0.0)) );
+                light->setDirection(rotation*osg::Vec3(0.0,1.0,0.0));
+            }
+
+            _lightSource->setLight(light);
+            _lightSource->setLocalStateSetModes((flags & ENABLED) ? osg::StateAttribute::ON : osg::StateAttribute::OFF); 
+
+            // Global light.
+            if (flags & GLOBAL)
+            {
+                // Apply light source to header node.
+                osg::Node* header = document.getHeaderNode();
+                if (header)
+                    _lightSource->setStateSetModes(*(header->getOrCreateStateSet()),osg::StateAttribute::ON);
+            }
+        }
+
+        if (_parent.valid())
+            _parent->addChild(*_lightSource);
+    }
+};
+
+RegisterRecordProxy<LightSource> g_LightSource(LIGHT_SOURCE_OP);
+
+
 } // end namespace
 
 
