@@ -11,14 +11,14 @@
 #include <osgDB/FileUtils>
 #include <osgDB/Registry>
 #include <osgDB/ReadFile>
-#include <osgDB/ReentrantMutex>
+#include <OpenThreads/ReentrantMutex>
 #include <osgUtil/Optimizer>
 
 #include "Registry.h"
 #include "Document.h"
 #include "RecordInputStream.h"
 
-#define SERIALIZER() OpenThreads::ScopedLock<osgDB::ReentrantMutex> lock(_serializerMutex)  
+#define SERIALIZER() OpenThreads::ScopedLock<OpenThreads::ReentrantMutex> lock(_serializerMutex)  
 
 using namespace flt;
 using namespace osg;
@@ -28,15 +28,19 @@ using namespace osgDB;
 class ReadExternalsVisitor : public osg::NodeVisitor
 {
     osg::ref_ptr<ReaderWriter::Options> _options;
+    bool _cloneExternalReferences;
 
 public:
 
     ReadExternalsVisitor(ReaderWriter::Options* options) :
         osg::NodeVisitor(osg::NodeVisitor::TRAVERSE_ALL_CHILDREN),
-        _options(options)
+        _options(options),
+        _cloneExternalReferences(false)
     {
+        if (options)
+            _cloneExternalReferences = (options->getOptionString().find("cloneExternalReferences")!=std::string::npos);
     }
-        
+
     virtual ~ReadExternalsVisitor() {}
 
     virtual void apply(ProxyNode& node)
@@ -50,14 +54,20 @@ public:
             std::string filename = node.getFileName(pos);
 
             // read external
-            osg::Node* external = osgDB::readNodeFile(filename,_options.get());
-            if (external)
-                node.addChild(external);
+            osg::ref_ptr<osg::Node> external = osgDB::readNodeFile(filename,_options.get());
+            if (external.valid())
+            {
+                if (_cloneExternalReferences)
+                    external = dynamic_cast<osg::Node*>(external->clone(osg::CopyOp(osg::CopyOp::DEEP_COPY_NODES)));
+
+                node.addChild(external.get());
+            }
         }
     }
 };
 
 
+
 class FLTReaderWriter : public ReaderWriter
 {
     public:
@@ -65,7 +75,7 @@ class FLTReaderWriter : public ReaderWriter
 
         virtual bool acceptsExtension(const std::string& extension) const
         {
-            return equalCaseInsensitive(extension,"flt");
+            return equalCaseInsensitive(extension,"flt") || extension.empty();
         }
 
         virtual ReadResult readObject(const std::string& file, const Options* options) const
@@ -85,7 +95,7 @@ class FLTReaderWriter : public ReaderWriter
 
             // in local cache?
             {
-                osg::Node* node = flt::Registry::instance()->getFromLocalCache(fileName);
+                osg::Node* node = flt::Registry::instance()->getExternalFromLocalCache(fileName);
                 if (node)
                     return ReadResult(node, ReaderWriter::ReadResult::FILE_LOADED_FROM_CACHE);
             }
@@ -112,7 +122,7 @@ class FLTReaderWriter : public ReaderWriter
             if (rr.success())
             {
                 // add to local cache.
-                flt::Registry::instance()->addToLocalCache(fileName,rr.getNode());
+                flt::Registry::instance()->addExternalToLocalCache(fileName,rr.getNode());
         
                 bool keepExternalReferences = false;
                 if (options)
@@ -159,6 +169,9 @@ class FLTReaderWriter : public ReaderWriter
             {
                 const char readerMsg[] = "flt reader option: ";
                 
+                document.setReplaceClampWithClampToEdge((options->getOptionString().find("clampToEdge")!=std::string::npos));
+                osg::notify(osg::DEBUG_INFO) << readerMsg << "clampToEdge=" << document.getReplaceClampWithClampToEdge() << std::endl;
+
                 document.setKeepExternalReferences((options->getOptionString().find("keepExternalReferences")!=std::string::npos));
                 osg::notify(osg::DEBUG_INFO) << readerMsg << "keepExternalReferences=" << document.getKeepExternalReferences() << std::endl;
 
@@ -171,11 +184,14 @@ class FLTReaderWriter : public ReaderWriter
                 document.setDefaultDOFAnimationState((options->getOptionString().find("dofAnimation")!=std::string::npos));
                 osg::notify(osg::DEBUG_INFO) << readerMsg << "dofAnimation=" << document.getDefaultDOFAnimationState() << std::endl;
 
+                document.setUseBillboardCenter((options->getOptionString().find("billboardCenter")!=std::string::npos));
+                osg::notify(osg::DEBUG_INFO) << readerMsg << "billboardCenter=" << document.getUseBillboardCenter() << std::endl;
+
                 document.setUseTextureAlphaForTransparancyBinning(options->getOptionString().find("noTextureAlphaForTransparancyBinning")==std::string::npos);
-                osg::notify(osg::DEBUG_INFO) << readerMsg << "noTextureAlphaForTransparancyBinning=" << document.getUseTextureAlphaForTransparancyBinning() << std::endl;
+                osg::notify(osg::DEBUG_INFO) << readerMsg << "noTextureAlphaForTransparancyBinning=" << !document.getUseTextureAlphaForTransparancyBinning() << std::endl;
 
                 document.setDoUnitsConversion((options->getOptionString().find("noUnitsConversion")==std::string::npos)); // default to true, unless noUnitsConversion is specified.
-                osg::notify(osg::DEBUG_INFO) << readerMsg << "noUnitsConversion=" << document.getDoUnitsConversion() << std::endl;
+                osg::notify(osg::DEBUG_INFO) << readerMsg << "noUnitsConversion=" << !document.getDoUnitsConversion() << std::endl;
 
                 if (document.getDoUnitsConversion())
                 {
@@ -202,8 +218,12 @@ class FLTReaderWriter : public ReaderWriter
                         document.setTexturePool( pools->getTexturePool(), true );
                     if (pools->getMaterialPool())
                         document.setMaterialPool( pools->getMaterialPool(), true );
+                    if (pools->getLightSourcePool())
+                        document.setLightSourcePool( pools->getLightSourcePool(), true );
                     if (pools->getLPAppearancePool())
                         document.setLightPointAppearancePool( pools->getLPAppearancePool(), true );
+                    if (pools->getLPAnimationPool())
+                        document.setLightPointAnimationPool( pools->getLPAnimationPool(), true );
                     if (pools->getShaderPool())
                         document.setShaderPool( pools->getShaderPool(), true );
                 }
@@ -224,7 +244,12 @@ class FLTReaderWriter : public ReaderWriter
             if (!document.getPreserveFace())
             {
                 osgUtil::Optimizer optimizer;
-                optimizer.optimize(document.getHeaderNode(),  osgUtil::Optimizer::SHARE_DUPLICATE_STATE | osgUtil::Optimizer::MERGE_GEOMETRY | osgUtil::Optimizer::MERGE_GEODES | osgUtil::Optimizer::TESSELATE_GEOMETRY );
+                optimizer.optimize(document.getHeaderNode(),
+                    osgUtil::Optimizer::SHARE_DUPLICATE_STATE |
+                    osgUtil::Optimizer::MERGE_GEOMETRY | 
+                    osgUtil::Optimizer::MERGE_GEODES | 
+                    osgUtil::Optimizer::TESSELLATE_GEOMETRY |
+                    osgUtil::Optimizer::STATIC_OBJECT_DETECTION);
             }
 
             return document.getHeaderNode();
@@ -237,20 +262,9 @@ class FLTReaderWriter : public ReaderWriter
             return WriteResult::FILE_NOT_HANDLED;
         }
 
-        virtual WriteResult writeNode(const Node& node,const std::string& fileName, const osgDB::ReaderWriter::Options* options) const
+        virtual WriteResult writeNode(const Node& /*node*/,const std::string& /*fileName*/, const osgDB::ReaderWriter::Options* /*options*/) const
         {
-            std::string ext = getFileExtension(fileName);
-            if (!acceptsExtension(ext)) return WriteResult::FILE_NOT_HANDLED;
-
-            // code for setting up the database path so that internally referenced file are searched for on relative paths. 
-            osg::ref_ptr<Options> local_opt = options ? static_cast<Options*>(options->clone(osg::CopyOp::SHALLOW_COPY)) : new Options;
-            if(local_opt->getDatabasePathList().empty())
-                local_opt->setDatabasePath(osgDB::getFilePath(fileName));
-
-            std::ofstream fout(fileName.c_str(), std::ios::out | std::ios::binary);
-            WriteResult result = writeNode(node, fout, local_opt.get());
-            fout.close();
-            return result;
+            return WriteResult::FILE_NOT_HANDLED;
         }
         
         virtual WriteResult writeObject(const Object& object,std::ostream& fout, const osgDB::ReaderWriter::Options* options) const
@@ -263,19 +277,16 @@ class FLTReaderWriter : public ReaderWriter
         virtual WriteResult writeNode(const Node& /*node*/,std::ostream& /*fout*/, const osgDB::ReaderWriter::Options* /*options*/) const
         {
             return WriteResult::FILE_NOT_HANDLED;
-
         }
 
     protected:
 
-        mutable osgDB::ReentrantMutex _serializerMutex;
+        mutable OpenThreads::ReentrantMutex _serializerMutex;
 };
 
 // now register with Registry to instantiate the above
 // reader/writer.
-RegisterReaderWriterProxy<FLTReaderWriter> g_FLTReaderWriterProxy;
-
-
+REGISTER_OSGPLUGIN(OpenFlight, FLTReaderWriter)
 
 
 
