@@ -23,17 +23,22 @@
 //#include <dom/domLibrary_effects.h>
 //#include <dom/domLibrary_materials.h>
 
+#ifdef WIN32
+#include "windows.h"
+#endif
+
 using namespace osgdae;
 
 void daeWriter::processMaterial( osg::StateSet *ss, domInstance_geometry *ig, const std::string &geoName )
 {
+    osg::ref_ptr<osg::StateSet> ssClean = CleanStateSet(ss); // Need to hold a ref to this or the materialMap.find() will delete it
     domBind_material *bm = daeSafeCast< domBind_material >( ig->createAndPlace( COLLADA_ELEMENT_BIND_MATERIAL ) );
     domBind_material::domTechnique_common *tc = daeSafeCast< domBind_material::domTechnique_common >( bm->createAndPlace( "technique_common" ) );
     domInstance_material *im = daeSafeCast< domInstance_material >( tc->createAndPlace( COLLADA_ELEMENT_INSTANCE_MATERIAL ) );
     std::string symbol = geoName + "_material";
     im->setSymbol( symbol.c_str() );
 
-    std::map< osg::StateSet*, domMaterial* >::iterator iter = materialMap.find( ss );
+    MaterialMap::iterator iter = materialMap.find( ssClean );
     if ( iter != materialMap.end() )
     {
         std::string url = "#" + std::string( iter->second->getId() );
@@ -47,7 +52,7 @@ void daeWriter::processMaterial( osg::StateSet *ss, domInstance_geometry *ig, co
     }
 
     domMaterial *mat = daeSafeCast< domMaterial >( lib_mats->createAndPlace( COLLADA_ELEMENT_MATERIAL ) );
-    std::string name = ss->getName();
+    std::string name = ssClean->getName();
     if ( name.empty() )
     {
         name = "material";
@@ -78,10 +83,10 @@ void daeWriter::processMaterial( osg::StateSet *ss, domInstance_geometry *ig, co
     pc_teq->setSid( "t0" );
     domProfile_COMMON::domTechnique::domPhong *phong = daeSafeCast< domProfile_COMMON::domTechnique::domPhong >( pc_teq->createAndPlace( "phong" ) );
 
-    osg::Texture *tex = static_cast<osg::Texture*>(ss->getTextureAttribute( 0, osg::StateAttribute::TEXTURE ));
-    if ( ss->getTextureAttribute( 1, osg::StateAttribute::TEXTURE ) != NULL )
+    osg::Texture *tex = static_cast<osg::Texture*>(ssClean->getTextureAttribute( 0, osg::StateAttribute::TEXTURE ));
+    if ( ssClean->getTextureAttribute( 1, osg::StateAttribute::TEXTURE ) != NULL )
     {
-        tex = static_cast<osg::Texture*>(ss->getTextureAttribute( 1, osg::StateAttribute::TEXTURE ));
+        tex = static_cast<osg::Texture*>(ssClean->getTextureAttribute( 1, osg::StateAttribute::TEXTURE ));
     }
     if ( tex != NULL && tex->getImage( 0 ) != NULL )
     {
@@ -92,16 +97,49 @@ void daeWriter::processMaterial( osg::StateSet *ss, domInstance_geometry *ig, co
 
         osg::Image *osgimg = tex->getImage( 0 );
         domImage::domInit_from *imgif = daeSafeCast< domImage::domInit_from >( img->createAndPlace( "init_from" ) );
-        std::string imgstr = "/" + osgDB::convertFileNameToUnixStyle( osgDB::findDataFile( osgimg->getFileName() ) );
+#ifdef WIN32
+        char path1[ MAX_PATH + 1 ];
+        char path2[ MAX_PATH + 1 ];
+        LPTSTR pFileName;
+        std::string imageFileName = osgDB::findDataFile( osgimg->getFileName() );
+        GetFullPathName( imageFileName.c_str(), MAX_PATH + 1, path1, &pFileName );
+        GetShortPathName( path1, path2, MAX_PATH + 1 );
+        GetLongPathName( path2, path1, MAX_PATH + 1 );
+        imageFileName = osgDB::convertFileNameToUnixStyle(path1);
+        if ((imageFileName.length() > 3) && (isalpha(imageFileName[0])) && (imageFileName[1] == ':') && (imageFileName[2] == '/'))
+        {
+            if (islower(imageFileName[0]))
+                imageFileName[0] = (char)_toupper(imageFileName[0]);
+            imageFileName = '/' + imageFileName;
+        }
+        daeURI imageFileUri( imageFileName.c_str() );
+        imageFileUri.validate();
+        std::string docFileName = doc->getDocumentURI()->getFilepath();
+        docFileName += doc->getDocumentURI()->getFile();
+        if ((docFileName.length() > 3) && (docFileName[0] == '/') && (isalpha(docFileName[1])) && (docFileName[2] == ':') && (docFileName[3] == '/'))
+            GetFullPathName( docFileName.c_str() + 1, MAX_PATH + 1, path1, &pFileName );
+        else
+            GetFullPathName( docFileName.c_str(), MAX_PATH + 1, path1, &pFileName );
+        GetShortPathName( path1, path2, MAX_PATH + 1 );
+        GetLongPathName( path2, path1, MAX_PATH + 1 );
+        docFileName = osgDB::convertFileNameToUnixStyle(path1);
+        if ((docFileName.length() > 3) && (isalpha(docFileName[0])) && (docFileName[1] == ':') && (docFileName[2] == '/'))
+        {
+            if (islower(docFileName[0]))
+                docFileName[0] = (char)_toupper(docFileName[0]);
+            docFileName = '/' + docFileName;
+        }
+        daeURI docFileUri( docFileName.c_str() );
+        docFileUri.validate();
+        imgif->setValue( imageFileUri.getURI() );
+        imgif->getValue().makeRelativeTo( &docFileUri );
+#else
+        std::string imgstr = osgDB::convertFileNameToUnixStyle( osgDB::findDataFile( osgimg->getFileName() ) );
         daeURI uri( imgstr.c_str() );
-        uri.validate();
+        uri.validate(); // This returns an absolute URI
         imgif->setValue( uri.getURI() );
-        //imgif->setValue( imgstr.c_str() );
-        //imgif->getValue().validate();
-        imgif->getValue().makeRelativeTo( doc->getDocumentURI() ); 
-        //imgif->setValue( osgimg->getFileName().c_str() );
-        //osg::notify( osg::WARN ) << "original img filename " << osgimg->getFileName() << std::endl;
-        //osg::notify( osg::WARN ) << "init_from filename " << imgstr << std::endl;
+        imgif->getValue().makeRelativeTo( doc->getDocumentURI() );
+#endif
 
 #ifndef EARTH_TEX
         domCommon_newparam_type *np = daeSafeCast< domCommon_newparam_type >( pc->createAndPlace( "newparam" ) );
@@ -237,8 +275,8 @@ void daeWriter::processMaterial( osg::StateSet *ss, domInstance_geometry *ig, co
         bvi->setInput_set( 0 );
 
         //take care of blending if any
-        osg::BlendFunc *bf = static_cast< osg::BlendFunc * >( ss->getAttribute( osg::StateAttribute::BLENDFUNC ) );
-        osg::BlendColor *bc = static_cast< osg::BlendColor * >( ss->getAttribute( osg::StateAttribute::BLENDCOLOR ) );
+        osg::BlendFunc *bf = static_cast< osg::BlendFunc * >( ssClean->getAttribute( osg::StateAttribute::BLENDFUNC ) );
+        osg::BlendColor *bc = static_cast< osg::BlendColor * >( ssClean->getAttribute( osg::StateAttribute::BLENDCOLOR ) );
         if ( bc != NULL )
         {
             domCommon_transparent_type *ctt = daeSafeCast< domCommon_transparent_type >( phong->createAndPlace( "transparent" ) );
@@ -263,7 +301,7 @@ void daeWriter::processMaterial( osg::StateSet *ss, domInstance_geometry *ig, co
         }
         
     }
-    osg::Material *osgmat = static_cast<osg::Material*>(ss->getAttribute( osg::StateAttribute::MATERIAL ));
+    osg::Material *osgmat = static_cast<osg::Material*>(ssClean->getAttribute( osg::StateAttribute::MATERIAL ));
     if ( osgmat != NULL )
     {
         const osg::Vec4 &eCol = osgmat->getEmissionFrontAndBack()?osgmat->getEmission( osg::Material::FRONT_AND_BACK ):osgmat->getEmission( osg::Material::FRONT );
@@ -288,7 +326,7 @@ void daeWriter::processMaterial( osg::StateSet *ss, domInstance_geometry *ig, co
 
         
         //### check if we really have a texture
-        if ((tex == NULL) || (!cot->getTexture()))
+        if ( phong->getDiffuse() == NULL )
         {
             cot = daeSafeCast< domCommon_color_or_texture_type >( phong->createAndPlace( "diffuse" ) );
             col = daeSafeCast< domCommon_color_or_texture_type_complexType::domColor >( cot->createAndPlace( "color" ) );
@@ -309,7 +347,7 @@ void daeWriter::processMaterial( osg::StateSet *ss, domInstance_geometry *ig, co
             domAny *any = (domAny*)(daeElement*)teq->createAndPlace( "color" );
 
             std::ostringstream colVal;
-            colVal << std::dec << " " << int(dCol[0]) << " " << int(dCol[1]) << " " << int(dCol[2]) << " " << int(dCol[3]);
+            colVal << dCol.r() << " " << dCol.g() << " " << dCol.b() << " " << dCol.a();
             any->setValue( colVal.str().c_str() );
         }
 
@@ -325,5 +363,22 @@ void daeWriter::processMaterial( osg::StateSet *ss, domInstance_geometry *ig, co
         f->setValue( shininess );
     }
 
-    materialMap.insert( std::make_pair( ss, mat ) );
+    materialMap.insert( std::make_pair( ssClean, mat ) );
 }
+
+osg::StateSet* daeWriter::CleanStateSet(osg::StateSet* pStateSet) const
+{
+    // TODO - clean out all the the attributes and modes not used to define materials
+    osg::StateSet* pCleanedStateSet = new osg::StateSet;
+    pCleanedStateSet->setTextureAttributeList(pStateSet->getTextureAttributeList());
+    if (NULL != pStateSet->getAttribute(osg::StateAttribute::BLENDFUNC))
+        pCleanedStateSet->setAttribute(pStateSet->getAttribute(osg::StateAttribute::BLENDFUNC));
+    if (NULL != pStateSet->getAttribute(osg::StateAttribute::BLENDCOLOR))
+        pCleanedStateSet->setAttribute(pStateSet->getAttribute(osg::StateAttribute::BLENDCOLOR));
+    if (NULL != pStateSet->getAttribute(osg::StateAttribute::MATERIAL))
+        pCleanedStateSet->setAttribute(pStateSet->getAttribute(osg::StateAttribute::MATERIAL));
+
+    return pCleanedStateSet;
+}
+
+
