@@ -29,13 +29,16 @@ public:
     /** get the singleton instance.*/
     static FreeTypeLibrary* instance();
 
-    osgText::Font* getFont(const std::string& fontfile,unsigned int index=0);
-    osgText::Font* getFont(std::istream& fontstream, unsigned int index=0);
+    osgText::Font* getFont(const std::string& fontfile,unsigned int index=0, unsigned int flags=0);
+    osgText::Font* getFont(std::istream& fontstream, unsigned int index=0, unsigned int flags=0);
     
     void removeFontImplmentation(FreeTypeFont* fontImpl) { _fontImplementationSet.erase(fontImpl); }
 
 protected:
 
+    /** Verify the correct character mapping for MS windows */
+    void  verifyCharacterMap(FT_Face face);
+
     /** protected constructor to ensure the only way to create the 
       * library is via the singleton instance method.*/
     FreeTypeLibrary();
