@@ -9,11 +9,13 @@
 #include <osgDB/ImageOptions>
 
 #include <OpenThreads/ScopedLock>
-#include <osgDB/ReentrantMutex>
+#include <OpenThreads/ReentrantMutex>
 
 #include <gdal_priv.h>
 
-#define SERIALIZER() OpenThreads::ScopedLock<osgDB::ReentrantMutex> lock(_serializerMutex)  
+#include "DataSetLayer.h"
+
+#define SERIALIZER() OpenThreads::ScopedLock<OpenThreads::ReentrantMutex> lock(_serializerMutex)  
 
 // From easyrgb.com
 float Hue_2_RGB( float v1, float v2, float vH )
@@ -35,15 +37,48 @@ class ReaderWriterGDAL : public osgDB::ReaderWriter
             return osgDB::equalCaseInsensitive(extension,"gdal") || osgDB::equalCaseInsensitive(extension,"gdal");
         }
 
+        virtual ReadResult readObject(const std::string& file, const osgDB::ReaderWriter::Options* options) const
+        {
+            if (osgDB::equalCaseInsensitive(osgDB::getFileExtension(file),"gdal"))
+            {
+                return readObject(osgDB::getNameLessExtension(file),options);
+            }
+
+            OpenThreads::ScopedLock<OpenThreads::ReentrantMutex> lock(_serializerMutex);
+
+            std::string fileName = osgDB::findDataFile( file, options );
+            if (fileName.empty()) return ReadResult::FILE_NOT_FOUND;
+
+            initGDAL();
+
+            // open a DataSetLayer.
+            osg::ref_ptr<GDALPlugin::DataSetLayer> dataset = new GDALPlugin::DataSetLayer(fileName);
+            dataset->setGdalReader(this);
+            
+            if (dataset->isOpen()) return dataset.release();
+            
+            return ReadResult::FILE_NOT_HANDLED;
+        }
+
         virtual ReadResult readImage(const std::string& fileName, const osgDB::ReaderWriter::Options* options) const
         {
-            OpenThreads::ScopedLock<osgDB::ReentrantMutex> lock(_serializerMutex);
+            if (osgDB::equalCaseInsensitive(osgDB::getFileExtension(fileName),"gdal"))
+            {
+                return readImage(osgDB::getNameLessExtension(fileName),options);
+            }
+
+            OpenThreads::ScopedLock<OpenThreads::ReentrantMutex> lock(_serializerMutex);
             return const_cast<ReaderWriterGDAL*>(this)->local_readImage(fileName, options);
         }
         
         virtual ReadResult readHeightField(const std::string& fileName, const osgDB::ReaderWriter::Options* options) const
         {
-            OpenThreads::ScopedLock<osgDB::ReentrantMutex> lock(_serializerMutex);
+            if (osgDB::equalCaseInsensitive(osgDB::getFileExtension(fileName),"gdal"))
+            {
+                return readHeightField(osgDB::getNameLessExtension(fileName),options);
+            }
+
+            OpenThreads::ScopedLock<OpenThreads::ReentrantMutex> lock(_serializerMutex);
             return const_cast<ReaderWriterGDAL*>(this)->local_readHeightField(fileName, options);
         }
 
@@ -76,8 +111,8 @@ class ReaderWriterGDAL : public osgDB::ReaderWriter
 
             int destX = 0;
             int destY = 0;
-            int destWidth = osg::minimum(dataWidth,1024);
-            int destHeight = osg::minimum(dataHeight,1024);
+            int destWidth = osg::minimum(dataWidth,4096);
+            int destHeight = osg::minimum(dataHeight,4096);
 //             int destWidth = osg::minimum(dataWidth,4096);
 //             int destHeight = osg::minimum(dataHeight,4096);
 
@@ -530,8 +565,8 @@ class ReaderWriterGDAL : public osgDB::ReaderWriter
 
             int destX = 0;
             int destY = 0;
-            int destWidth = osg::minimum(dataWidth,1024);
-            int destHeight = osg::minimum(dataHeight,1024);
+            int destWidth = osg::minimum(dataWidth,4096);
+            int destHeight = osg::minimum(dataHeight,4096);
 
             osgDB::ImageOptions::TexCoordRange* texCoordRange = 0;
 
@@ -711,7 +746,9 @@ class ReaderWriterGDAL : public osgDB::ReaderWriter
                         hf->setHeight(c,copy_r,temp);
                     }
                 }
-
+                hf->setOrigin(osg::Vec3(BottomLeft[0],-BottomLeft[1],0));
+                hf->setXInterval((BottomRight[0]-BottomLeft[0])/destWidth);
+                hf->setYInterval((TopLeft[1]-BottomLeft[1])/destHeight);
                 return hf;
             }
                    
@@ -719,7 +756,7 @@ class ReaderWriterGDAL : public osgDB::ReaderWriter
 
         }
 
-        void initGDAL()
+        void initGDAL() const
         {
             static bool s_initialized = false;
             if (!s_initialized)
@@ -729,7 +766,7 @@ class ReaderWriterGDAL : public osgDB::ReaderWriter
             }
         }
         
-        mutable osgDB::ReentrantMutex _serializerMutex;
+        mutable OpenThreads::ReentrantMutex _serializerMutex;
         
 };
 
