@@ -48,7 +48,7 @@
 extern  "C"
 {
     #include <gif_lib.h>
-};
+}
 
 #define ERR_NO_ERROR     0
 #define ERR_OPEN         1
@@ -398,4 +398,4 @@ class ReaderWriterGIF : public osgDB::ReaderWriter
 
 // now register with Registry to instantiate the above
 // reader/writer.
-osgDB::RegisterReaderWriterProxy<ReaderWriterGIF> g_readerWriter_GIF_Proxy;
+REGISTER_OSGPLUGIN(gif, ReaderWriterGIF)
