@@ -18,10 +18,13 @@
 #include "DataInputStream.h"
 #include "StateSet.h"
 #include "AlphaFunc.h"
+#include "BlendColor.h"
+#include "Stencil.h"
 #include "BlendFunc.h"
 #include "Depth.h"
 #include "Material.h"
 #include "CullFace.h"
+#include "ColorMask.h"
 #include "ClipPlane.h"
 #include "PolygonOffset.h"
 #include "PolygonMode.h"
@@ -46,10 +49,13 @@
 #include "Viewport.h"
 #include "Scissor.h"
 #include "Image.h"
+#include "PointSprite.h"
+#include "Multisample.h"
+
 
 #include "Group.h"
 #include "MatrixTransform.h"
-#include "CameraNode.h"
+#include "Camera.h"
 #include "CameraView.h"
 #include "Geode.h"
 #include "LightSource.h"
@@ -156,11 +162,6 @@ bool DataInputStream::readBool(){
     return c!=0;
 }
 
-unsigned int DataInputStream::getVersion()
-{
-    return( _version );
-}
-
 char DataInputStream::readChar(){
     char c;
     _istream->read(&c, CHARSIZE);
@@ -285,7 +286,8 @@ unsigned long DataInputStream::readULong(){
     return l;
 }
 
-double DataInputStream::readDouble(){
+double DataInputStream::readDouble()
+{
     double d;
     _istream->read((char*)&d, DOUBLESIZE);
     if (_istream->rdstate() & _istream->failbit)
@@ -296,20 +298,25 @@ double DataInputStream::readDouble(){
     return d;
 }
 
-std::string DataInputStream::readString(){
+std::string DataInputStream::readString()
+{
     std::string s;
     int size = readInt();
-    s.resize(size);
-    _istream->read((char*)s.c_str(), size);
-    //if (_istream->rdstate() & _istream->failbit)
-    //   throw Exception("DataInputStream::readString(): Failed to read string value.");
+    if (size != 0)
+    {
+        s.resize(size);
+        _istream->read((char*)s.c_str(), size);
+        //if (_istream->rdstate() & _istream->failbit)
+        //   throw Exception("DataInputStream::readString(): Failed to read string value.");
 
-    if (_verboseOutput) std::cout<<"read/writeString() ["<<s<<"]"<<std::endl;
-    
+        if (_verboseOutput) std::cout<<"read/writeString() ["<<s<<"]"<<std::endl;
+    }
+        
     return s;
 }
 
-void DataInputStream::readCharArray(char* data, int size){
+void DataInputStream::readCharArray(char* data, int size)
+{
     _istream->read(data, size);
     if (_istream->rdstate() & _istream->failbit)
         throw Exception("DataInputStream::readCharArray(): Failed to read char value.");
@@ -386,11 +393,24 @@ osg::Vec4d DataInputStream::readVec4d(){
 
 osg::Plane DataInputStream::readPlane(){
     osg::Plane v;
-    v[0]=readFloat();
-    v[1]=readFloat();
-    v[2]=readFloat();
-    v[3]=readFloat();
 
+    if (getVersion() <= VERSION_0018)
+    {
+        v[0]=readFloat();
+        v[1]=readFloat();
+        v[2]=readFloat();
+        v[3]=readFloat();
+    }
+    else
+    {
+        // assume double for planes even if Plane::value_type is float
+        // to ensure that the .ive format does vary.
+        v[0]=readDouble();
+        v[1]=readDouble();
+        v[2]=readDouble();
+        v[3]=readDouble();
+    }
+    
     if (_verboseOutput) std::cout<<"read/writePlane() ["<<v<<"]"<<std::endl;
     
     return v;
@@ -463,6 +483,8 @@ osg::Array* DataInputStream::readArray(){
 osg::IntArray* DataInputStream::readIntArray()
 {
     int size = readInt();
+    if (size == 0)
+        return NULL;
     osg::IntArray* a = new osg::IntArray(size);
     
     _istream->read((char*)&((*a)[0]), INTSIZE*size);
@@ -482,6 +504,8 @@ osg::IntArray* DataInputStream::readIntArray()
 osg::UByteArray* DataInputStream::readUByteArray()
 {
     int size = readInt();
+    if (size == 0)
+        return NULL;
     osg::UByteArray* a = new osg::UByteArray(size);
 
     _istream->read((char*)&((*a)[0]), CHARSIZE*size);
@@ -497,6 +521,8 @@ osg::UByteArray* DataInputStream::readUByteArray()
 osg::UShortArray* DataInputStream::readUShortArray()
 {
     int size = readInt();
+    if (size == 0)
+        return NULL;
     osg::UShortArray* a = new osg::UShortArray(size);
 
     _istream->read((char*)&((*a)[0]), SHORTSIZE*size);
@@ -517,6 +543,8 @@ osg::UShortArray* DataInputStream::readUShortArray()
 osg::UIntArray* DataInputStream::readUIntArray()
 {
     int size = readInt();
+    if (size == 0)
+        return NULL;
     osg::UIntArray* a = new osg::UIntArray(size);
 
     _istream->read((char*)&((*a)[0]), INTSIZE*size);
@@ -537,6 +565,8 @@ osg::UIntArray* DataInputStream::readUIntArray()
 osg::Vec4ubArray* DataInputStream::readVec4ubArray()
 {
     int size = readInt();
+    if (size == 0)
+        return NULL;
     osg::Vec4ubArray* a = new osg::Vec4ubArray(size);
 
     _istream->read((char*)&((*a)[0]), INTSIZE*size);
@@ -552,6 +582,8 @@ osg::Vec4ubArray* DataInputStream::readVec4ubArray()
 osg::FloatArray* DataInputStream::readFloatArray()
 {
     int size = readInt();
+    if (size == 0)
+        return NULL;
     
     osg::FloatArray* a = new osg::FloatArray(size);
     
@@ -573,6 +605,9 @@ osg::FloatArray* DataInputStream::readFloatArray()
 osg::Vec2Array* DataInputStream::readVec2Array()
 {
     int size = readInt();
+    if (size == 0)
+        return NULL;
+
     osg::Vec2Array* a = new osg::Vec2Array(size);
     
     _istream->read((char*)&((*a)[0]), FLOATSIZE*2*size);
@@ -596,6 +631,8 @@ osg::Vec2Array* DataInputStream::readVec2Array()
 osg::Vec3Array* DataInputStream::readVec3Array()
 {
     int size = readInt();
+    if (size == 0)
+        return NULL;
     osg::Vec3Array* a = new osg::Vec3Array(size);
 
     _istream->read((char*)&((*a)[0]), FLOATSIZE*3*size);
@@ -619,6 +656,8 @@ osg::Vec3Array* DataInputStream::readVec3Array()
 
 osg::Vec4Array* DataInputStream::readVec4Array(){
     int size = readInt();
+    if (size == 0)
+        return NULL;
     osg::Vec4Array* a = new osg::Vec4Array(size);
 
     _istream->read((char*)&((*a)[0]), FLOATSIZE*4*size);
@@ -640,6 +679,8 @@ osg::Vec4Array* DataInputStream::readVec4Array(){
 osg::Vec2bArray* DataInputStream::readVec2bArray()
 {
     int size = readInt();
+    if (size == 0)
+        return NULL;
     osg::Vec2bArray* a = new osg::Vec2bArray(size);
 
     _istream->read((char*)&((*a)[0]), CHARSIZE * 2 * size);
@@ -655,6 +696,8 @@ osg::Vec2bArray* DataInputStream::readVec2bArray()
 osg::Vec3bArray* DataInputStream::readVec3bArray()
 {
     int size = readInt();
+    if (size == 0)
+        return NULL;
     osg::Vec3bArray* a = new osg::Vec3bArray(size);
 
     _istream->read((char*)&((*a)[0]), CHARSIZE * 3 * size);
@@ -670,6 +713,8 @@ osg::Vec3bArray* DataInputStream::readVec3bArray()
 osg::Vec4bArray* DataInputStream::readVec4bArray()
 {
     int size = readInt();
+    if (size == 0)
+        return NULL;
     osg::Vec4bArray* a = new osg::Vec4bArray(size);
 
     _istream->read((char*)&((*a)[0]), CHARSIZE * 4 * size);
@@ -685,6 +730,8 @@ osg::Vec4bArray* DataInputStream::readVec4bArray()
 osg::Vec2sArray* DataInputStream::readVec2sArray()
 {
     int size = readInt();
+    if (size == 0)
+        return NULL;
     osg::Vec2sArray* a = new osg::Vec2sArray(size);
 
     _istream->read((char*)&((*a)[0]), SHORTSIZE * 2 * size);
@@ -709,6 +756,8 @@ osg::Vec2sArray* DataInputStream::readVec2sArray()
 osg::Vec3sArray* DataInputStream::readVec3sArray()
 {
     int size = readInt();
+    if (size == 0)
+        return NULL;
     osg::Vec3sArray* a = new osg::Vec3sArray(size);
 
     _istream->read((char*)&((*a)[0]), SHORTSIZE * 3 * size);
@@ -734,6 +783,8 @@ osg::Vec3sArray* DataInputStream::readVec3sArray()
 osg::Vec4sArray* DataInputStream::readVec4sArray()
 {
     int size = readInt();
+    if (size == 0)
+        return NULL;
     osg::Vec4sArray* a = new osg::Vec4sArray(size);
 
     _istream->read((char*)&((*a)[0]), SHORTSIZE * 4 * size);
@@ -921,6 +972,10 @@ osg::StateAttribute* DataInputStream::readStateAttribute()
         attribute = new osg::AlphaFunc();
         ((ive::AlphaFunc*)(attribute))->read(this);
     }
+    else if(attributeID == IVEBLENDCOLOR){
+        attribute = new osg::BlendColor();
+        ((ive::BlendColor*)(attribute))->read(this);
+    }
     else if(attributeID == IVEBLENDFUNC ||
             attributeID == IVEBLENDFUNCSEPARATE){
         attribute = new osg::BlendFunc();
@@ -946,6 +1001,10 @@ osg::StateAttribute* DataInputStream::readStateAttribute()
         attribute = new osg::CullFace();
         ((ive::CullFace*)(attribute))->read(this);
     }
+    else if(attributeID == IVECOLORMASK){
+        attribute = new osg::ColorMask();
+        ((ive::ColorMask*)(attribute))->read(this);
+    }
     else if(attributeID == IVECLIPPLANE){
         attribute = new osg::ClipPlane();
         ((ive::ClipPlane*)(attribute))->read(this);
@@ -1026,6 +1085,18 @@ osg::StateAttribute* DataInputStream::readStateAttribute()
         attribute = new osg::Program();
         ((ive::Program*)(attribute))->read(this);
     }
+    else if(attributeID == IVEPOINTSPRITE){
+        attribute = new osg::PointSprite();
+        ((ive::PointSprite*)(attribute))->read(this);
+    }
+    else if(attributeID == IVEMULTISAMPLE){
+        attribute = new osg::Multisample();
+        ((ive::Multisample*)(attribute))->read(this);
+    }
+    else if(attributeID == IVESTENCIL){
+        attribute = new osg::Stencil();
+        ((ive::Stencil*)(attribute))->read(this);
+    }
     else{
         throw Exception("Unknown StateAttribute in StateSet::read()");
     }
@@ -1202,9 +1273,9 @@ osg::Node* DataInputStream::readNode()
         node = new osg::MatrixTransform();
         ((ive::MatrixTransform*)(node))->read(this);
     }
-    else if(nodeTypeID== IVECAMERANODE){
-        node = new osg::CameraNode();
-        ((ive::CameraNode*)(node))->read(this);
+    else if(nodeTypeID== IVECAMERA){
+        node = new osg::Camera();
+        ((ive::Camera*)(node))->read(this);
     }
     else if(nodeTypeID== IVECAMERAVIEW){
         node = new osg::CameraView();
