@@ -9,7 +9,7 @@
  *
  *    HISTORY:        Created 11.03.2003
  *                    Updated for 1D textures - Don Burns 27.1.2004
- *                      Updated for light model - Stan Blinov at 25 august 7512 from World Creation (7.09.2004)
+ *                    Updated for light model - Stan Blinov at 25 august 7512 from World Creation (7.09.2004)
  *
  *    Copyright 2003 VR-C
  **********************************************************************/
@@ -19,9 +19,12 @@
 
 #include "StateSet.h"
 #include "AlphaFunc.h"
+#include "BlendColor.h"
+#include "Stencil.h"
 #include "BlendFunc.h"
 #include "Material.h"
 #include "CullFace.h"
+#include "ColorMask.h"
 #include "Depth.h"
 #include "ClipPlane.h"
 #include "PolygonOffset.h"
@@ -49,10 +52,12 @@
 #include "Viewport.h"
 #include "Scissor.h"
 #include "Image.h"
+#include "PointSprite.h"
+#include "Multisample.h"
 
 #include "Group.h"
 #include "MatrixTransform.h"
-#include "CameraNode.h"
+#include "Camera.h"
 #include "CameraView.h"
 #include "Geode.h"
 #include "LightSource.h"
@@ -271,10 +276,10 @@ void DataOutputStream::writeVec4d(const osg::Vec4d& v){
 
 void DataOutputStream::writePlane(const osg::Plane& v)
 {
-    writeFloat(v[0]);
-    writeFloat(v[1]);
-    writeFloat(v[2]);
-    writeFloat(v[3]);
+    writeDouble(v[0]);
+    writeDouble(v[1]);
+    writeDouble(v[2]);
+    writeDouble(v[3]);
     
     if (_verboseOutput) std::cout<<"read/writePlane() ["<<v<<"]"<<std::endl;
 }
@@ -653,6 +658,12 @@ void DataOutputStream::writeStateAttribute(const osg::StateAttribute* attribute)
         if(dynamic_cast<const osg::AlphaFunc*>(attribute)){
             ((ive::AlphaFunc*)(attribute))->write(this);
         }
+        else if(dynamic_cast<const osg::BlendColor*>(attribute)){
+            ((ive::BlendColor*)(attribute))->write(this);
+        }
+        else if(dynamic_cast<const osg::Stencil*>(attribute)){
+            ((ive::Stencil*)(attribute))->write(this);
+        }
         else if(dynamic_cast<const osg::BlendFunc*>(attribute)){
             ((ive::BlendFunc*)(attribute))->write(this);
         }
@@ -673,6 +684,10 @@ void DataOutputStream::writeStateAttribute(const osg::StateAttribute* attribute)
         else if(dynamic_cast<const osg::CullFace*>(attribute)){
             ((ive::CullFace*)(attribute))->write(this);
         }
+        // This is a ColorMask
+        else if(dynamic_cast<const osg::ColorMask*>(attribute)){
+            ((ive::ColorMask*)(attribute))->write(this);
+        }
         // this is a Cliplane
         else if(dynamic_cast<const osg::ClipPlane*>(attribute)){
             ((ive::ClipPlane*)(attribute))->write(this);
@@ -749,10 +764,18 @@ void DataOutputStream::writeStateAttribute(const osg::StateAttribute* attribute)
         else if(dynamic_cast<const osg::FrontFace*>(attribute)){
             ((ive::FrontFace*)(attribute))->write(this);
         }
-        // This is a FrontFace
+        // This is a Program
         else if(dynamic_cast<const osg::Program*>(attribute)){
             ((ive::Program*)(attribute))->write(this);
         }
+        // This is a PointSprite
+        else if(dynamic_cast<const osg::PointSprite*>(attribute)){
+            ((ive::PointSprite*)(attribute))->write(this);
+        }
+        // This is a Multisample
+        else if(dynamic_cast<const osg::Multisample*>(attribute)){
+            ((ive::Multisample*)(attribute))->write(this);
+        }
 
         else{
             std::string className = attribute->className();
@@ -922,8 +945,8 @@ void DataOutputStream::writeNode(const osg::Node* node)
         if(dynamic_cast<const osg::MatrixTransform*>(node)){
             ((ive::MatrixTransform*)(node))->write(this);
         }
-        else if(dynamic_cast<const osg::CameraNode*>(node)){
-            ((ive::CameraNode*)(node))->write(this);
+        else if(dynamic_cast<const osg::Camera*>(node)){
+            ((ive::Camera*)(node))->write(this);
         }
         else if(dynamic_cast<const osg::CameraView*>(node)){
             ((ive::CameraView*)(node))->write(this);
