@@ -49,7 +49,7 @@ extern "C"
 {
     #include <jpeglib.h>
     #include "jerror.h"
-};
+}
 
 #include <setjmp.h>
 #include <string.h>
@@ -826,4 +826,4 @@ class ReaderWriterJPEG : public osgDB::ReaderWriter
 
 // now register with Registry to instantiate the above
 // reader/writer.
-osgDB::RegisterReaderWriterProxy<ReaderWriterJPEG> g_readerWriter_JPEG_Proxy;
+REGISTER_OSGPLUGIN(jpeg, ReaderWriterJPEG)
