@@ -23,13 +23,78 @@ class OSGReaderWriter : public ReaderWriter
             return equalCaseInsensitive(extension,"osg");
         }
 
-        virtual ReadResult readObject(const std::string& fileName, const Options* opt) const { return readNode(fileName, opt); }
+        virtual ReadResult readObject(const std::string& file, const Options* opt) const
+        {
+            std::string ext = osgDB::getLowerCaseFileExtension(file);
+                        
+            if (equalCaseInsensitive(ext,"osgs"))
+            {   
+                std::istringstream fin(osgDB::getNameLessExtension(file));
+                if (fin) return readNode(fin,opt);
+                return ReadResult::ERROR_IN_READING_FILE;
+            }            
+            
+            if (!acceptsExtension(ext)) return ReadResult::FILE_NOT_HANDLED;
 
-        virtual ReadResult readObject(std::istream& fin, const Options* opt) const { return readNode(fin, opt); }
+            std::string fileName = osgDB::findDataFile( file, opt );
+            if (fileName.empty()) return ReadResult::FILE_NOT_FOUND;
+
+            // code for setting up the database path so that internally referenced file are searched for on relative paths. 
+            osg::ref_ptr<Options> local_opt = opt ? static_cast<Options*>(opt->clone(osg::CopyOp::SHALLOW_COPY)) : new Options;
+            local_opt->setDatabasePath(osgDB::getFilePath(fileName));
+
+            std::ifstream fin(fileName.c_str());
+            if (fin)
+            {
+                return readObject(fin, local_opt.get());
+            }
+            return 0L;
+        }               
+
+        virtual ReadResult readObject(std::istream& fin, const Options* options) const
+        {
+            fin.imbue(std::locale::classic());
+
+            Input fr;
+            fr.attach(&fin);
+            fr.setOptions(options);
+            
+            typedef std::vector<osg::Object*> ObjectList;
+            ObjectList objectList;
+
+            // load all nodes in file, placing them in a group.
+            while(!fr.eof())
+            {
+                Object *object = fr.readObject();
+                if (object) objectList.push_back(object);
+                else fr.advanceOverCurrentFieldOrBlock();
+            }
+
+            if  (objectList.empty())
+            {
+                return ReadResult("No data loaded");
+            }
+            else if (objectList.size()==1)
+            {
+                return objectList.front();
+            }
+            else
+            {
+                return objectList.front();
+            }
+        }
 
         virtual ReadResult readNode(const std::string& file, const Options* opt) const
         {
             std::string ext = osgDB::getLowerCaseFileExtension(file);
+                        
+            if (equalCaseInsensitive(ext,"osgs"))
+            {   
+                std::istringstream fin(osgDB::getNameLessExtension(file));
+                if (fin) return readNode(fin,opt);
+                return ReadResult::ERROR_IN_READING_FILE;
+            }            
+            
             if (!acceptsExtension(ext)) return ReadResult::FILE_NOT_HANDLED;
 
             std::string fileName = osgDB::findDataFile( file, opt );
@@ -50,6 +115,7 @@ class OSGReaderWriter : public ReaderWriter
         
         virtual ReadResult readNode(std::istream& fin, const Options* options) const
         {
+            fin.imbue(std::locale::classic());
 
             Input fr;
             fr.attach(&fin);
@@ -105,7 +171,7 @@ class OSGReaderWriter : public ReaderWriter
                     }
                     if (opt=="OutputTextureFiles")
                     {
-                        fout.setOutputTextureFiles(false);
+                        fout.setOutputTextureFiles(true);
                     }
                 }
             }
@@ -117,11 +183,14 @@ class OSGReaderWriter : public ReaderWriter
             if (!acceptsExtension(ext)) return WriteResult::FILE_NOT_HANDLED;
 
             Output fout(fileName.c_str());
-            fout.setOptions(options);
             if (fout)
             {
+                fout.setOptions(options);
+
                 setPrecision(fout,options);
 
+                fout.imbue(std::locale::classic());
+
                 fout.writeObject(obj);
                 fout.close();
                 return WriteResult::FILE_SAVED;
@@ -131,14 +200,17 @@ class OSGReaderWriter : public ReaderWriter
 
         virtual WriteResult writeObject(const Object& obj,std::ostream& fout, const osgDB::ReaderWriter::Options* options) const
         {
-            Output foutput;
-            foutput.setOptions(options);
-
-            std::ios &fios = foutput;
-            fios.rdbuf(fout.rdbuf());
 
             if (fout)
             {
+                Output foutput;
+                foutput.setOptions(options);
+
+                std::ios &fios = foutput;
+                fios.rdbuf(fout.rdbuf());
+
+                fout.imbue(std::locale::classic());
+
                 setPrecision(foutput,options);
 
                 foutput.writeObject(obj);
@@ -155,9 +227,12 @@ class OSGReaderWriter : public ReaderWriter
 
 
             Output fout(fileName.c_str());
-            fout.setOptions(options);
             if (fout)
             {
+                fout.setOptions(options);
+
+                fout.imbue(std::locale::classic());
+
                 setPrecision(fout,options);
 
                 fout.writeObject(node);
@@ -169,14 +244,18 @@ class OSGReaderWriter : public ReaderWriter
 
         virtual WriteResult writeNode(const Node& node,std::ostream& fout, const osgDB::ReaderWriter::Options* options) const
         {
-            Output foutput;
-            foutput.setOptions(options);
 
-            std::ios &fios = foutput;
-            fios.rdbuf(fout.rdbuf());
 
             if (fout)
             {
+                Output foutput;
+                foutput.setOptions(options);
+
+                std::ios &fios = foutput;
+                fios.rdbuf(fout.rdbuf());
+
+                foutput.imbue(std::locale::classic());
+
                 setPrecision(foutput,options);
 
                 foutput.writeObject(node);
@@ -189,4 +268,4 @@ class OSGReaderWriter : public ReaderWriter
 
 // now register with Registry to instantiate the above
 // reader/writer.
-RegisterReaderWriterProxy<OSGReaderWriter> g_OSGReaderWriterProxy;
+REGISTER_OSGPLUGIN(osg, OSGReaderWriter)
