@@ -1,3 +1,8 @@
+// Released under the OSGPL license, as part of the OpenSceneGraph distribution.
+//
+// ReaderWriter for sgi's .rgb format.
+// specification can be found at http://local.wasp.uwa.edu.au/~pbourke/dataformats/sgirgb/sgiversion.html
+
 #include <osg/Image>
 #include <osg/Notify>
 
@@ -259,7 +264,7 @@ static void RawImageGetRow(rawImageRec *raw, unsigned char *buf, int y, int z)
 
     if ((raw->type & 0xFF00) == 0x0100)
     {
-                raw->file->seekg((long) raw->rowStart[y+z*raw->sizeY], std::ios::beg);
+        raw->file->seekg((long) raw->rowStart[y+z*raw->sizeY], std::ios::beg);
         raw->file->read((char*)raw->tmp, (unsigned int)raw->rowSize[y+z*raw->sizeY]);
 
         iPtr = raw->tmp;
@@ -275,14 +280,24 @@ static void RawImageGetRow(rawImageRec *raw, unsigned char *buf, int y, int z)
                 tempShort++;
                 iPtr = reinterpret_cast<unsigned char *>(tempShort);
             }
+            
             if(raw->bpc != 1)
                 ConvertShort(&pixel, 1);
+
             count = (int)(pixel & 0x7F);
-            if (!count)
+            
+            // limit the count value to the remiaing row size
+            if (oPtr + count*raw->bpc > buf + raw->sizeX*raw->bpc)
+            {
+                count = ( (buf + raw->sizeX*raw->bpc) - oPtr ) / raw->bpc;
+            }
+                
+            if (count<=0)
             {
                 done = 1;
                 return;
             }
+            
             if (pixel & 0x80)
             {
                 while (count--)
@@ -356,6 +371,12 @@ static void RawImageGetData(rawImageRec *raw, unsigned char **data )
     //     if (width!=raw->sizeX) width += 4;
 
     // byte aligned.
+    
+    osg::notify(osg::INFO)<<"raw->sizeX = "<<raw->sizeX<<std::endl;
+    osg::notify(osg::INFO)<<"raw->sizeY = "<<raw->sizeY<<std::endl;
+    osg::notify(osg::INFO)<<"raw->sizeZ = "<<raw->sizeZ<<std::endl;
+    osg::notify(osg::INFO)<<"raw->bpc = "<<raw->bpc<<std::endl;
+    
     *data = new unsigned char [(raw->sizeX)*(raw->sizeY)*(raw->sizeZ)*(raw->bpc)];
 
     ptr = *data;
@@ -526,7 +547,10 @@ class ReaderWriterRGB : public osgDB::ReaderWriter
 
             GLenum pixelFormat = img.getPixelFormat();
 
-            raw.dim    = 
+            raw.dim    = 3;
+            raw.sizeX = img.s();
+            raw.sizeY = img.t();
+            raw.sizeZ =
                 pixelFormat == GL_COLOR_INDEX? 1 :
                 pixelFormat == GL_RED? 1 :
                 pixelFormat == GL_GREEN? 1 :
@@ -538,31 +562,48 @@ class ReaderWriterRGB : public osgDB::ReaderWriter
                 pixelFormat == GL_BGRA? 4 :
                 pixelFormat == GL_LUMINANCE? 1 :
                 pixelFormat == GL_LUMINANCE_ALPHA ? 2 : 1;
-
-            raw.sizeX = img.s();
-            raw.sizeY = img.t();
-            raw.sizeZ = raw.dim;
             raw.min = 0;
             raw.max = 0xFF;
             raw.wasteBytes = 0;
             strncpy( raw.name, name.c_str(), 80);
             raw.colorMap = 0;
+            raw.bpc = (img.getPixelSizeInBits()/raw.sizeZ)/8;
 
             int isize = img.getImageSizeInBytes();
             unsigned char *buffer = new unsigned char[isize];
-            unsigned char *dptr = buffer;
-            int i, j;
-            for( i = 0; i < raw.sizeZ; i++ )
+            if(raw.bpc == 1)
             {
-                const unsigned char *ptr = img.data();
-                ptr += i;
-                for( j = 0; j < isize/raw.sizeZ; j++ )
+                unsigned char *dptr = buffer;
+                int i, j;
+                for( i = 0; i < raw.sizeZ; ++i )
                 {
-                    *(dptr++) = *ptr;
-                    ptr += raw.sizeZ;
+                    const unsigned char *ptr = img.data();
+                    ptr += i;
+                    for( j = 0; j < isize/raw.sizeZ; ++j )
+                    {
+                        *(dptr++) = *ptr;
+                        ptr += raw.sizeZ;
+                    }
+                }
+            }
+            else
+            { // bpc == 2
+                unsigned short *dptr = reinterpret_cast<unsigned short*>(buffer);
+                int i, j;
+                for( i = 0; i < raw.sizeZ; ++i )
+                {
+                    const unsigned short *ptr = reinterpret_cast<const unsigned short*>(img.data());
+                    ptr += i;
+                    for( j = 0; j < isize/(raw.sizeZ*2); ++j )
+                    {
+                        *dptr = *ptr;
+                        ConvertShort(dptr++, 1);
+                        ptr += raw.sizeZ;
+                    }
                 }
             }
 
+        
             if( raw.needsBytesSwapped() )
                 raw.swapBytes();
 
@@ -611,4 +652,4 @@ class ReaderWriterRGB : public osgDB::ReaderWriter
 
 // now register with Registry to instantiate the above
 // reader/writer.
-osgDB::RegisterReaderWriterProxy<ReaderWriterRGB> g_readerWriter_RGB_Proxy;
+REGISTER_OSGPLUGIN(rgb, ReaderWriterRGB)
