@@ -42,7 +42,8 @@ void TXPPagedLOD::traverse(osg::NodeVisitor& nv)
             bool needToLoadChild = false;
             
             unsigned maxRangeSize = _rangeList.size();
-            if (maxRangeSize!=0 && forceUseOfFirstChild) maxRangeSize=1;
+            if (maxRangeSize!=0 && forceUseOfFirstChild)
+		maxRangeSize=1;
             
             for(unsigned int i=0;i<maxRangeSize;++i)
             {    
@@ -50,9 +51,9 @@ void TXPPagedLOD::traverse(osg::NodeVisitor& nv)
                 {
                     if (i<_children.size())
                     {
-                        if (updateTimeStamp) _perRangeDataList[i]._timeStamp=timeStamp;
+                        if (updateTimeStamp)
+			    _perRangeDataList[i]._timeStamp=timeStamp;
 
-                        //std::cout<<"PagedLOD::traverse() - Selecting child "<<i<<std::endl;
                         _children[i]->accept(nv);
                         lastChildTraversed = (int)i;
                     }
@@ -72,7 +73,8 @@ void TXPPagedLOD::traverse(osg::NodeVisitor& nv)
                 if (numChildren>0 && ((int)numChildren-1)!=lastChildTraversed)
                 {
                     //std::cout<<"    to child "<<numChildren-1<<std::endl;
-                    if (updateTimeStamp) _perRangeDataList[numChildren-1]._timeStamp=timeStamp;
+                    if (updateTimeStamp)
+			_perRangeDataList[numChildren-1]._timeStamp=timeStamp;
                     _children[numChildren-1]->accept(nv);
                 }
                 
@@ -100,3 +102,18 @@ void TXPPagedLOD::traverse(osg::NodeVisitor& nv)
     }
 }
 
+osg::BoundingSphere TXPPagedLOD::computeBound() const
+{
+    // Force calculation of entire bounding sphere; this will include any
+    // externally-referenced models which are attached to the tile.
+    // If this is not done, then externally referenced models will disappear
+    // when the tile they are attached to leaves the view volume.
+    osg::BoundingSphere result = osg::Group::computeBound();
+    
+    if (_centerMode==USER_DEFINED_CENTER && _radius>=0.0f)
+    {
+	float tempRadius = osg::maximum( _radius, result.radius() );
+        result = osg::BoundingSphere(_userDefinedCenter,tempRadius);
+    }
+    return result;
+}
