@@ -1,4 +1,4 @@
-/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2005 Robert Osfield 
+/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2006 Robert Osfield 
  *
  * This library is open source and may be redistributed and/or modified under  
  * the terms of the OpenSceneGraph Public License (OSGPL) version 0.0 or 
@@ -60,13 +60,13 @@ bool MultiSwitch::addChild( osg::Node *child)
             ++itr)
         {
             ValueList& values = *itr;
-            if (_children.size()>_values.size())
-	    {
-	        values.resize(_children.size(),_newChildDefaultValue);
-	        values[childPosition]=_newChildDefaultValue;
-	    }
+            if (_children.size()>values.size())
+            {
+                values.resize(_children.size(),_newChildDefaultValue);
+                values[childPosition]=_newChildDefaultValue;
+            }
         }
-	return true;
+        return true;
     }
     return false;
 }
@@ -80,18 +80,18 @@ bool MultiSwitch::insertChild( unsigned int index, osg::Node *child)
             ++itr)
         {
             ValueList& values = *itr;
-            if (index>=_values.size())
-	    {
-	        values.push_back(_newChildDefaultValue);
-	    }
+            if (index>=values.size())
+            {
+                values.push_back(_newChildDefaultValue);
+            }
             else
             {
-	        values.insert(values.begin()+index, _newChildDefaultValue);
+                values.insert(values.begin()+index, _newChildDefaultValue);
             }
         }
 
         
-	return true;
+        return true;
     }
     return false;
 }
