@@ -1,4 +1,3 @@
-;
 /* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2006 Robert Osfield 
  *
  * This library is open source and may be redistributed and/or modified under  
@@ -24,11 +23,20 @@
 #include <osgDB/FileNameUtils>
 #include <osg/GLU>
 
+#include <OpenThreads/ReentrantMutex>
+
 using namespace osgText;
 using namespace std;
 
 static osg::ApplicationUsageProxy Font_e0(osg::ApplicationUsage::ENVIRONMENTAL_VARIABLE,"OSG_TEXT_INCREMENTAL_SUBLOADING <type>","ON | OFF");
 
+static OpenThreads::ReentrantMutex s_FontFileMutex;
+
+Font::FontMutex* osgText::Font::getSerializeFontCallsMutex()
+{
+    static OpenThreads::Mutex s_FontCallsMutex;
+    return &s_FontCallsMutex;
+}
 
 std::string osgText::findFontFile(const std::string& str)
 {
@@ -36,6 +44,7 @@ std::string osgText::findFontFile(const std::string& str)
     std::string filename = osgDB::findDataFile(str);
     if (!filename.empty()) return filename;
 
+    OpenThreads::ScopedLock<OpenThreads::ReentrantMutex> lock(s_FontFileMutex);
 
     static osgDB::FilePathList s_FontFilePath;
     static bool initialized = false;
@@ -82,17 +91,23 @@ std::string osgText::findFontFile(const std::string& str)
     return std::string();
 }
 
-osgText::Font* osgText::readFontFile(const std::string& filename)
+osgText::Font* osgText::readFontFile(const std::string& filename, const osgDB::ReaderWriter::Options* userOptions)
 {
     if (filename=="") return 0;
 
     std::string foundFile = findFontFile(filename);
     if (foundFile.empty()) return 0;
     
-    osg::ref_ptr<osgDB::ReaderWriter::Options> options = new osgDB::ReaderWriter::Options;
-    options->setObjectCacheHint(osgDB::ReaderWriter::Options::CACHE_OBJECTS);
+    OpenThreads::ScopedLock<OpenThreads::Mutex> lock(s_FontFileMutex);
 
-    osg::Object* object = osgDB::readObjectFile(foundFile, options.get());
+    osg::ref_ptr<osgDB::ReaderWriter::Options> localOptions;
+    if (!userOptions)
+    {
+        localOptions = new osgDB::ReaderWriter::Options;
+        localOptions->setObjectCacheHint(osgDB::ReaderWriter::Options::CACHE_OBJECTS);
+    }
+
+    osg::Object* object = osgDB::readObjectFile(foundFile, userOptions ? userOptions : localOptions.get());
 
     // if the object is a font then return it.
     osgText::Font* font = dynamic_cast<osgText::Font*>(object);
@@ -103,15 +118,21 @@ osgText::Font* osgText::readFontFile(const std::string& filename)
     return 0;
 }
 
-osgText::Font* osgText::readFontStream(std::istream& stream)
+osgText::Font* osgText::readFontStream(std::istream& stream, const osgDB::ReaderWriter::Options* userOptions)
 {
-    osg::ref_ptr<osgDB::ReaderWriter::Options> options = new osgDB::ReaderWriter::Options;
-    options->setObjectCacheHint(osgDB::ReaderWriter::Options::CACHE_OBJECTS);
+    OpenThreads::ScopedLock<OpenThreads::Mutex> lock(s_FontFileMutex);
+
+    osg::ref_ptr<osgDB::ReaderWriter::Options> localOptions;
+    if (!userOptions)
+    {
+        localOptions = new osgDB::ReaderWriter::Options;
+        localOptions->setObjectCacheHint(osgDB::ReaderWriter::Options::CACHE_OBJECTS);
+    }
 
     // there should be a better way to get the FreeType ReaderWriter by name...
     osgDB::ReaderWriter *reader = osgDB::Registry::instance()->getReaderWriterForExtension("ttf");
     if (reader == 0) return 0;
-    osgDB::ReaderWriter::ReadResult rr = reader->readObject(stream, options.get());
+    osgDB::ReaderWriter::ReadResult rr = reader->readObject(stream, userOptions ? userOptions : localOptions.get());
     if (rr.error())
     {
         osg::notify(osg::WARN) << rr.message() << std::endl;
@@ -132,11 +153,13 @@ osgText::Font* osgText::readFontStream(std::istream& stream)
 
 
 Font::Font(FontImplementation* implementation):
+    osg::Object(true),
     _width(16),
     _height(16),
-    _margin(2),
-    _textureWidthHint(512),
-    _textureHeightHint(512),
+    _margin(1),
+    _marginRatio(0.02),
+    _textureWidthHint(1024),
+    _textureHeightHint(1024),
     _minFilterHint(osg::Texture::LINEAR_MIPMAP_LINEAR),
     _magFilterHint(osg::Texture::LINEAR)
 {
@@ -145,6 +168,16 @@ Font::Font(FontImplementation* implementation):
     _texenv = new osg::TexEnv;
     _stateset = new osg::StateSet;
     _stateset->setRenderingHint(osg::StateSet::TRANSPARENT_BIN);
+
+    char *ptr;
+    if( (ptr = getenv("OSG_MAX_TEXTURE_SIZE")) != 0)
+    {
+        unsigned int osg_max_size = atoi(ptr);
+
+        if (osg_max_size<_textureWidthHint) _textureWidthHint = osg_max_size;
+        if (osg_max_size<_textureHeightHint) _textureHeightHint = osg_max_size;
+    }
+
 }
 
 Font::~Font()
@@ -200,10 +233,29 @@ unsigned int Font::getGlyphImageMargin() const
     return _margin;
 }
 
+void Font::setGlyphImageMarginRatio(float ratio)
+{
+    _marginRatio = ratio;
+}
+
+float Font::getGlyphImageMarginRatio() const
+{
+    return _marginRatio;
+}
+
 void Font::setTextureSizeHint(unsigned int width,unsigned int height)
 {
     _textureWidthHint = width;
     _textureHeightHint = height;
+
+    char *ptr;
+    if( (ptr = getenv("OSG_MAX_TEXTURE_SIZE")) != 0)
+    {
+        unsigned int osg_max_size = atoi(ptr);
+
+        if (osg_max_size<_textureWidthHint) _textureWidthHint = osg_max_size;
+        if (osg_max_size<_textureHeightHint) _textureHeightHint = osg_max_size;
+    }
 }
 
 unsigned int Font::getTextureWidthHint() const
@@ -254,14 +306,46 @@ Font::Glyph* Font::getGlyph(unsigned int charcode)
     else return 0;
 }
 
+void Font::setThreadSafeRefUnref(bool threadSafe)
+{
+    Object::setThreadSafeRefUnref(threadSafe);
+    
+    if (_texenv.valid()) _texenv->setThreadSafeRefUnref(threadSafe);
+    if (_stateset.valid()) _stateset->setThreadSafeRefUnref(threadSafe);
+
+    for(GlyphTextureList::const_iterator itr=_glyphTextureList.begin();
+        itr!=_glyphTextureList.end();
+        ++itr)
+    {
+        (*itr)->setThreadSafeRefUnref(threadSafe);
+    }    
+}
+
+void Font::resizeGLObjectBuffers(unsigned int maxSize)
+{
+    if (_stateset.valid()) _stateset->resizeGLObjectBuffers(maxSize);
+
+    for(GlyphTextureList::const_iterator itr=_glyphTextureList.begin();
+        itr!=_glyphTextureList.end();
+        ++itr)
+    {
+        (*itr)->resizeGLObjectBuffers(maxSize);
+    }    
+}
+
 void Font::releaseGLObjects(osg::State* state) const
 {
-  for(GlyphTextureList::const_iterator itr=_glyphTextureList.begin();
-    itr!=_glyphTextureList.end();
-    ++itr)
-  {
-    (*itr)->releaseGLObjects(state);
-  }
+    if (_stateset.valid()) _stateset->releaseGLObjects(state);
+
+    for(GlyphTextureList::const_iterator itr=_glyphTextureList.begin();
+        itr!=_glyphTextureList.end();
+        ++itr)
+    {
+        (*itr)->releaseGLObjects(state);
+    }
+    
+    // const_cast<Font*>(this)->_glyphTextureList.clear();
+    // const_cast<Font*>(this)->_sizeGlyphMap.clear();
 }
 
 osg::Vec2 Font::getKerning(unsigned int leftcharcode,unsigned int rightcharcode, KerningType kerningType)
@@ -280,12 +364,8 @@ bool Font::hasVertical() const
 
 void Font::addGlyph(unsigned int width, unsigned int height, unsigned int charcode, Glyph* glyph)
 {
-
-    //cout << "charcode "<<(char)charcode<<"  "<<&_glyphTextureList<<endl;
-
     _sizeGlyphMap[SizePair(width,height)][charcode]=glyph;
     
-    
     int posX=0,posY=0;
     
     GlyphTexture* glyphTexture = 0;
@@ -313,6 +393,7 @@ void Font::addGlyph(unsigned int width, unsigned int height, unsigned int charco
 
         // reserve enough space for the glyphs.
         glyphTexture->setGlyphImageMargin(_margin);
+        glyphTexture->setGlyphImageMarginRatio(_marginRatio);
         glyphTexture->setTextureSize(_textureWidthHint,_textureHeightHint);
         glyphTexture->setFilter(osg::Texture::MIN_FILTER,_minFilterHint);
         glyphTexture->setFilter(osg::Texture::MAG_FILTER,_magFilterHint);
@@ -335,7 +416,8 @@ void Font::addGlyph(unsigned int width, unsigned int height, unsigned int charco
 
 
 Font::GlyphTexture::GlyphTexture():
-    _margin(2),
+    _margin(1),
+    _marginRatio(0.02f),
     _usedY(0),
     _partUsedX(0),
     _partUsedY(0)
@@ -357,9 +439,11 @@ int Font::GlyphTexture::compare(const osg::StateAttribute& rhs) const
 
 bool Font::GlyphTexture::getSpaceForGlyph(Glyph* glyph, int& posX, int& posY)
 {
-        
-    int width = glyph->s()+2*_margin;
-    int height = glyph->t()+2*_margin;
+    int maxAxis = std::max(glyph->s(), glyph->t());
+    int margin = _margin + (int)((float)maxAxis * _marginRatio);
+    
+    int width = glyph->s()+2*margin;
+    int height = glyph->t()+2*margin;
 
     // first check box (_partUsedX,_usedY) to (width,height)
     if (width <= (getTextureWidth()-_partUsedX) &&
@@ -368,8 +452,8 @@ bool Font::GlyphTexture::getSpaceForGlyph(Glyph* glyph, int& posX, int& posY)
         // can fit in existing row.
 
         // record the position in which the texture will be stored.
-        posX = _partUsedX+_margin;
-        posY = _usedY+_margin;        
+        posX = _partUsedX+margin;
+        posY = _usedY+margin;        
 
         // move used markers on.
         _partUsedX += width;
@@ -386,8 +470,8 @@ bool Font::GlyphTexture::getSpaceForGlyph(Glyph* glyph, int& posX, int& posY)
         _partUsedX = 0;
         _usedY = _partUsedY;
 
-        posX = _partUsedX+_margin;
-        posY = _usedY+_margin;        
+        posX = _partUsedX+margin;
+        posY = _usedY+margin;        
 
         // move used markers on.
         _partUsedX += width;
@@ -402,6 +486,7 @@ bool Font::GlyphTexture::getSpaceForGlyph(Glyph* glyph, int& posX, int& posY)
 
 void Font::GlyphTexture::addGlyph(Glyph* glyph, int posX, int posY)
 {
+    OpenThreads::ScopedLock<OpenThreads::Mutex> lock(_mutex);
 
     _glyphs.push_back(glyph);
     for(unsigned int i=0;i<_glyphsToSubload.size();++i)
@@ -412,9 +497,9 @@ void Font::GlyphTexture::addGlyph(Glyph* glyph, int posX, int posY)
     // set up the details of where to place glyph's image in the texture.
     glyph->setTexture(this);
     glyph->setTexturePosition(posX,posY);
-    unsigned int sizeAdjustment = 0; // was 1.
-    glyph->setMinTexCoord(osg::Vec2((float)(posX+_margin)/(float)(getTextureWidth()-sizeAdjustment),(float)(posY+_margin)/(float)(getTextureHeight()-sizeAdjustment)));
-    glyph->setMaxTexCoord(osg::Vec2((float)(posX+glyph->s()-_margin)/(float)(getTextureWidth()-sizeAdjustment),(float)(posY+glyph->t()-_margin)/(float)(getTextureHeight()-sizeAdjustment)));
+    unsigned int sizeAdjustment = 1;
+    glyph->setMinTexCoord(osg::Vec2((float)(posX)/(float)(getTextureWidth()-sizeAdjustment),(float)(posY)/(float)(getTextureHeight()-sizeAdjustment)));
+    glyph->setMaxTexCoord(osg::Vec2((float)(posX+glyph->s())/(float)(getTextureWidth()-sizeAdjustment),(float)(posY+glyph->t())/(float)(getTextureHeight()-sizeAdjustment)));
 }
 
 void Font::GlyphTexture::apply(osg::State& state) const
@@ -425,6 +510,8 @@ void Font::GlyphTexture::apply(osg::State& state) const
 
     if (contextID>=_glyphsToSubload.size())
     {
+        OpenThreads::ScopedLock<OpenThreads::Mutex> lock(_mutex);
+
         // graphics context is beyond the number of glyphsToSubloads, so
         // we must now copy the glyph list across, this is a potential
         // threading issue though is multiple applies are happening the
@@ -453,6 +540,15 @@ void Font::GlyphTexture::apply(osg::State& state) const
 
     if (newTextureObject)
     {
+        GLint maxTextureSize = 256;
+        glGetIntegerv(GL_MAX_TEXTURE_SIZE, &maxTextureSize);
+        if (maxTextureSize < getTextureWidth() || maxTextureSize < getTextureHeight())
+        {
+            osg::notify(osg::WARN)<<"Warning: osgText::Font texture size of ("<<getTextureWidth()<<", "<<getTextureHeight()<<") too large, unable to create font texture."<<std::endl;
+            osg::notify(osg::WARN)<<"         Maximum supported by hardward by native OpenGL implementation is ("<<maxTextureSize<<","<<maxTextureSize<<")."<<std::endl;
+            osg::notify(osg::WARN)<<"         Please set OSG_MAX_TEXTURE_SIZE lenvironment variable to "<<maxTextureSize<<" and re-run application."<<std::endl;
+            return;
+        }
         
         // being bound for the first time, need to allocate the texture
 
@@ -482,13 +578,26 @@ void Font::GlyphTexture::apply(osg::State& state) const
             // not mip mapping so no problems.
             break;
         }
+        
+        unsigned int imageDataSize = getTextureHeight()*getTextureWidth();
+        unsigned char* imageData = new unsigned char[imageDataSize];
+        for(unsigned int i=0; i<imageDataSize; ++i)
+        {
+            imageData[i] = 0;
+        }
+        
+
+//        osg::notify(osg::NOTICE)<<"Texture width = "<<getTextureWidth()<<std::endl;
+//        osg::notify(osg::NOTICE)<<"Texture height = "<<getTextureHeight()<<std::endl;
                
         // allocate the texture memory.
         glTexImage2D( GL_TEXTURE_2D, 0, GL_ALPHA,
                 getTextureWidth(), getTextureHeight(), 0,
                 GL_ALPHA,
                 GL_UNSIGNED_BYTE,
-                0 );
+                imageData );
+                
+        delete [] imageData;
     
     }
     else
@@ -508,20 +617,31 @@ void Font::GlyphTexture::apply(osg::State& state) const
     static bool s_subloadAllGlyphsTogether = false;
     if (!s_renderer)
     {
+        OpenThreads::ScopedLock<OpenThreads::Mutex> lock(_mutex);
+
         s_renderer = glGetString(GL_RENDERER);
         osg::notify(osg::INFO)<<"glGetString(GL_RENDERER)=="<<s_renderer<<std::endl;
-        if (strstr((const char*)s_renderer,"IMPACT")!=0)
+        if (s_renderer && strstr((const char*)s_renderer,"IMPACT")!=0)
         {
             // we're running on an Octane, so need to work around its
             // subloading bugs by loading all at once.
             s_subloadAllGlyphsTogether = true;
         }
         
-        const char* str = getenv("OSG_TEXT_INCREMENTAL_SUBLOADING");
-        if (str && (strcmp(str,"OFF")==0 || strcmp(str,"Off")==0 || strcmp(str,"Off")==0))
+        if (s_renderer && 
+            (strstr((const char*)s_renderer,"Radeon")!=0) || 
+            (strstr((const char*)s_renderer,"RADEON")!=0))
         {
+            // we're running on an ATI, so need to work around its
+            // subloading bugs by loading all at once.
             s_subloadAllGlyphsTogether = true;
         }
+
+        const char* str = getenv("OSG_TEXT_INCREMENTAL_SUBLOADING");
+        if (str)
+        {
+            s_subloadAllGlyphsTogether = strcmp(str,"OFF")==0 || strcmp(str,"Off")==0 || strcmp(str,"Off")==0;
+        }
     }
 
 
@@ -530,7 +650,8 @@ void Font::GlyphTexture::apply(osg::State& state) const
 
     if (!glyphsWereSubloading.empty() || newTextureObject)
     {
-    
+        OpenThreads::ScopedLock<OpenThreads::Mutex> lock(_mutex);
+
         if (!s_subloadAllGlyphsTogether)
         {
             if (newTextureObject)
@@ -612,7 +733,7 @@ void Font::GlyphTexture::apply(osg::State& state) const
     }
     else
     {
-        //std::cout << "no need to subload "<<std::endl;
+//        osg::notify(osg::INFO) << "no need to subload "<<std::endl;
     }
 
 
@@ -625,6 +746,18 @@ void Font::GlyphTexture::apply(osg::State& state) const
 
 }
 
+void Font::GlyphTexture::setThreadSafeRefUnref(bool threadSafe)
+{
+    osg::Texture2D::setThreadSafeRefUnref(threadSafe);
+}
+
+void Font::GlyphTexture::resizeGLObjectBuffers(unsigned int maxSize)
+{
+    osg::Texture2D::resizeGLObjectBuffers(maxSize);
+    _glyphsToSubload.resize(maxSize);
+}
+
+
 // all the methods in Font::Glyph have been made non inline because VisualStudio6.0 is STUPID, STUPID, STUPID PILE OF JUNK.
 Font::Glyph::Glyph() {}
 Font::Glyph::~Glyph() {}
