@@ -39,7 +39,9 @@ Text::Text():
     _characterSizeMode(OBJECT_COORDS),
     _maximumWidth(0.0f),
     _maximumHeight(0.0f),
+    _lineSpacing(0.0f),
     _alignment(BASE_LINE),
+    _axisAlignment(XY_PLANE),
     _autoRotateToScreen(false),
     _layout(LEFT_TO_RIGHT),
     _color(1.0f,1.0f,1.0f,1.0f),
@@ -72,9 +74,11 @@ Text::Text(const Text& text,const osg::CopyOp& copyop):
     _characterSizeMode(text._characterSizeMode),
     _maximumWidth(text._maximumWidth),
     _maximumHeight(text._maximumHeight),
+    _lineSpacing(text._lineSpacing),
     _text(text._text),
     _position(text._position),
     _alignment(text._alignment),
+    _axisAlignment(text._axisAlignment),
     _rotation(text._rotation),
     _autoRotateToScreen(text._autoRotateToScreen),
     _layout(text._layout),
@@ -147,6 +151,12 @@ void  Text::setMaximumHeight(float maximumHeight)
     _maximumHeight = maximumHeight;
     computeGlyphRepresentation();
 }
+
+void Text::setLineSpacing(float lineSpacing)
+{
+    _lineSpacing = lineSpacing;
+    computeGlyphRepresentation();
+}
     
 
 void Text::setText(const String& text)
@@ -191,32 +201,41 @@ void Text::setAlignment(AlignmentType alignment)
 
 void Text::setAxisAlignment(AxisAlignment axis)
 {
+    _axisAlignment = axis;
+
     switch(axis)
     {
     case XZ_PLANE:
+        setAutoRotateToScreen(false);
         setRotation(osg::Quat(osg::inDegrees(90.0f),osg::Vec3(1.0f,0.0f,0.0f))); 
         break;
     case REVERSED_XZ_PLANE:
+        setAutoRotateToScreen(false);
         setRotation(osg::Quat(osg::inDegrees(180.0f),osg::Vec3(0.0f,1.0f,0.0f))*
                     osg::Quat(osg::inDegrees(90.0f),osg::Vec3(1.0f,0.0f,0.0f))); 
         break;
     case YZ_PLANE:  
+        setAutoRotateToScreen(false);
         setRotation(osg::Quat(osg::inDegrees(90.0f),osg::Vec3(1.0f,0.0f,0.0f))*
                     osg::Quat(osg::inDegrees(90.0f),osg::Vec3(0.0f,0.0f,1.0f)));
         break;
     case REVERSED_YZ_PLANE:  
+        setAutoRotateToScreen(false);
         setRotation(osg::Quat(osg::inDegrees(180.0f),osg::Vec3(0.0f,1.0f,0.0f))*
                     osg::Quat(osg::inDegrees(90.0f),osg::Vec3(1.0f,0.0f,0.0f))*
                     osg::Quat(osg::inDegrees(90.0f),osg::Vec3(0.0f,0.0f,1.0f)));
         break;
     case XY_PLANE:
+        setAutoRotateToScreen(false);
         setRotation(osg::Quat());  // nop - already on XY plane.
         break;
     case REVERSED_XY_PLANE:
+        setAutoRotateToScreen(false);
         setRotation(osg::Quat(osg::inDegrees(180.0f),osg::Vec3(0.0f,1.0f,0.0f)));
         break;
     case SCREEN:
         setAutoRotateToScreen(true);
+        setRotation(osg::Quat());  // nop - already on XY plane.
         break;
     }
 }
@@ -330,8 +349,8 @@ String::iterator Text::computeLastCharacterOnLine(osg::Vec2& cursor, String::ite
         if (glyph)
         {
 
-           float width = (float)(glyph->s()-2*activefont->getGlyphImageMargin()) * wr;
-           //float height = (float)(glyph->t()-2*activefont->getGlyphImageMargin()) * hr;
+           float width = (float)(glyph->s()) * wr;
+           //float height = (float)(glyph->t()) * hr;
            #ifdef TREES_CODE_FOR_MAKING_SPACES_EDITABLE
            if (width == 0.0f)  width = glyph->getHorizontalAdvance() * wr;
            //if (height == 0.0f) height = glyph->getVerticalAdvance() * hr;
@@ -371,11 +390,13 @@ String::iterator Text::computeLastCharacterOnLine(osg::Vec2& cursor, String::ite
               case LEFT_TO_RIGHT:
               {
                 if (_maximumWidth>0.0f && cursor.x()+width>_maximumWidth) outOfSpace=true;
+                if(_maximumHeight>0.0f && cursor.y()<-_maximumHeight) outOfSpace=true;
                 break;
               }
               case RIGHT_TO_LEFT:
               {
                 if (_maximumWidth>0.0f && cursor.x()<-_maximumWidth) outOfSpace=true;
+                if(_maximumHeight>0.0f && cursor.y()<-_maximumHeight) outOfSpace=true;
                 break;
               }
               case VERTICAL:
@@ -433,6 +454,7 @@ String::iterator Text::computeLastCharacterOnLine(osg::Vec2& cursor, String::ite
     return lastChar;
 }
 
+
 void Text::computeGlyphRepresentation()
 {
     Font* activefont = getActiveFont();
@@ -447,6 +469,8 @@ void Text::computeGlyphRepresentation()
         computePositions(); //to reset the origin
         return;
     }
+    
+    OpenThreads::ScopedLock<Font::FontMutex> lock(*(Font::getSerializeFontCallsMutex()));
 
     // initialize bounding box, it will be expanded during glyph position calculation
     _textBB.init();
@@ -590,9 +614,9 @@ void Text::computeGlyphRepresentation()
                 Font::Glyph* glyph = activefont->getGlyph(charcode);
                 if (glyph)
                 {
+                    float width = (float)(glyph->s()) * wr;
+                    float height = (float)(glyph->t()) * hr;
 
-                    float width = (float)(glyph->s()-2*activefont->getGlyphImageMargin()) * wr;
-                    float height = (float)(glyph->t()-2*activefont->getGlyphImageMargin()) * hr;
                     #ifdef TREES_CODE_FOR_MAKING_SPACES_EDITABLE
                     if (width == 0.0f)  width = glyph->getHorizontalAdvance() * wr;
                     if (height == 0.0f) height = glyph->getVerticalAdvance() * hr;
@@ -693,7 +717,7 @@ void Text::computeGlyphRepresentation()
         {
           case LEFT_TO_RIGHT:
           {
-            startOfLine_coords.y() -= _characterHeight;
+            startOfLine_coords.y() -= _characterHeight * (1.0 + _lineSpacing);
             cursor = startOfLine_coords;
             previous_charcode = 0;
             _lineCount++;
@@ -701,7 +725,7 @@ void Text::computeGlyphRepresentation()
           }
           case RIGHT_TO_LEFT:
           {
-            startOfLine_coords.y() -= _characterHeight;
+            startOfLine_coords.y() -= _characterHeight * (1.0 + _lineSpacing);
             cursor = startOfLine_coords;
             previous_charcode = 0;
             _lineCount++;
@@ -709,7 +733,7 @@ void Text::computeGlyphRepresentation()
           }
           case VERTICAL:
           {
-            startOfLine_coords.x() += _characterHeight/_characterAspectRatio;
+            startOfLine_coords.x() += _characterHeight/_characterAspectRatio * (1.0 + _lineSpacing);
             cursor = startOfLine_coords;
             previous_charcode = 0;
             // because _lineCount is the max vertical no. of characters....
@@ -731,7 +755,7 @@ void Text::computeGlyphRepresentation()
 // Also sets avg_width and avg_height to 0.0f if the value is invalid.
 // This method is used several times in a loop for the same object which will produce the same values. 
 // Further optimization may try saving these values instead of recomputing them.
-bool Text::computeAverageGlypthWidthAndHeight(float& avg_width, float& avg_height) const
+bool Text::computeAverageGlyphWidthAndHeight(float& avg_width, float& avg_height) const
 {
     float width = 0.0f;
     float height = 0.0f;
@@ -784,7 +808,7 @@ void Text::computePositions()
     unsigned int size = osg::maximum(osg::DisplaySettings::instance()->getMaxNumberOfGraphicsContexts(),_autoTransformCache.size());
     
     // FIXME: OPTIMIZE: This would be one of the ideal locations to
-    // call computeAverageGlypthWidthAndHeight(). It is out of the contextID loop
+    // call computeAverageGlyphWidthAndHeight(). It is out of the contextID loop
     // so the value would be computed fewer times. But the code will need changes
     // to get the value down to the locations it is needed. (Either pass through parameters
     // or member variables, but we would need a system to know if the values are stale.)
@@ -969,7 +993,7 @@ void Text::computeBackdropPositions(unsigned int contextID) const
     // FIXME: OPTIMIZE: This function produces the same value regardless of contextID.
     // Since we tend to loop over contextID, we should cache this value some how
     // instead of recomputing it each time.
-    is_valid_size = computeAverageGlypthWidthAndHeight(avg_width, avg_height);
+    is_valid_size = computeAverageGlyphWidthAndHeight(avg_width, avg_height);
     
     // now apply matrix to the glyphs.
     for(TextureGlyphQuadMap::iterator titr=_textureGlyphQuadMap.begin();
@@ -1092,7 +1116,7 @@ void Text::computeBackdropBoundingBox() const
     
     // FIXME: OPTIMIZE: It is possible that this value has already been computed before
     // from previous calls to this function. This might be worth optimizing.
-    is_valid_size = computeAverageGlypthWidthAndHeight(avg_width, avg_height);
+    is_valid_size = computeAverageGlyphWidthAndHeight(avg_width, avg_height);
 
     // Finally, we have one more issue to deal with.
     // Now that the text takes more space, we need
@@ -1447,12 +1471,21 @@ void Text::computeColorGradientsPerCharacter() const
     }
 }
 
-void Text::drawImplementation(osg::State& state) const
+void Text::drawImplementation(osg::RenderInfo& renderInfo) const
+{
+    drawImplementation(*renderInfo.getState(), osg::Vec4(1.0f,1.0f,1.0f,1.0f));
+}
+
+void Text::drawImplementation(osg::State& state, const osg::Vec4& colorMultiplier) const
 {
     unsigned int contextID = state.getContextID();
 
     state.applyMode(GL_BLEND,true);
+#if 1   
     state.applyTextureMode(0,GL_TEXTURE_2D,true);
+#else
+    state.applyTextureMode(0,GL_TEXTURE_2D,false);
+#endif
     state.applyTextureAttribute(0,getActiveFont()->getTexEnv());
 
     if (_characterSizeMode!=OBJECT_COORDS || _autoRotateToScreen)
@@ -1470,8 +1503,8 @@ void Text::drawImplementation(osg::State& state) const
         const osg::Viewport* viewport = state.getCurrentViewport();
         if (viewport)
         {
-            width = viewport->width();
-            height = viewport->height();
+            width = static_cast<int>(viewport->width());
+            height = static_cast<int>(viewport->height());
         }
 
         bool doUpdate = atc._traversalNumber==-1;
@@ -1536,24 +1569,24 @@ void Text::drawImplementation(osg::State& state) const
             switch(_backdropImplementation)
             {
                 case POLYGON_OFFSET:
-                    renderWithPolygonOffset(state);
+                    renderWithPolygonOffset(state,colorMultiplier);
                     break;
                 case NO_DEPTH_BUFFER:
-                    renderWithNoDepthBuffer(state);
+                    renderWithNoDepthBuffer(state,colorMultiplier);
                     break;
                 case DEPTH_RANGE:
-                    renderWithDepthRange(state);
+                    renderWithDepthRange(state,colorMultiplier);
                     break;
                 case STENCIL_BUFFER:
-                    renderWithStencilBuffer(state);
+                    renderWithStencilBuffer(state,colorMultiplier);
                     break;
                 default:
-                    renderWithPolygonOffset(state);
+                    renderWithPolygonOffset(state,colorMultiplier);
             }
         }
         else
         {
-            renderOnlyForegroundText(state);
+            renderOnlyForegroundText(state,colorMultiplier);
         }
     }
 
@@ -1572,7 +1605,7 @@ void Text::drawImplementation(osg::State& state) const
             osg::Vec3 c01(osg::Vec3(_textBB.xMin(),_textBB.yMax(),_textBB.zMin())*matrix);
 
         
-            glColor4f(1.0f,1.0f,0.0f,1.0f);
+            glColor4fv(colorMultiplier.ptr());
             glBegin(GL_LINE_LOOP);
                 glVertex3fv(c00.ptr());
                 glVertex3fv(c10.ptr());
@@ -1584,7 +1617,7 @@ void Text::drawImplementation(osg::State& state) const
 
     if (_drawMode & ALIGNMENT)
     {
-        glColor4f(1.0f,0.0f,1.0f,1.0f);
+        glColor4fv(colorMultiplier.ptr());
 
         float cursorsize = _characterHeight*0.5f;
 
@@ -1634,17 +1667,31 @@ void Text::accept(osg::PrimitiveFunctor& pf) const
     
 }
 
-/** If State is non-zero, this function releases OpenGL objects for
-  * the specified graphics context. Otherwise, releases OpenGL objexts
-  * for all graphics contexts. */
+
+void Text::setThreadSafeRefUnref(bool threadSafe)
+{
+    Drawable::setThreadSafeRefUnref(threadSafe);
+
+    getActiveFont()->setThreadSafeRefUnref(threadSafe);
+}
+
+void Text::resizeGLObjectBuffers(unsigned int maxSize)
+{
+    Drawable::resizeGLObjectBuffers(maxSize);
+
+    _autoTransformCache.resize(maxSize);
+    
+    getActiveFont()->resizeGLObjectBuffers(maxSize);
+}
+
+
 void Text::releaseGLObjects(osg::State* state) const
 {
     Drawable::releaseGLObjects(state);
-    if (_font.valid()) _font->releaseGLObjects(state);
+    getActiveFont()->releaseGLObjects(state);
 }
 
 
-
 void Text::setBackdropType(BackdropType type)
 {
     if (_backdropType==type) return;
@@ -1895,7 +1942,7 @@ void Text::convertRgbToHsv( float rgb[], float hsv[] ) const
 
 }
 
-void Text::drawForegroundText(osg::State& state, const GlyphQuads& glyphquad) const
+void Text::drawForegroundText(osg::State& state, const GlyphQuads& glyphquad, const osg::Vec4& colorMultiplier) const
 {
     unsigned int contextID = state.getContextID();
 
@@ -1908,7 +1955,7 @@ void Text::drawForegroundText(osg::State& state, const GlyphQuads& glyphquad) co
         if(_colorGradientMode == SOLID)
         {
             state.disableColorPointer();
-            glColor4fv(_color.ptr());
+            glColor4f(colorMultiplier.r()*_color.r(),colorMultiplier.g()*_color.g(),colorMultiplier.b()*_color.b(),colorMultiplier.a()*_color.a());
         }
         else
         {
@@ -1920,7 +1967,7 @@ void Text::drawForegroundText(osg::State& state, const GlyphQuads& glyphquad) co
     }
 }
 
-void Text::renderOnlyForegroundText(osg::State& state) const
+void Text::renderOnlyForegroundText(osg::State& state, const osg::Vec4& colorMultiplier) const
 {
     for(TextureGlyphQuadMap::iterator titr=_textureGlyphQuadMap.begin();
         titr!=_textureGlyphQuadMap.end();
@@ -1931,13 +1978,13 @@ void Text::renderOnlyForegroundText(osg::State& state) const
 
         const GlyphQuads& glyphquad = titr->second;
 
-        drawForegroundText(state, glyphquad);
+        drawForegroundText(state, glyphquad, colorMultiplier);
     }
 
 }
 
 
-void Text::renderWithPolygonOffset(osg::State& state) const
+void Text::renderWithPolygonOffset(osg::State& state, const osg::Vec4& colorMultiplier) const
 {
     unsigned int contextID = state.getContextID();
 
@@ -1992,14 +2039,14 @@ void Text::renderWithPolygonOffset(osg::State& state) const
         // Reset the polygon offset so the foreground text is on top
         glPolygonOffset(0.0f,0.0f);
 
-        drawForegroundText(state, glyphquad);
+        drawForegroundText(state, glyphquad, colorMultiplier);
     }
 
     glPopAttrib();
 }
     
 
-void Text::renderWithNoDepthBuffer(osg::State& state) const
+void Text::renderWithNoDepthBuffer(osg::State& state, const osg::Vec4& colorMultiplier) const
 {
     unsigned int contextID = state.getContextID();
 
@@ -2042,14 +2089,14 @@ void Text::renderWithNoDepthBuffer(osg::State& state) const
             }
         }
 
-        drawForegroundText(state, glyphquad);
+        drawForegroundText(state, glyphquad, colorMultiplier);
     }
 
     glPopAttrib();
 }
 
 // This idea comes from Paul Martz's OpenGL FAQ: 13.050
-void Text::renderWithDepthRange(osg::State& state) const
+void Text::renderWithDepthRange(osg::State& state, const osg::Vec4& colorMultiplier) const
 {
     unsigned int contextID = state.getContextID();
 
@@ -2099,13 +2146,13 @@ void Text::renderWithDepthRange(osg::State& state) const
 
         glDepthRange(0.0, 1.0);
 
-        drawForegroundText(state, glyphquad);
+        drawForegroundText(state, glyphquad, colorMultiplier);
     }
 
     glPopAttrib();
 }
 
-void Text::renderWithStencilBuffer(osg::State& state) const
+void Text::renderWithStencilBuffer(osg::State& state, const osg::Vec4& colorMultiplier) const
 {
     /* Here are the steps:
      * 1) Disable drawing color
@@ -2256,7 +2303,7 @@ void Text::renderWithStencilBuffer(osg::State& state) const
             }
         }
 
-        drawForegroundText(state, glyphquad);
+        drawForegroundText(state, glyphquad, colorMultiplier);
     }
     
     glPopAttrib();
