@@ -98,15 +98,38 @@ CullVisitor::CullVisitor():
     _currentReuseRenderLeafIndex(0),
     _numberOfEncloseOverrideRenderBinDetails(0)
 {
-    // _nearFarRatio = 0.000005f;     
 }
 
+CullVisitor::CullVisitor(const CullVisitor& rhs):
+    NodeVisitor(rhs),
+    CullStack(rhs),
+    _currentStateGraph(NULL),
+    _currentRenderBin(NULL),
+    _computed_znear(FLT_MAX),
+    _computed_zfar(-FLT_MAX),
+    _currentReuseRenderLeafIndex(0),
+    _numberOfEncloseOverrideRenderBinDetails(0)
+{
+}
 
 CullVisitor::~CullVisitor()
 {
     reset();
 }
 
+osg::ref_ptr<CullVisitor>& CullVisitor::prototype()
+{
+    static osg::ref_ptr<CullVisitor> s_CullVisitor = new CullVisitor;
+    return s_CullVisitor;
+}
+
+CullVisitor* CullVisitor::create()
+{
+    return CullVisitor::prototype().valid() ? 
+           CullVisitor::prototype()->clone() :
+           new CullVisitor; 
+}
+
 
 void CullVisitor::reset()
 {
@@ -153,11 +176,14 @@ float CullVisitor::getDistanceToEyePoint(const Vec3& pos, bool withLODScale) con
     else return (pos-getEyeLocal()).length();
 }
 
+float CullVisitor::getDistanceToViewPoint(const Vec3& pos, bool withLODScale) const
+{
+    if (withLODScale) return (pos-getViewPointLocal()).length()*getLODScale();
+    else return (pos-getViewPointLocal()).length();
+}
+
 inline CullVisitor::value_type distance(const osg::Vec3& coord,const osg::Matrix& matrix)
 {
-
-    //std::cout << "distance("<<coord<<", "<<matrix<<")"<<std::endl;
-
     return -((CullVisitor::value_type)coord[0]*(CullVisitor::value_type)matrix(0,2)+(CullVisitor::value_type)coord[1]*(CullVisitor::value_type)matrix(1,2)+(CullVisitor::value_type)coord[2]*(CullVisitor::value_type)matrix(2,2)+matrix(3,2));
 }
 
@@ -170,9 +196,8 @@ float CullVisitor::getDistanceFromEyePoint(const osg::Vec3& pos, bool withLODSca
     else return dist;
 }
 
-void CullVisitor::popProjectionMatrix()
+void CullVisitor::computeNearPlane()
 {
-
     if (!_nearPlaneCandidateMap.empty())
     { 
     
@@ -196,7 +221,14 @@ void CullVisitor::popProjectionMatrix()
 
         // osg::Timer_t end_t = osg::Timer::instance()->tick();
         // osg::notify(osg::NOTICE)<<"Took "<<osg::Timer::instance()->delta_m(start_t,end_t)<<"ms to test "<<numTests<<" out of "<<_nearPlaneCandidateMap.size()<<std::endl;
+
+        _nearPlaneCandidateMap.clear();
     }
+}
+
+void CullVisitor::popProjectionMatrix()
+{
+    computeNearPlane();
 
     if (_computeNearFar && _computed_zfar>=_computed_znear)
     {
@@ -732,7 +764,7 @@ void CullVisitor::apply(Geode& node)
     // traverse any call callbacks and traverse any children.
     handle_cull_callbacks_and_traverse(node);
 
-    RefMatrix& matrix = getModelViewMatrix();
+    RefMatrix& matrix = *getModelViewMatrix();
     for(unsigned int i=0;i<node.getNumDrawables();++i)
     {
         Drawable* drawable = node.getDrawable(i);
@@ -740,7 +772,7 @@ void CullVisitor::apply(Geode& node)
 
         if( drawable->getCullCallback() )
         {
-            if( drawable->getCullCallback()->cull( this, drawable, _state.valid()?_state.get():NULL ) == true )
+            if( drawable->getCullCallback()->cull( this, drawable, &_renderInfo ) == true )
             continue;
         }
         
@@ -810,7 +842,7 @@ void CullVisitor::apply(Billboard& node)
     handle_cull_callbacks_and_traverse(node);
 
     const Vec3& eye_local = getEyeLocal();
-    const RefMatrix& modelview = getModelViewMatrix();
+    const RefMatrix& modelview = *getModelViewMatrix();
 
     for(unsigned int i=0;i<node.getNumDrawables();++i)
     {
@@ -820,6 +852,12 @@ void CullVisitor::apply(Billboard& node)
         // need to modify isCulled to handle the billboard offset.
         // if (isCulled(drawable->getBound())) continue;
 
+        if( drawable->getCullCallback() )
+        {
+            if( drawable->getCullCallback()->cull( this, drawable, &_renderInfo ) == true )
+                continue;
+        }
+
         RefMatrix* billboard_matrix = createOrReuseMatrix(modelview);
 
         node.computeMatrix(*billboard_matrix,eye_local,pos);
@@ -864,7 +902,7 @@ void CullVisitor::apply(LightSource& node)
     {
         if (node.getReferenceFrame()==osg::LightSource::RELATIVE_RF)
         {
-            RefMatrix& matrix = getModelViewMatrix();
+            RefMatrix& matrix = *getModelViewMatrix();
             addPositionedAttribute(&matrix,light);
         }
         else
@@ -886,7 +924,7 @@ void CullVisitor::apply(ClipNode& node)
     StateSet* node_state = node.getStateSet();
     if (node_state) pushStateSet(node_state);
 
-    RefMatrix& matrix = getModelViewMatrix();
+    RefMatrix& matrix = *getModelViewMatrix();
 
     const ClipNode::ClipPlaneList& planes = node.getClipPlaneList();
     for(ClipNode::ClipPlaneList::const_iterator itr=planes.begin();
@@ -911,7 +949,7 @@ void CullVisitor::apply(TexGenNode& node)
 
     if (node.getReferenceFrame()==osg::TexGenNode::RELATIVE_RF)
     {
-        RefMatrix& matrix = getModelViewMatrix();
+        RefMatrix& matrix = *getModelViewMatrix();
         addPositionedTextureAttribute(node.getTextureUnit(), &matrix ,node.getTexGen());
     }
     else
@@ -956,9 +994,9 @@ void CullVisitor::apply(Transform& node)
     StateSet* node_state = node.getStateSet();
     if (node_state) pushStateSet(node_state);
 
-    ref_ptr<RefMatrix> matrix = createOrReuseMatrix(getModelViewMatrix());
+    ref_ptr<RefMatrix> matrix = createOrReuseMatrix(*getModelViewMatrix());
     node.computeLocalToWorldMatrix(*matrix,this);
-    pushModelViewMatrix(matrix.get());
+    pushModelViewMatrix(matrix.get(), node.getReferenceFrame());
     
     handle_cull_callbacks_and_traverse(node);
 
@@ -1065,37 +1103,99 @@ void CullVisitor::apply(osg::ClearNode& node)
 
 }
 
-void CullVisitor::apply(osg::CameraNode& camera)
+namespace osgUtil
+{
+
+class RenderStageCache : public osg::Object
+{
+    public:
+    
+        RenderStageCache() {}
+        RenderStageCache(const RenderStageCache&, const osg::CopyOp&) {}
+        
+        META_Object(osgUtil, RenderStageCache);
+        
+        void setRenderStage(CullVisitor* cv, RenderStage* rs)
+        {
+            OpenThreads::ScopedLock<OpenThreads::Mutex> lock(_mutex);
+            _renderStageMap[cv] = rs;
+        }        
+
+        RenderStage* getRenderStage(osgUtil::CullVisitor* cv)
+        {
+            OpenThreads::ScopedLock<OpenThreads::Mutex> lock(_mutex);
+            return _renderStageMap[cv].get();
+        }
+        
+        typedef std::map<CullVisitor*, osg::ref_ptr<RenderStage> > RenderStageMap;
+        
+        OpenThreads::Mutex  _mutex;
+        RenderStageMap      _renderStageMap;
+};
+
+}
+
+void CullVisitor::apply(osg::Camera& camera)
 {
     // push the node's state.
     StateSet* node_state = camera.getStateSet();
     if (node_state) pushStateSet(node_state);
+
+    // Save current cull settings
+    CullSettings saved_cull_settings(*this);
+
+    // activate all active cull settings from this Camera
+    inheritCullSettings(camera);
+
+    // set the cull mask.
+    unsigned int savedTraversalMask = getTraversalMask();
+    bool mustSetCullMask = (camera.getInheritanceMask() & osg::CullSettings::CULL_MASK) == 0;
+    if (mustSetCullMask) setTraversalMask(camera.getCullMask());
+
+    RefMatrix& originalModelView = *getModelViewMatrix();
+
+    osg::RefMatrix* projection = 0;
+    osg::RefMatrix* modelview = 0;
+
+    if (camera.getReferenceFrame()==osg::Transform::RELATIVE_RF)
+    {
+        if (camera.getTransformOrder()==osg::Camera::POST_MULTIPLY)
+        {
+            projection = createOrReuseMatrix(*getProjectionMatrix()*camera.getProjectionMatrix());
+            modelview = createOrReuseMatrix(*getModelViewMatrix()*camera.getViewMatrix());
+        }
+        else // pre multiply 
+        {
+            projection = createOrReuseMatrix(camera.getProjectionMatrix()*(*getProjectionMatrix()));
+            modelview = createOrReuseMatrix(camera.getViewMatrix()*(*getModelViewMatrix()));
+        }
+    }
+    else
+    {
+        // an absolute reference frame
+        projection = createOrReuseMatrix(camera.getProjectionMatrix());
+        modelview = createOrReuseMatrix(camera.getViewMatrix());
+    }
+
+
+    if (camera.getViewport()) pushViewport(camera.getViewport());
+
+    // record previous near and far values.
+    float previous_znear = _computed_znear;
+    float previous_zfar = _computed_zfar;
     
-    // set the compute near far mode.
-    ComputeNearFarMode saved_compute_near_far_mode = getComputeNearFarMode();
-    setComputeNearFarMode( camera.getComputeNearFarMode());
+    // take a copy of the current near plane candidates
+    DistanceMatrixDrawableMap  previousNearPlaneCandidateMap;
+    previousNearPlaneCandidateMap.swap(_nearPlaneCandidateMap);
+
+    _computed_znear = FLT_MAX;
+    _computed_zfar = -FLT_MAX;
+
+    pushProjectionMatrix(projection);
+    pushModelViewMatrix(modelview, camera.getReferenceFrame());    
 
 
-    RefMatrix& originalModelView = getModelViewMatrix();
-
-    if (camera.getReferenceFrame()==osg::Transform::ABSOLUTE_RF)
-    {
-        pushProjectionMatrix(createOrReuseMatrix(camera.getProjectionMatrix()));
-        pushModelViewMatrix(createOrReuseMatrix(camera.getViewMatrix()));
-    }
-    else if (camera.getTransformOrder()==osg::CameraNode::POST_MULTIPLY)
-    {
-        pushProjectionMatrix(createOrReuseMatrix(getProjectionMatrix()*camera.getProjectionMatrix()));
-        pushModelViewMatrix(createOrReuseMatrix(getModelViewMatrix()*camera.getViewMatrix()));
-    }
-    else // pre multiply 
-    {
-        pushProjectionMatrix(createOrReuseMatrix(camera.getProjectionMatrix()*getProjectionMatrix()));
-        pushModelViewMatrix(createOrReuseMatrix(camera.getViewMatrix()*getModelViewMatrix()));
-    }
-
-
-    if (camera.getRenderOrder()==osg::CameraNode::NESTED_RENDER)
+    if (camera.getRenderOrder()==osg::Camera::NESTED_RENDER)
     {
         handle_cull_callbacks_and_traverse(camera);
     }
@@ -1106,17 +1206,26 @@ void CullVisitor::apply(osg::CameraNode& camera)
         // will do later.
         osgUtil::RenderStage* previous_stage = getCurrentRenderBin()->getStage();
 
-        unsigned int contextID = getState() ? getState()->getContextID() : 0;
+//        unsigned int contextID = getState() ? getState()->getContextID() : 0;
 
         // use render to texture stage.
         // create the render to texture stage.
-        osg::ref_ptr<osgUtil::RenderStage> rtts = dynamic_cast<osgUtil::RenderStage*>(camera.getRenderingCache(contextID));
+        osg::ref_ptr<osgUtil::RenderStageCache> rsCache = dynamic_cast<osgUtil::RenderStageCache*>(camera.getRenderingCache());
+        if (!rsCache)
+        {
+            rsCache = new osgUtil::RenderStageCache;
+            camera.setRenderingCache(rsCache.get());
+        }
+        
+        osg::ref_ptr<osgUtil::RenderStage> rtts = rsCache->getRenderStage(this);
         if (!rtts)
         {
             OpenThreads::ScopedLock<OpenThreads::Mutex> lock(*(camera.getDataChangeMutex()));
         
             rtts = new osgUtil::RenderStage;
-            rtts->setCameraNode(&camera);
+            rsCache->setRenderStage(this,rtts.get());
+
+            rtts->setCamera(&camera);
 
             if (camera.getDrawBuffer() != GL_NONE)
             {
@@ -1138,7 +1247,6 @@ void CullVisitor::apply(osg::CameraNode& camera)
                 rtts->setReadBuffer(previous_stage->getReadBuffer());
             }
 
-            camera.setRenderingCache(contextID, rtts.get());
         }
         else
         {
@@ -1149,7 +1257,14 @@ void CullVisitor::apply(osg::CameraNode& camera)
 
 
         // set up the background color and clear mask.
-        rtts->setClearColor(camera.getClearColor());
+        if (camera.getInheritanceMask() & CLEAR_COLOR)
+        {
+            rtts->setClearColor(camera.getClearColor());
+        }
+        else
+        {
+            rtts->setClearColor(previous_stage->getClearColor());
+        }
         rtts->setClearMask(camera.getClearMask());
         
         // set the color mask.
@@ -1164,7 +1279,7 @@ void CullVisitor::apply(osg::CameraNode& camera)
         // set up to charge the same PositionalStateContainer is the parent previous stage.
         osg::Matrix inhertiedMVtolocalMV;
         inhertiedMVtolocalMV.invert(originalModelView);
-        inhertiedMVtolocalMV.postMult(getModelViewMatrix());
+        inhertiedMVtolocalMV.postMult(*getModelViewMatrix());
         rtts->setInheritedPositionalStateContainerMatrix(inhertiedMVtolocalMV);
         rtts->setInheritedPositionalStateContainer(previous_stage->getPositionalStateContainer());
 
@@ -1175,24 +1290,11 @@ void CullVisitor::apply(osg::CameraNode& camera)
         // set the current renderbin to be the newly created stage.
         setCurrentRenderBin(rtts.get());
 
-        // set the cull traversal mask of camera node
-        osg::Node::NodeMask saved_mask = getCullMask();
-        if (camera.getInheritanceMask() & CULL_MASK)
-        {
-             setTraversalMask(camera.getCullMask());
-        }
-
         // traverse the subgraph
         {
             handle_cull_callbacks_and_traverse(camera);
         }
 
-        // restore the cull traversal mask of camera node
-        if (camera.getInheritanceMask() & CULL_MASK)
-        {
-            setTraversalMask(saved_mask);
-        }
-
         // restore the previous renderbin.
         setCurrentRenderBin(previousRenderBin);
      
@@ -1208,7 +1310,7 @@ void CullVisitor::apply(osg::CameraNode& camera)
         // dependancy list.
         switch(camera.getRenderOrder())
         {
-            case osg::CameraNode::PRE_RENDER:
+            case osg::Camera::PRE_RENDER:
                 getCurrentRenderBin()->getStage()->addPreRenderStage(rtts.get(),camera.getRenderOrderNum());
                 break;
             default:
@@ -1217,15 +1319,29 @@ void CullVisitor::apply(osg::CameraNode& camera)
         }
 
     }
-
+    
     // restore the previous model view matrix.
     popModelViewMatrix();
 
     // restore the previous model view matrix.
     popProjectionMatrix();
 
-    // restore the previous compute near far mode
-    setComputeNearFarMode(saved_compute_near_far_mode);
+
+    // restore the original near and far values
+    _computed_znear = previous_znear;
+    _computed_zfar = previous_zfar;
+
+    // swap back the near plane candidates
+    previousNearPlaneCandidateMap.swap(_nearPlaneCandidateMap);
+
+
+    if (camera.getViewport()) popViewport();
+
+    // restore the previous traversal mask settings
+    if (mustSetCullMask) setTraversalMask(savedTraversalMask);
+
+    // restore the previous cull settings
+    setCullSettings(saved_cull_settings);
 
     // pop the node's state off the render graph stack.    
     if (node_state) popStateSet();
