@@ -195,7 +195,7 @@ void RenderStage::addPostRenderStage(RenderStage* rs, int order)
     }
 }
 
-void RenderStage::drawPreRenderStages(osg::State& state,RenderLeaf*& previous)
+void RenderStage::drawPreRenderStages(osg::RenderInfo& renderInfo,RenderLeaf*& previous)
 {
     if (_preRenderList.empty()) return;
     
@@ -204,28 +204,30 @@ void RenderStage::drawPreRenderStages(osg::State& state,RenderLeaf*& previous)
         itr!=_preRenderList.end();
         ++itr)
     {
-        itr->second->draw(state,previous);
+        itr->second->draw(renderInfo,previous);
     }
     //cout << "Done Drawing prerendering stages "<<this<< "  "<<_viewport->x()<<","<< _viewport->y()<<","<< _viewport->width()<<","<< _viewport->height()<<std::endl;
 }
 
 
-void RenderStage::runCameraSetUp(osg::State& state)
+void RenderStage::runCameraSetUp(osg::RenderInfo& renderInfo)
 {
     _cameraRequiresSetUp = false;
 
     if (!_camera) return;
     
-    osg::CameraNode::RenderTargetImplementation renderTargetImplemntation = _camera->getRenderTargetImplementation();
-    osg::CameraNode::RenderTargetImplementation renderTargetFallback = _camera->getRenderTargetFallback();
+     osg::State& state = *renderInfo.getState();
 
-    osg::CameraNode::BufferAttachmentMap& bufferAttachements = _camera->getBufferAttachmentMap();
+    osg::Camera::RenderTargetImplementation renderTargetImplemntation = _camera->getRenderTargetImplementation();
+    osg::Camera::RenderTargetImplementation renderTargetFallback = _camera->getRenderTargetFallback();
+
+    osg::Camera::BufferAttachmentMap& bufferAttachements = _camera->getBufferAttachmentMap();
 
     // compute the required dimensions
-    int width = _viewport->x() + _viewport->width();
-    int height = _viewport->y() + _viewport->height();
+    int width = static_cast<int>(_viewport->x() + _viewport->width());
+    int height = static_cast<int>(_viewport->y() + _viewport->height());
     int depth = 1;
-    osg::CameraNode::BufferAttachmentMap::iterator itr;
+    osg::Camera::BufferAttachmentMap::iterator itr;
     for(itr = bufferAttachements.begin();
         itr != bufferAttachements.end();
         ++itr)
@@ -258,15 +260,11 @@ void RenderStage::runCameraSetUp(osg::State& state)
 
                 if (dataType==0) dataType = _imageReadPixelDataType;
                 if (dataType==0) dataType = GL_UNSIGNED_BYTE;
-
-                image->allocateImage(width, height, 1, pixelFormat, dataType);
-
             }
 
-            _imageReadPixelFormat = pixelFormat;
-            _imageReadPixelDataType = dataType;
-
-            setImage(itr->second._image.get());
+            _bufferAttachmentMap[itr->first]._imageReadPixelFormat = pixelFormat;
+            _bufferAttachmentMap[itr->first]._imageReadPixelDataType = dataType;
+            _bufferAttachmentMap[itr->first]._image = image;
         }
         
         if (itr->second._texture.valid())
@@ -317,14 +315,14 @@ void RenderStage::runCameraSetUp(osg::State& state)
         }
     }
     
-    if (renderTargetImplemntation==osg::CameraNode::FRAME_BUFFER_OBJECT)
+    if (renderTargetImplemntation==osg::Camera::FRAME_BUFFER_OBJECT)
     {
         osg::FBOExtensions* fbo_ext = osg::FBOExtensions::instance(state.getContextID(),true);
         bool fbo_supported = fbo_ext && fbo_ext->isSupported();
         
         if (fbo_supported && !_fbo)
         {
-            osg::notify(osg::INFO)<<"Setting up osg::CameraNode::FRAME_BUFFER_OBJECT"<<std::endl;
+            osg::notify(osg::INFO)<<"Setting up osg::Camera::FRAME_BUFFER_OBJECT"<<std::endl;
 
             OpenThreads::ScopedLock<OpenThreads::Mutex> lock(*(_camera->getDataChangeMutex()));
 
@@ -333,23 +331,23 @@ void RenderStage::runCameraSetUp(osg::State& state)
             bool colorAttached = false;
             bool depthAttached = false;
             bool stencilAttached = false;
-            for(osg::CameraNode::BufferAttachmentMap::iterator itr = bufferAttachements.begin();
+            for(osg::Camera::BufferAttachmentMap::iterator itr = bufferAttachements.begin();
                 itr != bufferAttachements.end();
                 ++itr)
             {
 
-                osg::CameraNode::BufferComponent buffer = itr->first;
-                osg::CameraNode::Attachment& attachment = itr->second;
+                osg::Camera::BufferComponent buffer = itr->first;
+                osg::Camera::Attachment& attachment = itr->second;
                 
                 switch(buffer)
                 {
-                    case(osg::CameraNode::DEPTH_BUFFER):
+                    case(osg::Camera::DEPTH_BUFFER):
                     {
                         fbo->setAttachment(GL_DEPTH_ATTACHMENT_EXT, osg::FrameBufferAttachment(attachment));
                         depthAttached = true;
                         break;
                     }
-                    case(osg::CameraNode::STENCIL_BUFFER):
+                    case(osg::Camera::STENCIL_BUFFER):
                     {
                         fbo->setAttachment(GL_STENCIL_ATTACHMENT_EXT, osg::FrameBufferAttachment(attachment));
                         stencilAttached = true;
@@ -357,7 +355,7 @@ void RenderStage::runCameraSetUp(osg::State& state)
                     }
                     default:
                     {
-                        fbo->setAttachment(GL_COLOR_ATTACHMENT0_EXT+(buffer-osg::CameraNode::COLOR_BUFFER0), osg::FrameBufferAttachment(attachment));
+                        fbo->setAttachment(GL_COLOR_ATTACHMENT0_EXT+(buffer-osg::Camera::COLOR_BUFFER0), osg::FrameBufferAttachment(attachment));
                         colorAttached = true;
                         break;
                     }
@@ -411,26 +409,26 @@ void RenderStage::runCameraSetUp(osg::State& state)
             if (renderTargetImplemntation<renderTargetFallback)
                 renderTargetImplemntation = renderTargetFallback;
             else
-                renderTargetImplemntation = osg::CameraNode::PIXEL_BUFFER_RTT;
+                renderTargetImplemntation = osg::Camera::PIXEL_BUFFER_RTT;
         }
     }
     
     // check whether PBuffer-RTT is supported or not
-    if (renderTargetImplemntation==osg::CameraNode::PIXEL_BUFFER_RTT && 
+    if (renderTargetImplemntation==osg::Camera::PIXEL_BUFFER_RTT && 
         !osg::isGLExtensionSupported(state.getContextID(), "WGL_ARB_render_texture"))
     {    
         if (renderTargetImplemntation<renderTargetFallback)
             renderTargetImplemntation = renderTargetFallback;
         else
-            renderTargetImplemntation = osg::CameraNode::PIXEL_BUFFER;
+            renderTargetImplemntation = osg::Camera::PIXEL_BUFFER;
     }
 
     // if any of the renderTargetImplementations require a seperate graphics context such as with pbuffer try in turn to
     // set up, but if each level fails then resort to the next level down.    
     while (!getGraphicsContext() &&
-           (renderTargetImplemntation==osg::CameraNode::PIXEL_BUFFER_RTT ||
-            renderTargetImplemntation==osg::CameraNode::PIXEL_BUFFER ||
-            renderTargetImplemntation==osg::CameraNode::SEPERATE_WINDOW) )
+           (renderTargetImplemntation==osg::Camera::PIXEL_BUFFER_RTT ||
+            renderTargetImplemntation==osg::Camera::PIXEL_BUFFER ||
+            renderTargetImplemntation==osg::Camera::SEPERATE_WINDOW) )
     {
         osg::ref_ptr<osg::GraphicsContext> context = getGraphicsContext();
         if (!context)
@@ -439,14 +437,14 @@ void RenderStage::runCameraSetUp(osg::State& state)
             // set up the traits of the graphics context that we want
             osg::ref_ptr<osg::GraphicsContext::Traits> traits = new osg::GraphicsContext::Traits;
 
-            traits->_width = width;
-            traits->_height = height;
+            traits->width = width;
+            traits->height = height;
 
-            // osg::notify(osg::NOTICE)<<"traits = "<<traits->_width<<" "<<traits->_height<<std::endl;
+            // osg::notify(osg::NOTICE)<<"traits = "<<traits->width<<" "<<traits->height<<std::endl;
 
-            traits->_pbuffer = (renderTargetImplemntation==osg::CameraNode::PIXEL_BUFFER || renderTargetImplemntation==osg::CameraNode::PIXEL_BUFFER_RTT);
-            traits->_windowDecoration = (renderTargetImplemntation==osg::CameraNode::SEPERATE_WINDOW);
-            traits->_doubleBuffer = (renderTargetImplemntation==osg::CameraNode::SEPERATE_WINDOW);
+            traits->pbuffer = (renderTargetImplemntation==osg::Camera::PIXEL_BUFFER || renderTargetImplemntation==osg::Camera::PIXEL_BUFFER_RTT);
+            traits->windowDecoration = (renderTargetImplemntation==osg::Camera::SEPERATE_WINDOW);
+            traits->doubleBuffer = (renderTargetImplemntation==osg::Camera::SEPERATE_WINDOW);
 
             osg::Texture* pBufferTexture = 0;
             GLenum bufferFormat = GL_NONE;
@@ -456,28 +454,28 @@ void RenderStage::runCameraSetUp(osg::State& state)
             bool colorAttached = false;
             bool depthAttached = false;
             bool stencilAttached = false;
-            for(osg::CameraNode::BufferAttachmentMap::iterator itr = bufferAttachements.begin();
+            for(osg::Camera::BufferAttachmentMap::iterator itr = bufferAttachements.begin();
                 itr != bufferAttachements.end();
                 ++itr)
             {
 
-                osg::CameraNode::BufferComponent buffer = itr->first;
-                osg::CameraNode::Attachment& attachment = itr->second;
+                osg::Camera::BufferComponent buffer = itr->first;
+                osg::Camera::Attachment& attachment = itr->second;
                 switch(buffer)
                 {
-                    case(osg::CameraNode::DEPTH_BUFFER):
+                    case(osg::Camera::DEPTH_BUFFER):
                     {
-                        traits->_depth = 24;
+                        traits->depth = 24;
                         depthAttached = true;
                         break;
                     }
-                    case(osg::CameraNode::STENCIL_BUFFER):
+                    case(osg::Camera::STENCIL_BUFFER):
                     {
-                        traits->_stencil = 8;
+                        traits->stencil = 8;
                         stencilAttached = true;
                         break;
                     }
-                    case(osg::CameraNode::COLOR_BUFFER):
+                    case(osg::Camera::COLOR_BUFFER):
                     {
                         if (attachment._internalFormat!=GL_NONE)
                         {
@@ -503,18 +501,19 @@ void RenderStage::runCameraSetUp(osg::State& state)
                         level = attachment._level;
                         face = attachment._face;
 
-                        if (renderTargetImplemntation==osg::CameraNode::PIXEL_BUFFER_RTT)
+                        if (renderTargetImplemntation==osg::Camera::PIXEL_BUFFER_RTT)
                         {
-                            traits->_target = bufferFormat;
-                            traits->_level = level;
-                            traits->_face = face;
-                            traits->_mipMapGeneration = attachment._mipMapGeneration;
+                            traits->target = attachment._texture.valid() ? attachment._texture->getTextureTarget() : 0;
+                            traits->format = bufferFormat;
+                            traits->level = level;
+                            traits->face = face;
+                            traits->mipMapGeneration = attachment._mipMapGeneration;
                         }
                         break;
                     }
                     default:
                     {
-                        if (renderTargetImplemntation==osg::CameraNode::SEPERATE_WINDOW)
+                        if (renderTargetImplemntation==osg::Camera::SEPERATE_WINDOW)
                             osg::notify(osg::NOTICE)<<"Warning: RenderStage::runCameraSetUp(State&) Window ";
                         else
                             osg::notify(osg::NOTICE)<<"Warning: RenderStage::runCameraSetUp(State&) Pbuffer ";
@@ -528,29 +527,30 @@ void RenderStage::runCameraSetUp(osg::State& state)
 
             if (!depthAttached)
             {                
-                traits->_depth = 24;
+                traits->depth = 24;
             }
 
             if (!colorAttached)
             {                
                 if (bufferFormat == GL_NONE) bufferFormat = GL_RGB;
 
-                traits->_red = 8;
-                traits->_green = 8;
-                traits->_blue = 8;
-                traits->_alpha = (bufferFormat==GL_RGBA) ? 8 : 0;
+                traits->red = 8;
+                traits->green = 8;
+                traits->blue = 8;
+                traits->alpha = (bufferFormat==GL_RGBA) ? 8 : 0;
             }
 
             // share OpenGL objects if possible...
             if (state.getGraphicsContext())
             {
-                traits->_sharedContext = state.getGraphicsContext();
+                traits->sharedContext = state.getGraphicsContext();
                 
-                const osg::GraphicsContext::Traits* sharedTraits = traits->_sharedContext->getTraits();
+                const osg::GraphicsContext::Traits* sharedTraits = traits->sharedContext->getTraits();
                 if (sharedTraits)
                 {
-                    traits->_displayNum = sharedTraits->_displayNum;
-                    traits->_screenNum = sharedTraits->_screenNum;
+                    traits->hostName = sharedTraits->hostName;
+                    traits->displayNum = sharedTraits->displayNum;
+                    traits->screenNum = sharedTraits->screenNum;
                 }
             }
 
@@ -572,7 +572,7 @@ void RenderStage::runCameraSetUp(osg::State& state)
                 setDrawBuffer(GL_FRONT);
                 setReadBuffer(GL_FRONT);
 
-                if (pBufferTexture && renderTargetImplemntation==osg::CameraNode::PIXEL_BUFFER_RTT)
+                if (pBufferTexture && renderTargetImplemntation==osg::Camera::PIXEL_BUFFER_RTT)
                 {
                    osg::notify(osg::INFO)<<"RenderStage::runCameraSetUp(State&) Assign graphis context to Texture"<<std::endl;
                    pBufferTexture->setReadPBuffer(context.get());
@@ -587,17 +587,17 @@ void RenderStage::runCameraSetUp(osg::State& state)
             {
                 osg::notify(osg::INFO)<<"Failed to aquire Graphics Context"<<std::endl;
                 
-                if (renderTargetImplemntation==osg::CameraNode::PIXEL_BUFFER_RTT)
+                if (renderTargetImplemntation==osg::Camera::PIXEL_BUFFER_RTT)
                 {
                     // fallback to using standard PBuffer, this will allow this while loop to continue
                     if (renderTargetImplemntation<renderTargetFallback)
                         renderTargetImplemntation = renderTargetFallback;
                     else
-                        renderTargetImplemntation = osg::CameraNode::PIXEL_BUFFER;
+                        renderTargetImplemntation = osg::Camera::PIXEL_BUFFER;
                 }
                 else 
                 {
-                    renderTargetImplemntation = osg::CameraNode::FRAME_BUFFER;
+                    renderTargetImplemntation = osg::Camera::FRAME_BUFFER;
                 }
             }
 
@@ -605,11 +605,11 @@ void RenderStage::runCameraSetUp(osg::State& state)
     }
     
     // finally if all else has failed, then the frame buffer fallback will come in to play.
-    if (renderTargetImplemntation==osg::CameraNode::FRAME_BUFFER)
+    if (renderTargetImplemntation==osg::Camera::FRAME_BUFFER)
     {
-        osg::notify(osg::INFO)<<"Setting up osg::CameraNode::FRAME_BUFFER"<<std::endl;
+        osg::notify(osg::INFO)<<"Setting up osg::Camera::FRAME_BUFFER"<<std::endl;
 
-        for(osg::CameraNode::BufferAttachmentMap::iterator itr = bufferAttachements.begin();
+        for(osg::Camera::BufferAttachmentMap::iterator itr = bufferAttachements.begin();
             itr != bufferAttachements.end();
             ++itr)
         {
@@ -620,8 +620,10 @@ void RenderStage::runCameraSetUp(osg::State& state)
 
 }
 
-void RenderStage::copyTexture(osg::State& state)
+void RenderStage::copyTexture(osg::RenderInfo& renderInfo)
 {
+    osg::State& state = *renderInfo.getState();
+
     if (_readBuffer != GL_NONE)
     {
         glReadBuffer(_readBuffer);
@@ -634,64 +636,67 @@ void RenderStage::copyTexture(osg::State& state)
     osg::TextureRectangle* textureRec = 0;
     osg::TextureCubeMap* textureCubeMap = 0;
 
-#if 1
     // use TexCopySubImage with the offset of the viewport into the texture
     // note, this path mirrors the pbuffer and fbo means for updating the texture.
     // Robert Osfield, 3rd August 2006.
     if ((texture2D = dynamic_cast<osg::Texture2D*>(_texture.get())) != 0)
     {
-        texture2D->copyTexSubImage2D(state,_viewport->x(),_viewport->y(), _viewport->x(),_viewport->y(),_viewport->width(),_viewport->height());
+        texture2D->copyTexSubImage2D(state,
+                                     static_cast<int>(_viewport->x()),
+                                     static_cast<int>(_viewport->y()),
+                                     static_cast<int>(_viewport->x()),
+                                     static_cast<int>(_viewport->y()),
+                                     static_cast<int>(_viewport->width()),
+                                     static_cast<int>(_viewport->height()));
     }
     else if ((textureRec = dynamic_cast<osg::TextureRectangle*>(_texture.get())) != 0)
     {
-        textureRec->copyTexSubImage2D(state,_viewport->x(),_viewport->y(), _viewport->x(),_viewport->y(),_viewport->width(),_viewport->height());
+        textureRec->copyTexSubImage2D(state,
+                                     static_cast<int>(_viewport->x()),
+                                     static_cast<int>(_viewport->y()),
+                                     static_cast<int>(_viewport->x()),
+                                     static_cast<int>(_viewport->y()),
+                                     static_cast<int>(_viewport->width()),
+                                     static_cast<int>(_viewport->height()));
     }
     else if ((texture1D = dynamic_cast<osg::Texture1D*>(_texture.get())) != 0)
     {
         // need to implement
-        texture1D->copyTexSubImage1D(state,_viewport->x(), _viewport->x(),_viewport->y(),_viewport->width());
+        texture1D->copyTexSubImage1D(state,
+                                     static_cast<int>(_viewport->x()),
+                                     static_cast<int>(_viewport->x()),
+                                     static_cast<int>(_viewport->y()),
+                                     static_cast<int>(_viewport->width()));
     }
     else if ((texture3D = dynamic_cast<osg::Texture3D*>(_texture.get())) != 0)
     {
         // need to implement
-        texture3D->copyTexSubImage3D(state, _viewport->x(), _viewport->y(), _face, _viewport->x(), _viewport->y(), _viewport->width(), _viewport->height());
+        texture3D->copyTexSubImage3D(state, 
+                                     static_cast<int>(_viewport->x()),
+                                     static_cast<int>(_viewport->y()),
+                                     _face,
+                                     static_cast<int>(_viewport->x()),
+                                     static_cast<int>(_viewport->y()),
+                                     static_cast<int>(_viewport->width()),
+                                     static_cast<int>(_viewport->height()));
     }
     else if ((textureCubeMap = dynamic_cast<osg::TextureCubeMap*>(_texture.get())) != 0)
     {
         // need to implement
-        textureCubeMap->copyTexSubImageCubeMap(state, _face, _viewport->x(), _viewport->y(), _viewport->x(),_viewport->y(),_viewport->width(),_viewport->height());
+        textureCubeMap->copyTexSubImageCubeMap(state, _face, 
+                                     static_cast<int>(_viewport->x()),
+                                     static_cast<int>(_viewport->y()),
+                                     static_cast<int>(_viewport->x()),
+                                     static_cast<int>(_viewport->y()),
+                                     static_cast<int>(_viewport->width()),
+                                     static_cast<int>(_viewport->height()));
     }
-#else
-    // use CopySubImage with the offset set to 0,0
-    // original code path.
-    if ((texture2D = dynamic_cast<osg::Texture2D*>(_texture.get())) != 0)
-    {
-        texture2D->copyTexImage2D(state,_viewport->x(),_viewport->y(),_viewport->width(),_viewport->height());
-    }
-    else if ((textureRec = dynamic_cast<osg::TextureRectangle*>(_texture.get())) != 0)
-    {
-        textureRec->copyTexImage2D(state,_viewport->x(),_viewport->y(),_viewport->width(),_viewport->height());
-    }
-    else if ((texture1D = dynamic_cast<osg::Texture1D*>(_texture.get())) != 0)
-    {
-        // need to implement
-        texture1D->copyTexImage1D(state,_viewport->x(),_viewport->y(),_viewport->width());
-    }
-    else if ((texture3D = dynamic_cast<osg::Texture3D*>(_texture.get())) != 0)
-    {
-        // need to implement
-        texture3D->copyTexSubImage3D(state, 0, 0, _face, _viewport->x(), _viewport->y(), _viewport->width(), _viewport->height());
-    }
-    else if ((textureCubeMap = dynamic_cast<osg::TextureCubeMap*>(_texture.get())) != 0)
-    {
-        // need to implement
-        textureCubeMap->copyTexSubImageCubeMap(state, _face, 0, 0, _viewport->x(),_viewport->y(),_viewport->width(),_viewport->height());
-    }
-#endif    
 }
 
-void RenderStage::drawInner(osg::State& state,RenderLeaf*& previous, bool& doCopyTexture)
+void RenderStage::drawInner(osg::RenderInfo& renderInfo,RenderLeaf*& previous, bool& doCopyTexture)
 {
+    osg::State& state = *renderInfo.getState();
+
     if (_drawBuffer != GL_NONE)
     {    
         glDrawBuffer(_drawBuffer);
@@ -711,7 +716,7 @@ void RenderStage::drawInner(osg::State& state,RenderLeaf*& previous, bool& doCop
     }
 
     // do the drawing itself.    
-    RenderBin::draw(state,previous);
+    RenderBin::draw(renderInfo,previous);
 
 
     if(state.getCheckForGLErrors()!=osg::State::NEVER_CHECK_GL_ERRORS)
@@ -730,30 +735,38 @@ void RenderStage::drawInner(osg::State& state,RenderLeaf*& previous, bool& doCop
     // now copy the rendered image to attached texture.
     if (doCopyTexture)
     {
-        copyTexture(state);
+        copyTexture(renderInfo);
     }
     
-    if (_image.valid())
+    std::map< osg::Camera::BufferComponent, Attachment>::const_iterator itr;
+    for(itr = _bufferAttachmentMap.begin();
+        itr != _bufferAttachmentMap.end();
+        ++itr)
     {
+         if (itr->second._image.valid())
+         {
 
-        if (_readBuffer != GL_NONE)
-        {
-            glReadBuffer(_readBuffer);
-        }
+             if (_readBuffer != GL_NONE)
+             {
+                 glReadBuffer(_readBuffer);
+             }
 
-        GLenum pixelFormat = _image->getPixelFormat();
-        if (pixelFormat==0) pixelFormat = _imageReadPixelFormat;
-        if (pixelFormat==0) pixelFormat = GL_RGB;
+             GLenum pixelFormat = itr->second._image->getPixelFormat();
+              if (pixelFormat==0) pixelFormat = _imageReadPixelFormat;
+             if (pixelFormat==0) pixelFormat = GL_RGB;
 
-        GLenum dataType = _image->getDataType();
-        if (dataType==0) dataType =  _imageReadPixelDataType;
-        if (dataType==0) dataType = GL_UNSIGNED_BYTE;       
-        
-        _image->readPixels(_viewport->x(), _viewport->y(),
-                           _viewport->width(), _viewport->height(), 
-                           pixelFormat, dataType);
+             GLenum dataType = itr->second._image->getDataType();
+              if (dataType==0) dataType =  _imageReadPixelDataType;
+             if (dataType==0) dataType = GL_UNSIGNED_BYTE;       
 
-    }
+             itr->second._image->readPixels(static_cast<int>(_viewport->x()),
+                                            static_cast<int>(_viewport->y()),
+                                            static_cast<int>(_viewport->width()),
+                                            static_cast<int>(_viewport->height()), 
+                                            pixelFormat, dataType);
+
+         }
+     }
        
     
     if (fbo_supported)
@@ -767,8 +780,8 @@ void RenderStage::drawInner(osg::State& state,RenderLeaf*& previous, bool& doCop
     if (fbo_supported && _camera)
     {
         // now generate mipmaps if they are required.
-        const osg::CameraNode::BufferAttachmentMap& bufferAttachements = _camera->getBufferAttachmentMap();
-        for(osg::CameraNode::BufferAttachmentMap::const_iterator itr = bufferAttachements.begin();
+        const osg::Camera::BufferAttachmentMap& bufferAttachements = _camera->getBufferAttachmentMap();
+        for(osg::Camera::BufferAttachmentMap::const_iterator itr = bufferAttachements.begin();
             itr != bufferAttachements.end();
             ++itr)
         {
@@ -782,28 +795,34 @@ void RenderStage::drawInner(osg::State& state,RenderLeaf*& previous, bool& doCop
     }
 }
 
-struct DrawInnerOperation : public osg::GraphicsThread::Operation
+struct DrawInnerOperation : public osg::Operation
 {
-    DrawInnerOperation(RenderStage* stage) : 
-        osg::GraphicsThread::Operation("DrawInnerStage",false),
-        _stage(stage) {}
+    DrawInnerOperation(RenderStage* stage, osg::RenderInfo& renderInfo) : 
+        osg::Operation("DrawInnerStage",false),
+        _stage(stage),
+        _renderInfo(renderInfo) {}
 
-    virtual void operator() (osg::GraphicsContext* context)
+    virtual void operator () (osg::Object* object)
     {
+        osg::GraphicsContext* context = dynamic_cast<osg::GraphicsContext*>(object);
+        if (!context) return;
+
         // osg::notify(osg::NOTICE)<<"DrawInnerOperation operator"<<std::endl;
         if (_stage && context)
         {
             RenderLeaf* previous = 0;
             bool doCopyTexture = false;
-            _stage->drawInner(*(context->getState()), previous, doCopyTexture);
+            _renderInfo.setState(context->getState());
+            _stage->drawInner(_renderInfo, previous, doCopyTexture);
         }
     }
     
     RenderStage* _stage;
+    RenderInfo _renderInfo;
 };
 
 
-void RenderStage::draw(osg::State& state,RenderLeaf*& previous)
+void RenderStage::draw(osg::RenderInfo& renderInfo,RenderLeaf*& previous)
 {
     if (_stageDrawnThisFrame) return;
 
@@ -811,18 +830,22 @@ void RenderStage::draw(osg::State& state,RenderLeaf*& previous)
 
     // note, SceneView does call to drawPreRenderStages explicitly
     // so there is no need to call it here.
-    drawPreRenderStages(state,previous);
+    drawPreRenderStages(renderInfo,previous);
     
     if (_cameraRequiresSetUp)
     {
-        runCameraSetUp(state);
+        runCameraSetUp(renderInfo);
     }
 
+    osg::State& state = *renderInfo.getState();
 
     osg::State* useState = &state;
     osg::GraphicsContext* callingContext = state.getGraphicsContext();
     osg::GraphicsContext* useContext = callingContext;
-    osg::GraphicsThread* useThread = 0;
+    osg::OperationThread* useThread = 0;
+    osg::RenderInfo useRenderInfo(renderInfo);
+    
+    RenderLeaf* saved_previous = previous;
 
     if (_graphicsContext.valid() && _graphicsContext != callingContext)
     {
@@ -830,14 +853,35 @@ void RenderStage::draw(osg::State& state,RenderLeaf*& previous)
         // now as an experiment.
         callingContext->releaseContext();
     
+        // osg::notify(osg::NOTICE)<<"  enclosing state before - "<<state.getStateSetStackSize()<<std::endl;
+
         useState = _graphicsContext->getState();
         useContext = _graphicsContext.get();
         useThread = useContext->getGraphicsThread();
+        useRenderInfo.setState(useState);
         
         // syncronize the frame stamps
         useState->setFrameStamp(const_cast<osg::FrameStamp*>(state.getFrameStamp()));
+
+        // map the DynamicObjectCount across to the new window
+        useState->setDynamicObjectCount(state.getDynamicObjectCount());
+        useState->setDynamicObjectRenderingCompletedCallback(state.getDynamicObjectRenderingCompletedCallback());
         
-        if (!useThread) useContext->makeCurrent();
+        if (!useThread) 
+        {
+            previous = 0;
+            useContext->makeCurrent();
+            
+            // osg::notify(osg::NOTICE)<<"  nested state before - "<<useState->getStateSetStackSize()<<std::endl;
+        }
+    }
+
+    unsigned int originalStackSize = useState->getStateSetStackSize();
+
+    if (_camera && _camera->getPreDrawCallback())
+    {
+        // if we have a camera with a post draw callback invoke it.
+        (*(_camera->getPreDrawCallback()))(*_camera);
     }
     
     bool doCopyTexture = _texture.valid() ? 
@@ -846,13 +890,40 @@ void RenderStage::draw(osg::State& state,RenderLeaf*& previous)
 
     if (useThread)
     {
-        useThread->add(new DrawInnerOperation( this ), true);
+#if 1
+        ref_ptr<osg::BlockAndFlushOperation> block = new osg::BlockAndFlushOperation;
+    
+        useThread->add(new DrawInnerOperation( this, renderInfo ));
+        
+        useThread->add(block.get());
+        
+        // wait till the DrawInnerOperations is complete.
+        block->block();
         
         doCopyTexture = false;
+        
+#else
+        useThread->add(new DrawInnerOperation( this, renderInfo ), true);
+        
+        doCopyTexture = false;
+#endif        
     }
     else
     {
-        drawInner( *useState, previous, doCopyTexture);
+        drawInner( useRenderInfo, previous, doCopyTexture);
+        
+        if (useRenderInfo.getUserData() != renderInfo.getUserData())
+        {
+            renderInfo.setUserData(useRenderInfo.getUserData());
+        }
+        
+    }
+
+    if (useState != &state)
+    {
+        // reset the local State's DynamicObjectCount
+        state.setDynamicObjectCount(useState->getDynamicObjectCount());
+        useState->setDynamicObjectRenderingCompletedCallback(0);
     }
 
 
@@ -865,7 +936,7 @@ void RenderStage::draw(osg::State& state,RenderLeaf*& previous)
             callingContext->makeContextCurrent(useContext);
         }
 
-        copyTexture(state);
+        copyTexture(renderInfo);
     }
 
     if (_camera && _camera->getPostDrawCallback())
@@ -876,35 +947,60 @@ void RenderStage::draw(osg::State& state,RenderLeaf*& previous)
 
     if (_graphicsContext.valid() && _graphicsContext != callingContext)
     {
-        if (!useThread) useContext->releaseContext();
+        useState->popStateSetStackToSize(originalStackSize);
+
+        if (!useThread)
+        {
+
+
+            // flush any command left in the useContex's FIFO
+            // to ensure that textures are updated before main thread commenses.
+            glFlush();
+            
+        
+            useContext->releaseContext();
+        }
     }
 
     if (callingContext && useContext != callingContext)
     {
         // restore the graphics context.
+        
+        previous = saved_previous;
+        
+        // osg::notify(osg::NOTICE)<<"  nested state after - "<<useState->getStateSetStackSize()<<std::endl;
+        // osg::notify(osg::NOTICE)<<"  enclosing state after - "<<state.getStateSetStackSize()<<std::endl;
+
         callingContext->makeCurrent();
     }
 
     // place the post draw here temprorarily while we figure out how
     // best to do SceneView.
-    drawPostRenderStages(state,previous);
+    drawPostRenderStages(renderInfo,previous);
 }
 
-void RenderStage::drawImplementation(osg::State& state,RenderLeaf*& previous)
+void RenderStage::drawImplementation(osg::RenderInfo& renderInfo,RenderLeaf*& previous)
 {
+    osg::State& state = *renderInfo.getState();
 
     if (!_viewport)
     {
         notify(FATAL) << "Error: cannot draw stage due to undefined viewport."<< std::endl;
         return;
     }
-     
+
+    // push the stages camera so that drawing code can querry it     
+    if (_camera) renderInfo.pushCamera(_camera);
+
     // set up the back buffer.
     state.applyAttribute(_viewport.get());
 
 #define USE_SISSOR_TEST
 #ifdef USE_SISSOR_TEST
-    glScissor( _viewport->x(), _viewport->y(), _viewport->width(), _viewport->height() );
+    glScissor( static_cast<int>(_viewport->x()),
+               static_cast<int>(_viewport->y()),
+               static_cast<int>(_viewport->width()),
+               static_cast<int>(_viewport->height()) );
     //cout << "    clearing "<<this<< "  "<<_viewport->x()<<","<< _viewport->y()<<","<< _viewport->width()<<","<< _viewport->height()<<std::endl;
     
     glEnable( GL_SCISSOR_TEST );
@@ -952,12 +1048,16 @@ void RenderStage::drawImplementation(osg::State& state,RenderLeaf*& previous)
     }
 
     // draw the children and local.
-    RenderBin::drawImplementation(state,previous);
+    RenderBin::drawImplementation(renderInfo,previous);
 
     state.apply();
+
+    // pop the render stages camera.
+    if (_camera) renderInfo.popCamera();
+
 }
 
-void RenderStage::drawPostRenderStages(osg::State& state,RenderLeaf*& previous)
+void RenderStage::drawPostRenderStages(osg::RenderInfo& renderInfo,RenderLeaf*& previous)
 {
     if (_postRenderList.empty()) return;
     
@@ -966,7 +1066,7 @@ void RenderStage::drawPostRenderStages(osg::State& state,RenderLeaf*& previous)
         itr!=_postRenderList.end();
         ++itr)
     {
-        itr->second->draw(state,previous);
+        itr->second->draw(renderInfo,previous);
     }
     //cout << "Done Drawing prerendering stages "<<this<< "  "<<_viewport->x()<<","<< _viewport->y()<<","<< _viewport->width()<<","<< _viewport->height()<<std::endl;
 }
@@ -1002,3 +1102,32 @@ bool RenderStage::getStats(Statistics& stats) const
     }
     return statsCollected;
 }
+
+void RenderStage::attach(osg::Camera::BufferComponent buffer, osg::Image* image)
+{
+    _bufferAttachmentMap[buffer]._image = image;
+}
+
+unsigned int RenderStage::computeNumberOfDynamicRenderLeaves() const
+{
+    unsigned int count = 0;
+
+    for(RenderStageList::const_iterator pre_itr = _preRenderList.begin();
+        pre_itr != _preRenderList.end();
+        ++pre_itr)
+    {
+        count += pre_itr->second->computeNumberOfDynamicRenderLeaves();
+    }
+
+    count += RenderBin::computeNumberOfDynamicRenderLeaves();
+
+    for(RenderStageList::const_iterator post_itr = _postRenderList.begin();
+        post_itr != _postRenderList.end();
+        ++post_itr)
+    {
+        count += post_itr->second->computeNumberOfDynamicRenderLeaves();
+    }
+    
+    return count;
+}
+
